@@ -144,10 +144,10 @@ static void BusyTimesLimitReport(KHE_SOLN soln, KHE_AVAIL_SOLVER as,
   /* one row for each avail node */
   *total = 0;
   leftovers = KheInstanceTimeCount(KheSolnInstance(soln));
-  count = KheAvailSolverMaxBusyTimesAvailNodeCount(as);
+  count = KheAvailSolverMaxBusyTimesAvailNodeCount(as, r);
   for( i = 0;  i < count;  i++ )
   {
-    KheAvailSolverMaxBusyTimesAvailNode(as, i, &type, &limit, &ts, &m);
+    KheAvailSolverMaxBusyTimesAvailNode(as, r, i, &type, &limit, &ts, &m);
     HTMLTableRowBegin(html);
 
     /* limit */
@@ -339,8 +339,8 @@ static void BusyTimesTotalReport(KHE_SOLN soln, KHE_RESOURCE r,
 static void WorkloadLimitReport(KHE_SOLN soln, KHE_AVAIL_SOLVER as,
   KHE_RESOURCE r, HTML html, float *total)
 {
-  int i, limit, count;  KHE_AVAIL_NODE_TYPE type;  KHE_TIME_SET ts;
-  KHE_MONITOR m;
+  int i, count;  KHE_AVAIL_NODE_TYPE type;  KHE_TIME_SET ts;
+  KHE_MONITOR m;  float limit;
 
   /* begin table */
   HTMLParagraphBegin(html);
@@ -356,15 +356,15 @@ static void WorkloadLimitReport(KHE_SOLN soln, KHE_AVAIL_SOLVER as,
 
   /* one row for each avail node */
   *total = 0.0;
-  count = KheAvailSolverMaxWorkloadAvailNodeCount(as);
+  count = KheAvailSolverMaxWorkloadAvailNodeCount(as, r);
   for( i = 0;  i < count;  i++ )
   {
-    KheAvailSolverMaxWorkloadAvailNode(as, i, &type, &limit, &ts, &m);
+    KheAvailSolverMaxWorkloadAvailNode(as, r, i, &type, &limit, &ts, &m);
     HTMLTableRowBegin(html);
 
     /* limit */
     HTMLTableEntryRightBegin(html);
-    HTMLText(html, "%.1f", (float) limit);
+    HTMLText(html, "%.1f", limit);
     *total += limit;
     HTMLTableEntryEnd(html);
 
@@ -529,7 +529,7 @@ static void ResourceAvailabilityReportHTML(KHE_SOLN soln, KHE_RESOURCE r,
 
   /* set the avail solver */
   as = KheSolnAvailSolver(soln);
-  KheAvailSolverSetResource(as, r);
+  /* KheAvailSolverSetResource(as, r); */
 
   /* display the resource name as a heading */
   HTMLParagraphBegin(html);
@@ -538,7 +538,7 @@ static void ResourceAvailabilityReportHTML(KHE_SOLN soln, KHE_RESOURCE r,
 
   /* print an optional max busy times report */
   done_count = 0;
-  if( KheAvailSolverMaxBusyTimes(as, &limit) )
+  if( KheAvailSolverMaxBusyTimes(as, r, &limit) )
   {
     BusyTimesLimitReport(soln, as, r, html, &max_busy_times);
     BusyTimesTotalReport(soln, r, html, &busy_times);
@@ -550,7 +550,7 @@ static void ResourceAvailabilityReportHTML(KHE_SOLN soln, KHE_RESOURCE r,
   }
 
   /* print an optional max workload report */
-  if( KheAvailSolverMaxWorkload(as, &lim) )
+  if( KheAvailSolverMaxWorkload(as, r, &lim) )
   {
     WorkloadLimitReport(soln, as, r, html, &max_workload);
     WorkloadTotalReport(soln, r, html, &workload);
@@ -571,6 +571,48 @@ static void ResourceAvailabilityReportHTML(KHE_SOLN soln, KHE_RESOURCE r,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheResourceHasAvailability(KHE_SOLN soln, KHE_RESOURCE r)           */
+/*                                                                           */
+/*  Return true if there is something to say about the availability of       */
+/*  resource r.                                                              */
+/*                                                                           */
+/*****************************************************************************/
+
+static bool KheResourceHasAvailability(KHE_SOLN soln, KHE_RESOURCE r)
+{
+  KHE_AVAIL_SOLVER as;  int limit;  float lim;
+  as = KheSolnAvailSolver(soln);
+  /* KheAvailSolverSetResource(as, r); */
+  return KheAvailSolverMaxBusyTimes(as, r, &limit) ||
+    KheAvailSolverMaxWorkload(as, r, &lim);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool ResourceTypeHasAvailability(KHE_SOLN soln, KHE_RESOURCE_TYPE rt)    */
+/*                                                                           */
+/*  Return true if there is something to say about the availability of       */
+/*  resources of type rt.                                                    */
+/*                                                                           */
+/*****************************************************************************/
+
+static bool ResourceTypeHasAvailability(KHE_SOLN soln, KHE_RESOURCE_TYPE rt)
+{
+  int i;  KHE_RESOURCE r;
+
+  for( i = 0;  i < KheResourceTypeResourceCount(rt);  i++ )
+  {
+    r = KheResourceTypeResource(rt, i);
+    if( KheResourceHasAvailability(soln, r) )
+      return true;
+  }
+  return false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void ResourceTypeAvailabilityReportHTML(KHE_SOLN soln,                   */
@@ -592,11 +634,22 @@ static void ResourceTypeAvailabilityReportHTML(KHE_SOLN soln,
   HTMLHeadingEnd(html);
   HTMLParagraphEnd(html);
 
-  /* reports */
-  for( i = 0;  i < KheResourceTypeResourceCount(rt);  i++ )
+  if( ResourceTypeHasAvailability(soln, rt) )
   {
-    r = KheResourceTypeResource(rt, i);
-    ResourceAvailabilityReportHTML(soln, r, html);
+    /* print one report for each resource */
+    for( i = 0;  i < KheResourceTypeResourceCount(rt);  i++ )
+    {
+      r = KheResourceTypeResource(rt, i);
+      ResourceAvailabilityReportHTML(soln, r, html);
+    }
+  }
+  else
+  {
+    /* say that nothing is needed */
+    HTMLParagraphBegin(html);
+    HTMLText(html, "There are no limits on the availability of resources "
+     "of type %s.", KheResourceTypeId(rt));
+    HTMLParagraphEnd(html);
   }
 }
 
@@ -683,7 +736,7 @@ void AvailabilityReportHTML(COMMAND c, HA_ARENA_SET as)
   if( DEBUG3 )
     fprintf(stderr, "[ AvailabilityReportHTML(cgi, as)\n");
 
-  archive = ReadAndVerifyArchive(c, true, KHE_SOLN_ORDINARY, as);
+  archive = ReadAndVerifyArchive(c, true, false, KHE_SOLN_ORDINARY, as);
   snprintf(buff, 100, "%s Availability Report",
     KheArchiveId(archive) != NULL && strlen(KheArchiveId(archive)) < 70 ?
     KheArchiveId(archive) : "HSEval");
