@@ -35,6 +35,7 @@
 #include <stdarg.h>
 
 #define DEBUG1 0
+#define DEBUG2 0
 
 /*****************************************************************************/
 /*                                                                           */
@@ -163,6 +164,7 @@ static void CommandInitRerun(COMMAND c, bool rerun)
     fprintf(c->u.post.rerun, "rm -f core*\n");
     fprintf(c->u.post.rerun, "MALLOC_CHECK_=2 exec ./hseval.cgi <<EOF\n");
     chmod("./rerun", S_IRUSR | S_IWUSR | S_IXUSR | S_IROTH | S_IWOTH | S_IXOTH);
+    fflush(c->u.post.rerun);
   }
   else
     c->u.post.rerun = NULL;
@@ -203,6 +205,8 @@ COMMAND CommandBeginCGI(bool rerun, COMMAND_PAGE_BEGIN_FN page_begin_fn,
     res->u.get.query_string = getenv("QUERY_STRING");
     if( res->u.get.query_string == NULL )
       res->u.get.query_string = res->u.get.null_query_string;
+    if( DEBUG1 )
+      fprintf(stderr, "  query_string: \"%s\"\n", res->u.get.query_string);
   }
   if( DEBUG1 )
     fprintf(stderr, "] CommandBeginCGI returning (type %d)\n", res->type);
@@ -329,6 +333,10 @@ bool CommandNextPair(COMMAND c, char **key, char **val, FILE **file_val)
 
       /* read one mime separator into sep, exiting if not present */
       p = fgets(line, MAX_LINE, stdin);  /* p unused, keep compiler happy */
+      if( p == NULL )
+	line[0] = '\0';
+      if( DEBUG2 )
+	fprintf(stderr, "  CommandNextPair at (a)\n"); 
       if( c->u.post.rerun != NULL )
 	fputs(line, c->u.post.rerun);
       if( line[0] == '\n' || line[0] == '\0' )
@@ -346,10 +354,14 @@ bool CommandNextPair(COMMAND c, char **key, char **val, FILE **file_val)
 	strcpy(c->u.post.mime_sep, line);
       else if( strcmp(line, c->u.post.mime_sep) != 0 )
 	CommandError(c, "mime separator changed or not found where expected");
+      if( DEBUG2 )
+	fprintf(stderr, "  CommandNextPair at (b)\n"); 
 
       /* read header lines */
       *key = NULL;
-      p = fgets(line, MAX_LINE, stdin);  /* p unused, keep compiler happy */
+      p = fgets(line, MAX_LINE, stdin);
+      if( p == NULL )
+	line[0] = '\0';
       if( c->u.post.rerun != NULL )
 	fputs(line, c->u.post.rerun);
       while( line[0] != '\n' && line[0] != '\r' )
@@ -367,10 +379,14 @@ bool CommandNextPair(COMMAND c, char **key, char **val, FILE **file_val)
 	    *q = *p;
 	  *q = '\0';
 	}
-	p = fgets(line, MAX_LINE, stdin);  /* p unused, keep compiler happy */
+	p = fgets(line, MAX_LINE, stdin);
+	if( p == NULL )
+	  line[0] = '\0';
 	if( c->u.post.rerun != NULL )
 	  fputs(line, c->u.post.rerun);
       }
+      if( DEBUG2 )
+	fprintf(stderr, "  CommandNextPair at (c)\n"); 
 
       /* read the next line from stdin and make that the value */
       if( strcmp(*key, "file") == 0 )
@@ -382,15 +398,26 @@ bool CommandNextPair(COMMAND c, char **key, char **val, FILE **file_val)
       else
       {
 	/* this one is not a file, value is the next line */
-	p = fgets(line, MAX_LINE, stdin);  /* p unused; keep compiler happy */
+	p = fgets(line, MAX_LINE, stdin);
+	if( p == NULL )
+	  line[0] = '\0';
+	if( DEBUG2 )
+	  fprintf(stderr, "  CommandNextPair at (e): \"%s\"\n", line); 
 	if( c->u.post.rerun != NULL )
 	  fputs(line, c->u.post.rerun);
+	if( DEBUG2 )
+	  fprintf(stderr, "  CommandNextPair at (f)\n"); 
 	for(p = &line[strlen(line)-1]; p >= line && (*p=='\n' || *p=='\r'); p--)
 	  *p = '\0';
-	*val = malloc(sizeof(char) * (p - line + 1));
+	if( DEBUG2 )
+	  fprintf(stderr, "  CommandNextPair p - line + 2 = %ld: \"%s\"\n",
+	    p - line + 2, line); 
+	*val = malloc(sizeof(char) * (p - line + 2)); /* bug fix JeffK 7/1/25 */
 	strcpy(*val, line);
 	*file_val = NULL;
       }
+      if( DEBUG2 )
+	fprintf(stderr, "  CommandNextPair at (z)\n"); 
       break;
 
     case COMMAND_LINE:
