@@ -183,14 +183,14 @@ static void DisplayFrontPage(void)
   HTMLBigHeading(html, "The HSEval High School Timetable Evaluator");
   HTMLFormBeginUpload(html, true);
 
+  /* ***
   HTMLParagraphBegin(html);
   HTMLColouredBoxBeginFullWidth(html, LightRed);
-  HTMLText(html, "This version of HSEval has a completely rewritten");
-  HTMLText(html, "timetable display module.  The timetables look much the");
-  HTMLText(html, "same as before, but the various kinds are more consistent");
-  HTMLText(html, "with each other.");
+  HTMLText(html, "This is a new version of HSEval running on a new server.");
+  HTMLText(html, "Please report problems by email to jeff@it.usyd.edu.au.");
   HTMLColouredBoxEnd(html);
   HTMLParagraphEnd(html);
+  *** */
 
   HTMLParagraphBegin(html);
   HTMLText(html, "This is the HSEval High School Timetable Evaluator.  It");
@@ -536,7 +536,7 @@ static void soln_fn(KHE_SOLN soln, void *impl)
 *** */
 
 KHE_ARCHIVE ReadAndVerifyArchive(COMMAND c, bool allow_invalid_solns,
-  KHE_SOLN_TYPE soln_type, HA_ARENA_SET as)
+  bool resource_type_partitions, KHE_SOLN_TYPE soln_type, HA_ARENA_SET as)
 {
   char *op, *val;  KHE_ARCHIVE archive;  KML_ERROR ke;  FILE *fp;
   if( !CommandNextPair(c, &op, &val, &fp) || strcmp(op, "file") )
@@ -546,11 +546,11 @@ KHE_ARCHIVE ReadAndVerifyArchive(COMMAND c, bool allow_invalid_solns,
   *** */
     CommandError(c, "uploaded file expected but missing");
   /* ***
-  if( !KheArchiveReadIncremental(fp, as, &archive, &ke, false, false, false,
+  if( !KheArch iveReadIncremental(fp, as, &archive, &ke, false, false, false,
 	false, allow_invalid_solns, CommandRerunFile(c), NULL, NULL, NULL,
 	NULL, placeholder_solns ? soln_fn : NULL, NULL) )
   *** */
-  if( !KheArchiveRead(fp, as, &archive, &ke, false, false, false,
+  if( !KheArchiveRead(fp, as, &archive, &ke, false, resource_type_partitions,
 	false, allow_invalid_solns, soln_type, CommandRerunFile(c)) )
     CommandError(c, "line %d col %d: %s", KmlErrorLineNum(ke),
       KmlErrorColNum(ke), KmlErrorString(ke));
@@ -570,7 +570,7 @@ KHE_ARCHIVE ReadAndVerifyArchive(COMMAND c, bool allow_invalid_solns,
 static void Report(COMMAND c, bool txt_suffix, HA_ARENA_SET as)
 {
   KHE_ARCHIVE archive;
-  archive = ReadAndVerifyArchive(c, false, KHE_SOLN_ORDINARY, as);
+  archive = ReadAndVerifyArchive(c, false, false, KHE_SOLN_ORDINARY, as);
   fprintf(stdout, "Content-Type: text/%s\n", txt_suffix ? "plain" : "xml");
   fprintf(stdout, "Content-Disposition: attachment; filename=hseval.%s\n\n",
     txt_suffix ? "txt" : "xml");
@@ -589,7 +589,7 @@ static void Report(COMMAND c, bool txt_suffix, HA_ARENA_SET as)
 static void Summary(COMMAND c, PRINT_FORMAT print_format, HA_ARENA_SET as)
 {
   KHE_ARCHIVE archive;
-  archive = ReadAndVerifyArchive(c, true, KHE_SOLN_BASIC_PLACEHOLDER, as);
+  archive = ReadAndVerifyArchive(c, true, false, KHE_SOLN_BASIC_PLACEHOLDER,as);
   SummaryEvalPrint(archive, print_format, as);
 }
 
@@ -605,7 +605,7 @@ static void Summary(COMMAND c, PRINT_FORMAT print_format, HA_ARENA_SET as)
 static void Ranking(COMMAND c, PRINT_FORMAT print_format, HA_ARENA_SET as)
 {
   KHE_ARCHIVE archive;
-  archive = ReadAndVerifyArchive(c, true, KHE_SOLN_BASIC_PLACEHOLDER, as);
+  archive = ReadAndVerifyArchive(c, true, false, KHE_SOLN_BASIC_PLACEHOLDER,as);
   RankingEvalPrint(archive, print_format, as);
 }
 
@@ -621,7 +621,7 @@ static void Ranking(COMMAND c, PRINT_FORMAT print_format, HA_ARENA_SET as)
 static void ReportHTML(COMMAND c, HA_ARENA_SET as)
 {
   KHE_ARCHIVE archive;  HTML html;  char buff[100];
-  archive = ReadAndVerifyArchive(c, true, KHE_SOLN_ORDINARY, as);
+  archive = ReadAndVerifyArchive(c, true, false, KHE_SOLN_ORDINARY, as);
   snprintf(buff, 100, "%s Evaluations Summary",
     KheArchiveId(archive) != NULL && strlen(KheArchiveId(archive)) < 60 ?
     KheArchiveId(archive) : "HSeval");
@@ -658,7 +658,7 @@ static void ReportHTML(COMMAND c, HA_ARENA_SET as)
 static void TimetablesHTML(COMMAND c, bool with_event_groups, HA_ARENA_SET as)
 {
   KHE_ARCHIVE archive;  HTML html;  char buff[100];
-  archive = ReadAndVerifyArchive(c, true, KHE_SOLN_ORDINARY, as);
+  archive = ReadAndVer ifyArchive(c, true, KHE_SOLN_ORDINARY, as);
   snprintf(buff, 100, "%s Solution Timetables",
     KheArchiveId(archive) != NULL && strlen(KheArchiveId(archive)) < 60 ?
     KheArchiveId(archive) : "HSeval");
@@ -916,7 +916,7 @@ static void Run(HA_ARENA_SET as, HA_ARENA a)
   COMMAND c;  char *op, *val;  FILE *junk;
   if( DEBUG1 )
     fprintf(stderr, "[ Run()\n");
-  c = CommandBeginCGI(HSEVAL_RERUN, &PageBegin, &PageEnd, a);
+  c = Command BeginCGI(HSEVAL_RERUN, &PageBegin, &PageEnd, a);
   if( !CommandNextPair(c, &op, &val, &junk) || strcmp(op, "op") != 0 )
     DisplayFrontPage();
   ** ***
@@ -1121,13 +1121,13 @@ static void WriteTable(KHE_ARCHIVE archive, char *options, HA_ARENA_SET as)
 {
   PRINT_FORMAT fmt;
   bool show_costs, show_relative, show_times, show_averages, show_best_count;
-  bool non_empty, soft_costs_only;  TABLE table;  HTML html;  char *p;
-  HA_ARENA a;  char *instance_id;
+  bool non_empty, soft_costs_only, average_costs;  TABLE table;  HTML html;
+  char *p; HA_ARENA a;  char *instance_id;
 
   /* get the options */
   fmt = PRINT_LATEX;
-  show_costs = show_times = soft_costs_only = show_relative = false;
-  show_averages = show_best_count = false;
+  show_costs = show_times = soft_costs_only = average_costs = false;
+  show_relative = show_averages = show_best_count = false;
   instance_id = NULL;
   for( p = options;  *p != '\0';  p++ ) switch( *p )
   {
@@ -1161,6 +1161,11 @@ static void WriteTable(KHE_ARCHIVE archive, char *options, HA_ARENA_SET as)
       soft_costs_only = true;
       break;
 
+    case 'd':
+
+      average_costs = true;
+      break;
+
     case 'a':
 
       show_averages = true;
@@ -1186,10 +1191,11 @@ static void WriteTable(KHE_ARCHIVE archive, char *options, HA_ARENA_SET as)
   }
 
   /* generate the table */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   /* a = HaAren aMake(); */
-  table = EvalTableBuild(archive, show_costs, soft_costs_only, show_relative,
-    show_times, show_averages, instance_id, show_best_count, &non_empty, a);
+  table = EvalTableBuild(archive, show_costs, soft_costs_only, average_costs,
+    show_relative, show_times, show_averages, instance_id, show_best_count,
+    &non_empty, a);
   /* ***
   if( show_times )
     table = RunningTimeTableBuild(archive, show_averages, &non_empty);
@@ -1218,7 +1224,7 @@ static void WriteTable(KHE_ARCHIVE archive, char *options, HA_ARENA_SET as)
       HnAbort("HSEval internal error (fmt = %d)", fmt);
       break;
   }
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
   /* HaArenaD elete(a); */
 }
 
@@ -1238,7 +1244,7 @@ static KHE_ARCHIVE ReadArchive(char *file_name)
   fp = OpenFile(file_name);
   if( fp == NULL )
     HnFatal("khe: cannot open archive file %s", file_name);
-  if( !KheArchiveRead(fp, &res, &ke, false, false, false, NULL) )
+  if( !KheArchi veRead(fp, &res, &ke, false, false, false, NULL) )
     HnFatal("%s:%d:%d: %s", file_name, KmlErrorLineNum(ke),
       KmlErrorColNum(ke), KmlErrorString(ke));
   return res;
@@ -1276,8 +1282,8 @@ int main(int argc, char *argv[])
   setrlimit(RLIMIT_CORE, &rlim_rec);
 
   /* make an arena and an arena set */
-  a = HaArenaMake(false);
-  as = HaArenaSetMake(a);
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);
 
   /* do what the command line arguments request */
   if( argc == 1 )
@@ -1306,7 +1312,7 @@ int main(int argc, char *argv[])
 	  HnFatal("hseval: missing archive file name after -r");
 	pos = 2;
         archive = KheArchiveReadFromCommandLine(argc, argv, &pos, as, false,
-	  false, false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
+	  false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
 	if( pos != argc )
           HnFatal("hseval: unexpected command line argument \"%s\"", argv[pos]);
 	KheArchiveWrite(archive, true, stdout);
@@ -1344,7 +1350,7 @@ int main(int argc, char *argv[])
 	  HnFatal("hseval: missing archive file name after -t");
 	pos = 2;
         archive = KheArchiveReadFromCommandLine(argc, argv, &pos,
-	  as, false, false, false, false, false, KHE_SOLN_ORDINARY, NULL);
+	  as, false, false, false, false, KHE_SOLN_ORDINARY, NULL);
 	if( pos != argc )
           HnFatal("hseval: unexpected command line argument \"%s\"", argv[pos]);
 	WriteTable(archive, &argv[1][2], as);
