@@ -975,7 +975,7 @@ void RankingEvalPrint(KHE_ARCHIVE archive, PRINT_FORMAT print_format,
 {
   HTML html;  TABLE ranking_table;  HA_ARENA a;
   /* KHE_ARCHIVE_METADATA md; */  char buff[100];
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   switch( print_format )
   {
     case PRINT_HTML:
@@ -1102,5 +1102,5 @@ void RankingEvalPrint(KHE_ARCHIVE archive, PRINT_FORMAT print_format,
 
       HnAssert(false, "hseval internal error (RankingEvalPrint)");
   }
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
 }
