@@ -63,37 +63,40 @@ static void EndTable(KHE_CONSTRAINT c, int points, KHE_COST cost,
   /* end the table */
   hard_cost = KheHardCost(cost);
   soft_cost = KheSoftCost(cost);
-  HTMLTableRowBegin(html);
-  HTMLTableEntryBegin(html);
-  HTMLHSpace(html, 6);
-  HTMLTextBold(html, "Total");
-  if( points > 0 )
-    HTMLText(html, points == 1 ? "(%d point)" : "(%d points)", points);
-  HTMLTableEntryEnd(html);
-  /* ***
-  HTMLTableEntryBegin(html);
-  HTMLHSpace(html, 2);
-  HTMLTableEntryEnd(html);
-  *** */
-  HTMLTableEntryBegin(html);
-  HTMLHSpace(html, 2);
-  HTMLTableEntryEnd(html);
-  HTMLTableEntryBegin(html);
-  HTMLHSpace(html, 2);
-  HTMLTableEntryEnd(html);
-  HTMLTableEntryCentredBegin(html);
-  if( hard_cost > 0 )
-    HTMLText(html, "%d", hard_cost);
-  else
+  if( points > 1 )
+  {
+    HTMLTableRowBegin(html);
+    HTMLTableEntryBegin(html);
+    HTMLHSpace(html, 6);
+    HTMLTextBold(html, "Total");
+    if( points > 0 )
+      HTMLText(html, points == 1 ? "(%d point)" : "(%d points)", points);
+    HTMLTableEntryEnd(html);
+    /* ***
+    HTMLTableEntryBegin(html);
     HTMLHSpace(html, 2);
-  HTMLTableEntryEnd(html);
-  HTMLTableEntryCentredBegin(html);
-  if( soft_cost > 0 )
-    HTMLText(html, "%d", soft_cost);
-  else
+    HTMLTableEntryEnd(html);
+    *** */
+    HTMLTableEntryBegin(html);
     HTMLHSpace(html, 2);
-  HTMLTableEntryEnd(html);
-  HTMLTableRowEnd(html);
+    HTMLTableEntryEnd(html);
+    HTMLTableEntryBegin(html);
+    HTMLHSpace(html, 2);
+    HTMLTableEntryEnd(html);
+    HTMLTableEntryCentredBegin(html);
+    if( hard_cost > 0 )
+      HTMLText(html, "%d", hard_cost);
+    else
+      HTMLHSpace(html, 2);
+    HTMLTableEntryEnd(html);
+    HTMLTableEntryCentredBegin(html);
+    if( soft_cost > 0 )
+      HTMLText(html, "%d", soft_cost);
+    else
+      HTMLHSpace(html, 2);
+    HTMLTableEntryEnd(html);
+    HTMLTableRowEnd(html);
+  }
   HTMLTableEnd(html);
   HTMLParagraphEnd(html);
 
@@ -125,34 +128,37 @@ static void EndLowerBoundTable(KHE_CONSTRAINT c, int points, KHE_COST cost,
   /* end the table */
   hard_cost = KheHardCost(cost);
   soft_cost = KheSoftCost(cost);
-  HTMLTableRowBegin(html);
-  HTMLTableEntryBegin(html);
-  HTMLHSpace(html, 6);
-  HTMLTextBold(html, "Total");
-  if( points > 0 )
-    HTMLText(html, points == 1 ? "(%d point)" : "(%d points)", points);
-  HTMLTableEntryEnd(html);
-  HTMLTableEntryBegin(html);
-  HTMLHSpace(html, 2);
-  HTMLTableEntryEnd(html);
-  /* *** calculation
-  HTMLTableEntryBegin(html);
-  HTMLHSpace(html, 2);
-  HTMLTableEntryEnd(html);
-  *** */
-  HTMLTableEntryCentredBegin(html);
-  if( hard_cost > 0 )
-    HTMLText(html, "%d", hard_cost);
-  else
+  if( points > 1 )
+  {
+    HTMLTableRowBegin(html);
+    HTMLTableEntryBegin(html);
+    HTMLHSpace(html, 6);
+    HTMLTextBold(html, "Total");
+    if( points > 0 )
+      HTMLText(html, points == 1 ? "(%d point)" : "(%d points)", points);
+    HTMLTableEntryEnd(html);
+    HTMLTableEntryBegin(html);
     HTMLHSpace(html, 2);
-  HTMLTableEntryEnd(html);
-  HTMLTableEntryCentredBegin(html);
-  if( soft_cost > 0 )
-    HTMLText(html, "%d", soft_cost);
-  else
+    HTMLTableEntryEnd(html);
+    /* *** calculation
+    HTMLTableEntryBegin(html);
     HTMLHSpace(html, 2);
-  HTMLTableEntryEnd(html);
-  HTMLTableRowEnd(html);
+    HTMLTableEntryEnd(html);
+    *** */
+    HTMLTableEntryCentredBegin(html);
+    if( hard_cost > 0 )
+      HTMLText(html, "%d", hard_cost);
+    else
+      HTMLHSpace(html, 2);
+    HTMLTableEntryEnd(html);
+    HTMLTableEntryCentredBegin(html);
+    if( soft_cost > 0 )
+      HTMLText(html, "%d", soft_cost);
+    else
+      HTMLHSpace(html, 2);
+    HTMLTableEntryEnd(html);
+    HTMLTableRowEnd(html);
+  }
   HTMLTableEnd(html);
   HTMLParagraphEnd(html);
 
@@ -803,44 +809,28 @@ static void SolnReportHTML(KHE_SOLN soln, HTML html, HA_ARENA a)
 static void SolnGroupReportHTML(KHE_SOLN_GROUP soln_group, HTML html,
   HA_ARENA a)
 {
-  int i;  KHE_SOLN soln;
+  int i;  KHE_SOLN soln;  char buff[1000];
 
-  /* header information */
-  char buff[1000];
+  /* start segment */
   sprintf(buff, "Solution Group %s", KheSolnGroupId(soln_group));
-  HTMLHeading(html, buff);
+  HTMLSegmentBegin(html, KheSolnGroupId(soln_group), buff);
+
+  /* solution group metadata */
   HTMLParagraphBegin(html);
   HTMLText(html, KheSolnGroupMetaDataText(soln_group));
-  /* ***
-  md = KheSolnGroupMetaData(soln_group);
-  HTMLTextNoBreak(html, KheSolnGroupMetaDataDescription(md));
-  HTMLTextNoBreak(html, ".  Contributed by ");
-  HTMLTextNoBreak(html, KheSolnGroupMetaDataContributor(md));
-  HTMLText(html, ",");
-  HTMLTextNoBreak(html, KheSolnGroupMetaDataDate(md));
-  HTMLText(html, ".");
-  if( KheSolnGroupMetaDataRemarks(md) != NULL )
-    HTMLText(html, KheSolnGroupMetaDataRemarks(md));
-  *** */
   HTMLParagraphEnd(html);
 
-  /* optional citation information */
-  /* ***
-  if( publication != NULL )
-  {
-    HTMLParagraphBegin(html);
-    HTMLTextBold(html, "Publication.");
-    HTMLText(html, publication);
-    HTMLParagraphEnd(html);
-  }
-  *** */
-
-  /* the solutions */
+  /* solutions */
   for( i = 0;  i < KheSolnGroupSolnCount(soln_group);  i++ )
   {
+    if( i > 0 )
+      HTMLHorizontalRule(html);
     soln = KheSolnGroupSoln(soln_group, i);
     SolnReportHTML(soln, html, a);
   }
+
+  /* end segment */
+  HTMLSegmentEnd(html);
 }
 
 
@@ -854,7 +844,7 @@ static void SolnGroupReportHTML(KHE_SOLN_GROUP soln_group, HTML html,
 
 void ArchiveReportHTML(KHE_ARCHIVE archive, HTML html, HA_ARENA_SET as)
 {
-  KHE_SOLN_GROUP soln_group;  int i;  HA_ARENA a;
+  KHE_SOLN_GROUP soln_group;  int i;  HA_ARENA a;  char buff[1000];
   if( KheArchiveSolnGroupCount(archive) == 0 )
   {
     HTMLParagraphBegin(html);
@@ -863,14 +853,30 @@ void ArchiveReportHTML(KHE_ARCHIVE archive, HTML html, HA_ARENA_SET as)
   }
   else
   {
-    a = HaArenaSetArenaBegin(as, false);
+    a = HaArenaMake(as);
+
+    /* print table of contents if more than one solution group */
+    if( KheArchiveSolnGroupCount(archive) >= 2 )
+    {
+      HTMLParagraphBegin(html);
+      for( i = 0;  i < KheArchiveSolnGroupCount(archive);  i++ )
+      {
+	if( i > 0 )
+	  HTMLNewLine(html);
+	soln_group = KheArchiveSolnGroup(archive, i);
+        sprintf(buff, "Solution Group %s", KheSolnGroupId(soln_group));
+	HTMLJumpInternal(html, KheSolnGroupId(soln_group), buff);
+      }
+      HTMLParagraphEnd(html);
+    }
+    HTMLHorizontalRule(html);
+
+    /* print solution groups */
     for( i = 0;  i < KheArchiveSolnGroupCount(archive);  i++ )
     {
       soln_group = KheArchiveSolnGroup(archive, i);
-      HTMLHorizontalRule(html);
       SolnGroupReportHTML(soln_group, html, a);
     }
-    HaArenaSetArenaEnd(as, a);
-    HTMLHorizontalRule(html);
+    HaArenaDelete(a);
   }
 }
