@@ -152,6 +152,29 @@ static AVERAGE_INFO AverageInfoMake(int len, HA_ARENA a)
 /*                                                                           */
 /*****************************************************************************/
 
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheSolnGroupHasRunningTimes(KHE_SOLN_GROUP sg)                      */
+/*                                                                           */
+/*  Return true if at least one of the solutions of sg has a running time.   */
+/*                                                                           */
+/*****************************************************************************/
+
+/* *** currently unused
+static bool KheSolnGroupHasRunningTimes(KHE_SOLN_GROUP sg)
+{
+  KHE_SOLN soln;  int i;  float junk;
+  for( i = 0;  i < KheSolnGroupSolnCount(sg);  i++ )
+  {
+    soln = KheSolnGroupSoln(sg, i);
+    if( KheSolnHasRunningTime(soln, &junk) )
+      return true;
+  }
+  return false;
+}
+*** */
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*****************************************************************************/
@@ -164,6 +187,7 @@ static void TableAddAverageRow(TABLE table, KHE_ARCHIVE archive,
   bool best;  AVERAGE hard_av, soft_av, rel_av, time_av;
   double hard_av_val, soft_av_val, rel_av_val, time_av_val;
   double best_hard_av_val, best_soft_av_val /* , best_time_av_val */;
+  /* KHE_SOLN_GROUP soln_group; */
 
   /* make row and "Average" entry */
   row = TableRowMake(a);
@@ -253,7 +277,7 @@ static void TableAddAverageRow(TABLE table, KHE_ARCHIVE archive,
 	entry = TableEntryMakeText(TABLE_RIGHT, TABLE_ROMAN, "", a);
       TableRowAddEntry(row, entry);
     }
-    if( show_times )
+    if( show_times /* && KheSolnGroupHasRunningTimes(soln_group) */ )
     {
       if( time_av->count == wanted )
       {
@@ -289,11 +313,15 @@ static void TableAddAverageRow(TABLE table, KHE_ARCHIVE archive,
 /*      Only effective when show_costs is true.  Show an integral soft       */
 /*      cost only, except show "inf." when hard cost > 0.                    */
 /*                                                                           */
+/*    average_costs                                                          */
+/*      Only effective when show_costs is true.  Show the average of the     */
+/*      costs of the solutions of each instance, not the best costs.         */
+/*                                                                           */
 /*    show_relative                                                          */
 /*      Show relative solution costs.                                        */
 /*                                                                           */
 /*    show_times                                                             */
-/*      Show running times.                                                  */
+/*      Show running times, unless there are none.                           */
 /*                                                                           */
 /*    show_averages                                                          */
 /*      Show an averages row at the bottom of the table.                     */
@@ -323,9 +351,9 @@ static char *RemovePercent(char *str)
 }
 
 TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
-  bool soft_costs_only, bool show_relative, bool show_times,
-  bool show_averages, char *instance_id, bool show_best_count,
-  bool *non_empty, HA_ARENA a)
+  bool soft_costs_only, bool average_costs, bool show_relative,
+  bool show_times, bool show_averages, char *instance_id,
+  bool show_best_count, bool *non_empty, HA_ARENA a)
 {
   TABLE res;  int i, j, best_best;   KHE_SOLN_GROUP soln_group;  KHE_SOLN soln;
   KHE_INSTANCE ins;  TABLE_ENTRY entry;  TABLE_ROW row;  TABLE_FONT font;
@@ -336,7 +364,8 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
   AVERAGE hard_av, soft_av, rel_av, time_av, best_av;
   /* double hard_av_val, soft_av_val, time_av_val; */
   /* double best_hard_av_val, best_soft_av_val, best_time_av_val; */
-  int hard_cost, soft_cost, span;  /* bool best; */  AVERAGE_INFO ai;
+  int hard_cost, soft_cost, best_hard_cost, best_soft_cost, span, max_span;
+  /* bool best; */  AVERAGE_INFO ai;
 
   HnAssert(show_costs || show_times, "EvalTableBuild: neither costs nor times");
 
@@ -369,10 +398,8 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
   }
   *** */
 
-  /* spanning columns count */
-  span = (show_costs ? 1 : 0) + (show_relative ? 1 : 0) + (show_times ? 1 : 0);
-
   /* first row */
+  max_span = 1;
   row = TableRowMake(a);
   TableAddRow(res, row);
   entry = TableEntryMakeInt(TABLE_LEFT, TABLE_BOLD, "Instances (%d)",
@@ -381,6 +408,8 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
   for( i = 0;  i < KheArchiveSolnGroupCount(archive);  i++ )
   {
     soln_group = KheArchiveSolnGroup(archive, i);
+    span = (show_costs ? 1 : 0) + (show_relative ? 1 : 0) +
+      (show_times /* && KheSolnGroupHasRunningTimes(soln_group */ ? 1 : 0);
     str = RemovePercent(KheSolnGroupId(soln_group));
     entry = TableEntryMakeText(TABLE_CENTRE, TABLE_BOLD, str, a);
     if( span > 1 )
@@ -388,11 +417,13 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
       TableEntrySetHSpan(entry, span);
       TableEntrySetUnderline(entry, true);
     }
+    if( span > max_span )
+      max_span = span;
     TableRowAddEntry(row, entry);
   }
 
   /* optional second row */
-  if( span > 1 )
+  if( max_span > 1 )
   {
     row = TableRowMake(a);
     TableAddRow(res, row);
@@ -400,6 +431,7 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
     TableRowAddEntry(row, entry);
     for( i = 0;  i < KheArchiveSolnGroupCount(archive);  i++ )
     {
+      soln_group = KheArchiveSolnGroup(archive, i);
       if( show_costs )
       {
 	entry = TableEntryMakeText(TABLE_RIGHT, TABLE_BOLD, "Cost", a);
@@ -410,7 +442,7 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
 	entry = TableEntryMakeText(TABLE_RIGHT, TABLE_BOLD, "Rel.", a);
 	TableRowAddEntry(row, entry);
       }
-      if( show_times )
+      if( show_times /* && KheSolnGroupHasRunningTimes(soln_group) */ )
       {
 	entry = TableEntryMakeText(TABLE_RIGHT, TABLE_BOLD, "Time", a);
 	TableRowAddEntry(row, entry);
@@ -446,6 +478,8 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
 	}
       }
     }
+    best_hard_cost = KheHardCost(best_cost);
+    best_soft_cost = KheSoftCost(best_cost);
 
     /* span entries of instance row for each solution group */
     for( j = 0;  j < KheArchiveSolnGroupCount(archive);  j++ )
@@ -459,7 +493,7 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
 	  TableRowAddEntry(row, entry);
 	if( show_relative )
 	  TableRowAddEntry(row, entry);
-	if( show_times )
+	if( show_times /* && KheSolnGroupHasRunningTimes(soln_group) */ )
 	  TableRowAddEntry(row, entry);
       }
       else
@@ -472,7 +506,7 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
 	    TableRowAddEntry(row, entry);
 	  if( show_relative )
 	    TableRowAddEntry(row, entry);
-	  if( show_times )
+	  if( show_times /* && KheSolnGroupHasRunningTimes(soln_group) */ )
 	    TableRowAddEntry(row, entry);
 	  *non_empty = true;
 	}
@@ -489,11 +523,21 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
 	  AverageAdd(soft_av, soft_cost);
 
 	  /* find relative cost and add to average, but only if not hard */
-	  if( hard_cost == 0 && best_cost > 0 )
+	  if( hard_cost == 0 )
 	  {
-	    rel_cost = (float) KheSolnCost(soln) / (float) best_cost;
-	    rel_av = HaArray(ai->rel_cost_averages, j);
-	    AverageAdd(rel_av, rel_cost);
+	    HnAssert(best_hard_cost == 0, "EvalTableBuild internal error");
+	    if( best_soft_cost > 0 )
+	    {
+	      rel_cost = (float) soft_cost / (float) best_soft_cost;
+	      rel_av = HaArray(ai->rel_cost_averages, j);
+	      AverageAdd(rel_av, rel_cost);
+	    }
+	    else if( soft_cost == 0 )
+	    {
+	      /* we're pretending that 0 / 0 = 1.0 */
+	      rel_av = HaArray(ai->rel_cost_averages, j);
+	      AverageAdd(rel_av, 1.0);
+	    }
 	  }
 
 	  /* find optional running time and add to average if present */
@@ -526,20 +570,25 @@ TABLE EvalTableBuild(KHE_ARCHIVE archive, bool show_costs,
 	  /* relative cost entry, if requested */
 	  if( show_relative )
 	  {
-	    if( hard_cost == 0 && best_cost > 0 )
-	    {
-	      rel_cost = (float) KheSolnCost(soln) / (float) best_cost;
+	    if( best_hard_cost > 0 )
+              entry = TableEntryMakeDouble(TABLE_RIGHT, TABLE_ROMAN, "%.2f",
+		(float) hard_cost / (float) best_hard_cost, a);
+	    else if( hard_cost > 0 )
+	      entry = TableEntryMakeText(TABLE_RIGHT, TABLE_ROMAN, "inf.", a);
+	    else if( best_soft_cost > 0 )
 	      entry = TableEntryMakeDouble(TABLE_RIGHT, TABLE_ROMAN, "%.2f",
-		rel_cost, a);
-	    }
+		(float) soft_cost / (float) best_soft_cost, a);
+	    else if( soft_cost > 0 )
+	      entry = TableEntryMakeText(TABLE_RIGHT, TABLE_ROMAN, "inf.", a);
 	    else
-	      entry = TableEntryMakeText(TABLE_RIGHT, TABLE_ROMAN, "", a);
+	      entry = TableEntryMakeDouble(TABLE_RIGHT, TABLE_ROMAN, "%.2f",
+		1.0, a);
 	    TableRowAddEntry(row, entry);
 	    *non_empty = true;
 	  }
 
 	  /* running time entry, if requested */
-	  if( show_times )
+	  if( show_times /* && KheSolnGroupHasRunningTimes(soln_group) */ )
 	  {
 	    if( KheSolnHasRunningTime(soln, &rt) )
 	    {
@@ -975,10 +1024,9 @@ static TABLE InstanceTableBuild(KHE_ARCHIVE archive, HA_ARENA a)
   }
 
   /* caption */
-  TableAddCaptionLine(res, "For each instance, the number of times, teachers,");
-  TableAddCaptionLine(res, "rooms, classes (groups of students), individual");
-  TableAddCaptionLine(res, "students, other resources, and events; the total");
-  TableAddCaptionLine(res, "duration of events (D); and the total number of");
+  TableAddCaptionLine(res, "For each instance, the number of times, resources");
+  TableAddCaptionLine(res, "of various kinds, and events; plus the total");
+  TableAddCaptionLine(res, "duration of events (D), and the total number of");
   TableAddCaptionLine(res, "points of application of constraints (C).");
   if( KheArchiveInstanceCount(archive) >= 6 )
   {
@@ -1317,11 +1365,11 @@ static TABLE DensityTableBuild(KHE_ARCHIVE archive, bool first,
 void SummaryEvalPrint(KHE_ARCHIVE archive, PRINT_FORMAT print_format,
   HA_ARENA_SET as)
 {
-  HTML html;  TABLE eval_table, running_time_table, ins_table;
+  HTML html;  TABLE eval_table, ins_table;
   TABLE  constraint_table, density_table;
   char buff[100];  KHE_CONSTRAINT_TAG half_tag;  bool non_empty;
-  HA_ARENA a;
-  a = HaArenaSetArenaBegin(as, false);
+  HA_ARENA a;  EVAL_TABLE et;
+  a = HaArenaMake(as);
   switch( print_format )
   {
     case PRINT_HTML:
@@ -1357,15 +1405,13 @@ void SummaryEvalPrint(KHE_ARCHIVE archive, PRINT_FORMAT print_format,
       *** */
 
       /* evaluation tables */
-      eval_table = EvalTableBuild(archive, true, false, false, false, true,
-	NULL, false, &non_empty, a);
-      if( TableRowCount(eval_table) <= 1 )
+      if( KheArchiveInstanceCount(archive) == 0 )
       {
 	HTMLParagraphBegin(html);
 	HTMLText(html, "This archive contains no instances.");
 	HTMLParagraphEnd(html);
       }
-      else if( TableColCount(eval_table) <= 1 )
+      else if( KheArchiveSolnGroupCount(archive) == 0 )
       {
 	HTMLParagraphBegin(html);
 	HTMLText(html, "This archive contains no solution groups.");
@@ -1373,10 +1419,30 @@ void SummaryEvalPrint(KHE_ARCHIVE archive, PRINT_FORMAT print_format,
       }
       else
       {
-	/* full eval_table */
-	TablePrintHTML(eval_table, html);
+	/* cost table */
+	/* ***
+	eval_table = EvalTableBuild(archive, true, false, false, true,
+	  false, true, NULL, false, &non_empty, a);
+	*** */
+	et = EvalTableMake(archive, NULL, a);
+
+	/* best costs table */
+	EvalTableAddColumnFormat(et, EVAL_COLUMN_DATA_BEST_COST, false);
+	EvalTableAddColumnFormat(et, EVAL_COLUMN_DATA_RELATIVE_BEST_COST,false);
+	TablePrintHTML(EvalTableMakeTable(et, true, false), html);
+	EvalTableClearColumnFormats(et);
+
+	/* average costs table */
+	/* *** correct but not wanted just now
+	EvalTableAddColumnFormat(et, EVAL_COLUMN_DATA_AVERAGE_COST, false);
+	EvalTableAddColumnFormat(et, EVAL_COLUMN_DATA_RELATIVE_AVERAGE_COST,
+	  false);
+	TablePrintHTML(EvalTableMakeTable(et, true, false), html);
+	EvalTableClearColumnFormats(et);
+	*** */
 
 	/* eval_table after uncompetitive columns are reduced */
+	/* ***
 	if( TableRemoveNonBoldColumns(eval_table) > 0 )
 	{
 	  TableSetHeader(eval_table,
@@ -1386,13 +1452,22 @@ void SummaryEvalPrint(KHE_ARCHIVE archive, PRINT_FORMAT print_format,
 	  TableAddCaptionLine(eval_table, "with no bold entries.");
 	  TablePrintHTML(eval_table, html);
 	}
+	*** */
 
 	/* running time table */
-        running_time_table = EvalTableBuild(archive, false, false, false,
-	  true, true, NULL, false, &non_empty, a);
+	/* et = EvalTableMake(archive, true, NULL, a); */
+	EvalTableAddColumnFormat(et, EVAL_COLUMN_DATA_AVERAGE_RUNNING_TIME,
+	  false);
+	TablePrintHTML(EvalTableMakeTable(et, true, false), html);
+	EvalTableClearColumnFormats(et);
 	/* ***
-        running_time_table = RunningTimeTableBuild(archive, true, &non_empty);
-        *** */
+	 EvalTableBuild(KHE_ARCHIVE archive, bool show_best_costs,
+  bool soft_costs_only, bool average_costs, bool show_relative,
+  bool show_times, bool show_averages, char *instance_id,
+  bool show_best_count, bool *non_empty, HA_ARENA a);
+	running_time_table = EvalTableBuild(archive, false, false, false,
+	  false, true, true, NULL, false, &non_empty, a);
+	running_time_table = RunningTimeTableBuild(archive, true, &non_empty);
 	if( non_empty )
 	  TablePrintHTML(running_time_table, html);
 	else
@@ -1401,13 +1476,14 @@ void SummaryEvalPrint(KHE_ARCHIVE archive, PRINT_FORMAT print_format,
 	  HTMLText(html, "This archive contains no running times.");
 	  HTMLParagraphEnd(html);
 	}
+	*** */
       }
 
       /* invalid solutions */
       ArchiveReportInvalidSolns(archive, html);
 
       /* instances summary */
-      if( TableRowCount(eval_table) > 1 )
+      if( KheArchiveInstanceCount(archive) > 0 )
       {
 	/* instances */
 	ins_table = InstanceTableBuild(archive, a);
@@ -1448,8 +1524,8 @@ void SummaryEvalPrint(KHE_ARCHIVE archive, PRINT_FORMAT print_format,
 	"Content-Disposition: attachment; filename=hseval.tex\n\n");
 
       /* evaluation tables */
-      eval_table = EvalTableBuild(archive, true, false, false, false, true,
-	NULL, false, &non_empty, a);
+      eval_table = EvalTableBuild(archive, true, false, false, false,
+	false, true, NULL, false, &non_empty, a);
       if( TableRowCount(eval_table) <= 1 )
 	printf("%%%s\n", "This archive contains no instances.");
       else if( TableColCount(eval_table) <= 1 )
@@ -1499,5 +1575,5 @@ void SummaryEvalPrint(KHE_ARCHIVE archive, PRINT_FORMAT print_format,
 
       HnAssert(false, "hseval internal error (SummaryEvalPrint)");
   }
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
 }
