@@ -46,6 +46,10 @@
 typedef struct avail_info_rec {
   bool		show_avail_times;
   bool		show_avail_workload;
+  int		total_positive_avail_times;
+  int		total_negative_avail_times;
+  float		total_positive_avail_workload;
+  float		total_negative_avail_workload;
 } *AVAIL_INFO;
 
 
@@ -254,6 +258,10 @@ static void ResourceTypeInitAvail(KHE_RESOURCE_TYPE rt, KHE_SOLN soln,
 {
   float avail_workload;  int avail_times, i;  KHE_RESOURCE r;
   ai->show_avail_times = ai->show_avail_workload = false;
+  ai->total_positive_avail_times = 0;
+  ai->total_negative_avail_times = 0;
+  ai->total_positive_avail_workload = 0.0;
+  ai->total_negative_avail_workload = 0.0;
   for( i = 0;  i < KheResourceTypeResourceCount(rt);  i++ )
   {
     r = KheResourceTypeResource(rt, i);
@@ -275,25 +283,73 @@ static void ResourceTypeInitAvail(KHE_RESOURCE_TYPE rt, KHE_SOLN soln,
 /*  Display avail_times (if less than INT_MAX) and avail_workload (if        */
 /*  less than FLT_MAX) onto html.                                            */
 /*                                                                           */
+/*  Also, if accumulate is true, accumulate totals.                          */
+/*                                                                           */
 /*****************************************************************************/
 
 static void DisplayTimesAndWorkload(AVAIL_INFO ai, int avail_times,
-  float avail_workload, HTML html)
+  float avail_workload, bool accumulate, HTML html)
 {
-  if( ai->show_avail_times )
+  bool show_avail_times, show_workload;
+  show_avail_times = (ai->show_avail_times && avail_times < INT_MAX);
+  show_workload = (ai->show_avail_workload && avail_workload < FLT_MAX);
+
+  /* display */
+  if( show_avail_times )
   {
-    if( ai->show_avail_workload )
+    if( show_workload )
       HTMLText(html, "%d; %.1f", avail_times, avail_workload);
     else
       HTMLText(html, "%d", avail_times);
   }
   else
   {
-    if( ai->show_avail_workload )
+    if( show_workload )
       HTMLText(html, "%.1f", avail_workload);
     else
       HTMLHSpace(html, 2);
   }
+
+  /* optionally accumulate totals */
+  if( accumulate )
+  {
+    if( show_avail_times )
+    {
+      if( avail_times > 0 )
+	ai->total_positive_avail_times += avail_times;
+      else if( avail_times < 0 )
+	ai->total_negative_avail_times += avail_times;
+    }
+    if( show_workload )
+    {
+      if( avail_workload > 0.0 )
+	ai->total_positive_avail_workload += avail_workload;
+      else if( avail_workload < 0.0 )
+	ai->total_negative_avail_workload += avail_workload;
+    }
+  }
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void DisplayTotalTimesAndWorkload(AVAIL_INFO ai, HTML html)              */
+/*                                                                           */
+/*  Display a paragraph showing total available times and workload.          */
+/*                                                                           */
+/*****************************************************************************/
+
+static void DisplayTotalTimesAndWorkload(AVAIL_INFO ai, HTML html)
+{
+  if( ai->show_avail_times || ai->show_avail_workload )
+  {
+    if( ai->show_avail_times )
+      HTMLText(html, "Total available times: positive %d, negative %d.\n",
+        ai->total_positive_avail_times, ai->total_negative_avail_times);
+    if( ai->show_avail_workload )
+      HTMLText(html,"Total available workload: positive %.1f, negative %.1f.\n",
+        ai->total_positive_avail_workload, ai->total_negative_avail_workload);
+  }
 }
 
 
@@ -319,8 +375,10 @@ static void AvailResourceGroupFn(KHE_RESOURCE_GROUP rg, KHE_SOLN soln,
     r = KheResourceGroupResource(rg, 0);
     KheResourceAvailableBusyTimes(soln, r, &total_durn);
     KheResourceAvailableWorkload(soln, r, &total_workload);
+    DisplayTimesAndWorkload(ai, total_durn, total_workload, true, html);
   }
-  DisplayTimesAndWorkload(ai, total_durn, total_workload, html);
+  else
+    DisplayTimesAndWorkload(ai, total_durn, total_workload, false, html);
 }
 
 
@@ -447,6 +505,20 @@ static void ResourceBusyTimesAndEvents(KHE_SOLN soln, KHE_RESOURCE r,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  void DisplayTotalItalic(int total_italic, HTML html)                     */
+/*                                                                           */
+/*  Display the total number of italic entries.                              */
+/*                                                                           */
+/*****************************************************************************/
+
+static void DisplayTotalItalic(int total_italic, HTML html)
+{
+  HTMLText(html, "Total italic entries: %d.\n", total_italic);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void BuildPermutedTimes(ARRAY_KHE_TIME *permuted_times,                  */
@@ -581,8 +653,8 @@ static void ResourceTypeTimetables(KHE_RESOURCE_TYPE rt, KHE_SOLN soln,
   bool with_event_groups, char *constraints_str, HTML html, HA_ARENA a)
 {
   ARRAY_KHE_TIME permuted_times;  SPACE space;  KHE_INSTANCE ins;
-  SPACE_TIME_DIM std;  int i;  KHE_TIME time;  bool allow_spanning;
-  struct avail_info_rec avail_rec;
+  SPACE_TIME_DIM std;  int i, total_italic;  KHE_TIME time;
+  bool allow_spanning;  struct avail_info_rec avail_rec;
 
   /* heading (omit if just one resource type) */
   ins = KheSolnInstance(soln);
@@ -707,6 +779,11 @@ static void ResourceTypeTimetables(KHE_RESOURCE_TYPE rt, KHE_SOLN soln,
     SpaceCellFormat(space, SPACE_CELL_TIME_GROUP_EMPTY,            Black);
     SpaceCellFormat(space, SPACE_CELL_RESOURCE_GROUP_UNAVAIL_HARD, Red);
     SpaceCellFormat(space, SPACE_CELL_RESOURCE_GROUP_UNAVAIL_SOFT, LightRed);
+    SpaceCellFormat(space, SPACE_CELL_RESOURCE_GROUP_MUST_BE_BUSY_HARD, Blue);
+    SpaceCellFormat(space, SPACE_CELL_RESOURCE_GROUP_MUST_BE_BUSY_SOFT,
+      LightBlue);
+    SpaceCellFormat(space, SPACE_CELL_UNREQUIRED_ASSIGNED,         LightYellow);
+    SpaceCellFormat(space, SPACE_CELL_RESOURCE_GROUP_UNAVAIL_PART, LightPink);
     if( constraints_str != NULL )
     {
       SpaceCellFormat2(space, SPACE_CELL_CONSTRAINT_VIOLATED,
@@ -726,8 +803,17 @@ static void ResourceTypeTimetables(KHE_RESOURCE_TYPE rt, KHE_SOLN soln,
 
   /* add tasks, display, and delete */
   SpaceAddAllTasks(space);
-  SpaceDisplay(space, html);
+  SpaceDisplay(space, &total_italic, html);
   SpaceDelete(space);
+
+  /* print total available times and workload */
+  if( planning )
+  {
+    HTMLParagraphBegin(html);
+    DisplayTotalTimesAndWorkload(&avail_rec, html);
+    DisplayTotalItalic(total_italic, html);
+    HTMLParagraphEnd(html);
+  }
 }
 
 
@@ -747,7 +833,7 @@ void SolutionTimetables(KHE_SOLN soln, bool planning,
 {
   HA_ARENA a;  /* KHE_INSTANCE ins; */  /* ARRAY_WEEK weeks; */
   KHE_RESOURCE_TYPE rt;  KHE_INSTANCE ins;  int i;  bool compress_days;
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   /* ins = KheSolnInstance(soln); */
 
   /* print solution header */
@@ -777,7 +863,7 @@ void SolutionTimetables(KHE_SOLN soln, bool planning,
 	  highlight_splits, with_event_groups, constraints_str, html, a);
     }
   }
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
 }
 
 
@@ -800,9 +886,11 @@ void SolutionGroupTimetables(KHE_SOLN_GROUP soln_group, bool planning,
 {
   char buff[1000];  int i;  KHE_SOLN soln;
 
-  /* heading and solution group metadata */
+  /* start segment */
   sprintf(buff, "Solution Group %s", KheSolnGroupId(soln_group));
-  HTMLHeading(html, buff);
+  HTMLSegmentBegin(html, KheSolnGroupId(soln_group), buff);
+
+  /* solution group metadata */
   HTMLParagraphBegin(html);
   HTMLText(html, KheSolnGroupMetaDataText(soln_group));
   HTMLParagraphEnd(html);
@@ -818,6 +906,9 @@ void SolutionGroupTimetables(KHE_SOLN_GROUP soln_group, bool planning,
     if( KheSolnType(soln) == KHE_SOLN_ORDINARY )
       KheSolnTypeReduce(soln, KHE_SOLN_BASIC_PLACEHOLDER, NULL);
   }
+
+  /* end segment */
+  HTMLSegmentEnd(html);
 }
 
 
@@ -845,7 +936,7 @@ void Timetables(COMMAND c, bool planning, bool highlight_splits,
       bool_show(with_event_groups), constraints_str);
 
   /* get the archive */
-  archive = ReadAndVerifyArchive(c, true, KHE_SOLN_ORDINARY, as);
+  archive = ReadAndVerifyArchive(c, true, planning, KHE_SOLN_ORDINARY, as);
 
   /* page header */
   snprintf(buff, 1000, "%s %s Timetables",
@@ -868,22 +959,30 @@ void Timetables(COMMAND c, bool planning, bool highlight_splits,
       HTMLParagraphBegin(html);
       HTMLText(html, "For each solution in the uploaded XML archive, this");
       HTMLText(html, "page contains one planning timetable for each resource");
-      HTMLText(html, "type of the corresponding instance.");
+      HTMLText(html, "type of the corresponding instance.  This is a large");
+      HTMLText(html, "table with one row for each resource, and one column");
+      HTMLText(html, "for each time or each day.  Where a resource type is");
+      HTMLText(html, "divisible into completely independent parts (parts");
+      HTMLText(html, "which could have been separate resource types), each");
+      HTMLText(html, "part is given its own table.");
       HTMLParagraphEnd(html);
 
       HTMLParagraphBegin(html);
       HTMLText(html, "The Avail column, where present, shows an integer");
-      HTMLText(html, "number of available times, based on cluster busy");
-      HTMLText(html, "times and limit busy times constraints, or a decimal");
-      HTMLText(html, "amount of available workload, based on limit workload");
-      HTMLText(html, "constraints.  These are heuristic estimates, and are");
-      HTMLText(html, "occasionally higher than the true values, but never");
-      HTMLText(html, "lower.  A negative value proves that the resource is");
-      HTMLText(html, "overloaded.  In Unassigned rows the Avail column shows");
-      HTMLText(html, "the unassigned times or workload.  For more information");
-      HTMLText(html, "about how availability is calculated, see the");
-      HTMLText(html, "Availability Report, reached from the front page");
-      HTMLText(html, "of HSEval.");
+      HTMLText(html, "number of available times, determined by resource");
+      HTMLText(html, "constraints, or a decimal amount of available workload,");
+      HTMLText(html, "determined by limit workload constraints, or both.");
+      HTMLText(html, "Only non-trivial values are shown.  These are heuristic");
+      HTMLText(html, "estimates, and are occasionally higher than the true");
+      HTMLText(html, "values, but never lower.  A negative value proves that");
+      HTMLText(html, "its resource is overloaded.  In Unassigned rows the");
+      HTMLText(html, "Avail column shows the unassigned times or workload.");
+      HTMLText(html, "Below the table, Total available times shows the sum of");
+      HTMLText(html, "the positive values in the Avail column (not counting");
+      HTMLText(html, "Unassigned rows), and the sum of the negative values");
+      HTMLText(html, "in the Avail column.  For more information about how");
+      HTMLText(html, "availability is calculated, see the Availability");
+      HTMLText(html, "Report, reached from the front page of HSEval.");
       HTMLParagraphEnd(html);
     }
     else
@@ -909,15 +1008,22 @@ void Timetables(COMMAND c, bool planning, bool highlight_splits,
 
     HTMLParagraphBegin(html);
     HTMLText(html, "The background colour of each timetable cell is red if");
-    HTMLText(html, "the resource is unavailable then (dark red for hard");
-    HTMLText(html, "constraints, light red for soft ones), or else light");
-    HTMLText(html, "green if the cell contains no events, or else the");
-    HTMLText(html, "colour of the first event depicted (white if that event");
-    HTMLText(html, "has no colour).");
+    HTMLText(html, "the resource is unavailable for some reason (possibly");
+    HTMLText(html, "history) for all of the times that the cell represents");
+    HTMLText(html, "(dark red for hard constraints, light red for soft ones),");
+    HTMLText(html, "or else blue if history demands that the resource be busy");
+    HTMLText(html, "during at least one of those times (dark blue for hard");
+    HTMLText(html, "constraints, light blue for");
+    HTMLText(html, "soft ones), or else light yellow if the cell contains a");
+    HTMLText(html, "task printed in italics (see below), or else light pink");
+    HTMLText(html, "if the resource is unavailable for some of those times");
+    HTMLText(html, "but not all (hard or soft), or else light green if the");
+    HTMLText(html, "cell contains no events, or else the colour of the first");
+    HTMLText(html, "event depicted (white if that event has no colour).");
     if( constraints_str != NULL && constraints_str[0] != '\0' )
     {
-      HTMLText(html, "Constraints whose name or Id includes \"%s\"",
-	constraints_str);
+      HTMLText(html, "As requested, constraints whose name or Id includes "
+	"\"%s\"", constraints_str);
       HTMLText(html, "are highlighted:  the background colour of their");
       HTMLText(html, "cells is blue where one of them is violated, and");
       HTMLText(html, "for cluster busy times constraints it is light blue");
@@ -930,23 +1036,46 @@ void Timetables(COMMAND c, bool planning, bool highlight_splits,
     HTMLParagraphEnd(html);
 
     HTMLParagraphBegin(html);
-    HTMLText(html, "The text in each cell appears in bold italic font if");
-    HTMLText(html, "the event resource it indicates is subject to an event");
-    HTMLText(html, "resource constraint which is violated (the event");
-    HTMLText(html, "resource may be unassigned when an assignment is");
-    HTMLText(html, "wanted, assigned an unpreferred resource, part of");
-    HTMLText(html, "a split assignment, etc.), or else in italic when");
-    HTMLText(html, "unassigning the event resource would not violate an");
-    HTMLText(html, "event resource constraint, or else in roman.  It");
-    HTMLText(html, "may be abbreviated if it exceeds the column width,");
-    HTMLText(html, "or begins with a time name which is redundant in the");
-    HTMLText(html, "timetable context.");
+    HTMLText(html, "Each line of text in each cell represents one event");
+    HTMLText(html, "resource, by giving the name of the enclosing event.");
+    HTMLText(html, "Each line appears in bold italic font if the event");
+    HTMLText(html, "resource it names is subject to an event resource");
+    HTMLText(html, "constraint which is violated (the event resource may");
+    HTMLText(html, "be unassigned when an assignment is wanted, assigned an");
+    HTMLText(html, "unpreferred resource, part of a split assignment, etc.),");
+    HTMLText(html, "or else in italic when unassigning the event resource");
+    HTMLText(html, "would not violate an event resource constraint or");
+    HTMLText(html, "preassignment, or else in roman.  The name may be");
+    HTMLText(html, "abbreviated if it exceeds the column width, or begins");
+    HTMLText(html, "with a time name which is redundant in the context of");
+    HTMLText(html, "the printed table.");
+    if( planning )
+    {
+      HTMLText(html, "Below the table, Total Italic is the number of entries");
+      HTMLText(html,"printed in italic, signifying that unassigning would not");
+      HTMLText(html, "violate an event resource constraint or preassignment.");
+    }
     HTMLParagraphEnd(html);
 
+    /* print table of contents if more than one solution group */
+    if( KheArchiveSolnGroupCount(archive) >= 2 )
+    {
+      HTMLParagraphBegin(html);
+      for( i = 0;  i < KheArchiveSolnGroupCount(archive);  i++ )
+      {
+	if( i > 0 )
+	  HTMLNewLine(html);
+	soln_group = KheArchiveSolnGroup(archive, i);
+        sprintf(buff, "Solution Group %s", KheSolnGroupId(soln_group));
+	HTMLJumpInternal(html, KheSolnGroupId(soln_group), buff);
+      }
+      HTMLParagraphEnd(html);
+    }
+    HTMLHorizontalRule(html);
+
     /* print solution groups */
     for( i = 0;  i < KheArchiveSolnGroupCount(archive);  i++ )
     {
-      HTMLHorizontalRule(html);
       soln_group = KheArchiveSolnGroup(archive, i);
       SolutionGroupTimetables(soln_group, planning, highlight_splits,
 	with_event_groups, constraints_str, html, as);
