@@ -163,15 +163,6 @@ static int KheFactMod(int d, int c)
     c1f *= i;
   return ((d / c1f) + (d % c1f)) % c;
 }
-/* ***
-static int KheOffset4Mod(int d, int c) {
-  return (d + ((c >= 4) && (d >= 12)) + ((c >= 5) && (d >= 60))) % c; }
-static int KheOffset2Mod(int d, int c) { return (d + (c >= 4)) % c; }
-static int KhePrimeMod(int d, int c) { return (103 * d) % c; }
-static int KheSquareMod(int d, int c) { return (d * d) % c; }
-static int KheOffset3Mod(int d, int c) { return (d + (c >= 5)) % c; }
-static int KheDiv(int d, int c) { return ((d + 1) / c) % c; }
-*** */
 
 
 /*****************************************************************************/
@@ -191,16 +182,6 @@ static void KheDiversificationTest(int max_cols, FILE *fp)
   KheDivTest(&KheFact, "(d / fact(c-1)) % c", max_cols, 2,fp);
   KheDivTest(&KheFactMod, "((d / fact(c - 1)) + (d % fact(c - 1))) % c",
     max_cols, 2,fp);
-  /* ***
-  fprintf(fp, "\n");
-  KheDivTest(&KheOffset4Mod,
-    "(d + ((c >= 4) && (d >= 12)) + ((c >= 5) && (d >= 60))) % c",
-    max_cols, 2,fp);
-  KheDivTest(&KheOffset2Mod, "(d + (c >= 4)) % c", max_cols, 2, fp);
-  KheDivTest(&KhePrimeMod, "(103 * d) % c", max_cols, 2, fp);
-  KheDivTest(&KheSquareMod, "(d * d) % c", max_cols, 2, fp);
-  KheDivTest(&KheDiv, "((d + 1) / c) % c", 2, fp);
-  *** */
 }
 
 
@@ -220,19 +201,23 @@ static void KheUsageMessageAndExit(void)
   p("  khe -a <archive>           Audit and fix archive, write to stdout");
   p("  khe -e <archive>           Explode archive into one per instance");
   p("  khe -E <archive>           Explode archive into one per country");
+  p("  khe -m <id> <archive> ...  Merge archives into one with the given <id>");
   p("  khe -w <archive>           Write archive to stdout, neatly formatted");
   p("");
   p("Operations for solving archives");
   p("");
   p("  khe -r <archive> ...       Solve archive multiple times (see below)");
+  p("  khe -R <archive> ...       Solve archive multiple times (see below)");
   p("  khe -s <archive> <opts>    Solve archive once (see below)");
+  p("  khe -S <archive> <opts>    Solve archive once (see below)");
   p("");
   p("Operations for testing aspects of KHE");
   p("");
   p("  khe -b                     Test exponential backoff");
-  p("  khe -c <archive>           Test KheSolnCopy() on archive");
+  p("  khe -c <archive>           Test KheSolnCopy, KheSolnDelete on archive");
   p("  khe -d                     Test diversification functions");
   p("  khe -n                     Test the SSet module");
+  p("  khe -g                     Test the multiset generator module");
   p("  khe [ -u ]                 Write this usage message to stderr");
   p("  khe -v                     Write version number to stderr");
   p("");
@@ -242,11 +227,13 @@ static void KheUsageMessageAndExit(void)
   p("");
   p("  -x<id>{,<id>}              Delete instances with these Ids");
   p("  -i<id>{,<id>}              Include only instances with these Ids");
+  p("  -n<int>                    Include only the first <int> instances");
   p("  -X<id>{,<id>}              Delete solution groups with these Ids");
   p("  -I<id>{,<id>}              Include only solution groups with these Ids");
   p("");
-  p("Options -x and -i may not be used together.  Options -X and -I may not");
-  p("be used together.  As a special case, -X without ids means delete all");
+  p("Deleting a solution group includes deleting its solutions.  Options");
+  p("-x, -i, and -n may not be used together.  Options -X and -I may");
+  p("not be used together.  As a special case, -X without ids deletes all");
   p("solution groups.  These modifications occur only in the in-memory copy;");
   p("they do not affect the original file.  They are done before the archive");
   p("is used for anything else (exploded, written, solved, or whatever).");
@@ -254,13 +241,13 @@ static void KheUsageMessageAndExit(void)
   p("For -s, the <opts> part consists of any number of elements separated");
   p("by spaces.  Each element has the form <key>=<value>, or just <key>,");
   p("which is equivalent to <key>=true.  These pairs go into the options");
-  p("table passed to solvers when solving archive.  For example,");
+  p("table passed to solvers when solving <archive>.  For example,");
   p("");
-  p("  khe -s <archive> soln_group=test ps_threads=4 ps_make=4 ps_keep=1");
+  p("  khe -s <archive> ps_soln_group=test ps_threads=4 ps_make=8 ps_keep=1");
   p("");
   p("causes the output archive to have a solution group with name test,");
   p("created using 4 threads running in parallel.  For each instance,");
-  p("4 solutions are made and the best 1 of those is kept.  Consult the");
+  p("8 solutions are made and the best 1 of those is kept.  Consult the");
   p("User's Guide for the full list of options used by KHE's solvers.");
   p("");
   p("The -r flag allows an archive to be solved multiple times:");
@@ -273,8 +260,29 @@ static void KheUsageMessageAndExit(void)
   p("of options to be tried, each producing its own solution group.");
   p("");
   p("One can also use -S and -R instead of -s and -r.  The only difference");
-  p("is that the resource_type_partitions and limit_busy_recode parameters");
-  p("of the read are set to true.  This is recommended when solving.");
+  p("is that the resource_type_partitions parameter of the read is set to");
+  p("true.  This is recommended when solving.");
+  p("");
+  p("Furthermore, -s, -S, -r, and -R can be immediately followed by an o,");
+  p("as in -Ro for example.  This means that the solutions of the archive");
+  p("are to be read as ordinary solutions, suitable for detailed analysis");
+  p("and further solving, not merely as writable placeholders.");
+  /* ***
+  p("");
+  p("In all cases one can insert a -o option immediately after the command:");
+  p("");
+  p("  khe -o <file_name> -r ...");
+  p("");
+  p("for example.  This causes any output to go to a new file called");
+  p("<file_name> rather than to stdout.  But when solving it has a");
+  p("further meaning.  After each instance is solved, the result archive");
+  p("is written to <file_name>; and before the solve begins, if <file_name>");
+  p("exists it replaces <archive> as the archive being read, and any solves");
+  p("for which <file_name> already contains at least one solution are not");
+  p("done.  In this way, one can interrupt a long solve of many instances");
+  p("and restart it without losing very much, unless the interrupt happens");
+  p("while the archive is being written.");
+  *** */
   exit(1);
 }
 
@@ -406,6 +414,9 @@ static void KheExplodeArchive(KHE_ARCHIVE archive, HA_ARENA_SET as)
     }
     KheArchiveWrite(archive2, false, fp);
     fclose(fp);
+
+    /* delete the new archive */
+    KheArchiveDelete(archive2);   /* added JeffK 11/25 */
   }
 }
 
@@ -431,13 +442,13 @@ static char *KheCountryPrefix(char *str, HA_ARENA a)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheTestWriteArchive(KHE_ARCHIVE archive, char *name_prefix)         */
+/*  void KheWriteArchiveToName(KHE_ARCHIVE archive, char *name_prefix)       */
 /*                                                                           */
 /*  Write archive to name_prefix.xml.                                        */
 /*                                                                           */
 /*****************************************************************************/
 
-static void KheTestWriteArchive(KHE_ARCHIVE archive, char *name_prefix)
+static void KheWriteArchiveToName(KHE_ARCHIVE archive, char *name_prefix)
 {
   FILE *fp;  char fname[500];
   snprintf(fname, 500, "%s.xml", name_prefix);
@@ -466,7 +477,7 @@ static void KheExplodeArchiveByCountry(KHE_ARCHIVE archive, HA_ARENA_SET as)
   curr_archive = NULL;  prev_prefix = NULL;
   KheArchiveMetaData(archive, &a_name, &a_contributor, &a_date,
     &a_description, &a_remarks);
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   while( KheArchiveInstanceCount(archive) > 0 )
   {
     ins = KheArchiveInstance(archive, 0);
@@ -480,7 +491,10 @@ static void KheExplodeArchiveByCountry(KHE_ARCHIVE archive, HA_ARENA_SET as)
       {
 	/* write curr_archive, if any */
 	if( curr_archive != NULL )
-          KheTestWriteArchive(curr_archive, prev_prefix);
+	{
+          KheWriteArchiveToName(curr_archive, prev_prefix);
+	  KheArchiveDelete(curr_archive);   /* added JeffK 11/25 */
+	}
 
 	/* start off a new archive */
 	/* ***
@@ -499,8 +513,11 @@ static void KheExplodeArchiveByCountry(KHE_ARCHIVE archive, HA_ARENA_SET as)
     }
   }
   if( curr_archive != NULL )
-    KheTestWriteArchive(curr_archive, prev_prefix);
-  HaArenaSetArenaEnd(as, a);
+  {
+    KheWriteArchiveToName(curr_archive, prev_prefix);
+    KheArchiveDelete(curr_archive);   /* added JeffK 11/25 */
+  }
+  HaArenaDelete(a);
 }
 
 
@@ -535,6 +552,105 @@ static void GetSolveOptions(KHE_OPTIONS options, int argc,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  FILE *WriteFile(char *file_name)                                         */
+/*                                                                           */
+/*  Return a file open for writing, either file_name if non-NULL, or         */
+/*  else stdout.                                                             */
+/*                                                                           */
+/*****************************************************************************/
+
+/* ***
+static FILE *WriteFile(char *file_name)
+{
+  FILE *fp;
+  if( file_name != NULL )
+  {
+    fp = fopen(file_name, "w");
+    HnAssert(fp != NULL, "khe: cannot open \"%s\" for writing", file_name);
+  }
+  else
+    fp = stdout;
+  return fp;
+}
+*** */
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void SkipArchiveOnCommandLine(int argc, char *argv[], int *pos)          */
+/*                                                                           */
+/*  Move *pos over an <archive> on the command line.                         */
+/*                                                                           */
+/*****************************************************************************/
+
+/* ***
+static void SkipArchiveOnCommandLine(int argc, char *argv[], int *pos)
+{
+  ** skip <archive> **
+  (*pos)++;
+
+  ** skip -x, -i, -n, -X, and -I options **
+  while( argv[*pos][0] == '-' && (argv[*pos][1] == 'x' || argv[*pos][1] == 'i'
+      || argv[*pos][1] == 'n' || argv[*pos][1] == 'X' || argv[*pos][1] == 'I') )
+    (*pos)++;
+}
+*** */
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_ARCHIVE GetArchive(int argc, char *argv[], int *pos,                 */
+/*    HA_ARENA_SET as, bool audit_and_fix,                                   */
+/*    bool resource_type_partitions, bool infer_resource_partitions,         */
+/*    bool allow_invalid_solns, KHE_SOLN_TYPE soln_type, FILE *echo_fp,      */
+/*    HA_ARENA a)                                                            */
+/*                                                                           */
+/*  Read an archive from the command line with optional timing.              */
+/*                                                                           */
+/*****************************************************************************/
+
+static KHE_ARCHIVE GetArchive(int argc, char *argv[], int *pos,
+  HA_ARENA_SET as, bool audit_and_fix,
+  bool resource_type_partitions, bool infer_resource_partitions,
+  bool allow_invalid_solns, KHE_SOLN_TYPE soln_type, FILE *echo_fp,
+  /* char *out_file_name, */ HA_ARENA a)
+{
+  KHE_TIMER timer;  /* FILE *fp; */  KHE_ARCHIVE archive;  /* KML_ERROR ke; */
+
+  if( DEBUG4 )
+    timer = KheTimerMake("archive_read", KHE_NO_TIME, a);
+  /* ***
+  if( out_file_name != NULL && (fp = fopen(out_file_name, "r")) != NULL )
+  {
+    ** use out_file_name as the initial archive **
+    if( !KheArchiveRead(fp, as, &archive, &ke, audit_and_fix,
+	resource_type_partitions, infer_resource_partitions,
+	allow_invalid_solns, soln_type, echo_fp) )
+      HnFatal("%s:%d:%d: %s\n", out_file_name, KmlErrorLineNum(ke),
+	KmlErrorColNum(ke), KmlErrorString(ke));
+    fclose(fp);
+
+    ** skip <archive> and its -n etc. options **
+    SkipArchiveOnCommandLine(argc, argv, pos);
+  }
+  else
+  {
+  *** */
+
+  /* use the <archive> command line option as the initial archive */
+  archive = KheArchiveReadFromCommandLine(argc, argv, pos, as, audit_and_fix,
+    resource_type_partitions, infer_resource_partitions, allow_invalid_solns,
+    soln_type, echo_fp);
+  /* } */
+  if( DEBUG4 )
+    fprintf(stderr, "  archive read time: %.1f secs\n",
+      KheTimerElapsedTime(timer));
+  return archive;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  int main(int argc, char *argv[])                                         */
@@ -545,40 +661,59 @@ static void GetSolveOptions(KHE_OPTIONS options, int argc,
 
 int main(int argc, char *argv[])
 {
-  KHE_ARCHIVE archive;  KHE_SOLN soln;  char op;  bool no_print;
-  KHE_SOLN_GROUP soln_group;  int i, j, pos;  KHE_TIMER timer;
-  KHE_OPTIONS options, roptions;  HA_ARENA a;  HA_ARENA_SET as;
+  KHE_ARCHIVE res_archive, archive;  KHE_SOLN soln;  char op, op2;
+  bool no_print;  KHE_SOLN_GROUP soln_group;  int i, j, pos /* , s */;
+  KHE_OPTIONS options, roptions;  HA_ARENA a;  HA_ARENA_SET as;  char *id;
+  /* char *results[20], *out_file_name */;
+  KHE_SOLN_TYPE soln_type;
 
   if( DEBUG_MALLOC )
     mallopt(M_CHECK_ACTION, 3);
 
-  /* decide which operation is wanted */
-  a = HaArenaMake(false);
-  as = HaArenaSetMake(a);
-  if( argc == 1 )
-    op = 'u';
-  else if( argc >= 2 && argv[1][0] == '-' && argv[1][1] != '\0' )
-    op = argv[1][1];
+  /* get the optional -o argument and set pos */
+  /* ***
+  if( argc >= 3 && strcmp(argv[1], "-o") == 0 )
+  {
+    ** khe -o <file_name> **
+    out_file_name = argv[2];
+    pos = 3;
+  }
   else
   {
-    fprintf(stderr, "khe: unexpected first option %s\n", argv[1]);
-    op = 'u';
+    out_file_name = NULL;
+    pos = 1;
+  }
+  *** */
+  pos = 1;
+
+  /* decide which operation is wanted */
+  if( argc == 1 )
+    op = 'u', op2 = '\0';
+  else if( argc >= 2 && argv[pos][0] == '-' && argv[pos][1] != '\0' )
+    op = argv[pos][1], op2 = argv[pos][2];
+  else
+  {
+    fprintf(stderr, "khe: unexpected first option %s\n", argv[pos]);
+    op = 'u', op2 = '\0';
   }
   if( DEBUG1 )
     fprintf(stderr, "khe -%c (argc = %d)\n", op, argc);
 
   /* do the operation */
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);
   switch( op )
   {
     case 'a':
 
       /* khe -a <archive> */
-      pos = 2;
+      pos++;
       archive = KheArchiveReadFromCommandLine(argc, argv, &pos, as, true,
-	false, false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
+	false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
       if( pos != argc )
 	HnAbort("khe -a:  unexpected options on command line after <archive>");
-      KheArchiveWrite(archive, true, stdout);
+      KheArchiveWrite(archive, true, stdout /*WriteFile(out_file_name)*/);
+      KheArchiveDelete(archive);  /* added by JeffK 11/25, suggested by Ortiz */
       break;
 
     case 'b':
@@ -586,15 +721,15 @@ int main(int argc, char *argv[])
       /* khe -b */
       if( argc != 2 )
         KheUsageMessageAndExit();
-      KheBackoffTest(stdout);
+      KheBackoffTest(stdout /*WriteFile(out_file_name)*/);
       break;
 
     case 'c':
 
       /* khe -c <archive> */
-      pos = 2;
+      pos++;
       archive = KheArchiveReadFromCommandLine(argc, argv, &pos,
-	as, false, false, false, false, false, KHE_SOLN_ORDINARY, NULL);
+	as, false, false, false, false, KHE_SOLN_ORDINARY, NULL);
       if( pos != argc )
 	HnAbort("khe -c:  unexpected options on command line after <archive>");
       for( i = 0;  i < KheArchiveSolnGroupCount(archive);  i++ )
@@ -604,8 +739,10 @@ int main(int argc, char *argv[])
 	{
 	  soln = KheSolnGroupSoln(soln_group, j);
 	  soln = KheSolnCopy(soln, as);
+	  KheSolnDelete(soln);  /* added by JeffK 11/25 */
 	}
       }
+      KheArchiveDelete(archive);  /* added by JeffK 11/25, suggested by Ortiz */
       break;
 
     case 'd':
@@ -613,90 +750,97 @@ int main(int argc, char *argv[])
       /* khe -d */
       if( argc != 2 )
         KheUsageMessageAndExit();
-      KheDiversificationTest(3, stdout);
+      KheDiversificationTest(3, stdout /*WriteFile(out_file_name)*/);
       break;
 
     case 'e':
 
       /* khe -e <archive> */
-      pos = 2;
+      pos++;
       archive = KheArchiveReadFromCommandLine(argc, argv, &pos, as, false,
-	false, false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
+	false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
       if( pos != argc )
 	HnAbort("khe -e:  unexpected options on command line after <archive>");
       KheExplodeArchive(archive, as);
+      KheArchiveDelete(archive);  /* added by JeffK 11/25, suggested by Ortiz */
       break;
 
     case 'E':
 
       /* khe -E <archive> */
-      pos = 2;
+      pos++;
       archive = KheArchiveReadFromCommandLine(argc, argv, &pos, as, false,
-	false, false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
+	false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
       if( pos != argc )
 	HnAbort("khe -E:  unexpected options on command line after <archive>");
       KheExplodeArchiveByCountry(archive, as);
+      KheArchiveDelete(archive);  /* added by JeffK 11/25, suggested by Ortiz */
       break;
 
-    /* *** withdrawn
     case 'g':
-    case 'h':
 
-      ** khe -g<letters> <archive> **
-      ** khe -h<letters> <archive> **
-      pos = 2;
-      archive = KheArchiveReadFromCommandLine(argc, argv, &pos,
-	true, true, false, NULL);
-      if( pos != argc )
-	HnAbort("khe -%c:  unexpected options on command line after <archive>",
-	  op);
-      len = strlen(argv[1]);
-      for( i = 2;  i < len;  i++ )
-	KheBenchmark(archive, &KheGeneralSolve2020, "KHE18",
-	  "Jeffrey H. Kingston", argv[1][i],
-	  op == 'g' ? KHE_STATS_TABLE_LOUT : KHE_STATS_TABLE_LATEX);
+      /* khe -g */
+      KheMulisetGeneratorTest(a);
       break;
-    *** */
 
-    /* ***
-    case 'l':
-
-      ** khe -l **
-      if( argc != 2 )
-        KheUsageMessageAndExit();
-      LSetTest(stdout);
-      break;
-    *** */
-
-    /* ***
     case 'm':
-
-      ** khe -m **
-      if( argc != 2 )
-        KheUsageMessageAndExit();
-      MTest(stdout);
+     
+      /* khe -m <id> <archive> ... <archive> */
+      if( argc < 4 )
+	HnAbort("khe -m:  too few command line arguments");
+      pos++;
+      id = argv[pos];
+      pos++;
+      archive = KheArchiveReadFromCommandLine(argc, argv, &pos, as,
+	false, false, false, true, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
+      res_archive = KheArchiveMake(id, KheArchiveModel(archive), as);
+      KheArchiveSetMetaData(res_archive, id, "khe", KheDateToday(),
+	"Meld of other archives", NULL);
+      KheArchiveMeld(res_archive, archive);
+      KheArchiveDelete(archive);
+      while( pos < argc )
+      {
+	archive = KheArchiveReadFromCommandLine(argc, argv, &pos, as,
+	  false, false, false, true, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
+	KheArchiveMeld(res_archive, archive);
+	KheArchiveDelete(archive);
+      }
+      KheArchiveWrite(res_archive, false, stdout);
+      KheArchiveDelete(res_archive);
+      /* ***
+      ** khe -m: malloc test **
+      fprintf(stderr, "[ malloc test; sizeof(void *) = %ld\n",
+	sizeof(void *));
+      for( i = 0, s = 1;  i < 20;  i++, s *= 2 )
+      {
+	results[i] = malloc(s);
+	if( i > 0 )
+	  fprintf(stderr, "    %p - %p = %ld (%ld words), overhead %ld bytes\n",
+	    results[i], results[i-1], results[i] - results[i-1],
+	    (results[i] - results[i-1]) / sizeof(void *),
+	    (results[i] - results[i-1]) - s/2);
+	fprintf(stderr, "  malloc(%d bytes, %ld words) = %p\n", s,
+	  s / sizeof(void *), results[i]);
+      }
+      fprintf(stderr, "]\n");
+      exit(0);
+      *** */
       break;
-    *** */
 
     case 'n':
 
       /* khe -n */
-      if( argc != 2 )
-        KheUsageMessageAndExit();
-      SSetTest(stdout);
+      SSetTest(stdout /*WriteFile(out_file_name)*/);
       break;
 
     case 'r':
     case 'R':
 
       /* khe -r <archive> <options> -s <options> -s <options> ... */
-      pos = 2;
-      timer = KheTimerMake("archive_read", KHE_NO_TIME, a);
-      archive = KheArchiveReadFromCommandLine(argc, argv, &pos, as, false,
-	op == 'R', true, op == 'R', false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
-      if( DEBUG4 )
-	fprintf(stderr, "  archive read time: %.1f secs\n",
-	  KheTimerElapsedTime(timer));
+      pos++;
+      soln_type =(op2=='o' ? KHE_SOLN_ORDINARY : KHE_SOLN_WRITABLE_PLACEHOLDER);
+      archive = GetArchive(argc, argv, &pos, as, false, op == 'R', true,
+	false, soln_type, NULL, /* out_file_name, */ a);
       roptions = KheOptionsMake(a);
       GetSolveOptions(roptions, argc, argv, &pos);
       no_print = KheOptionsGetBool(roptions, "no_print", false);
@@ -709,46 +853,55 @@ int main(int argc, char *argv[])
 	options = KheOptionsCopy(roptions, a);
 	GetSolveOptions(options, argc, argv, &pos);
 	/* soln_group = KheTestMakeSolnGroup(archive, options); */
-	KheArchiveParallelSolve(archive, &KheGeneralSolve2020,
-	  options, KHE_SOLN_WRITABLE_PLACEHOLDER, as);
+	KheArchiveParallelSolve(archive, &KheGeneralSolve2024,
+	  options, KHE_SOLN_WRITABLE_PLACEHOLDER, /* out_file_name, */ as);
       }
-      if( !no_print )
+      if( !no_print /* && out_file_name == NULL */ )
       {
-	KheTimerResetStartTime(timer);
-	KheArchiveWrite(archive, false, stdout);
+	/* ***
+	if( DEBUG4 )
+	  KheTimerResetStartTime(timer);
+	*** */
+	KheArchiveWrite(archive, false, stdout /*WriteFile(out_file_name)*/ );
+	/* ***
 	if( DEBUG4 )
 	  fprintf(stderr, "  archive write time: %.1f secs\n",
 	    KheTimerElapsedTime(timer));
+	*** */
       }
+      KheArchiveDelete(archive);
       break;
 
     case 's':
     case 'S':
 
       /* khe -s <archive> <options> */
-      pos = 2;
-      timer = KheTimerMake("archive_read", KHE_NO_TIME, a);
-      archive = KheArchiveReadFromCommandLine(argc, argv, &pos, as, false,
-	op == 'S', true, op == 'S', false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
-      if( DEBUG4 )
-	fprintf(stderr, "  archive read time: %.1f secs\n",
-	  KheTimerElapsedTime(timer));
+      pos++;
+      soln_type =(op2=='o' ? KHE_SOLN_ORDINARY : KHE_SOLN_WRITABLE_PLACEHOLDER);
+      archive = GetArchive(argc, argv, &pos, as, false, op == 'R', true,
+	false, soln_type, NULL, /* out_file_name, */ a);
       options = KheOptionsMake(a);
       GetSolveOptions(options, argc, argv, &pos);
       if( pos != argc )
 	HnAbort("khe -w:  unexpected options on command line after <archive>");
       no_print = KheOptionsGetBool(options, "no_print", false);
       /* soln_group = KheTestMakeSolnGroup(archive, options); */
-      KheArchiveParallelSolve(archive, &KheGeneralSolve2020, options,
-	KHE_SOLN_WRITABLE_PLACEHOLDER, as);
-      if( !no_print )
+      KheArchiveParallelSolve(archive, &KheGeneralSolve2024, options,
+	KHE_SOLN_WRITABLE_PLACEHOLDER, /* out_file_name, */ as);
+      if( !no_print /* && out_file_name == NULL */ )
       {
-	KheTimerResetStartTime(timer);
-	KheArchiveWrite(archive, false, stdout);
+	/* ***
+	if( DEBUG4 )
+	  KheTimerResetStartTime(timer);
+	*** */
+	KheArchiveWrite(archive, false, stdout /*WriteFile(out_file_name)*/);
+	/* ***
 	if( DEBUG4 )
 	  fprintf(stderr, "  archive write time: %.1f secs\n",
 	    KheTimerElapsedTime(timer));
+	*** */
       }
+      KheArchiveDelete(archive);
       break;
 
     case 'u':
@@ -764,12 +917,13 @@ int main(int argc, char *argv[])
     case 'w':
 
       /* khe -w <archive> */
-      pos = 2;
+      pos++;
       archive = KheArchiveReadFromCommandLine(argc, argv, &pos, as, false,
-	false, false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
+	false, false, false, KHE_SOLN_WRITABLE_PLACEHOLDER, NULL);
       if( pos != argc )
 	HnAbort("khe -w:  unexpected options on command line after <archive>");
-      KheArchiveWrite(archive, false, stdout);
+      KheArchiveWrite(archive, false, stdout /*WriteFile(out_file_name)*/);
+      KheArchiveDelete(archive);
       break;
 
     default:
@@ -778,4 +932,5 @@ int main(int argc, char *argv[])
       KheUsageMessageAndExit();
   }
   HaArenaDelete(a);
+  HaArenaSetDelete(as);  /* added by JeffK 11/25, suggested by Ortiz */
 }
