@@ -107,7 +107,7 @@ char *NrcVersionBanner(void)
 NRC_ARCHIVE NrcArchiveMake(char *id, HA_ARENA_SET as)
 {
   NRC_ARCHIVE res;  HA_ARENA a;
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   HaMake(res, a);
   res->arena = a;
   res->arena_set = as;
@@ -136,7 +136,7 @@ NRC_ARCHIVE NrcArchiveMake(char *id, HA_ARENA_SET as)
 
 HA_ARENA NrcArchiveArenaBegin(NRC_ARCHIVE archive)
 {
-  return HaArenaSetArenaBegin(archive->arena_set, false);
+  return HaArenaMake(archive->arena_set);
 }
 
 
@@ -150,7 +150,7 @@ HA_ARENA NrcArchiveArenaBegin(NRC_ARCHIVE archive)
 
 void NrcArchiveArenaEnd(NRC_ARCHIVE archive, HA_ARENA a)
 {
-  HaArenaSetArenaEnd(archive->arena_set, a);
+  HaArenaDelete(a);
 }
 
 
