@@ -176,7 +176,7 @@ NRC_SOLN NrcSolnMake(NRC_INSTANCE ins, HA_ARENA_SET as)
   NRC_SOLN res;  NRC_SOLN_DAY sd;  NRC_DAY d;  int i;  HA_ARENA a;
   HnAssert(NrcInstanceComplete(ins),
     "NrcSolnMake called on instance lacking NrcIinstanceMakeEnd");
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   HaMake(res, a);
   res->arena = a;
   res->instance = ins;
