@@ -1576,7 +1576,7 @@ NRC_INSTANCE CQ14ConvertInstance(INSTANCE_MODEL ins_model,
   CQ_INFO ii;  char *ins_id;  HA_ARENA a;  NRC_INSTANCE res;
 
   /* make an instance info object, initially more or less empty */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   HaMake(ii, a);
   ii->arena = a;
   ii->file_name = instance_file_name;
@@ -1615,7 +1615,7 @@ NRC_INSTANCE CQ14ConvertInstance(INSTANCE_MODEL ins_model,
   /* all done */
   NrcInstanceMakeEnd(ii->ins);
   res = ii->ins;
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
   return res;
 }
 
@@ -1769,7 +1769,7 @@ void CQ14ConvertSoln(SOLN_MODEL soln_model, char *soln_file_name,
   }
 
   /* read XML from the file, and exit if can't */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   if( !KmlReadFile(fp, NULL, &soln_elt, &ke, a) )
   {
     KmlWarning(ke, soln_file_name);
@@ -1927,7 +1927,7 @@ void CQ14ConvertSoln(SOLN_MODEL soln_model, char *soln_file_name,
    SolnModelAddSoln(soln_model, archive, id, secs, soln);
   if( DEBUG12 )
     Warning(soln_file_name, -1, "including solution");
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
   /* KmlFree(soln_elt); */
   if( DEBUG8 )
     fprintf(stderr, "] CQ14ConvertSoln returning\n");
