@@ -1780,7 +1780,7 @@ NRC_INSTANCE INRC1ConvertInstance(INSTANCE_MODEL ins_model,
   char *ins_id;  KML_ELT elt;  HA_ARENA a;  NRC_INSTANCE res;
 
   /* make an instance info object, initially more or less empty */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   HaMake(ii, a);
   ii->arena = a;
   ii->file_name = instance_file_name;
@@ -1842,7 +1842,7 @@ NRC_INSTANCE INRC1ConvertInstance(INSTANCE_MODEL ins_model,
   AddShiftOffRequests(ii);
   NrcInstanceMakeEnd(ii->ins);
   res = ii->ins;
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
   return res;
 }
 
@@ -1882,7 +1882,7 @@ void INRC1ConvertSoln(SOLN_MODEL soln_model, char *soln_file_name,
   }
 
   /* read XML from the file, and exit if can't */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   if( !KmlReadFile(fp, NULL, &soln_elt, &ke, a) )
     KmlFatalError(ke, soln_file_name);
   fclose(fp);
@@ -1942,5 +1942,5 @@ void INRC1ConvertSoln(SOLN_MODEL soln_model, char *soln_file_name,
 
   /* add soln to the appropriate soln group */
   SolnModelAddSoln(soln_model, archive, "", 0.0, soln);
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
 }
