@@ -1356,7 +1356,7 @@ NRC_INSTANCE INRC2ConvertInstance(INSTANCE_MODEL ins_model,
     fprintf(stderr, "[ INRC2ConvertInstance(%s)\n", instance_file_name);
 
   /* make an instance info object, initially more or less empty */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   ii = INRC2InfoMake(a);
 
   /* dismantle instance_file_name */
@@ -1462,7 +1462,7 @@ NRC_INSTANCE INRC2ConvertInstance(INSTANCE_MODEL ins_model,
   ins_id = HnStringMake(a, "C2-%s-w%s-h%s", sc_id, wk_id, hy_id);
   INRC2BuildInstance(ii, ins_id, as);
   ins = ii->ins;
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
   if( DEBUG1 )
     fprintf(stderr, "] INRC2ConvertInstance returning\n");
   return ins;
@@ -1527,7 +1527,7 @@ void INRC2ConvertSoln(SOLN_MODEL soln_model, char *soln_file_name,
   }
 
   /* read XML from the file, and exit if can't */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   if( !KmlReadFile(fp, NULL, &soln_elt, &ke, a) )
     KmlFatalError(ke, soln_file_name);
   fclose(fp);
@@ -1601,7 +1601,7 @@ void INRC2ConvertSoln(SOLN_MODEL soln_model, char *soln_file_name,
 
   /* add soln to the appropriate soln group */
   SolnModelAddSoln(soln_model, archive, "", 0.0, soln);
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
 }
 
 
@@ -1620,14 +1620,15 @@ void INRC2ConvertSoln(SOLN_MODEL soln_model, char *soln_file_name,
 /*****************************************************************************/
 #define is_digit(c) ((c) >= '0' && (c) <= '9')
 
-static bool GrabInt(char **p, char *val)
+static bool GrabInt(char **p, char **val, HA_ARENA a)
 {
-  if( **p == '\0' || !sscanf(*p, "%[0-9]", val) )
-    return false;
-  *p += strlen(val);
+  char buff[13];
+  if( **p == '\0' || !sscanf(*p, "%12[0-9]", buff) )
+    return *val = "", false;
+  *p += strlen(buff);
   while( **p != '\0' && !is_digit(**p) )
     (*p)++;
-  return true;
+  return *val = HnStringMake(a, "%s", buff), true;
 }
 
 
@@ -1681,8 +1682,8 @@ NRC_INSTANCE INRC2ConvertInstanceStatic(INSTANCE_MODEL ins_model,
 {
   char *suffix, *p;
   int len, suffix_len, weeks, count;  char save_suffix;
-  char field1[500], field2[500], field3[500], field4[500], tag[500],ins_id[500];
-  char dir_name[500], sc_file_name[500], hy_file_name[500], wk_file_name[500];
+  char *field1, *field2, *field3, *field4, *tag, *ins_id;
+  char *dir_name, *sc_file_name, *hy_file_name, *wk_file_name;
   INRC2_INFO ii;  HA_ARENA a;  NRC_INSTANCE ins;
 
   if( DEBUG7 )
@@ -1690,7 +1691,7 @@ NRC_INSTANCE INRC2ConvertInstanceStatic(INSTANCE_MODEL ins_model,
       instance_file_name);
 
   /* make an instance info object, initially more or less empty */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   ii = INRC2InfoMake(a);
 
   /* let suffix point to just after the last / in the file name */
@@ -1704,41 +1705,47 @@ NRC_INSTANCE INRC2ConvertInstanceStatic(INSTANCE_MODEL ins_model,
   HnAssert(suffix_len >= 7,
     "INRC2ConvertInstanceStatic: file name suffix \"%s\" is too short", suffix);
   p = suffix;
-  GrabInt(&p, field1);
-  GrabInt(&p, field2);
-  GrabInt(&p, field3);
-  sprintf(tag, "n%sw%s", field1, field2);
+  GrabInt(&p, &field1, a);
+  GrabInt(&p, &field2, a);
+  GrabInt(&p, &field3, a);
+  tag = HnStringMake(a, "n%sw%s", field1, field2);
+  /* sprintf(tag, "n%sw%s", field1, field2); */
   if( DEBUG7 )
     fprintf(stderr, "  tag \"%s\"\n", tag);
 
   /* build the directory name */
   save_suffix = *suffix;
   *suffix = '\0';
-  sprintf(dir_name, "%s%s", instance_file_name, tag);
+  dir_name = HnStringMake(a, "%s%s", instance_file_name, tag);
+  /* sprintf(dir_name, "%s%s", instance_file_name, tag); */
   *suffix = save_suffix;
   if( DEBUG7 )
     fprintf(stderr, "  dir \"%s\"\n", dir_name);
 
   /* scenario file */
-  sprintf(sc_file_name, "%s/Sc-%s.xml", dir_name, tag);
+  sc_file_name = HnStringMake(a, "%s/Sc-%s.xml", dir_name, tag);
+  /* sprintf(sc_file_name, "%s/Sc-%s.xml", dir_name, tag); */
   ii->sc_file = FileMake(sc_file_name, "scenario", a);
   if( DEBUG7 )
     fprintf(stderr, "  sc_file_name \"%s\"\n", sc_file_name);
 
   /* history file */
-  sprintf(hy_file_name, "%s/H0-%s-%s.xml", dir_name, tag, field3);
+  hy_file_name = HnStringMake(a, "%s/H0-%s-%s.xml", dir_name, tag, field3);
+  /* sprintf(hy_file_name, "%s/H0-%s-%s.xml", dir_name, tag, field3); */
   if( DEBUG7 )
     fprintf(stderr, "  hy_file_name \"%s\"\n", hy_file_name);
   ii->hy_file = FileMake(hy_file_name, "history", a);
 
   /* weekly file names, and also instanced id */
-  sprintf(ins_id, "INRC2-%s-%s-%s-", field2, field1, field3);
+  ins_id = HnStringMake(a, "INRC2-%s-%s-%s-", field2, field1, field3);
+  /* sprintf(ins_id, "INRC2-%s-%s-%s-", field2, field1, field3); */
   sscanf(field2, "%d", &weeks);
   count = 0;
-  while( GrabInt(&p, field4) )
+  while( GrabInt(&p, &field4, a) )
   {
     count++;
-    sprintf(wk_file_name, "%s/WD-%s-%s.xml", dir_name, tag, field4);
+    wk_file_name = HnStringMake(a, "%s/WD-%s-%s.xml", dir_name, tag, field4);
+    /* sprintf(wk_file_name, "%s/WD-%s-%s.xml", dir_name, tag, field4); */
     strcpy(&ins_id[strlen(ins_id)], field4);
     if( DEBUG7 )
       fprintf(stderr, "  wk_file_name %d: \"%s\"\n", count, wk_file_name);
@@ -1756,7 +1763,7 @@ NRC_INSTANCE INRC2ConvertInstanceStatic(INSTANCE_MODEL ins_model,
   /* build the instance and return it */
   INRC2BuildInstance(ii, ins_id, as);
   ins = ii->ins;
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
   if( DEBUG7 )
     fprintf(stderr, "] INRC2ConvertInstanceStatic returning\n");
   return ins;
@@ -1773,18 +1780,19 @@ NRC_INSTANCE INRC2ConvertInstanceStatic(INSTANCE_MODEL ins_model,
 /*                                                                           */
 /*****************************************************************************/
 
-static char *SolnFileNameToInstanceIdOmer(char *soln_file_name)
+static char *SolnFileNameToInstanceIdOmer(char *soln_file_name, HA_ARENA a)
 {
-  char *suffix, *p;  char field1[500], field2[500], field3[500];
-  static char res[500];
+  char *suffix, *p, *field1, *field2, *field3, *res;
   suffix = FileNameSuffix(soln_file_name);
   p = &suffix[1];
-  GrabInt(&p, field1);
-  GrabInt(&p, field2);
-  GrabInt(&p, field3);
-  sprintf(res, "INRC2-%s-%s-%s-", field2, field1, field3);
-  while( GrabInt(&p, field1) )
-    strcpy(&res[strlen(res)], field1);
+  GrabInt(&p, &field1, a);
+  GrabInt(&p, &field2, a);
+  GrabInt(&p, &field3, a);
+  res = HnStringMake(a, "INRC2-%s-%s-%s-", field2, field1, field3);
+  /* sprintf(res, "INRC2-%s-%s-%s-", field2, field1, field3); */
+  while( GrabInt(&p, &field1, a) )
+    res = HnStringMake(a, "%s%s", res, field1);
+    /* strcpy(&res[strlen(res)], field1); */
   return res;
 }
 
@@ -1802,18 +1810,19 @@ static char *SolnFileNameToInstanceIdOmer(char *soln_file_name)
 /*                                                                           */
 /*****************************************************************************/
 
-static char *SolnFileNameToInstanceIdSchaerf(char *soln_file_name)
+static char *SolnFileNameToInstanceIdSchaerf(char *soln_file_name, HA_ARENA a)
 {
-  char *suffix, *p;  char field1[500], field2[500], field3[500];
-  static char res[500];
+  char *suffix, *p, *field1, *field2, *field3, *res;
   suffix = FileNameSuffix(soln_file_name);
   p = suffix;
-  GrabInt(&p, field1);
-  GrabInt(&p, field2);
-  GrabInt(&p, field3);
-  sprintf(res, "INRC2-%s-%s-%s-", field2, field1, field3);
-  while( GrabInt(&p, field1) )
-    strcpy(&res[strlen(res)], field1);
+  GrabInt(&p, &field1, a);
+  GrabInt(&p, &field2, a);
+  GrabInt(&p, &field3, a);
+  res = HnStringMake(a, "INRC2-%s-%s-%s-", field2, field1, field3);
+  /* sprintf(res, "INRC2-%s-%s-%s-", field2, field1, field3); */
+  while( GrabInt(&p, &field1, a) )
+    res = HnStringMake(a, "%s%s", res, field1);
+    /* strcpy(&res[strlen(res)], field1); */
   return res;
 }
 
@@ -1968,11 +1977,11 @@ void INRC2ConvertSolnOmer(SOLN_MODEL soln_model, char *soln_file_name,
   if( DEBUG8 )
     fprintf(stderr, "[ INRC2ConvertSolnOmer(-, %s, %s, -, -)\n",
       soln_file_name, ins == NULL ? "NULL" : NrcInstanceId(ins));
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
 
   /* get the instance */
   HnAssert(ins == NULL, "INRC2ConvertSolnOmer passed non-NULL instance");
-  ins_id = SolnFileNameToInstanceIdOmer(soln_file_name);
+  ins_id = SolnFileNameToInstanceIdOmer(soln_file_name, a);
   if( !NrcArchiveRetrieveInstance(archive, ins_id, &ins) )
     HnAbort("INRC2ConvertSolnOmer: cannot retrieve instance \"%s\"", ins_id);
   if( DEBUG8 )
@@ -1992,7 +2001,7 @@ void INRC2ConvertSolnOmer(SOLN_MODEL soln_model, char *soln_file_name,
 
   /* add soln to the appropriate soln group and return */
   SolnModelAddSoln(soln_model, archive, "", 0.0, soln);
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
   if( DEBUG8 )
     fprintf(stderr, "] INRC2ConvertSolnOmer returning\n");
 }
@@ -2017,11 +2026,11 @@ void INRC2ConvertSolnSchaerf(SOLN_MODEL soln_model, char *soln_file_name,
   if( DEBUG8 )
     fprintf(stderr, "[ INRC2ConvertSolnSchaerf(-, %s, %s, -, -)\n",
       soln_file_name, ins == NULL ? "NULL" : NrcInstanceId(ins));
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
 
   /* get the instance */
   HnAssert(ins == NULL, "INRC2ConvertSolnSchaerf passed non-NULL instance");
-  ins_id = SolnFileNameToInstanceIdSchaerf(soln_file_name);
+  ins_id = SolnFileNameToInstanceIdSchaerf(soln_file_name, a);
   if( !NrcArchiveRetrieveInstance(archive, ins_id, &ins) )
     HnAbort("INRC2ConvertSolnSchaerf: cannot retrieve instance \"%s\"", ins_id);
   if( DEBUG8 )
@@ -2041,7 +2050,7 @@ void INRC2ConvertSolnSchaerf(SOLN_MODEL soln_model, char *soln_file_name,
 
   /* add soln to the appropriate soln group and return */
   SolnModelAddSoln(soln_model, archive, "", 0.0, soln);
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
   if( DEBUG8 )
     fprintf(stderr, "] INRC2ConvertSolnSchaerf returning\n");
 }
