@@ -298,8 +298,8 @@ int main(int argc, char *argv[])
   }
 
   /* initialize various state variables */
-  a = HaArenaMake(false);
-  as = HaArenaSetMake(a);
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);
   HnTableInit(instances, a);
   archive = NULL;
   out_fp = NULL;
