@@ -412,7 +412,7 @@ void Map(char *map_file_name, HA_ARENA_SET as)
   HA_ARRAY_NSTRING lines;  bool match;  MAP_SET map_set;  HA_ARENA a;
 
   /* read the maps into the map table */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   map_fp = fopen(map_file_name, "r");
   if( map_fp == NULL )
     HnAbort("nrconv: cannot open map file \"%s\"", map_file_name);
@@ -515,5 +515,5 @@ void Map(char *map_file_name, HA_ARENA_SET as)
     }
     GetLines(&lines, 1, stdin, a);
   }
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
 }
