@@ -474,7 +474,7 @@ bool HpTableImplContainsNext(HP_GROUP *table, int *pos)
 
 void HpTableImplDelete(HP_GROUP *table, int pos)
 {
-  HaAssert(table->keys[pos] > (void *) 0x1, L"HpTableDelete: no entry at pos");
+  HnAssert(table->keys[pos] > (void *) 0x1, "HpTableDelete: no entry at pos");
   if( DEBUG1 )
     fprintf(stderr, "[ HpTableImplDelete(%p, %d %p)\n", (void *) table,
       pos, table->keys[pos]);
