@@ -34,6 +34,7 @@
 #define DEBUG2 0
 #define DEBUG3 0
 #define DEBUG4 0
+#define DEBUG5 0
 
 
 /*****************************************************************************/
@@ -89,7 +90,6 @@ typedef struct khe_read_state_rec {
   bool			audit_and_fix;
   bool			resource_type_partitions;
   bool			infer_resource_partitions;
-  bool			limit_busy_recode;
   bool			allow_invalid_solns;
   KHE_SOLN_TYPE		soln_type;
   KHE_ARCHIVE_FN	archive_begin_fn;
@@ -189,7 +189,7 @@ static void KheIndexedInstanceDelete(KHE_INDEXED_INSTANCE ii)
 KHE_ARCHIVE KheArchiveMake(char *id, KHE_MODEL model, HA_ARENA_SET as)
 {
   KHE_ARCHIVE res;  HA_ARENA a;
-  a = HaArenaSetArenaBegin(as, true);
+  a = HaArenaMake(as);
   HaMake(res, a);
   res->arena = a;
   res->arena_set = as;
@@ -210,6 +210,101 @@ KHE_ARCHIVE KheArchiveMake(char *id, KHE_MODEL model, HA_ARENA_SET as)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheArchiveDelete(KHE_ARCHIVE archive)                               */
+/*                                                                           */
+/*  Delete archive along with its instances, solution groups, and solutions. */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheArchiveDelete(KHE_ARCHIVE archive)
+{
+  KheArchiveClear(archive);
+  HaArenaDelete(archive->arena);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheArchiveClear(KHE_ARCHIVE archive)                                */
+/*                                                                           */
+/*  Clear archive's instances and solution groups, leaving it empty except   */
+/*  for its metadata.                                                        */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheArchiveClear(KHE_ARCHIVE archive)
+{
+  KHE_INSTANCE ins;
+
+  /* delete archive's solution groups */
+  while( HaArrayCount(archive->soln_group_array) > 0 )
+    KheSolnGroupDelete(HaArrayFirst(archive->soln_group_array), true);
+
+  /* delete archive's instances, either completely or just from archive */
+  while( HaArrayCount(archive->instance_array) > 0 )
+  {
+    ins = HaArrayFirst(archive->instance_array)->instance;
+    if( KheInstanceArchiveCount(ins) == 1 )
+      KheInstanceDelete(ins);
+    else
+      KheArchiveDeleteInstance(archive, ins);
+  }
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheArchiveMeld(KHE_ARCHIVE dst_archive, KHE_ARCHIVE src_archive)    */
+/*                                                                           */
+/*  Meld the instances, solution groups, and solutions of src_archive        */
+/*  into dst_archive.                                                        */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheArchiveMeld(KHE_ARCHIVE dst_archive, KHE_ARCHIVE src_archive)
+{
+  int i, j;  KHE_INSTANCE ins;  KHE_SOLN_GROUP src_sg, dst_sg;  char *id;
+  KHE_SOLN soln;
+
+  /* check that the models are equal */
+  HnAssert(dst_archive->model == src_archive->model,
+    "KheArchiveMeld: dst_archive and src_archive have different models");
+
+  /* meld the instances */
+  for( i = 0;  i < KheArchiveInstanceCount(src_archive);  i++ )
+  {
+    ins = KheArchiveInstance(src_archive, i);
+    if( !KheArchiveAddInstance(dst_archive, ins) )
+      HnAbort("KheArchiveMeld:  cannot add instance \"%s\" of src_archive "
+	"to dst_archive", KheInstanceId(ins));
+  }
+
+  /* meld the soln groups */
+  for( i = 0;  i < KheArchiveSolnGroupCount(src_archive);  i++ )
+  {
+    /* get src_sg, a soln group from src_archive */
+    src_sg = KheArchiveSolnGroup(src_archive, i);
+
+    /* get (or make) dst_sg, a soln group from dst_archive */
+    id = KheSolnGroupId(src_sg);
+    if( !KheArchiveRetrieveSolnGroup(dst_archive, id, &dst_sg) )
+    {
+      if( !KheSolnGroupMake(dst_archive, id, &dst_sg) )
+	HnAbort("KheArchiveMeld internal error");
+    }
+
+    /* copy the solutions of src_sg into dst_sg */
+    for( j = 0;  j < KheSolnGroupSolnCount(src_sg);  j++ )
+    {
+      soln = KheSolnGroupSoln(src_sg, j);
+      KheSolnGroupAddSoln(dst_sg, KheSolnCopy(soln, dst_archive->arena_set));
+    }
+  }
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  char *KheArchiveId(KHE_ARCHIVE archive)                                  */
@@ -427,9 +522,9 @@ KHE_MODEL KheArchiveModel(KHE_ARCHIVE archive)
 /*                                                                           */
 /*****************************************************************************/
 
-HA_ARENA KheArchiveArenaBegin(KHE_ARCHIVE archive, bool large)
+HA_ARENA KheArchiveArenaBegin(KHE_ARCHIVE archive)
 {
-  return HaArenaSetArenaBegin(archive->arena_set, large);
+  return HaArenaMake(archive->arena_set);
 }
 
 
@@ -443,7 +538,7 @@ HA_ARENA KheArchiveArenaBegin(KHE_ARCHIVE archive, bool large)
 
 void KheArchiveArenaEnd(KHE_ARCHIVE archive, HA_ARENA a)
 {
-  HaArenaSetArenaEnd(archive->arena_set, a);
+  HaArenaDelete(a);
 }
 
 
@@ -499,6 +594,10 @@ void KheArchiveDeleteInstance(KHE_ARCHIVE archive, KHE_INSTANCE ins)
   int pos, i;  char *id;  KHE_SOLN_GROUP sg;
   KHE_INDEXED_INSTANCE ii, ii2;
 
+  if( DEBUG5 )
+    fprintf(stderr, "[ KheArchiveDeleteInstance(%s, %s)\n",
+      KheArchiveId(archive), KheInstanceId(ins));
+
   /* find ins's indexed instance, either by retrieving by id or searching; */
   /* also delete the indexed instance from archive's table */
   ii = NULL;
@@ -530,6 +629,9 @@ void KheArchiveDeleteInstance(KHE_ARCHIVE archive, KHE_INSTANCE ins)
   }
   HaArrayDeleteLast(archive->instance_array);
 
+  /* delete archive from ins */
+  KheInstanceDeleteArchive(ins, archive);
+
   /* delete solutions for ins from archive's solution groups */
   HaArrayForEach(archive->soln_group_array, sg, i)
     KheSolnGroupDeleteInstance(sg, ii->instance, ii->index);
@@ -556,6 +658,8 @@ void KheArchiveDeleteInstance(KHE_ARCHIVE archive, KHE_INSTANCE ins)
   HaArrayForEach(archive->soln_group_array, sg, i)
     KheSolnGroupDeleteSolnsForInstance(sg, ins);
   *** */
+  if( DEBUG5 )
+    fprintf(stderr, "] KheArchiveDeleteInstance returning\n");
 }
 
 
@@ -783,6 +887,7 @@ bool KheArchiveMetaDataMakeFromKml(KML_ELT md_elt, KHE_ARCHIVE archive,
   return true;
 }
 
+
 /*****************************************************************************/
 /*                                                                           */
 /*  bool KheArchiveEltToArchive(KML_ELT archive_elt, KHE_ARCHIVE *archive,   */
@@ -835,6 +940,95 @@ static bool KheArchiveEltToArchive(KML_ELT archive_elt, HA_ARENA_SET as,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheArchiveEltCheckArchive(KML_ELT archive_elt,                      */
+/*    KHE_ARCHIVE archive, KML_ERROR *ke, HA_ARENA a)                        */
+/*                                                                           */
+/*  Like KheArchiveEltToArchive except that it checks archive_elt against    */
+/*  an existing archive rather than converting it into an archive.           */
+/*                                                                           */
+/*  Arena a is used only for error messages.                                 */
+/*                                                                           */
+/*  Implementation note.  Because this function is called only by            */
+/*  segmented reads, there are no instances and no solution groups           */
+/*  under archive_elt when it is called.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+static bool KheArchiveEltCheckArchive(KML_ELT archive_elt,
+  KHE_ARCHIVE archive, KML_ERROR *ke, HA_ARENA a)
+{
+  /* char *id; */  KHE_MODEL model;
+
+  /* fail if archive_elt has problems */
+  if( strcmp(KmlLabel(archive_elt), "HighSchoolTimetableArchive") == 0 )
+    model = KHE_MODEL_HIGH_SCHOOL_TIMETABLE;
+  else if( strcmp(KmlLabel(archive_elt), "EmployeeScheduleArchive") == 0 )
+    model = KHE_MODEL_EMPLOYEE_SCHEDULE;
+  else
+  {
+    *ke = KmlErrorMake(a, KmlLineNum(archive_elt), KmlColNum(archive_elt),
+      "when loading, file does not begin with <HighSchoolTimetableArchive>"
+      " or <EmployeeScheduleArchive>");
+    return false;
+  }
+  /* NB Instances opening tag will have been read by this time */
+  if( !KmlCheck(archive_elt, "+Id : +MetaData +Instances +SolutionGroups", ke) )
+    return false;
+
+  /* check id */
+  /* *** I've decided not to check the Id
+  if( KmlAttributeCount(archive_elt) == 0 )
+  {
+    ** no Id present in file, archive must not have one either **
+    if( KheArchiveId(archive) != NULL )
+    {
+      *ke = KmlErrorMake(a, KmlLineNum(archive_elt), KmlColNum(archive_elt),
+	"when loading, file has no archive Id but archive has Id \"%s\"",
+	KheArchiveId(archive));
+      return false;
+    }
+  }
+  else
+  {
+    ** Id present in file, archive must have the same one **
+    id = KmlAttributeValue(archive_elt, 0);
+    if( KheArchiveId(archive) == NULL )
+    {
+      *ke = KmlErrorMake(a, KmlLineNum(archive_elt), KmlColNum(archive_elt),
+	"when loading, file has archive Id \"%s\" but archive has none", id);
+      return false;
+    }
+    else if( strcmp(id, KheArchiveId(archive)) != 0 )
+    {
+      *ke = KmlErrorMake(a, KmlLineNum(archive_elt), KmlColNum(archive_elt),
+	"when loading, file has archive Id \"%s\" but archive has Id \"%s\"",
+	id, KheArchiveId(archive));
+      return false;
+    }
+  }
+  *** */
+
+  /* check model */
+  if( model != KheArchiveModel(archive) )
+  {
+    if( model == KHE_MODEL_HIGH_SCHOOL_TIMETABLE )
+      *ke = KmlErrorMake(a, KmlLineNum(archive_elt), KmlColNum(archive_elt),
+	"when loading, file begins with <HighSchoolTimetableArchive>"
+	" but archive begins with <EmployeeScheduleArchive>");
+    else
+      *ke = KmlErrorMake(a, KmlLineNum(archive_elt), KmlColNum(archive_elt),
+	"when loading, file begins with <EmployeeScheduleArchive>"
+	" but archive begins with <HighSchoolTimetableArchive>");
+    return false;
+  }
+
+  /* all good */
+  return *ke = NULL, true;
+}
+
+
 /* *** old version that includes instances and solution groups
 static bool KheArchiveEltToArchive(KML_ELT archive_elt, KHE_ARCHIVE *archive,
   bool audit_and_fix, bool infer_resource_partitions, ** bool with_monitors, **
@@ -864,7 +1058,7 @@ static bool KheArchiveEltToArchive(KML_ELT archive_elt, KHE_ARCHIVE *archive,
   if( KmlContainsChild(archive_elt, "MetaData", &metadata_elt) &&
       !KheArchiveMetaDataMakeFromKml(metadata_elt, res, ke) )
   {
-    HnAssert(*ke != NULL, "KheArchiveEltToArchive internal error 1");
+    HnAssert(*ke != NULL, "KheArchiveEltToArc hive internal error 1");
     return false;
   }
 
@@ -879,7 +1073,7 @@ static bool KheArchiveEltToArchive(KML_ELT archive_elt, KHE_ARCHIVE *archive,
       if( !KheInstanceM akeFromKml(instance_elt, model, audit_and_fix,
 	    infer_resource_partitions, &ins, ke) )
       {
-	HnAssert(*ke != NULL, "KheArchiveEltToArchive internal error 2");
+	HnAssert(*ke != NULL, "KheArchiveE ltToArchive internal error 2");
 	return false;
       }
       if( !KheArchiveAddInstance(res, ins) )
@@ -899,7 +1093,7 @@ static bool KheArchiveEltToArchive(KML_ELT archive_elt, KHE_ARCHIVE *archive,
       if( !KheSolnGroupMakeFromKml(soln_group_elt, res,
 	    ** with_monitors, ** allow_invalid_solns, ke) )
       {
-	HnAssert(*ke != NULL, "KheArchiveEltToArchive internal error 3");
+	HnAssert(*ke != NULL, "KheArchiveEltToA rchive internal error 3");
 	return false;
       }
     }
@@ -915,7 +1109,7 @@ static bool KheArchiveEltToArchive(KML_ELT archive_elt, KHE_ARCHIVE *archive,
 /*                                                                           */
 /*  bool KheArchiveRead(FILE *fp, HA_ARENA_SET as, KHE_ARCHIVE *archive,     */
 /*    KML_ERROR *ke, bool audit_and_fix, bool resource_type_partitions,      */
-/*    bool infer_resource_partitions, bool limit_busy_recode,                */
+/*    bool infer_resource_partitions,                                        */
 /*    bool allow_invalid_solns, KHE_SOLN_TYPE soln_type, FILE *echo_fp)      */
 /*                                                                           */
 /*  Read *archive from fp.                                                   */
@@ -924,13 +1118,41 @@ static bool KheArchiveEltToArchive(KML_ELT archive_elt, KHE_ARCHIVE *archive,
 
 bool KheArchiveRead(FILE *fp, HA_ARENA_SET as, KHE_ARCHIVE *archive,
   KML_ERROR *ke, bool audit_and_fix, bool resource_type_partitions,
-  bool infer_resource_partitions, bool limit_busy_recode,
-  bool allow_invalid_solns, KHE_SOLN_TYPE soln_type, FILE *echo_fp)
+  bool infer_resource_partitions, bool allow_invalid_solns,
+  KHE_SOLN_TYPE soln_type, FILE *echo_fp)
 {
   return KheArchiveReadIncremental(fp, as, archive, ke, audit_and_fix,
-    resource_type_partitions, infer_resource_partitions, limit_busy_recode,
-    allow_invalid_solns, soln_type, echo_fp, NULL, NULL, NULL, NULL, NULL,
-    NULL);
+    resource_type_partitions, infer_resource_partitions, allow_invalid_solns,
+    soln_type, echo_fp, NULL, NULL, NULL, NULL, NULL, NULL);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheArchiveLoad(KHE_ARCHIVE archive, FILE *fp,                       */
+/*    KML_ERROR *ke, bool audit_and_fix, bool resource_type_partitions,      */
+/*    bool infer_resource_partitions, bool allow_invalid_solns,              */
+/*    KHE_SOLN_TYPE soln_type, FILE *echo_fp)                                */
+/*                                                                           */
+/*  Like KheArchiveRead except done into an existing archive.                */
+/*                                                                           */
+/*****************************************************************************/
+static bool KheArchiveLoadIncremental(KHE_ARCHIVE archive, FILE *fp,
+  KML_ERROR *ke, bool audit_and_fix,
+  bool resource_type_partitions, bool infer_resource_partitions,
+  bool allow_invalid_solns, KHE_SOLN_TYPE soln_type, FILE *echo_fp,
+  KHE_ARCHIVE_FN archive_begin_fn, KHE_ARCHIVE_FN archive_end_fn,
+  KHE_SOLN_GROUP_FN soln_group_begin_fn,
+  KHE_SOLN_GROUP_FN soln_group_end_fn, KHE_SOLN_FN soln_fn, void *impl);
+
+bool KheArchiveLoad(KHE_ARCHIVE archive, FILE *fp,
+  KML_ERROR *ke, bool audit_and_fix, bool resource_type_partitions,
+  bool infer_resource_partitions, bool allow_invalid_solns,
+  KHE_SOLN_TYPE soln_type, FILE *echo_fp)
+{
+  return KheArchiveLoadIncremental(archive, fp, ke, audit_and_fix,
+    resource_type_partitions, infer_resource_partitions, allow_invalid_solns,
+    soln_type, echo_fp, NULL, NULL, NULL, NULL, NULL, NULL);
 }
 
 
@@ -1165,7 +1387,7 @@ static void KheReadEltFn(KML_ELT elt, KML_READ_INFO ri)
     if( !KheReadInArchive(krs) )
     {
       ** this must be the very first instance **
-      if( !KheArchiveEltToArchive(KmlParent(KmlParent(elt)), &archive,
+      if( !KheArchiveEltToArch ive(KmlParent(KmlParent(elt)), &archive,
 	    krs->audit_and_fix, krs->infer_resource_partitions,
 	    ** krs->with_monitors, ** krs->allow_invalid_solns, &ke,
 	    KmlReadArena(ri)) )
@@ -1220,7 +1442,7 @@ static void KheReadEltFn(KML_ELT elt, KML_READ_INFO ri)
     if( !KheReadInArchive(krs) )
     {
       ** this must be the very first solution **
-      if( !KheArchiveEltToArchive(KmlParent(KmlParent(KmlParent(elt))),
+      if( !KheArchiveEltToAr chive(KmlParent(KmlParent(KmlParent(elt))),
 	    &archive, krs->audit_and_fix, krs->infer_resource_partitions,
 	    ** krs->with_monitors, ** krs->allow_invalid_solns, &ke,
 	    KmlReadArena(ri)) )
@@ -1300,7 +1522,7 @@ static void KheReadEltFn(KML_ELT elt, KML_READ_INFO ri)
     {
       ** this must be the very first solution group, and it must contain **
       ** no solutions, which is unlikely but legal **
-      if( !KheArchiveEltToArchive(KmlParent(KmlParent(elt)), &archive,
+      if( !KheArchiveEltToA rchive(KmlParent(KmlParent(elt)), &archive,
 	    krs->audit_and_fix, krs->infer_resource_partitions,
 	    ** krs->with_monitors, ** krs->allow_invalid_solns, &ke,
 	    KmlReadArena(ri)) )
@@ -1354,11 +1576,27 @@ static void KheArchiveSegmentBeginFn(KML_SEGMENT ks)
   KML_READER kr;  KHE_READ_STATE krs;  KML_ERROR ke;
   kr = KmlSegmentReader(ks);
   krs = (KHE_READ_STATE) KmlReaderImpl(kr);
-  if( !KheArchiveEltToArchive(KmlSegmentRoot(ks), krs->arena_set,
-	&krs->archive, &ke, KmlReaderArena(kr)) )
+  if( krs->archive != NULL )
   {
-    HnAssert(ke != NULL, "KheArchiveSegmentBeginFn internal error 2");
-    KmlReaderFail(kr, ke);
+    /* loading into an existing archive */
+    if( !KheArchiveEltCheckArchive(KmlSegmentRoot(ks), krs->archive,
+      &ke, KmlReaderArena(kr)) )
+    {
+      /* KmlSegmentRoot(ks) not compatible with existing archive */
+      HnAssert(ke != NULL, "KheArchiveSegmentBeginFn internal error 2");
+      KmlReaderFail(kr, ke);
+    }
+  }
+  else
+  {
+    /* creating a new archive */
+    if( !KheArchiveEltToArchive(KmlSegmentRoot(ks), krs->arena_set,
+	&krs->archive, &ke, KmlReaderArena(kr)) )
+    {
+      /* can't convert KmlSegmentRoot(ks) into an archive */
+      HnAssert(ke != NULL, "KheArchiveSegmentBeginFn internal error 2");
+      KmlReaderFail(kr, ke);
+    }
   }
   if( krs->archive_begin_fn != NULL )
     krs->archive_begin_fn(krs->archive, krs->impl);
@@ -1382,7 +1620,7 @@ static void KheInstanceSegmentFn(KML_SEGMENT ks)
   ins_elt = KmlSegmentRoot(ks);
   if( !KheInstanceMakeFromKml(ins_elt, KheArchiveModel(krs->archive),
 	krs->arena_set, krs->audit_and_fix, krs->resource_type_partitions,
-	krs->infer_resource_partitions, krs->limit_busy_recode, &ins, &ke) )
+	krs->infer_resource_partitions, &ins, &ke) )
   {
     HnAssert(ke != NULL, "KheArchiveEltToArchive internal error 2");
     KmlReaderFail(kr, ke);
@@ -1448,7 +1686,7 @@ static void KheSolnSegmentFn(KML_SEGMENT ks)
 /*  bool KheArchiveReadIncremental(FILE *fp, HA_ARENA_SET as,                */
 /*    KHE_ARCHIVE *archive, KML_ERROR *ke, bool audit_and_fix,               */
 /*    bool resource_type_partitions, bool infer_resource_partitions,         */
-/*    bool limit_busy_recode, bool allow_invalid_solns, FILE *echo_fp,       */
+/*    bool allow_invalid_solns, FILE *echo_fp,                               */
 /*    KHE_ARCHIVE_FN archive_begin_fn, KHE_ARCHIVE_FN archive_end_fn,        */
 /*    KHE_SOLN_GROUP_FN soln_group_begin_fn,                                 */
 /*    KHE_SOLN_GROUP_FN soln_group_end_fn, KHE_SOLN_FN soln_fn, void *impl)  */
@@ -1478,8 +1716,7 @@ static void KheSolnSegmentFn(KML_SEGMENT ks)
 bool KheArchiveReadIncremental(FILE *fp, HA_ARENA_SET as,
   KHE_ARCHIVE *archive, KML_ERROR *ke, bool audit_and_fix,
   bool resource_type_partitions, bool infer_resource_partitions,
-  bool limit_busy_recode, bool allow_invalid_solns,
-  KHE_SOLN_TYPE soln_type, FILE *echo_fp,
+  bool allow_invalid_solns, KHE_SOLN_TYPE soln_type, FILE *echo_fp,
   KHE_ARCHIVE_FN archive_begin_fn, KHE_ARCHIVE_FN archive_end_fn,
   KHE_SOLN_GROUP_FN soln_group_begin_fn,
   KHE_SOLN_GROUP_FN soln_group_end_fn, KHE_SOLN_FN soln_fn, void *impl)
@@ -1495,7 +1732,6 @@ bool KheArchiveReadIncremental(FILE *fp, HA_ARENA_SET as,
   krs_rec.audit_and_fix = audit_and_fix;
   krs_rec.resource_type_partitions = resource_type_partitions;
   krs_rec.infer_resource_partitions = infer_resource_partitions;
-  krs_rec.limit_busy_recode = limit_busy_recode;
   krs_rec.allow_invalid_solns = allow_invalid_solns;
   krs_rec.soln_type = soln_type;
   krs_rec.archive_begin_fn = archive_begin_fn;
@@ -1506,7 +1742,7 @@ bool KheArchiveReadIncremental(FILE *fp, HA_ARENA_SET as,
   krs_rec.impl = impl;
 
   /* set up for reading incrementally */
-  a = HaArenaSetArenaBegin(as, false);
+  a = HaArenaMake(as);
   kr = KmlReaderMake((void *) &krs_rec, as, a);
   KmlReaderDeclareSegmentBegin(kr, archive_str, &KheArchiveSegmentBeginFn);
   KmlReaderDeclareSegment(kr, instance_str, &KheInstanceSegmentFn);
@@ -1536,7 +1772,73 @@ bool KheArchiveReadIncremental(FILE *fp, HA_ARENA_SET as,
       res ? "true" : "false");
   if( DEBUG3 )
     fprintf(stderr, "] KheArchiveReadIncremental returning\n");
-  HaArenaSetArenaEnd(as, a);
+  HaArenaDelete(a);
+  return res;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*****************************************************************************/
+
+static bool KheArchiveLoadIncremental(KHE_ARCHIVE archive, FILE *fp,
+  KML_ERROR *ke, bool audit_and_fix,
+  bool resource_type_partitions, bool infer_resource_partitions,
+  bool allow_invalid_solns, KHE_SOLN_TYPE soln_type, FILE *echo_fp,
+  KHE_ARCHIVE_FN archive_begin_fn, KHE_ARCHIVE_FN archive_end_fn,
+  KHE_SOLN_GROUP_FN soln_group_begin_fn,
+  KHE_SOLN_GROUP_FN soln_group_end_fn, KHE_SOLN_FN soln_fn, void *impl)
+{
+  struct khe_read_state_rec krs_rec;  bool res;  KML_READER kr;  HA_ARENA a;
+  if( DEBUG3 )
+    fprintf(stderr, "[ KheArchiveLoadIncremental\n");
+
+  /* initialize incremental read data */
+  krs_rec.arena_set = archive->arena_set;
+  krs_rec.archive = archive;
+  krs_rec.soln_group = NULL;
+  krs_rec.audit_and_fix = audit_and_fix;
+  krs_rec.resource_type_partitions = resource_type_partitions;
+  krs_rec.infer_resource_partitions = infer_resource_partitions;
+  krs_rec.allow_invalid_solns = allow_invalid_solns;
+  krs_rec.soln_type = soln_type;
+  krs_rec.archive_begin_fn = archive_begin_fn;
+  krs_rec.archive_end_fn = archive_end_fn;
+  krs_rec.soln_group_begin_fn = soln_group_begin_fn;
+  krs_rec.soln_group_end_fn = soln_group_end_fn;
+  krs_rec.soln_fn = soln_fn;
+  krs_rec.impl = impl;
+
+  /* set up for reading incrementally */
+  a = HaArenaMake(archive->arena_set);
+  kr = KmlReaderMake((void *) &krs_rec, archive->arena_set, a);
+  KmlReaderDeclareSegmentBegin(kr, archive_str, &KheArchiveSegmentBeginFn);
+  KmlReaderDeclareSegment(kr, instance_str, &KheInstanceSegmentFn);
+  KmlReaderDeclareSegmentBegin(kr, soln_group_str, &KheSolnGroupSegmentBeginFn);
+  KmlReaderDeclareSegment(kr, soln_str, &KheSolnSegmentFn);
+  KmlReaderDeclareSegmentEnd(kr, &KheSolnGroupSegmentEndFn);
+  KmlReaderDeclareSegmentEnd(kr, &KheArchiveSegmentEndFn);
+
+  /* read fp incrementally, calling back after each high-level thing */
+  *ke = NULL;
+  if( KmlReaderReadFileSegmented(kr, fp, echo_fp, ke) )
+  {
+    /* the incremental read worked */
+    HnAssert(*ke == NULL, "KheArchiveLoadIncremental internal error 1");
+    res = true;
+  }
+  else
+  {
+    /* the incremental read did not work */
+    HnAssert(*ke != NULL, "KheArchiveLoadIncremental internal error 2");
+    res = false;
+  }
+  if( DEBUG3 )
+    fprintf(stderr, "KheArchiveLoadIncremental returning %s\n",
+      res ? "true" : "false");
+  if( DEBUG3 )
+    fprintf(stderr, "] KheArchiveLoadIncremental returning\n");
+  HaArenaDelete(a);
   return res;
 }
 
@@ -1601,6 +1903,31 @@ static void ArchiveIncludeInstances(KHE_ARCHIVE archive, char *ids)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  void ArchiveIncludeFirstInstances(KHE_ARCHIVE archive, char *count_str)  */
+/*                                                                           */
+/*  Include only the first count_str instances.                              */
+/*                                                                           */
+/*****************************************************************************/
+
+static void ArchiveIncludeFirstInstances(KHE_ARCHIVE archive, char *count_str)
+{
+  KHE_INSTANCE ins;  int count;
+
+  /* make sure all the named instances are present; build included_instances */
+  if( sscanf(count_str, "%d", &count) != 1 || count < 0 )
+    HnFatal("khe: invalid -i value \"%s\"", count_str);
+
+  /* delete all instances beyond count */
+  while( KheArchiveInstanceCount(archive) > count )
+  {
+    ins = KheArchiveInstance(archive, count);
+    KheArchiveDeleteInstance(archive, ins);
+  }
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void ArchiveExcludeSolutionGroups(KHE_ARCHIVE archive, char *ids)        */
@@ -1616,14 +1943,14 @@ static void ArchiveExcludeSolutionGroups(KHE_ARCHIVE archive, char *ids)
   {
     /* special case, means delete all solution groups */
     while( KheArchiveSolnGroupCount(archive) > 0 )
-      KheSolnGroupDelete(KheArchiveSolnGroup(archive, 0));
+      KheSolnGroupDelete(KheArchiveSolnGroup(archive, 0), true);
   }
   else
   {
     /* ordinary case, delete each solution group named by the option */
     for( id = strtok(ids, ",");  id != NULL;  id = strtok(NULL, ",") )
       if( KheArchiveRetrieveSolnGroup(archive, id, &soln_group) )
-	KheSolnGroupDelete(soln_group);
+	KheSolnGroupDelete(soln_group, true);
       else
 	HnFatal("khe: no solution group \"%s\" in archive", id);
   }
@@ -1657,7 +1984,7 @@ static void ArchiveIncludeSolutionGroups(KHE_ARCHIVE archive, char *ids)
     soln_group = KheArchiveSolnGroup(archive, i);
     if( !HaArrayContains(included_soln_groups, soln_group, &junk) )
     {
-      KheSolnGroupDelete(soln_group);
+      KheSolnGroupDelete(soln_group, true);
       i--;
     }
   }
@@ -1670,7 +1997,7 @@ static void ArchiveIncludeSolutionGroups(KHE_ARCHIVE archive, char *ids)
 /*  KHE_ARCHIVE KheArchiveReadFromCommandLine(int argc, char *argv[],        */
 /*    int *pos, HA_ARENA_SET as, bool audit_and_fix,                         */
 /*    bool resource_type_partitions, bool infer_resource_partitions,         */
-/*    bool limit_busy_recode, bool allow_invalid_solns, FILE *echo_fp)       */
+/*    bool allow_invalid_solns, FILE *echo_fp)                               */
 /*                                                                           */
 /*  Read an archive, returning only if successful.   The archive name is     */
 /*  at argv[*pos], and following options cause it to be updated.             */
@@ -1680,42 +2007,45 @@ static void ArchiveIncludeSolutionGroups(KHE_ARCHIVE archive, char *ids)
 KHE_ARCHIVE KheArchiveReadFromCommandLine(int argc, char *argv[],
   int *pos, HA_ARENA_SET as, bool audit_and_fix,
   bool resource_type_partitions, bool infer_resource_partitions,
-  bool limit_busy_recode, bool allow_invalid_solns,
-  KHE_SOLN_TYPE soln_type, FILE *echo_fp)
+  bool allow_invalid_solns, KHE_SOLN_TYPE soln_type, FILE *echo_fp)
 {
   FILE *fp;  KHE_ARCHIVE res;  KML_ERROR ke;
-  bool seen_x, seen_i, seen_X, seen_I;
+  bool seen_x, seen_i, seen_n, seen_X, seen_I;
 
   /* open the archive file for reading */
   if( *pos >= argc )
-    HnFatal("khe: archive name missing from command line");
+    HnFatal("%s: archive name missing from command line", argv[0]);
   if( strcmp(argv[*pos], "-") == 0 )
     fp = stdin;
   else
   {
     fp = fopen(argv[*pos], "r");
     if( fp == NULL )
-      HnFatal("khe: cannot open file \"%s\" for reading", argv[*pos]);
+      HnFatal("%s: cannot open file \"%s\" for reading", argv[0], argv[*pos]);
   }
 
   /* read the archive file */
   if( !KheArchiveRead(fp, as, &res, &ke, audit_and_fix,
         resource_type_partitions, infer_resource_partitions,
-	limit_busy_recode, allow_invalid_solns, soln_type, echo_fp) )
+	allow_invalid_solns, soln_type, echo_fp) )
     HnFatal("%s:%d:%d: %s\n", argv[*pos], KmlErrorLineNum(ke),
       KmlErrorColNum(ke), KmlErrorString(ke));
+  if( fp != stdin )
+    fclose(fp);
 
   /* process zero or more following -x, -i, -X, and -I options */
-  seen_x = seen_i = seen_X = seen_I = false;
+  seen_x = seen_i = seen_n = seen_X = seen_I = false;
   for( (*pos)++;  *pos < argc && argv[*pos][0] == '-';  (*pos)++ )
     switch( argv[*pos][1] )
     {
       case 'x':
 
 	if( seen_x )
-	  HnFatal("khe: -x option used twice");
+	  HnFatal("%s: -x option used twice", argv[0]);
 	if( seen_i )
-	  HnFatal("khe: -x and -i used together");
+	  HnFatal("%s: -x and -i used together", argv[0]);
+	if( seen_n )
+	  HnFatal("%s: -x and -n used together", argv[0]);
 	seen_x = true;
 	ArchiveExcludeInstances(res, &argv[*pos][2]);
 	break;
@@ -1723,19 +2053,33 @@ KHE_ARCHIVE KheArchiveReadFromCommandLine(int argc, char *argv[],
       case 'i':
 
 	if( seen_i )
-	  HnFatal("khe: -i option used twice");
+	  HnFatal("%s: -i option used twice", argv[0]);
 	if( seen_x )
-	  HnFatal("khe: -i and -x used together");
+	  HnFatal("%s: -i and -x used together", argv[0]);
+	if( seen_n )
+	  HnFatal("%s: -i and -n used together", argv[0]);
 	seen_i = true;
 	ArchiveIncludeInstances(res, &argv[*pos][2]);
 	break;
 
+      case 'n':
+
+	if( seen_n )
+	  HnFatal("%s: -n option used twice", argv[0]);
+	if( seen_i )
+	  HnFatal("%s: -i and -n used together", argv[0]);
+	if( seen_x )
+	  HnFatal("%s: -n and -x used together", argv[0]);
+	seen_n = true;
+	ArchiveIncludeFirstInstances(res, &argv[*pos][2]);
+	break;
+
       case 'X':
 
 	if( seen_X )
-	  HnFatal("khe: -X option used twice");
+	  HnFatal("%s: -X option used twice", argv[0]);
 	if( seen_I )
-	  HnFatal("khe: -X and -I used together");
+	  HnFatal("%s: -X and -I used together", argv[0]);
 	seen_X = true;
 	ArchiveExcludeSolutionGroups(res, &argv[*pos][2]);
 	break;
@@ -1743,9 +2087,9 @@ KHE_ARCHIVE KheArchiveReadFromCommandLine(int argc, char *argv[],
       case 'I':
 
 	if( seen_I )
-          HnFatal("khe: -I option used twice");
+          HnFatal("%s: -I option used twice", argv[0]);
 	if( seen_X )
-          HnFatal("khe: -I and -X used together");
+          HnFatal("%s: -I and -X used together", argv[0]);
 	seen_I = true;
 	ArchiveIncludeSolutionGroups(res, &argv[*pos][2]);
 	break;
@@ -1862,6 +2206,10 @@ static void KheArchiveWriteHeaderAndInstances(KHE_ARCHIVE archive, KML_FILE kf)
 /*  Write archive to fp, which must be open for writing.  Include reports    */
 /*  in the written archive if with_reports is true.                          */
 /*                                                                           */
+/*  Implementation note.  We use the archive's arena to hold the KML         */
+/*  file object (added by JeffK 11/25, suggested by Ortiz).  That's OK,      */
+/*  because the file object is not used after this function returns.         */
+/*                                                                           */
 /*****************************************************************************/
 
 void KheArchiveWrite(KHE_ARCHIVE archive, bool with_reports, FILE *fp)
@@ -1871,7 +2219,7 @@ void KheArchiveWrite(KHE_ARCHIVE archive, bool with_reports, FILE *fp)
   /* header and instances */
   if( DEBUG3 )
     fprintf(stderr, "[ KheArchiveWrite\n");
-  kf = KmlMakeFile(fp, 0, 2);
+  kf = KmlMakeFile(fp, 0, 2, archive->arena);
   KheArchiveWriteHeaderAndInstances(archive, kf);
 
   /* soln groups */
@@ -1907,7 +2255,7 @@ void KheArchiveWriteSolnGroup(KHE_ARCHIVE archive,
   /* header and instances */
   HnAssert(KheSolnGroupArchive(soln_group) == archive,
     "KheArchiveWriteSolnGroup:  soln_group not in archive");
-  kf = KmlMakeFile(fp, 0, 2);
+  kf = KmlMakeFile(fp, 0, 2, archive->arena);
   KheArchiveWriteHeaderAndInstances(archive, kf);
 
   /* soln group */
@@ -1933,7 +2281,7 @@ void KheArchiveWriteWithoutSolnGroups(KHE_ARCHIVE archive, FILE *fp)
   KML_FILE kf;
 
   /* header and instances */
-  kf = KmlMakeFile(fp, 0, 2);
+  kf = KmlMakeFile(fp, 0, 2, archive->arena);
   KheArchiveWriteHeaderAndInstances(archive, kf);
 
   /* close header and exit */
