@@ -70,7 +70,9 @@ KHE_ASSIGN_RESOURCE_MONITOR KheAssignResourceMonitorMake(
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_ASSIGN_RESOURCE_MONITOR_TAG);
+    KHE_ASSIGN_RESOURCE_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->deviation = 0;
   res->event_resource_in_soln = ers;
   res->constraint = c;
@@ -125,6 +127,7 @@ void KheAssignResourceMonitorCopyPhase2(KHE_ASSIGN_RESOURCE_MONITOR m)
   {
     m->copy = NULL;
     KheMonitorCopyCommonFieldsPhase2((KHE_MONITOR) m);
+    KheEventResourceInSolnCopyPhase2(m->event_resource_in_soln);
   }
 }
 
@@ -251,6 +254,80 @@ void KheAssignResourceMonitorAttachCheck(KHE_ASSIGN_RESOURCE_MONITOR m)
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAssignResourceMonitorTimeRange(KHE_ASSIGN_RESOURCE_MONITOR m,    */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for assign resource monitor m.             */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAssignResourceMonitorTimeRange(KHE_ASSIGN_RESOURCE_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_EVENT_RESOURCE er;
+  er = KheAssignResourceMonitorEventResource(m);
+  return KheEventResourceTimeRange(er, m->soln, first_time, last_time);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheAssignResourceMonitorSetSweepTime(                               */
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for assign resource monitor m.          */
+/*                                                                           */
+/*  Implementation note.  Implementing this function by attaching and        */
+/*  detaching is not a great way to do it:  if a monitor is detached for     */
+/*  some other reason, it goes wrong.  But it's good enough to go on with.   */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheAssignResourceMonitorSetSweepTime(
+  KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)
+{
+  KHE_TIME first_time, last_time;
+  if( KheAssignResourceMonitorTimeRange(m, &first_time, &last_time)
+      && (t == NULL || KheTimeIndex(t) < KheTimeIndex(last_time)) )
+  {
+    /* m needs to be cut off */
+    if( m->attached )
+      KheAssignResourceMonitorDetachFromSoln(m);
+  }
+  else
+  {
+    /* m needs to be not cut off */
+    if( !m->attached )
+      KheAssignResourceMonitorAttachToSoln(m);
+  }
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAssignResourceMonitorSweepTimeRange(                             */
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME *first_time,                   */
+/*    KHE_TIME *last_time)                                                   */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for assign resource monitor m.        */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAssignResourceMonitorSweepTimeRange(
+  KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return KheAssignResourceMonitorTimeRange(m, first_time, last_time);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -278,7 +355,7 @@ void KheAssignResourceMonitorAddTask(KHE_ASSIGN_RESOURCE_MONITOR m,
   {
     m->deviation += KheTaskDuration(task);
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -305,7 +382,7 @@ void KheAssignResourceMonitorDeleteTask(KHE_ASSIGN_RESOURCE_MONITOR m,
     HnAssert(m->deviation >= 0,
       "KheAssignResourceMonitorDeleteTask internal error");
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -367,7 +444,7 @@ void KheAssignResourceMonitorAssignResource(KHE_ASSIGN_RESOURCE_MONITOR m,
   HnAssert(m->deviation >= 0,
     "KheAssignResourceMonitorAssignTime internal error");
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
@@ -388,7 +465,7 @@ void KheAssignResourceMonitorUnAssignResource(KHE_ASSIGN_RESOURCE_MONITOR m,
       KheAssignResourceMonitorId(m), m->deviation);
   m->deviation += KheTaskDuration(task);
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
