@@ -69,7 +69,9 @@ KHE_ASSIGN_TIME_MONITOR KheAssignTimeMonitorMake(KHE_EVENT_IN_SOLN es,
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_ASSIGN_TIME_MONITOR_TAG);
+    KHE_ASSIGN_TIME_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->deviation = 0;
   res->event_in_soln = es;
   res->constraint = c;
@@ -125,6 +127,7 @@ void KheAssignTimeMonitorCopyPhase2(KHE_ASSIGN_TIME_MONITOR m)
   {
     m->copy = NULL;
     KheMonitorCopyCommonFieldsPhase2((KHE_MONITOR) m);
+    KheEventInSolnCopyPhase2(m->event_in_soln);
   }
   if( DEBUG1 )
     fprintf(stderr, "KheAssignTimeMonitorCopyPhase2(%p)\n", (void *) m);
@@ -246,6 +249,62 @@ void KheAssignTimeMonitorAttachCheck(KHE_ASSIGN_TIME_MONITOR m)
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAssignTimeMonitorTimeRange(KHE_ASSIGN_TIME_MONITOR m,            */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAssignTimeMonitorTimeRange(KHE_ASSIGN_TIME_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_EVENT e;
+  e = KheAssignTimeMonitorEvent(m);
+  return KheEventTimeRange(e, m->soln, first_time, last_time);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheAssignTimeMonitorSetSweepTime(KHE_ASSIGN_TIME_MONITOR m,         */
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheAssignTimeMonitorSetSweepTime(KHE_ASSIGN_TIME_MONITOR m,
+  KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAssignTimeMonitorSweepTimeRange(KHE_ASSIGN_TIME_MONITOR m,       */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for assign resource monitor m.        */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAssignTimeMonitorSweepTimeRange(KHE_ASSIGN_TIME_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -271,7 +330,7 @@ void KheAssignTimeMonitorAddMeet(KHE_ASSIGN_TIME_MONITOR m, KHE_MEET meet)
   {
     m->deviation += KheMeetDuration(meet);
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -297,7 +356,7 @@ void KheAssignTimeMonitorDeleteMeet(KHE_ASSIGN_TIME_MONITOR m, KHE_MEET meet)
     HnAssert(m->deviation >= 0,
       "KheAssignTimeMonitorDeleteMeet internal error");
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -381,7 +440,7 @@ void KheAssignTimeMonitorAssignTime(KHE_ASSIGN_TIME_MONITOR m,
   HnAssert(m->deviation >= 0,
     "KheAssignTimeMonitorAssignTime internal error");
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
@@ -405,7 +464,7 @@ void KheAssignTimeMonitorUnAssignTime(KHE_ASSIGN_TIME_MONITOR m,
       assigned_time_index);
   m->deviation += KheMeetDuration(meet);
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
