@@ -75,7 +75,7 @@ static void KheAvoidClashesMonitorSetLowerBound(KHE_AVOID_CLASHES_MONITOR m)
 
   if( events_durn > cycle_durn )
   {
-    m->lower_bound = KheConstraintCost((KHE_CONSTRAINT) m->constraint,
+    m->lower_bound = KheMonitorDevToCost((KHE_MONITOR) m,
       events_durn - cycle_durn);
     if( DEBUG2 )
       fprintf(stderr, "  setting avoid clashes lower bound of %s to %.5f\n",
@@ -102,12 +102,14 @@ KHE_AVOID_CLASHES_MONITOR KheAvoidClashesMonitorMake(
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_AVOID_CLASHES_MONITOR_TAG);
+    KHE_AVOID_CLASHES_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->resource_in_soln = rs;
   res->constraint = c;
   /* ***
   res->separate =
-    (KheConstraintCostFunction((KHE_CONSTRAINT) c) != KHE_SUM_COST_FUNCTION);
+    (KheConstrai ntCostFunction((KHE_CONSTRAINT) c) != KHE_SUM_COST_FUNCTION);
   KheDevMonitorInit(&res->separate_dev_monitor);
   *** */
   res->deviation = 0;
@@ -281,6 +283,71 @@ void KheAvoidClashesMonitorAttachCheck(KHE_AVOID_CLASHES_MONITOR m)
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAvoidClashesMonitorTimeRange(KHE_AVOID_CLASHES_MONITOR m,        */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAvoidClashesMonitorTimeRange(KHE_AVOID_CLASHES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_INSTANCE ins;
+  ins = KheSolnInstance(m->soln);
+  if( KheInstanceTimeCount(ins) == 0 )
+    return *first_time = *last_time = NULL, false;
+  else
+    return *first_time = KheInstanceTime(ins, 0),
+      *last_time = KheInstanceTime(ins, KheInstanceTimeCount(ins) - 1), true;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheAvoidClashesMonitorSetSweepTime(KHE_AVOID_CLASHES_MONITOR m,     */
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*  Adding a sweep time does not change the cost of m, assuming that         */
+/*  times beyond the sweep time are unassigned.                              */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheAvoidClashesMonitorSetSweepTime(KHE_AVOID_CLASHES_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAvoidClashesMonitorSweepTimeRange(KHE_AVOID_CLASHES_MONITOR m,   */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for m.                                */
+/*                                                                           */
+/*  Adding a sweep time does not change the cost of m, assuming that         */
+/*  times beyond the sweep time are unassigned.                              */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAvoidClashesMonitorSweepTimeRange(KHE_AVOID_CLASHES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -330,7 +397,7 @@ void KheAvoidClashesMonitorFlush(KHE_AVOID_CLASHES_MONITOR m)
   if( m->new_deviation != m->deviation )
   {
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->new_deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->new_deviation));
     m->deviation = m->new_deviation;
   }
 }
@@ -343,7 +410,7 @@ void KheAvoidClashesMonitorFlush(KHE_AVOID_CLASHES_MONITOR m)
     if( KheDevMonitorHasChanged(&m->separate_dev_monitor) )
     {
       KheMonitorChangeCost((KHE_MONITOR) m,
-        KheConstraintCostMulti((KHE_CONSTRAINT) m->constraint,
+        KheConstrai ntCostMulti((KHE_CONSTRAINT) m->constraint,
 	  KheDevMonitorDevs(&m->separate_dev_monitor)));
       KheDevMonitorFlush(&m->separate_dev_monitor);
     }
@@ -353,7 +420,7 @@ void KheAvoidClashesMonitorFlush(KHE_AVOID_CLASHES_MONITOR m)
     if( m->new_deviation != m->deviation )
     {
       KheMonitorChangeCost((KHE_MONITOR) m,
-        KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->new_deviation));
+        KheMonitorDevToCost((KHE_MONITOR) m, m->new_deviation));
       m->deviation = m->new_deviation;
     }
   }
