@@ -78,7 +78,9 @@ KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR KheAvoidSplitAssignmentsMonitorMake(
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR_TAG);
+    KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->constraint = c;
   res->eg_index = eg_index;
   HaArrayInit(res->resources, a);
@@ -378,6 +380,88 @@ void KheAvoidSplitAssignmentsMonitorAttachCheck(
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAvoidSplitAssignmentsMonitorTimeRange(                           */
+/*    KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR m, KHE_TIME *first_time,           */
+/*    KHE_TIME *last_time)                                                   */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for avoid split assts monitor m.           */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAvoidSplitAssignmentsMonitorTimeRange(
+  KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR m, KHE_TIME *first_time,
+  KHE_TIME *last_time)
+{
+  int i, count;  KHE_EVENT_RESOURCE er;  KHE_TIME ft, lt;
+  *first_time = *last_time = NULL;
+  count = KheAvoidSplitAssignmentsConstraintEventResourceCount(m->constraint,
+    m->eg_index);
+  for( i = 0;  i < count;  i++ )
+  {
+    er = KheAvoidSplitAssignmentsConstraintEventResource(m->constraint,
+      m->eg_index, i);
+    if( !KheEventResourceTimeRange(er, m->soln, &ft, &lt) )
+      return false;
+    if( *first_time == NULL || KheTimeIndex(ft) < KheTimeIndex(*first_time) )
+      *first_time = ft;
+    if( *last_time == NULL || KheTimeIndex(lt) > KheTimeIndex(*last_time) )
+      *last_time = lt;
+  }
+  return *first_time != NULL;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheAvoidSplitAssignmentsMonitorSetSweepTime(                        */
+/*    KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR m, KHE_TIME t)                     */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for avoid split assignments monitor m.  */
+/*                                                                           */
+/*  Implementation note.  Installing a cutoff time makes no difference       */
+/*  to this monitor, because an unassigned task has the same effect on       */
+/*  it as an open task, so there is nothing to do here.                      */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheAvoidSplitAssignmentsMonitorSetSweepTime(
+  KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAvoidSplitAssignmentsMonitorSweepTimeRange(                      */
+/*    KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR m, KHE_TIME *first_time,           */
+/*    KHE_TIME *last_time)                                                   */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for avoid split assts monitor m.      */
+/*                                                                           */
+/*  Implementation note.  Installing a cutoff time makes no difference       */
+/*  to this monitor, because an unassigned task has the same effect on       */
+/*  it as an open task, so there is nothing to do here.                      */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAvoidSplitAssignmentsMonitorSweepTimeRange(
+  KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR m, KHE_TIME *first_time,
+  KHE_TIME *last_time)
+{
+  /* nothing to do */
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "fix and unfix"                                                */
@@ -599,8 +683,7 @@ void KheAvoidSplitAssignmentsMonitorAssignResource(
     HaArrayAddLast(m->multiplicities, 1);
     if( HaArrayCount(m->resources) >= 2 )
       KheMonitorChangeCost((KHE_MONITOR) m,
-	KheConstraintCost((KHE_CONSTRAINT) m->constraint,
-	HaArrayCount(m->resources) - 1));
+	KheMonitorDevToCost((KHE_MONITOR) m, HaArrayCount(m->resources) - 1));
   }
 }
 
@@ -635,8 +718,7 @@ void KheAvoidSplitAssignmentsMonitorUnAssignResource(
     }
     if( HaArrayCount(m->resources) >= 1 )
       KheMonitorChangeCost((KHE_MONITOR) m,
-	KheConstraintCost((KHE_CONSTRAINT) m->constraint,
-	HaArrayCount(m->resources) - 1));
+	KheMonitorDevToCost((KHE_MONITOR) m, HaArrayCount(m->resources) - 1));
   }
 }
 
