@@ -145,7 +145,7 @@ void KheAvoidUnavailableTimesConstraintFinalize(
     HaArrayForEach(c->times, t, i)
       SSetInsert(ss, KheTimeIndex(t));
     c->unavailable_times = KheTimeGroupMakeAndFinalize(c->instance,
-      KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, NULL, false);
+      KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, /* NULL, */ false, a);
     if( DEBUG1 )
     {
       fprintf(stderr, "KheAvoidUnavailableTimesConstraintFinalize (a) %p: ",
@@ -159,7 +159,7 @@ void KheAvoidUnavailableTimesConstraintFinalize(
   SSetUnion(ss, *KheTimeGroupTimeSet(KheInstanceFullTimeGroup(c->instance)));
   SSetDifference(ss, *KheTimeGroupTimeSet(c->unavailable_times));
   c->available_times = KheTimeGroupMakeAndFinalize(c->instance,
-    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, NULL, false);
+    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, false, a);
   if( DEBUG1 )
   {
     fprintf(stderr, "KheAvoidUnavailableTimesConstraintFinalize (b) %p: ",
