@@ -64,18 +64,22 @@ struct khe_avoid_unavailable_times_monitor_rec {
 static void KheAvoidUnavailableTimesMonitorSetLowerBound(
   KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m)
 {
-  KHE_RESOURCE r;  KHE_TIME_GROUP tg;  KHE_CONSTRAINT mc;
+  KHE_RESOURCE r;  KHE_TIME_GROUP tg;  /* KHE_CONSTRAINT mc; */
   int events_durn, avoid_durn, d;
 
   /* if r does not have a highly weighted avoid clashes constraint, quit */
   r = KheAvoidUnavailableTimesMonitorResource(m);
-  mc = (KHE_CONSTRAINT) m->constraint;
-  if( !KheResourceHasAvoidClashesConstraint(r, KheConstraintCombinedWeight(mc)))
+  /* mc = (KHE_CONSTRAINT) m->constraint; */
+  if( !KheResourceHasAvoidClashesConstraint(r, m->combined_weight))
     return;  /* no highly weighted avoid clashes constraint */
+  /* ***
+  if( !KheResourceHasAvoidClashesConstraint(r,K heConstraintCombinedWeight(mc)))
+    return;
+  *** */
 
   /* find the duration of highly weighted preassigned events */
-  events_durn = KheResourcePreassignedEventsDuration(r,
-    KheConstraintCombinedWeight(mc));
+  events_durn = KheResourcePreassignedEventsDuration(r, m->combined_weight);
+    /* Khe ConstraintCombinedWeight(mc)); */
 
   /* find the number of times to be avoided */
   tg = KheAvoidUnavailableTimesConstraintUnavailableTimes(m->constraint);
@@ -84,7 +88,7 @@ static void KheAvoidUnavailableTimesMonitorSetLowerBound(
   /* Let d be the number of deviations, and set lower_bound if needed */
   d = events_durn + avoid_durn - KheInstanceTimeCount(KheSolnInstance(m->soln));
   if( d > 0 )
-    m->lower_bound = KheConstraintCost((KHE_CONSTRAINT) m->constraint, d);
+    m->lower_bound = KheMonitorDevToCost((KHE_MONITOR) m, d);
 }
 
 
@@ -106,7 +110,9 @@ KHE_AVOID_UNAVAILABLE_TIMES_MONITOR KheAvoidUnavailableTimesMonitorMake(
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_AVOID_UNAVAILABLE_TIMES_MONITOR_TAG);
+    KHE_AVOID_UNAVAILABLE_TIMES_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->deviation = 0;
   res->resource_in_soln = rs;
   res->constraint = c;
@@ -283,6 +289,76 @@ void KheAvoidUnavailableTimesMonitorDetachFromSoln(
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAvoidUnavailableTimesMonitorTimeRange(                           */
+/*    KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m,                                 */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAvoidUnavailableTimesMonitorTimeRange(
+  KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_AVOID_UNAVAILABLE_TIMES_CONSTRAINT c;  KHE_TIME_GROUP tg;
+  c = KheAvoidUnavailableTimesMonitorConstraint(m);
+  tg = KheAvoidUnavailableTimesConstraintUnavailableTimes(c);
+  if( KheTimeGroupTimeCount(tg) == 0 )
+    return *first_time = *last_time = NULL, false;
+  else
+    return *first_time = KheTimeGroupTime(tg, 0),
+      *last_time = KheTimeGroupTime(tg, KheTimeGroupTimeCount(tg) - 1), true;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheAvoidUnavailableTimesMonitorSetSweepTime(                        */
+/*    KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m,                                 */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*  Adding a sweep time does not change the cost of m, assuming that         */
+/*  times beyond the sweep time are unassigned.                              */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheAvoidUnavailableTimesMonitorSetSweepTime(KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheAvoidUnavailableTimesMonitorSweepTimeRange(                      */
+/*    KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m,                                 */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for m.                                */
+/*                                                                           */
+/*  Adding a sweep time does not change the cost of m, assuming that         */
+/*  times beyond the sweep time are unassigned.                              */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheAvoidUnavailableTimesMonitorSweepTimeRange(
+  KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -293,7 +369,7 @@ void KheAvoidUnavailableTimesMonitorDetachFromSoln(
 /*                                                                           */
 /*  void KheAvoidUnavailableTimesMonitorAddBusyAndIdle(                      */
 /*    KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m, int index,                      */
-/*    int busy_count, int idle_count, float workload)                        */
+/*    KHE_BUSY_AND_IDLE busy_and_idle)                                       */
 /*                                                                           */
 /*  Receive a report of a new lot of busy times, and pass it on.  Idle       */
 /*  times are not used by this monitor.                                      */
@@ -302,13 +378,13 @@ void KheAvoidUnavailableTimesMonitorDetachFromSoln(
 
 void KheAvoidUnavailableTimesMonitorAddBusyAndIdle(
   KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m, int index,
-  int busy_count, int idle_count, float workload)
+  KHE_BUSY_AND_IDLE busy_and_idle)
 {
   HnAssert(m->deviation == 0,
     "KheAvoidUnavailableTimesMonitorAddBusyAndIdle internal error");
-  m->deviation = busy_count;
+  m->deviation = busy_and_idle->busy_count;
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
@@ -316,7 +392,7 @@ void KheAvoidUnavailableTimesMonitorAddBusyAndIdle(
 /*                                                                           */
 /*  void KheAvoidUnavailableTimesMonitorDeleteBusyAndIdle(                   */
 /*    KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m, int index,                      */
-/*    int busy_count, int idle_count, float workload)                        */
+/*    KHE_BUSY_AND_IDLE busy_and_idle)                                       */
 /*                                                                           */
 /*  Receive a report of a lot of busy times being deleted, and pass it on.   */
 /*  Idle times are not used by this monitor.                                 */
@@ -325,9 +401,10 @@ void KheAvoidUnavailableTimesMonitorAddBusyAndIdle(
 
 void KheAvoidUnavailableTimesMonitorDeleteBusyAndIdle(
   KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m, int index,
-  int busy_count, int idle_count, float workload)
+  KHE_BUSY_AND_IDLE busy_and_idle)
 {
-  HnAssert(m->cost==KheConstraintCost((KHE_CONSTRAINT)m->constraint, busy_count),
+  HnAssert(m->cost == KheMonitorDevToCost((KHE_MONITOR) m,
+    busy_and_idle->busy_count),
     "KheAvoidUnavailableTimesMonitorDeleteBusyAndIdle internal error");
   m->deviation = 0;
   KheMonitorChangeCost((KHE_MONITOR) m, 0);
@@ -338,9 +415,8 @@ void KheAvoidUnavailableTimesMonitorDeleteBusyAndIdle(
 /*                                                                           */
 /*  void KheAvoidUnavailableTimesMonitorChangeBusyAndIdle(                   */
 /*    KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m, int index,                      */
-/*    int old_busy_count, int new_busy_count,                                */
-/*    int old_idle_count, int new_idle_count,                                */
-/*    float old_workload, float new_workload)                                */
+/*    KHE_BUSY_AND_IDLE old_busy_and_idle,                                   */
+/*    KHE_BUSY_AND_IDLE new_busy_and_idle)                                   */
 /*                                                                           */
 /*  Receive a report of a change in the number of busy and idle times,       */
 /*  and pass it on.  Idle times are not used by this monitor.                */
@@ -349,17 +425,15 @@ void KheAvoidUnavailableTimesMonitorDeleteBusyAndIdle(
 
 void KheAvoidUnavailableTimesMonitorChangeBusyAndIdle(
   KHE_AVOID_UNAVAILABLE_TIMES_MONITOR m, int index,
-  int old_busy_count, int new_busy_count,
-  int old_idle_count, int new_idle_count,
-  float old_workload, float new_workload)
+  KHE_BUSY_AND_IDLE old_busy_and_idle, KHE_BUSY_AND_IDLE new_busy_and_idle)
 {
-  HnAssert(old_busy_count == m->deviation,
+  HnAssert(old_busy_and_idle->busy_count == m->deviation,
     "KheAvoidUnavailableTimesMonitorChangeBusyAndIdle internal error");
-  if( old_busy_count != new_busy_count )
+  if( old_busy_and_idle->busy_count != new_busy_and_idle->busy_count )
   {
-    m->deviation = new_busy_count;
+    m->deviation = new_busy_and_idle->busy_count;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
