@@ -48,7 +48,7 @@ struct khe_cluster_busy_times_constraint_rec {
   int				minimum;		/* minimum           */
   int				maximum;		/* maximum           */
   bool				allow_zero;		/* allow zero        */
-  bool				limit_busy_recode;	/* recodes limit busy*/
+  /* bool			limit_busy_recode; */	/* recodes limit busy*/
   /* bool			limits_whole_cycle; */	/* limits whole cycle*/
   ARRAY_KHE_TIME_GROUP		time_groups;		/* time groups       */
   int				min_time_index;		/* min time index    */
@@ -72,19 +72,19 @@ struct khe_cluster_busy_times_constraint_rec {
 /*                                                                           */
 /*  bool KheClusterBusyTimesConstraintMakeInternal(KHE_INSTANCE ins,         */
 /*    char *id, char *name, bool required, int weight, KHE_COST_FUNCTION cf, */
-/*    KHE_TIME_GROUP applies_to_tg, int minimum, int maximum,bool allow_zero,*/
-/*    bool limit_busy_recode, KHE_CLUSTER_BUSY_TIMES_CONSTRAINT *c)          */
+/*    KHE_TIME_GROUP applies_to_tg, int minimum, int maximum,                */
+/*    bool allow_zero, KHE_CLUSTER_BUSY_TIMES_CONSTRAINT *c)                 */
 /*                                                                           */
 /*  Make a new cluster busy times constraint with these attributes, add it   */
 /*  to ins, and return it.  This is the internal form that does not require  */
-/*  the instance to be unfinalized, and that sets limit_busy_recode.         */
+/*  the instance to be unfinalized.                                          */
 /*                                                                           */
 /*****************************************************************************/
 
 bool KheClusterBusyTimesConstraintMakeInternal(KHE_INSTANCE ins,
   char *id, char *name, bool required, int weight, KHE_COST_FUNCTION cf,
-  KHE_TIME_GROUP applies_to_tg, int minimum, int maximum, bool allow_zero,
-  bool limit_busy_recode, KHE_CLUSTER_BUSY_TIMES_CONSTRAINT *c)
+  KHE_TIME_GROUP applies_to_tg, int minimum, int maximum,
+  bool allow_zero, KHE_CLUSTER_BUSY_TIMES_CONSTRAINT *c)
 {
   KHE_CLUSTER_BUSY_TIMES_CONSTRAINT res;  KHE_CONSTRAINT cc;  HA_ARENA a;
   int i, count;
@@ -120,7 +120,6 @@ bool KheClusterBusyTimesConstraintMakeInternal(KHE_INSTANCE ins,
   res->minimum = minimum;
   res->maximum = maximum;
   res->allow_zero = allow_zero;
-  res->limit_busy_recode = limit_busy_recode;
   /* res->limits_whole_cycle = false; */ /* placeholder until finalized */
   HaArrayInit(res->time_groups, a);
   res->min_time_index = -1;
@@ -159,28 +158,10 @@ bool KheClusterBusyTimesConstraintMake(KHE_INSTANCE ins, char *id,
   HnAssert(KheInstanceFinalized(ins) == KHE_FINALIZED_NONE,
     "KheClusterBusyTimesConstraintMake called after KheInstanceMakeEnd");
   return KheClusterBusyTimesConstraintMakeInternal(ins, id, name, required,
-    weight, cf, applies_to_tg, minimum, maximum, allow_zero, false, c);
+    weight, cf, applies_to_tg, minimum, maximum, allow_zero, c);
 }
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheClusterBusyTimesConstraintSetLimitBusyRecode(                    */
-/*    KHE_CLUSTER_BUSY_TIMES_CONSTRAINT c)                                   */
-/*                                                                           */
-/*  Set the limit busy recode attribute of c.                                */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** replaced by KheClusterBusyTimesConstraintMakeInternal
-void KheClusterBusyTimesConstraintSetLimitBusyRecode(
-  KHE_CLUSTER_BUSY_TIMES_CONSTRAINT c)
-{
-  c->limit_busy_recode = true;
-}
-*** */
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  KHE_TIME_GROUP KheClusterBusyTimesConstraintAppliesToTimeGroup(          */
@@ -252,11 +233,13 @@ bool KheClusterBusyTimesConstraintAllowZero(
 /*                                                                           */
 /*****************************************************************************/
 
+/* ***
 bool KheClusterBusyTimesConstraintLimitBusyRecode(
   KHE_CLUSTER_BUSY_TIMES_CONSTRAINT c)
 {
   return c->limit_busy_recode;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -1162,8 +1145,10 @@ void KheClusterBusyTimesConstraintWrite(KHE_CLUSTER_BUSY_TIMES_CONSTRAINT c,
 {
   KHE_RESOURCE_GROUP rg;  KHE_RESOURCE r;  int i, val;
   KHE_TIME_GROUP tg;  char buff[50];
-  if( c->limit_busy_recode )  /* don't print if recodes limit busy */
+  /* ***
+  if( c->limit_busy_recode )  ** don't print if recodes limit busy **
     return;
+  *** */
   KmlBegin(kf, "ClusterBusyTimesConstraint");
   HnAssert(c->id != NULL,
     "KheArchiveWrite: Id missing in ClusterBusyTimesConstraint");
@@ -1269,10 +1254,10 @@ void KheClusterBusyTimesConstraintDebug(
       tg_id = (c->applies_to_tg == NULL ? "null" :
         KheTimeGroupId(c->applies_to_tg) == NULL ? "-" :
         KheTimeGroupId(c->applies_to_tg));
-      fprintf(fp, "%*s[ applies_to %s, %d-%d, allow_zero %s, recode %s\n",
+      fprintf(fp, "%*s[ applies_to %s, %d-%d, allow_zero %s\n",
 	indent, "", tg_id, c->minimum, c->maximum,
-	c->allow_zero ? "true" : "false",
-	c->limit_busy_recode ? "true" : "false");
+	c->allow_zero ? "true" : "false");
+	/* c->limi t_busy_recode ? "true" : "false"); */
       HaArrayForEach(c->resource_groups, rg, i)
         fprintf(fp, "%*s  %s\n", indent, "",
 	  KheResourceGroupId(rg) != NULL ? KheResourceGroupId(rg) : "-");
