@@ -205,6 +205,38 @@ KHE_COST_FUNCTION KheConstraintCostFunction(KHE_CONSTRAINT c)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_COST KheConstraintDevToCost(KHE_CONSTRAINT c, int dev)               */
+/*                                                                           */
+/*  Convert dev into a cost for c.                                           */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_COST KheConstraintDevToCost(KHE_CONSTRAINT c, int dev)
+{
+  switch( c->cost_function )
+  {
+    case KHE_STEP_COST_FUNCTION:
+
+        return dev > 0 ? c->combined_weight : 0;
+
+    case KHE_LINEAR_COST_FUNCTION:
+
+        return dev * c->combined_weight;
+
+    case KHE_QUADRATIC_COST_FUNCTION:
+
+	return dev * dev * c->combined_weight;
+
+    default:
+
+	HnAbort("KheConstraintDevToCost internal error");
+	return 0;  /* keep compiler happy */
+  }
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  int KheConstraintIndex(KHE_CONSTRAINT c)                                 */
@@ -1025,15 +1057,13 @@ void KheConstraintResetResourceOfType(KHE_CONSTRAINT c)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  bool KheConstraintFinalize(KHE_CONSTRAINT c, bool limit_busy_recode,     */
-/*    KML_ERROR *ke)                                                         */
+/*  bool KheConstraintFinalize(KHE_CONSTRAINT c, KML_ERROR *ke)              */
 /*                                                                           */
 /*  Finalize c (at end of instance read).                                    */
 /*                                                                           */
 /*****************************************************************************/
 
-bool KheConstraintFinalize(KHE_CONSTRAINT c, bool limit_busy_recode,
-  KML_ERROR *ke)
+bool KheConstraintFinalize(KHE_CONSTRAINT c, KML_ERROR *ke)
 {
   if( DEBUG1 )
     fprintf(stderr, "    KheConstraintFinalize(%s %s)\n",
@@ -1112,7 +1142,7 @@ bool KheConstraintFinalize(KHE_CONSTRAINT c, bool limit_busy_recode,
 
     case KHE_LIMIT_BUSY_TIMES_CONSTRAINT_TAG:
       KheLimitBusyTimesConstraintFinalize(
-	(KHE_LIMIT_BUSY_TIMES_CONSTRAINT) c, limit_busy_recode);
+	(KHE_LIMIT_BUSY_TIMES_CONSTRAINT) c);
       break;
 
     case KHE_LIMIT_WORKLOAD_CONSTRAINT_TAG:
@@ -2399,6 +2429,7 @@ void KheConstraintWriteCommonFields(KHE_CONSTRAINT c, KML_FILE kf)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** replaced by KheMonitorDevToCost
 KHE_COST KheConstraintCost(KHE_CONSTRAINT c, int dev)
 {
   switch( c->cost_function )
@@ -2418,6 +2449,7 @@ KHE_COST KheConstraintCost(KHE_CONSTRAINT c, int dev)
     default:
 
 	HnAbort("KheConstraintCost internal error");
-	return 0;  /* keep compiler happy */
+	return 0;  ** keep compiler happy **
   }
 }
+*** */
