@@ -359,7 +359,7 @@ void KheSolnSplitCycleMeet(KHE_SOLN soln)
   HA_ARRAY_BOOL break_after;  bool break_here;  KHE_MEET cycle_meet, junk;
 
   ins = KheSolnInstance(soln);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   if( DEBUG1 )
     fprintf(stderr, "[ KheSolnSplitCycleMeet(soln of %s)\n",
       KheInstanceId(ins));
