@@ -72,7 +72,9 @@ KHE_DISTRIBUTE_SPLIT_EVENTS_MONITOR KheDistributeSplitEventsMonitorMake(
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_DISTRIBUTE_SPLIT_EVENTS_MONITOR_TAG);
+    KHE_DISTRIBUTE_SPLIT_EVENTS_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->event_in_soln = es;
   res->constraint = c;
   res->duration = KheDistributeSplitEventsConstraintDuration(c);
@@ -134,6 +136,7 @@ void KheDistributeSplitEventsMonitorCopyPhase2(
   {
     m->copy = NULL;
     KheMonitorCopyCommonFieldsPhase2((KHE_MONITOR) m);
+    KheEventInSolnCopyPhase2(m->event_in_soln);
   }
 }
 
@@ -240,7 +243,7 @@ void KheDistributeSplitEventsMonitorAttachToSoln(
   m->attached = true;
   m->deviation = m->minimum;
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   KheEventInSolnAttachMonitor(m->event_in_soln, (KHE_MONITOR) m);
 }
 
@@ -289,6 +292,66 @@ void KheDistributeSplitEventsMonitorAttachCheck(
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheDistributeSplitEventsMonitorTimeRange(                           */
+/*    KHE_DISTRIBUTE_SPLIT_EVENTS_MONITOR m,                                 */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheDistributeSplitEventsMonitorTimeRange(
+  KHE_DISTRIBUTE_SPLIT_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_EVENT e;
+  e = KheDistributeSplitEventsMonitorEvent(m);
+  return KheEventTimeRange(e, m->soln, first_time, last_time);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheDistributeSplitEventsMonitorSetSweepTime(                        */
+/*    KHE_DISTRIBUTE_SPLIT_EVENTS_MONITOR m, KHE_TIME t)                     */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheDistributeSplitEventsMonitorSetSweepTime(
+  KHE_DISTRIBUTE_SPLIT_EVENTS_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheDistributeSplitEventsMonitorSweepTimeRange(                      */
+/*    KHE_DISTRIBUTE_SPLIT_EVENTS_MONITOR m,                                 */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for m.                                */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheDistributeSplitEventsMonitorSweepTimeRange(
+  KHE_DISTRIBUTE_SPLIT_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -364,7 +427,7 @@ void KheDistributeSplitEventsMonitorAddMeet(
     {
       m->deviation += delta_deviation;
       KheMonitorChangeCost((KHE_MONITOR) m,
-	KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+	KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
     }
   }
 }
@@ -393,7 +456,7 @@ void KheDistributeSplitEventsMonitorDeleteMeet(
     {
       m->deviation += delta_deviation;
       KheMonitorChangeCost((KHE_MONITOR) m,
-	KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+	KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
     }
   }
 }
@@ -430,7 +493,7 @@ void KheDistributeSplitEventsMonitorSplitMeet(
   {
     m->deviation += delta_deviation;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -475,7 +538,7 @@ void KheDistributeSplitEventsMonitorMergeMeet(
   {
     m->deviation += delta_deviation;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
