@@ -307,14 +307,14 @@ void KheEvennessHandlerDetachAllEvennessMonitors(KHE_EVENNESS_HANDLER eh)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheEvennessHandleSetAllEvennessMonitorWeights(                      */
+/*  void KheEvennessHandlerSetAllEvennessMonitorWeights(                     */
 /*    KHE_EVENNESS_HANDLER eh, KHE_COST weight)                              */
 /*                                                                           */
 /*  Set the weights of all evenness monitors.                                */
 /*                                                                           */
 /*****************************************************************************/
 
-void KheEvennessHandleSetAllEvennessMonitorWeights(
+void KheEvennessHandlerSetAllEvennessMonitorWeights(
   KHE_EVENNESS_HANDLER eh, KHE_COST weight)
 {
   KHE_PARTITION_HANDLER ph;  KHE_EVENNESS_MONITOR m;  int i, j;
