@@ -82,7 +82,8 @@ KHE_EVENNESS_MONITOR KheEvennessMonitorMake(KHE_SOLN soln,
   a = KheSolnArena(soln);
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
-  KheMonitorInitCommonFields((KHE_MONITOR) res, soln, KHE_EVENNESS_MONITOR_TAG);
+  KheMonitorInitCommonFields((KHE_MONITOR) res, soln, KHE_EVENNESS_MONITOR_TAG,
+    KHE_LINEAR_COST_FUNCTION, KheCost(1, 0));
   res->partition = partition;
   res->time = time;
   res->limit = KheResourceGroupResourceCount(partition) -
