@@ -298,11 +298,15 @@ int KheEventIndex(KHE_EVENT e)
 
 void KheEventFinalize(KHE_EVENT e)
 {
-  KHE_EVENT_RESOURCE er, prev_er;  int i, equiv_class;
+  KHE_EVENT_RESOURCE er /* , prev_er */;  int i /* , equiv_class */;
   if( DEBUG2 )
     fprintf(stderr, "[ KheEventFinalize(%s)\n", e->id != NULL ? e->id : "-");
 
   /* finalize the event resources and set their equivalence classes */
+  HaArrayForEach(e->event_resources, er, i)
+    KheEventResourceFinalize(er);
+
+  /* *** old version that created equivalence classes
   prev_er = NULL;
   equiv_class = -1;
   HaArrayForEach(e->event_resources, er, i)
@@ -313,6 +317,7 @@ void KheEventFinalize(KHE_EVENT e)
     KheEventResourceSetEquivClass(er, equiv_class);
     prev_er = er;
   }
+  *** */
 
   if( DEBUG2 )
     fprintf(stderr, "] KheEventFinalize(%s)\n", e->id != NULL ? e->id : "-");
@@ -422,7 +427,7 @@ bool KheEventSimilar(KHE_EVENT e1, KHE_EVENT e2)
   ARRAY_KHE_RESOURCE_GROUP domains1, domains2;  bool res;  HA_ARENA a;
   if( e1 == e2 )
     return true;
-  a = KheInstanceArenaBegin(e1->instance, false);
+  a = KheInstanceArenaBegin(e1->instance);
   HaArrayInit(domains1, a);
   HaArrayInit(domains2, a);
   res = KheEventPartitionSimilar(e1, e2, &domains1, &domains2);
@@ -750,6 +755,43 @@ KHE_CONSTRAINT KheEventConstraint(KHE_EVENT e, int i)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheEventTimeRange(KHE_EVENT e, KHE_SOLN soln,                       */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Return the time range of e in soln.                                      */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheEventTimeRange(KHE_EVENT e, KHE_SOLN soln,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  int i;  KHE_MEET meet;  KHE_TIME ft, lt;
+  *first_time = *last_time = NULL;
+  for( i = 0;  i < KheEventMeetCount(soln, e);  i++ )
+  {
+    meet = KheEventMeet(soln, e, i);
+    HnAssert(meet != NULL, "KheEventTimeRange internal error");
+    ft = KheMeetAsstTime(meet);
+    if( ft == NULL )
+      return false;
+    lt = KheTimeNeighbour(ft, KheMeetDuration(meet) - 1);
+    if( *first_time == NULL || KheTimeIndex(ft) < KheTimeIndex(*first_time) )
+      *first_time = ft;
+    if( *last_time == NULL || KheTimeIndex(lt) > KheTimeIndex(*last_time) )
+      *last_time = lt;
+  }
+  return *first_time != NULL;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "reading and writing"                                          */
