@@ -32,6 +32,8 @@
 #define DEBUG2 0
 #define DEBUG3 0
 #define DEBUG4 0
+#define DEBUG5 0
+#define DEBUG6 0
 
 
 /*****************************************************************************/
@@ -64,9 +66,9 @@ struct khe_event_in_soln_rec {
 static void KheEventInSolnMeetsDebug(KHE_EVENT_IN_SOLN es, FILE *fp)
 {
   int i;  KHE_MEET m;
-  fprintf(fp, "%p:[", (void *) es);
+  fprintf(fp, "%d meets[", HaArrayCount(es->meets));
   HaArrayForEach(es->meets, m, i)
-    fprintf(fp, "%s%p", i == 0 ? "" : ", ", (void *) m);
+    fprintf(fp, "%s%s%p", i == 0 ? "" : ", ", KheMeetId(m), (void *) m);
   fprintf(fp, "]");
 }
 
@@ -108,7 +110,8 @@ KHE_EVENT_IN_SOLN KheEventInSolnMake(KHE_SOLN soln, KHE_EVENT e
   res->copy = NULL;
   if( DEBUG4 )
   {
-    fprintf(stderr, "KheEventInSolnMake(%p, %p) = ", (void *) soln, (void *) e);
+    fprintf(stderr, "KheEventInSolnMake(%p, %s %p) = ", (void *) soln,
+      KheEventId(e), (void *) e);
     KheEventInSolnMeetsDebug(res, stderr);
     fprintf(stderr, "\n");
   }
@@ -353,10 +356,12 @@ KHE_EVENT_IN_SOLN KheEventInSolnCopyPhase1(KHE_EVENT_IN_SOLN es, HA_ARENA a)
 
 void KheEventInSolnCopyPhase2(KHE_EVENT_IN_SOLN es)
 {
-  int i;  KHE_EVENT_RESOURCE_IN_SOLN ers;  KHE_MONITOR m;
+  int i;  KHE_EVENT_RESOURCE_IN_SOLN ers;  KHE_MONITOR m;  KHE_MEET meet;
   if( es->copy != NULL )
   {
     es->copy = NULL;
+    HaArrayForEach(es->meets, meet, i)
+      KheMeetCopyPhase2(meet);
     HaArrayForEach(es->event_resources_in_soln, ers, i)
       KheEventResourceInSolnCopyPhase2(ers);
     if( es->timetable_monitor != NULL )
@@ -463,16 +468,17 @@ void KheEventInSolnSplitMeet(KHE_EVENT_IN_SOLN es, KHE_MEET meet1,
     "KheEventInSolnSplitMeet internal error 1");
   HnAssert(KheMeetDuration(meet2) > 0,
     "KheEventInSolnSplitMeet internal error 2");
-  if( DEBUG4 )
+  if( DEBUG6 )
   {
-    fprintf(stderr, "[ KheEventInSolnSplitMeet(");
+    fprintf(stderr, "[ KheEventInSolnSplitMeet(%s ", KheEventId(es->event));
     KheEventInSolnMeetsDebug(es, stderr);
-    fprintf(stderr, ", %p, %p)\n", (void *) meet1, (void *) meet2);
+    fprintf(stderr, ", %s %p, %s %p)\n",
+      KheMeetId(meet1), (void *) meet1, KheMeetId(meet2), (void *) meet2);
   }
   HaArrayAddLast(es->meets, meet2);
   HaArrayForEach(es->attached_monitors, m, i)
     KheMonitorSplitMeet(m, meet1, meet2);
-  if( DEBUG4 )
+  if( DEBUG6 )
   {
     fprintf(stderr, "] KheEventInSolnSplitMeet returning, es = ");
     KheEventInSolnMeetsDebug(es, stderr);
@@ -495,11 +501,12 @@ void KheEventInSolnMergeMeet(KHE_EVENT_IN_SOLN es, KHE_MEET meet1,
   KHE_MEET meet2)
 {
   KHE_MEET x;  int i;  KHE_MONITOR m;
-  if( DEBUG4 )
+  if( DEBUG6 )
   {
-    fprintf(stderr, "[ KheEventInSolnMergeMeet(");
+    fprintf(stderr, "[ KheEventInSolnMergeMeet(%s ", KheEventId(es->event));
     KheEventInSolnMeetsDebug(es, stderr);
-    fprintf(stderr, ", %p, %p)\n", (void *) meet1, (void *) meet2);
+    fprintf(stderr, ", %s %p, %s %p)\n",
+      KheMeetId(meet1), (void *) meet1, KheMeetId(meet2), (void *) meet2);
   }
   HaArrayForEach(es->attached_monitors, m, i)
     KheMonitorMergeMeet(m, meet1, meet2);
@@ -510,7 +517,7 @@ void KheEventInSolnMergeMeet(KHE_EVENT_IN_SOLN es, KHE_MEET meet1,
       HnAbort("KheEventInSolnMergeMeet internal error");
     HaArrayPut(es->meets, i, x);
   }
-  if( DEBUG4 )
+  if( DEBUG6 )
   {
     fprintf(stderr, "] KheEventInSolnMergeMeet returning, es = ");
     KheEventInSolnMeetsDebug(es, stderr);
@@ -533,7 +540,12 @@ void KheEventInSolnAssignTime(KHE_EVENT_IN_SOLN es,
 {
   KHE_MONITOR m;  int i;
   HaArrayForEach(es->attached_time_asst_monitors, m, i)
+  {
+    if( DEBUG5 )
+      fprintf(stderr, "  KheEventInSolnAssignTime(es, %s, %d) calling "
+	"KheMonitorAssignTime\n", KheMeetId(meet), assigned_time_index);
     KheMonitorAssignTime(m, meet, assigned_time_index);
+  }
 }
 
 
