@@ -48,8 +48,9 @@ struct khe_event_resource_rec {
   KHE_RESOURCE		preassigned_resource;	/* preassigned resource      */
   char			*role;			/* optional role             */
   int			workload;		/* workload (compulsory!)    */
+  float			workload_per_time;	/* workload per time         */
   int			event_index;		/* index in event            */
-  int			event_equiv_class;	/* equiv class within event  */
+  /* int		event_equiv_class; */	/* equiv class within event  */
   int			instance_index;		/* index in instance         */
   ARRAY_KHE_CONSTRAINT	constraints;		/* constraints on this e.r.  */
   HA_ARRAY_INT		constraint_eg_indexes;	/* eg_indexes of asac's      */
@@ -103,6 +104,8 @@ bool KheEventResourceMake(KHE_EVENT event, KHE_RESOURCE_TYPE rt,
     KheResourceTypeDemandNotAllPreassigned(rt);
   res->role = HnStringCopy(role, a);
   res->workload = workload;
+  res->workload_per_time = (float) workload / (float) KheEventDuration(event);
+  KheResourceTypeAddMaxWorkloadPerTime(rt, res->workload_per_time);
   if( preassigned_resource != NULL )
     KheResourceAddPreassignedEventResource(preassigned_resource, res);
   res->hard_domain = NULL;  /* set later, when finalizing */ 
@@ -111,7 +114,7 @@ bool KheEventResourceMake(KHE_EVENT event, KHE_RESOURCE_TYPE rt,
   HaArrayInit(res->constraints, a);
   HaArrayInit(res->constraint_eg_indexes, a);
   KheEventAddEventResource(event, res, &res->event_index);
-  res->event_equiv_class = -1;  /* set later, when finalizing */
+  /* res->event_equiv_class = -1; */  /* set later, when finalizing */
   res->instance_index = KheInstanceEventResourceCount(ins);
   KheInstanceAddEventResource(ins, res);
   *er = res;
@@ -221,6 +224,20 @@ int KheEventResourceWorkload(KHE_EVENT_RESOURCE er)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  float KheEventResourceWorkloadPerTime(KHE_EVENT_RESOURCE er)             */
+/*                                                                           */
+/*  Return the workload per time of er.                                      */
+/*                                                                           */
+/*****************************************************************************/
+
+float KheEventResourceWorkloadPerTime(KHE_EVENT_RESOURCE er)
+{
+  return er->workload_per_time;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  int KheEventResourceEventIndex(KHE_EVENT_RESOURCE er)                    */
@@ -980,11 +997,12 @@ KHE_RESOURCE_GROUP KheEventResourceHardAndSoftDomain(
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** withdrawn
 bool KheEventResourceDoEquiv(KHE_EVENT_RESOURCE er1, KHE_EVENT_RESOURCE er2)
 {
   KHE_CONSTRAINT c1, c2;  int i;
 
-  /* er1 and er2 must have a lot of attributes in common */
+  ** er1 and er2 must have a lot of attributes in common **
   if( er1->event != er2->event )
     return false;
   if( er1->resource_type != er2->resource_type )
@@ -994,7 +1012,7 @@ bool KheEventResourceDoEquiv(KHE_EVENT_RESOURCE er1, KHE_EVENT_RESOURCE er2)
   if( er1->workload != er2->workload )
     return false;
 
-  /* and they must have equivalent constraints (assumed sorted) */
+  ** and they must have equivalent constraints (assumed sorted) **
   if( HaArrayCount(er1->constraints) != HaArrayCount(er2->constraints) )
     return false;
   for( i = 0;  i < HaArrayCount(er1->constraints);  i++ )
@@ -1005,9 +1023,10 @@ bool KheEventResourceDoEquiv(KHE_EVENT_RESOURCE er1, KHE_EVENT_RESOURCE er2)
       return false;
   }
 
-  /* all good */
+  ** all good **
   return true;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -1018,6 +1037,7 @@ bool KheEventResourceDoEquiv(KHE_EVENT_RESOURCE er1, KHE_EVENT_RESOURCE er2)
 /*                                                                           */
 /*****************************************************************************/
 
+/* ***
 void KheEventResourceSetEquivClass(KHE_EVENT_RESOURCE er, int val)
 {
   er->event_equiv_class = val;
@@ -1028,6 +1048,7 @@ void KheEventResourceSetEquivClass(KHE_EVENT_RESOURCE er, int val)
     fprintf(stderr, ", %d)\n", val);
   }
 }
+*** */
 
 
 /*****************************************************************************/
@@ -1041,11 +1062,51 @@ void KheEventResourceSetEquivClass(KHE_EVENT_RESOURCE er, int val)
 /*                                                                           */
 /*****************************************************************************/
 
+/* ***
 bool KheEventResourceEquivalent(KHE_EVENT_RESOURCE er1, KHE_EVENT_RESOURCE er2)
 {
   return er1->event == er2->event &&
     er1->event_equiv_class == er2->event_equiv_class;
 }
+*** */
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheEventResourceTimeRange(KHE_EVENT_RESOURCE er,                    */
+/*    KHE_SOLN soln, KHE_TIME *first_time, KHE_TIME *last_time)              */
+/*                                                                           */
+/*  Return the time range of er in soln.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheEventResourceTimeRange(KHE_EVENT_RESOURCE er,
+  KHE_SOLN soln, KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  int i;  KHE_TASK task;  KHE_MEET meet;  KHE_TIME ft, lt;
+  *first_time = *last_time = NULL;
+  for( i = 0;  i < KheEventResourceTaskCount(soln, er);  i++ )
+  {
+    task = KheEventResourceTask(soln, er, i);
+    meet = KheTaskMeet(task);
+    HnAssert(meet != NULL, "KheEventResourceTimeRange internal error");
+    ft = KheMeetAsstTime(meet);
+    if( ft == NULL )
+      return false;
+    lt = KheTimeNeighbour(ft, KheMeetDuration(meet) - 1);
+    if( *first_time == NULL || KheTimeIndex(ft) < KheTimeIndex(*first_time) )
+      *first_time = ft;
+    if( *last_time == NULL || KheTimeIndex(lt) > KheTimeIndex(*last_time) )
+      *last_time = lt;
+  }
+  return *first_time != NULL;
+}
 
 
 /*****************************************************************************/
