@@ -193,6 +193,7 @@ void KheEventResourceInSolnCopyPhase2(KHE_EVENT_RESOURCE_IN_SOLN ers)
   if( ers->copy != NULL )
   {
     ers->copy = NULL;
+    KheEventInSolnCopyPhase2(ers->event_in_soln);
     HaArrayForEach(ers->all_monitors, m, i)
       KheMonitorCopyPhase2(m);
     HaArrayForEach(ers->attached_monitors, m, i)
@@ -602,7 +603,7 @@ void KheEventResourceInSolnDebug(KHE_EVENT_RESOURCE_IN_SOLN ers,
   int verbosity, int indent, FILE *fp)
 {
   int i;  KHE_TASK task;  KHE_MONITOR m;  char *role;
-  KHE_RESOURCE_TYPE rt;
+  KHE_RESOURCE_TYPE rt;  KHE_RESOURCE r;
   if( verbosity >= 1 )
   {
     if( indent >= 0 )
@@ -629,6 +630,15 @@ void KheEventResourceInSolnDebug(KHE_EVENT_RESOURCE_IN_SOLN ers,
       }
       fprintf(fp, "%s:%s", KheEventInSolnId(ers->event_in_soln),
 	role != NULL ? role : "~");
+      fprintf(fp, " asst (");
+      HaArrayForEach(ers->tasks, task, i)
+      {
+	if( i > 0 )
+	  fprintf(fp, ", ");
+	r = KheTaskAsstResource(task);
+	fprintf(fp, "%s", r == NULL ? "-" : KheResourceId(r));
+      }
+      fprintf(fp, ")");
     }
   }
 }
