@@ -29,6 +29,8 @@
 /*****************************************************************************/
 #include "khe_interns.h"
 
+#define DEBUG2 0
+#define DEBUG3 0
 #define DEBUG4 0
 #define DEBUG5 0
 
@@ -231,11 +233,14 @@ KHE_EVENT_TIMETABLE_MONITOR KheEventTimetableMonitorMake(KHE_SOLN soln,
   KHE_EVENT_GROUP eg)
 {
   KHE_EVENT_TIMETABLE_MONITOR res;  int i;  KHE_EVENT e;  HA_ARENA a;
+  if( DEBUG3 )
+    fprintf(stderr, "  calling KheEventTimetableMonitorMake(soln, eg %d)\n",
+      KheEventGroupEventCount(eg));
   a = KheSolnArena(soln);
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_EVENT_TIMETABLE_MONITOR_TAG);
+    KHE_EVENT_TIMETABLE_MONITOR_TAG, KHE_LINEAR_COST_FUNCTION, 0);
   HaArrayInit(res->events_in_soln, a);
   for( i = 0;  i < KheEventGroupEventCount(eg);  i++ )
   {
@@ -383,6 +388,15 @@ int KheEventTimetableMonitorTimeMeetCount(KHE_EVENT_TIMETABLE_MONITOR etm,
   KHE_TIME_CELL tc;
   KheEventTimetableMonitorAddTimeCells(etm);
   tc = HaArray(etm->time_cells, KheTimeIndex(time));
+  if( DEBUG2 )
+  {
+    KHE_MEET m;  int i;
+    fprintf(stderr,"[ KheEventTimetableMonitorTimeMeetCount(etm, %s, %d meets):\n",
+      KheTimeId(time), HaArrayCount(tc->meets));
+    HaArrayForEach(tc->meets, m, i)
+      fprintf(stderr, "  meet %s\n", KheMeetId(m));
+    fprintf(stderr,"]\n");
+  }
   return HaArrayCount(tc->meets);
 }
 
@@ -601,8 +615,13 @@ void KheEventTimetableMonitorAddMeet(KHE_EVENT_TIMETABLE_MONITOR etm,
   KHE_MEET meet)
 {
   if( KheMeetAssignedTimeIndex(meet) != NO_TIME_INDEX )
+  {
+    if( DEBUG3 )
+      fprintf(stderr, "  KheEventTimetableMonitorAddMeet(etm, %s) calling "
+	"KheEventTimetableMonitorAssignTime\n", KheMeetId(meet));
     KheEventTimetableMonitorAssignTime(etm, meet,
       KheMeetAssignedTimeIndex(meet));
+  }
 }
 
 
@@ -736,6 +755,10 @@ void KheEventTimetableMonitorAssignTime(KHE_EVENT_TIMETABLE_MONITOR etm,
       HaArrayForEach(etm->link_events_monitors, lem, j)
 	KheLinkEventsMonitorAssignNonClash(lem, assigned_time_index + i);
     }
+    if( DEBUG3 )
+      fprintf(stderr, "KheEventTimetableMonitorAssignTime(%s, %d) at tc %d"
+	" (%d meets before)\n", KheMeetId(meet), assigned_time_index,
+	assigned_time_index + i, HaArrayCount(tc->meets));
     HaArrayAddLast(tc->meets, meet);
   }
   HaArrayForEach(etm->link_events_monitors, lem, i)
@@ -780,6 +803,10 @@ void KheEventTimetableMonitorUnAssignTime(KHE_EVENT_TIMETABLE_MONITOR etm,
       }
       HnAbort("KheEventTimetableMonitorUnAssignTime internal error 1");
     }
+    if( DEBUG3 )
+      fprintf(stderr, "KheEventTimetableMonitorUnAssignTime(%s, %d) at tc %d"
+	" (%d meets before)\n", KheMeetId(meet), assigned_time_index,
+	assigned_time_index + i, HaArrayCount(tc->meets));
     HaArrayDeleteAndShift(tc->meets, pos);
     if( HaArrayCount(tc->meets) == 0 )
     {
@@ -997,7 +1024,7 @@ void KheEventTimetableMonitorPrintTimetable(KHE_EVENT_TIMETABLE_MONITOR etm,
 
   /* find the weeks; make just one if there are no weeks */
   ins = KheSolnInstance(etm->soln);
-  a = KheSolnArenaBegin(etm->soln, false);
+  a = KheSolnArenaBegin(etm->soln);
   HaArrayInit(weeks, a);
   for( i = 0;  i < KheInstanceTimeGroupCount(ins);  i++ )
   {
