@@ -32,6 +32,7 @@
 #define DEBUG1 0
 #define DEBUG2 0
 #define DEBUG3 0
+#define DEBUG4 0
 
 /*****************************************************************************/
 /*                                                                           */
@@ -51,6 +52,63 @@ struct khe_group_monitor_rec {
 /*                                                                           */
 /*****************************************************************************/
 
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheGroupMonitorCheck(KHE_GROUP_MONITOR gm)                          */
+/*                                                                           */
+/*  Check gm.                                                                */
+/*                                                                           */
+/*****************************************************************************/
+
+static void KheGroupMonitorCheck(KHE_GROUP_MONITOR gm, KHE_MONITOR change_m,
+  KHE_COST old_cost, KHE_COST new_cost)
+{
+  KHE_COST total_cost;  int i;  KHE_MONITOR_LINK link;  KHE_MONITOR m;
+  if( DEBUG4 && !KheSolnMatchingIsActive(gm->soln) &&
+      KheSolnType(gm->soln) == KHE_SOLN_ORDINARY )
+  {
+    /* check children */
+    total_cost = 0;
+    HaArrayForEach(gm->child_links, link, i)
+    {
+      m = link->child;
+      total_cost += KheMonitorCost(m);
+    }
+    if( total_cost != gm->cost )
+    {
+      if( change_m != NULL )
+      {
+	fprintf(stderr, "KheGroupMonitorCheck failing (m %.5f -> %.5f):\n",
+	  KheCostShowWide(old_cost), KheCostShowWide(new_cost));
+	KheMonitorDebug(change_m, 4, 2, stderr);
+      }
+      else
+	fprintf(stderr, "KheGroupMonitorCheck failing:\n");
+      KheGroupMonitorDebug(gm, 4, 2, stderr);
+      HnAbort("KheGroupMonitorCheck(%s) internal error:"
+      " (a) total_cost %.8f, gm->cost %.8f", KheMonitorId((KHE_MONITOR) gm),
+      KheCostShowWide(total_cost), KheCostShowWide(gm->cost));
+    }
+
+    /* check defects */
+    total_cost = 0;
+    HaArrayForEach(gm->defect_links, link, i)
+    {
+      m = link->child;
+      total_cost += KheMonitorCost(m);
+    }
+    if( total_cost != gm->cost )
+    {
+      fprintf(stderr, "KheGroupMonitorCheck failing");
+      KheGroupMonitorDebug(gm, 4, 2, stderr);
+      HnAbort("KheGroupMonitorCheck(%s) internal error:"
+      " (b) total_cost %.8f, gm->cost %.8f", KheMonitorId((KHE_MONITOR) gm),
+      KheCostShowWide(total_cost), KheCostShowWide(gm->cost));
+    }
+  }
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  KHE_GROUP_MONITOR KheGroupMonitorMake(KHE_SOLN soln, int sub_tag,        */
@@ -85,11 +143,13 @@ KHE_GROUP_MONITOR KheGroupMonitorMake(KHE_SOLN soln, int sub_tag,
     HaArrayClear(res->defect_links);
     HaArrayClear(res->traces);
   }
-  KheMonitorInitCommonFields((KHE_MONITOR) res, soln, KHE_GROUP_MONITOR_TAG);
+  KheMonitorInitCommonFields((KHE_MONITOR) res, soln, KHE_GROUP_MONITOR_TAG,
+    KHE_LINEAR_COST_FUNCTION, 0);
   /* MArrayInit(res->defect_links_copy); */
   res->sub_tag = sub_tag;
   res->sub_tag_label = HnStringCopy(sub_tag_label, a);
   res->copy = NULL;
+  KheGroupMonitorCheck(res, NULL, 0, 0);
   if( DEBUG1 )
     fprintf(stderr, "] KheGroupMonitorMake returning %p\n", (void *) res);
   return res;
@@ -113,6 +173,7 @@ void KheGroupMonitorDelete(KHE_GROUP_MONITOR gm)
 {
   if( DEBUG1 )
     fprintf(stderr, "[ KheGroupMonitorDelete(%p)\n", (void *) gm);
+  KheGroupMonitorCheck(gm, NULL, 0, 0);
   HnAssert(gm != (KHE_GROUP_MONITOR) gm->soln,
     "KheGroupMonitorDelete:  gm is soln");
   HnAssert(HaArrayCount(gm->traces) == 0,
@@ -191,6 +252,7 @@ KHE_GROUP_MONITOR KheGroupMonitorCopyPhase1(KHE_GROUP_MONITOR gm, HA_ARENA a)
       copy->sub_tag = gm->sub_tag;
       copy->sub_tag_label = gm->sub_tag_label;
       copy->copy = NULL;
+      KheGroupMonitorCheck(gm->copy, NULL, 0, 0);
     }
     return gm->copy;
   }
@@ -464,6 +526,8 @@ static void KheGroupMonitorChangeLowerBound(KHE_GROUP_MONITOR gm,
 /*  Add m as a child to gm.                                                  */
 /*                                                                           */
 /*****************************************************************************/
+void KheGroupMonitorChangeCostInternal(KHE_GROUP_MONITOR gm, KHE_MONITOR m,
+  KHE_MONITOR_LINK link, KHE_COST old_cost, KHE_COST new_cost, bool check);
 
 void KheGroupMonitorAddChildMonitor(KHE_GROUP_MONITOR gm, KHE_MONITOR m)
 {
@@ -471,6 +535,7 @@ void KheGroupMonitorAddChildMonitor(KHE_GROUP_MONITOR gm, KHE_MONITOR m)
 
   /* make sure m's cost is up to date (IMPORTANT - this is a nasty bug fix!) */
   KheMonitorCost(m);
+  KheGroupMonitorCheck(gm, NULL, 0, 0);
 
   /* make sure the add would not cause a monitor cycle */
   HnAssert(!KheGroupMonitorCycle(gm, m),
@@ -497,7 +562,8 @@ void KheGroupMonitorAddChildMonitor(KHE_GROUP_MONITOR gm, KHE_MONITOR m)
 
   /* change gm's cost and add m to defects, if m has non-zero cost */
   if( KheMonitorCost(m) > 0 )
-    KheGroupMonitorChangeCost(gm, m, link, 0, KheMonitorCost(m));
+    KheGroupMonitorChangeCostInternal(gm, m, link, 0, KheMonitorCost(m), false);
+  KheGroupMonitorCheck(gm, NULL, 0, 0);
 }
 
 
@@ -515,10 +581,12 @@ void KheGroupMonitorDeleteChildMonitor(KHE_GROUP_MONITOR gm, KHE_MONITOR m)
   KHE_MONITOR_LINK link, link2;
 
   /* change gm's cost and remove m from defects, if m has non-zero cost */
+  KheGroupMonitorCheck(gm, NULL, 0, 0);
   if( !KheMonitorHasParentLink(m, gm, &link) )
     HnAbort("KheGroupMonitorDeleteChildMonitor: gm not parent of m");
   if( KheMonitorCost(m) > 0 )
-    KheGroupMonitorChangeCost(gm, m, link, KheMonitorCost(m), 0);
+    KheGroupMonitorChangeCostInternal(gm, m, link, KheMonitorCost(m), 0,
+      false);
 
   /* update gm's lower bound */
   if( KheMonitorLowerBound(m) > 0 )
@@ -538,6 +606,7 @@ void KheGroupMonitorDeleteChildMonitor(KHE_GROUP_MONITOR gm, KHE_MONITOR m)
 
   /* free link */
   KheSolnAddMonitorLinkToFreeList(gm->soln, link);
+  KheGroupMonitorCheck(gm, NULL, 0, 0);
 }
 
 
@@ -745,6 +814,7 @@ KHE_COST KheGroupMonitorCostByType(KHE_GROUP_MONITOR gm, KHE_MONITOR_TAG tag,
   int *defect_count)
 {
   KHE_COST res;  int i, dc;  KHE_MONITOR_LINK link;  KHE_MONITOR m;
+  KheGroupMonitorCheck(gm, NULL, 0, 0);
   res = 0;  *defect_count = 0;
   if( tag == KHE_GROUP_MONITOR_TAG )
     return res;
@@ -763,6 +833,7 @@ KHE_COST KheGroupMonitorCostByType(KHE_GROUP_MONITOR gm, KHE_MONITOR_TAG tag,
       *defect_count += dc;
     }
   }
+  KheGroupMonitorCheck(gm, NULL, 0, 0);
   return res;
 }
 
@@ -772,7 +843,8 @@ KHE_COST KheGroupMonitorCostByType(KHE_GROUP_MONITOR gm, KHE_MONITOR_TAG tag,
 /*  void KheGroupMonitorCostByTypeDebug(KHE_GROUP_MONITOR gm,                */
 /*    int verbosity, int indent, FILE *fp)                                   */
 /*                                                                           */
-/*  Debug print of gm's cost onto fp.                                        */
+/*  Debug print of gm's cost onto fp, although not if gm's soln is a         */
+/*  placeholder.                                                             */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -780,8 +852,9 @@ void KheGroupMonitorCostByTypeDebug(KHE_GROUP_MONITOR gm,
   int verbosity, int indent, FILE *fp)
 {
   int tag, defect_count, total_defect_count;  KHE_COST cost, total_cost;
-  if( verbosity >= 1 && indent >= 0 )
+  if(verbosity >= 1 && indent >= 0 && KheSolnType(gm->soln)==KHE_SOLN_ORDINARY)
   {
+    KheGroupMonitorCheck(gm, NULL, 0, 0);
     fprintf(fp, "%*s%-31s %9s %13s\n", indent, "", gm->sub_tag_label,
       "Defects", "Cost");
     fprintf(fp, "%*s-------------------------------------------------------\n",
@@ -801,8 +874,9 @@ void KheGroupMonitorCostByTypeDebug(KHE_GROUP_MONITOR gm,
     fprintf(fp, "%*s%-34s %6d %13.5f\n", indent, "", "Total",
       total_defect_count, KheCostShow(total_cost));
     HnAssert(total_cost == gm->cost, "KheGroupMonitorCostByTypeDebug "
-      " internal error (total_cost %.5f, gm->cost %.5f)\n",
+      "internal error (total_cost %.5f, gm->cost %.5f)\n",
       KheCostShow(total_cost), KheCostShow(gm->cost));
+    KheGroupMonitorCheck(gm, NULL, 0, 0);
   }
 }
 
@@ -897,8 +971,8 @@ void KheGroupMonitorEndTrace(KHE_GROUP_MONITOR gm, KHE_TRACE t)
 /*                                                                           */
 /*****************************************************************************/
 
-void KheGroupMonitorChangeCost(KHE_GROUP_MONITOR gm, KHE_MONITOR m,
-  KHE_MONITOR_LINK link, KHE_COST old_cost, KHE_COST new_cost)
+void KheGroupMonitorChangeCostInternal(KHE_GROUP_MONITOR gm, KHE_MONITOR m,
+  KHE_MONITOR_LINK link, KHE_COST old_cost, KHE_COST new_cost, bool check)
 {
   KHE_TRACE t;  int i;  KHE_COST delta_cost;  KHE_MONITOR_LINK link2;
   KHE_GROUP_MONITOR prnt;
@@ -947,9 +1021,28 @@ void KheGroupMonitorChangeCost(KHE_GROUP_MONITOR gm, KHE_MONITOR m,
   HaArrayForEach(gm->parent_links, link2, i)
     KheGroupMonitorChangeCost(link2->parent, (KHE_MONITOR) gm, link2,
       gm->cost - delta_cost, gm->cost);
+  if( check )
+    KheGroupMonitorCheck(gm, m, old_cost, new_cost);
   if( DEBUG2 )
     fprintf(stderr, "] KheGroupMonitorChangeCost returning\n");
 }
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheGroupMonitorChangeCost(KHE_GROUP_MONITOR gm, KHE_MONITOR m,      */
+/*    KHE_COST old_cost, KHE_COST new_cost)                                  */
+/*                                                                           */
+/*  Let gm know that its child monitor m (with the given link) has changed   */
+/*  its cost from old_cost to new_cost.                                      */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheGroupMonitorChangeCost(KHE_GROUP_MONITOR gm, KHE_MONITOR m,
+  KHE_MONITOR_LINK link, KHE_COST old_cost, KHE_COST new_cost)
+{
+  KheGroupMonitorChangeCostInternal(gm, m, link, old_cost, new_cost, true);
+}
 
 
 /*****************************************************************************/
@@ -1094,7 +1187,7 @@ void KheGroupMonitorDefectTypeDebug(KHE_GROUP_MONITOR gm,
 void KheGroupMonitorDebug(KHE_GROUP_MONITOR gm, int verbosity,
   int indent, FILE *fp)
 {
-  int i;  KHE_MONITOR_LINK link;
+  int i;  KHE_MONITOR_LINK link;  KHE_MONITOR m;  KHE_COST total_cost;
   if( verbosity >= 1 )
   {
     KheMonitorDebugWithTagBegin((KHE_MONITOR) gm,
@@ -1106,9 +1199,17 @@ void KheGroupMonitorDebug(KHE_GROUP_MONITOR gm, int verbosity,
     if( indent >= 0 && verbosity >= 2 && HaArrayCount(gm->child_links) > 0 )
     {
       fprintf(fp, "\n");
+      total_cost = 0;
       HaArrayForEach(gm->child_links, link, i)
-        KheMonitorDebug(link->child, verbosity, indent + 2, fp);
+      {
+	m = link->child;
+        KheMonitorDebug(m, verbosity, indent + 2, fp);
+	total_cost += KheMonitorCost(m);
+      }
       KheMonitorDebugEnd((KHE_MONITOR) gm, false, indent, fp);
+      HnAssert(gm->cost == total_cost,
+	"KheGroupMonitorDebug: cost (%.5f) != total child cost (%.5f)",
+        gm->cost, total_cost);
     }
     else
       KheMonitorDebugEnd((KHE_MONITOR) gm, true, indent, fp);
