@@ -704,8 +704,14 @@ KHE_LAYER KheLayerCopyPhase1(KHE_LAYER layer, HA_ARENA a)
 
 void KheLayerCopyPhase2(KHE_LAYER layer)
 {
+  KHE_NODE child_node;  int i;
   if( layer->copy != NULL )
+  {
     layer->copy = NULL;
+    KheNodeCopyPhase2(layer->parent_node);
+    HaArrayForEach(layer->child_nodes, child_node, i)
+      KheNodeCopyPhase2(child_node);
+  }
 }
 
 
