@@ -47,7 +47,7 @@ struct khe_limit_busy_times_constraint_rec {
   int				minimum;		/* minimum           */
   int				maximum;		/* maximum           */
   bool				allow_zero;		/* allow_zero        */
-  bool				limit_busy_recode;	/* recode as cluster */
+  /* bool			limit_busy_recode; */	/* recode as cluster */
   bool				limits_whole_cycle;	/* limits whole cycle*/
   ARRAY_KHE_TIME_GROUP		time_groups;		/* time groups       */
   KHE_TIME_GROUP		domain;			/* time groups union */
@@ -96,10 +96,6 @@ bool KheLimitBusyTimesConstraintMake(KHE_INSTANCE ins, char *id,
   }
   a = KheInstanceArena(ins);
   count = KheInstanceResourceTypeCount(ins);
-  /* ***
-  res = HaAlloc(a, sizeof(struct khe_limit_busy_times_constraint_rec)
-    + (count - 1) * sizeof(int));
-  *** */
   HaMake(res, a);
   KheConstraintInitCommonFields((KHE_CONSTRAINT) res,
     KHE_LIMIT_BUSY_TIMES_CONSTRAINT_TAG, ins, id, name, required, weight, cf,a);
@@ -111,7 +107,7 @@ bool KheLimitBusyTimesConstraintMake(KHE_INSTANCE ins, char *id,
   res->minimum = minimum;
   res->maximum = maximum;
   res->allow_zero = allow_zero;
-  res->limit_busy_recode = false;  /* placeholder until finalized */
+  /* res->limit_busy_recode = false; */  /* placeholder until finalized */
   res->limits_whole_cycle = false; /* placeholder until finalized */
   HaArrayInit(res->time_groups, a);
   res->domain = NULL;
@@ -280,12 +276,13 @@ bool KheLimitBusyTimesConstraintRecode(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** not recoding now
 static char *KheRecodedName(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
   KHE_TIME_GROUP tg, int tg_index, HA_ARENA a)
 {
   char *id;  KHE_CONSTRAINT cc;  int j;
 
-  /* first attempt */
+  ** first attempt **
   if( tg == NULL )
     id =  HnStringMake(a, "%s:R", c->id);
   else if( KheTimeGroupId(tg) == NULL )
@@ -293,7 +290,7 @@ static char *KheRecodedName(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
   else
     id =  HnStringMake(a, "%s:%s", c->id, KheTimeGroupId(tg));
 
-  /* further attempts */
+  ** further attempts **
   for( j = 1;  KheInstanceRetrieveConstraint(c->instance, id, &cc);  j++ )
   {
     if( tg == NULL )
@@ -305,6 +302,7 @@ static char *KheRecodedName(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
   }
   return id;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -347,12 +345,12 @@ void KheLimitBusyTimesConstraintResetResourceOfType(
 /*                                                                           */
 /*****************************************************************************/
 
-void KheLimitBusyTimesConstraintFinalize(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
-  bool limit_busy_recode)
+void KheLimitBusyTimesConstraintFinalize(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c)
+  /* bool limit_busy_recode) */
 {
-  int tg_count, max_index, base_index, i, j, time_count, index;  HA_ARENA a;
-  KHE_TIME_GROUP tg;  SSET ss;  KHE_RESOURCE_GROUP rg;  KHE_RESOURCE r;
-  KHE_CLUSTER_BUSY_TIMES_CONSTRAINT cbtc;  char *id, *name;
+  int tg_count, max_index, base_index, i, time_count, index;  HA_ARENA a;
+  KHE_TIME_GROUP tg;  SSET ss;  /* KHE_RESOURCE_GROUP rg;  KHE_RESOURCE r; */
+  /* KHE_CLUSTER_BUSY_TIMES_CONSTRAINT cbtc;  char *id, *name; */
 
   /* build c->domain */
   if( DEBUG1 )
@@ -362,7 +360,7 @@ void KheLimitBusyTimesConstraintFinalize(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
   HaArrayForEach(c->time_groups, tg, i)
     SSetUnion(ss, *KheTimeGroupTimeSet(tg));
   c->domain = KheTimeGroupMakeAndFinalize(c->instance,
-    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, NULL, false);
+    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, /* NULL, */ false, a);
   /* ***
   c->domain = KheTimeGroup MakeInternal(c->instance,
     ** KHE_TIME_GROUP_TYPE_CONSTRUCTED, ** NULL, KHE_TIME_GROUP_KIND_AUTO,
@@ -420,6 +418,7 @@ void KheLimitBusyTimesConstraintFinalize(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
   }
 
   /* recode, if requested and c has a non-trivial minimum limit */
+  /* *** this is not exact when the cost function is not linear
   if( limit_busy_recode && c->minimum > 0 )
   {
     c->limit_busy_recode = true;
@@ -431,26 +430,26 @@ void KheLimitBusyTimesConstraintFinalize(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
     }
     HaArrayForEach(c->time_groups, tg, i)
     {
-      /* make a unique id for the recoded constraint, and a name */
+      ** make a unique id for the recoded constraint, and a name **
       if( HaArrayCount(c->time_groups) == 1 )
         id = KheRecodedName(c, NULL, 0, a);
       else
         id = KheRecodedName(c, tg, i, a);
       name = (c->name==NULL ? NULL : HnStringMake(a, "%s (recoded)", c->name));
 
-      /* make the constraint and mark it as a recoding */
+      ** make the constraint and mark it as a recoding **
       if( !KheClusterBusyTimesConstraintMakeInternal(c->instance, id, name,
 	    c->required, c->weight, c->cost_function, c->applies_to_tg,
 	    c->minimum, c->maximum, c->allow_zero, true, &cbtc) )
 	HnAbort("KheLimitBusyTimesConstraintFinalize internal error 1");
-      /* KheClusterBusyTimesConstraintSetLimitBusyRecode(cbtc); */
+      ** KheClusterBusyTimesConstraintSetLimitBusyRecode(cbtc); **
 
-      /* add one positive time group from each time in tg */
+      ** add one positive time group from each time in tg **
       for( j = 0;  j < KheTimeGroupTimeCount(tg);  j++ )
 	KheClusterBusyTimesConstraintAddTimeGroup(cbtc,
 	  KheTimeSingletonTimeGroup(KheTimeGroupTime(tg, j)), KHE_POSITIVE);
 
-      /* add resource groups and resources */
+      ** add resource groups and resources **
       HaArrayForEach(c->resource_groups, rg, j)
 	KheClusterBusyTimesConstraintAddResourceGroup(cbtc, rg);
       HaArrayForEach(c->resources, r, j)
@@ -461,6 +460,7 @@ void KheLimitBusyTimesConstraintFinalize(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
     if( DEBUG2 )
       fprintf(stderr, "] end KheLimitBusyTimesConstraintFinalize recoding\n");
   }
+  *** */
   if( DEBUG1 )
     fprintf(stderr, "] KheLimitBusyTimesConstraintFinalize\n");
 }
@@ -775,8 +775,10 @@ void KheLimitBusyTimesConstraintMakeAndAttachMonitors(
   KHE_LIMIT_BUSY_TIMES_CONSTRAINT c, KHE_SOLN soln)
 {
   int i, j;  KHE_RESOURCE_GROUP rg;  KHE_RESOURCE r;
+  /* ***
   if( !c->limit_busy_recode )
   {
+  *** */
     HaArrayForEach(c->resource_groups, rg, i)
     {
       for( j = 0;  j < KheResourceGroupResourceCount(rg);  j++ )
@@ -787,7 +789,9 @@ void KheLimitBusyTimesConstraintMakeAndAttachMonitors(
     }
     HaArrayForEach(c->resources, r, i)
       DoLimitBusyTimesConstraintMonitorsForResource(soln, c, r);
+  /* ***
   }
+  *** */
 }
 
 
@@ -979,10 +983,10 @@ void KheLimitBusyTimesConstraintDebug(KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
       tg_id = (c->applies_to_tg == NULL ? "null" :
         KheTimeGroupId(c->applies_to_tg) == NULL ? "-" :
         KheTimeGroupId(c->applies_to_tg));
-      fprintf(fp, "%*s[ applies_to %s, %d-%d, allow_zero %s, recode %s\n",
+      fprintf(fp, "%*s[ applies_to %s, %d-%d, allow_zero %s\n",
 	indent, "", tg_id, c->minimum, c->maximum,
-	c->allow_zero ? "true" : "false",
-	c->limit_busy_recode ? "true" : "false");
+	c->allow_zero ? "true" : "false");
+	/* c->limit_busy_recode ? "true" : "false"); */
       HaArrayForEach(c->resource_groups, rg, i)
         fprintf(fp, "%*s  %s\n", indent, "",
 	  KheResourceGroupId(rg) != NULL ? KheResourceGroupId(rg) : "-");
