@@ -100,17 +100,17 @@ static void KheLimitBusyTimesMonitorSetLowerBound(
   KHE_LIMIT_BUSY_TIMES_MONITOR m)
 {
   KHE_RESOURCE r;  int events_durn, min_in_domain, max_in_domain;
-  KHE_TIME_GROUP domain;  KHE_INSTANCE ins;  KHE_CONSTRAINT mc;
+  KHE_TIME_GROUP domain;  KHE_INSTANCE ins;  /* KHE_CONSTRAINT mc; */
 
   /* if r does not have a highly weighted avoid clashes constraint, quit */
   r = KheLimitBusyTimesMonitorResource(m);
-  mc = (KHE_CONSTRAINT) m->constraint;
-  if( !KheResourceHasAvoidClashesConstraint(r, KheConstraintCombinedWeight(mc)))
+  /* mc = (KHE_CONSTRAINT) m->constraint; */
+  if( !KheResourceHasAvoidClashesConstraint(r, m->combined_weight))
     return;  /* no highly weighted avoid clashes constraint */
 
   /* find the duration of highly weighted preassigned events */
-  events_durn = KheResourcePreassignedEventsDuration(r,
-    KheConstraintCombinedWeight(mc));
+  events_durn = KheResourcePreassignedEventsDuration(r, m->combined_weight);
+    /* Khe ConstraintCombinedWeight(mc)); */
 
   /* min_in_domain = the min number of domain times occupied by workload */
   ins = KheResourceInstance(r);
@@ -123,7 +123,7 @@ static void KheLimitBusyTimesMonitorSetLowerBound(
     KheLimitBusyTimesConstraintTimeGroupCount(m->constraint);
   if( min_in_domain > max_in_domain )
   {
-    m->lower_bound = KheConstraintCost((KHE_CONSTRAINT) m->constraint,
+    m->lower_bound = KheMonitorDevToCost((KHE_MONITOR) m,
       min_in_domain - max_in_domain);
     if( DEBUG2 )
       fprintf(stderr, "  setting limit busy times lower bound of %s to %.5f\n",
@@ -157,7 +157,9 @@ KHE_LIMIT_BUSY_TIMES_MONITOR KheLimitBusyTimesMonitorMake(
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_LIMIT_BUSY_TIMES_MONITOR_TAG);
+    KHE_LIMIT_BUSY_TIMES_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->resource_in_soln = rs;
   res->constraint = c;
   res->deviation = 0;
@@ -214,7 +216,6 @@ KHE_LIMIT_BUSY_TIMES_MONITOR KheLimitBusyTimesMonitorCopyPhase1(
     copy->resource_in_soln = KheResourceInSolnCopyPhase1(m->resource_in_soln,a);
     copy->constraint = m->constraint;
     copy->deviation = m->deviation;
-    /* copy->new_deviation = m->new_deviation; */
     copy->offset = m->offset;
     copy->minimum = m->minimum;
     copy->maximum = m->maximum;
@@ -230,11 +231,6 @@ KHE_LIMIT_BUSY_TIMES_MONITOR KheLimitBusyTimesMonitorCopyPhase1(
 	KheMonitoredTimeGroupCopyPhase1(mtg, a));
     copy->first_time = m->first_time;
     copy->last_time = m->last_time;
-    /* ***
-    MArrayInit(copy->defective_busy_counts);
-    HaArrayForEach(m->defective_busy_counts, busy_count, i)
-      HaArrayAddLast(copy->defective_busy_counts, busy_count);
-    *** */
     copy->copy = NULL;
   }
   return m->copy;
@@ -265,34 +261,6 @@ void KheLimitBusyTimesMonitorCopyPhase2(KHE_LIMIT_BUSY_TIMES_MONITOR m)
 }
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheLimitBusyTimesMonitorDelete(KHE_LIMIT_BUSY_TIMES_MONITOR m)      */
-/*                                                                           */
-/*  Delete m.                                                                */
-/*                                                                           */
-/*****************************************************************************/
-
-/* ***
-void KheLimitBusyTimesMonitorDelete(KHE_LIMIT_BUSY_TIMES_MONITOR m)
-{
-  if( DEBUG1_R(m->resource_in_soln) )
-  {
-    fprintf(stderr, "  KheLimitBusyTimesMonitorDelete(m) ");
-    KheLimitBusyTimesMonitorDebugCost(m, stderr);
-  }
-  if( m->attached )
-    KheLimitBusyTimesMonitorDetachFromSoln(m);
-  KheMonitorDeleteAllParentMonitors((KHE_MONITOR) m);
-  KheResourceInSolnDeleteMonitor(m->resource_in_soln, (KHE_MONITOR) m);
-  KheSolnDeleteMonitor(m->soln, (KHE_MONITOR) m);
-  MArrayFree(m->monitored_time_groups);
-  MArrayFree(m->defective_time_groups);
-  MFree(m);
-}
-*** */
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  KHE_RESOURCE_IN_SOLN KheLimitBusyTimesMonitorResourceInSoln(             */
@@ -382,9 +350,10 @@ int KheLimitBusyTimesMonitorTimeGroupCount(
 KHE_TIME_GROUP KheLimitBusyTimesMonitorTimeGroup(
   KHE_LIMIT_BUSY_TIMES_MONITOR m, int i, int *busy_count)
 {
-  KHE_MONITORED_TIME_GROUP mtg;
+  KHE_MONITORED_TIME_GROUP mtg;  KHE_BUSY_AND_IDLE busy_and_idle;
   mtg = HaArray(m->monitored_time_groups, i);
-  *busy_count = KheMonitoredTimeGroupBusyCount(mtg);
+  busy_and_idle = KheMonitoredTimeGroupBusyAndIdle(mtg);
+  *busy_count = busy_and_idle->busy_count;
   return KheMonitoredTimeGroupTimeGroup(mtg);
 }
 
@@ -398,6 +367,7 @@ KHE_TIME_GROUP KheLimitBusyTimesMonitorTimeGroup(
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** replaced by KheLimitBusyTimesMonitorSweepTimeRange
 void KheLimitBusyTimesMonitorRange(KHE_LIMIT_BUSY_TIMES_MONITOR m,
   KHE_TIME *first_time, KHE_TIME *last_time)
 {
@@ -431,6 +401,7 @@ void KheLimitBusyTimesMonitorRange(KHE_LIMIT_BUSY_TIMES_MONITOR m,
   *first_time = m->first_time;
   *last_time = m->last_time;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -491,6 +462,95 @@ void KheLimitBusyTimesMonitorDetachFromSoln(KHE_LIMIT_BUSY_TIMES_MONITOR m)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLimitBusyTimesMonitorTimeRange(KHE_LIMIT_BUSY_TIMES_MONITOR m,   */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Return the time range of m.                                              */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLimitBusyTimesMonitorTimeRange(KHE_LIMIT_BUSY_TIMES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  int i, first_index, last_index, index, count;  KHE_TIME_GROUP tg;
+  KHE_INSTANCE ins;  KHE_MONITORED_TIME_GROUP mtg;
+  if( m->first_time == NULL )
+  {
+    first_index = INT_MAX;
+    last_index = 0;
+    HaArrayForEach(m->monitored_time_groups, mtg, i)
+    {
+      tg = KheMonitoredTimeGroupTimeGroup(mtg);
+      count = KheTimeGroupTimeCount(tg);
+      if( count > 0 )
+      {
+	index = KheTimeIndex(KheTimeGroupTime(tg, 0));
+	if( index < first_index )
+          first_index = index;
+	index = KheTimeIndex(KheTimeGroupTime(tg, count - 1));
+	if( index > last_index )
+          last_index = index;
+      }
+    }
+    if( first_index < INT_MAX )
+    {
+      ins = KheSolnInstance(m->soln);
+      m->first_time = KheInstanceTime(ins, first_index);
+      m->last_time = KheInstanceTime(ins, last_index);
+    }
+  }
+  *first_time = m->first_time;
+  *last_time = m->last_time;
+  return *first_time != NULL;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheLimitBusyTimesMonitorSetSweepTime(                               */
+/*    KHE_LIMIT_BUSY_TIMES_MONITOR m, KHE_TIME t)                            */
+/*                                                                           */
+/*  Set the sweep time of m.                                                 */
+/*                                                                           */
+/*  Implementation note.  This simply passes t on to each monitored time     */
+/*  group, which calls back if its value (including open count) changes.     */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheLimitBusyTimesMonitorSetSweepTime(
+  KHE_LIMIT_BUSY_TIMES_MONITOR m, KHE_TIME t)
+{
+  KHE_MONITORED_TIME_GROUP mtg;  int i;
+  HaArrayForEach(m->monitored_time_groups, mtg, i)
+    KheMonitoredTimeGroupSetSweepTime(mtg, t);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLimitBusyTimesMonitorSweepTimeRange(                             */
+/*    KHE_LIMIT_BUSY_TIMES_MONITOR m, KHE_TIME *first_time,                  */
+/*    KHE_TIME *last_time)                                                   */
+/*                                                                           */
+/*  Return the sweep time range of m.                                        */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLimitBusyTimesMonitorSweepTimeRange(
+  KHE_LIMIT_BUSY_TIMES_MONITOR m, KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return KheLimitBusyTimesMonitorTimeRange(m, first_time, last_time);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "ceiling"                                                      */
@@ -545,21 +605,21 @@ int KheLimitBusyTimesMonitorCeiling(KHE_LIMIT_BUSY_TIMES_MONITOR m)
 /*****************************************************************************/
 /*                                                                           */
 /*  int KheLimitBusyTimesMonitorDev(KHE_LIMIT_BUSY_TIMES_MONITOR m,          */
-/*    int busy_count)                                                        */
+/*    int busy_count, int open_count)                                        */
 /*                                                                           */
-/*  Work out the deviations caused by this busy_count.                       */
+/*  Work out the deviations caused by this busy_count and open_count.        */
 /*                                                                           */
 /*****************************************************************************/
 
 static int KheLimitBusyTimesMonitorDev(KHE_LIMIT_BUSY_TIMES_MONITOR m,
-  int busy_count)
+  int busy_count, int open_count)
 {
   if( (m->allow_zero && busy_count == 0) || busy_count > m->ceiling )
     return 0;
-  else if( busy_count < m->minimum )
-    return m->minimum - busy_count;
   else if( busy_count > m->maximum )
     return busy_count - m->maximum;
+  else if( busy_count + open_count < m->minimum )
+    return m->minimum - (busy_count + open_count);
   else
     return 0;
 }
@@ -567,85 +627,83 @@ static int KheLimitBusyTimesMonitorDev(KHE_LIMIT_BUSY_TIMES_MONITOR m,
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheLimitBusyTimesMonitorFlush(KHE_LIMIT_BUSY_TIMES_MONITOR m)       */
-/*                                                                           */
-/*  Flush m, assuming that there has been a change in deviation.             */
-/*                                                                           */
-/*****************************************************************************/
-
-/* ***
-static void KheLimitBusyTimesMonitorFlush(KHE_LIMIT_BUSY_TIMES_MONITOR m)
-{
-  if( DEBUG1_R(m->resource_in_soln) )
-  {
-    fprintf(stderr, "[ KheLimitBusyTimesMonitorFlush(m), init ");
-    KheLimitBusyTimesMonitorDebugCost(m, stderr);
-  }
-  KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->new_deviation));
-  m->deviation = m->new_deviation;
-  if( DEBUG1_R(m->resource_in_soln) )
-  {
-    fprintf(stderr, "] KheLimitBusyTimesMonitorFlush(m), final ");
-    KheLimitBusyTimesMonitorDebugCost(m, stderr);
-  }
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheLimitBusyTimesMonitorAddMonitoredTimeGroup(                      */
+/*  void KheLimitBusyTimesMonitorAddDefective(                               */
 /*    KHE_LIMIT_BUSY_TIMES_MONITOR m, KHE_MONITORED_TIME_GROUP mtg)          */
 /*                                                                           */
-/*  Add mtg to m.                                                            */
+/*  Add mtg to m's list of defective time groups, in sorted order.           */
 /*                                                                           */
 /*****************************************************************************/
 
-/* ***
-void KheLimitBusyTimesMonitorAddMonitoredTimeGroup(
+static void KheLimitBusyTimesMonitorAddDefective(
   KHE_LIMIT_BUSY_TIMES_MONITOR m, KHE_MONITORED_TIME_GROUP mtg)
 {
-  HaArrayAddLast(m->monitored_time_groups, mtg);
+  KHE_MONITORED_TIME_GROUP mtg2;  int i;  KHE_TIME_GROUP tg, tg2;
+  tg = KheMonitoredTimeGroupTimeGroup(mtg);
+  HaArrayAddLast(m->defective_time_groups, NULL);
+  for( i = HaArrayCount(m->defective_time_groups) - 2;  i >= 0;  i-- )
+  {
+    mtg2 = HaArray(m->defective_time_groups, i);
+    tg2 = KheMonitoredTimeGroupTimeGroup(mtg2);
+    if( KheTimeGroupTypedCmp(tg2, tg) )
+      break;
+    HaArrayPut(m->defective_time_groups, i + 1, mtg2);
+  }
+  HaArrayPut(m->defective_time_groups, i + 1, mtg);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheLimitBusyTimesMonitorDeleteDefective(                            */
+/*    KHE_LIMIT_BUSY_TIMES_MONITOR m, KHE_MONITORED_TIME_GROUP mtg)          */
+/*                                                                           */
+/*  Delete mtg from m's list of defective time groups.  It must be there.    */
+/*                                                                           */
+/*****************************************************************************/
+
+static void KheLimitBusyTimesMonitorDeleteDefective(
+  KHE_LIMIT_BUSY_TIMES_MONITOR m, KHE_MONITORED_TIME_GROUP mtg)
+{
+  int pos;
+  if( !HaArrayContains(m->defective_time_groups, mtg, &pos) )
+    HnAbort("KheLimitBusyTimesMonitorDeleteDefective internal error");
+  HaArrayDeleteAndShift(m->defective_time_groups, pos);
 }
-*** */
 
 
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheLimitBusyTimesMonitorAddBusyAndIdle(                             */
 /*    KHE_LIMIT_BUSY_TIMES_MONITOR m, int index,                             */
-/*    int busy_count, int idle_count, float workload)                        */
+/*    KHE_BUSY_AND_IDLE busy_and_idle)                                       */
 /*                                                                           */
-/*  Add a busy value for one monitored time group.                           */
+/*  Add a busy and idle value for one monitored time group.                  */
 /*                                                                           */
 /*****************************************************************************/
 
 void KheLimitBusyTimesMonitorAddBusyAndIdle(
   KHE_LIMIT_BUSY_TIMES_MONITOR m, int index,
-  int busy_count, int idle_count, float workload)
+  KHE_BUSY_AND_IDLE busy_and_idle)
 {
-  int dev /* , pos */;  KHE_MONITORED_TIME_GROUP mtg;
+  int dev;  KHE_MONITORED_TIME_GROUP mtg;
   if( DEBUG1_R(m->resource_in_soln) )
   {
     fprintf(stderr, "[ KheLimitBusyTimesMonitorAddBusyAndIdle(m, %d, %d), ",
-      busy_count, idle_count);
+      busy_and_idle->busy_count, busy_and_idle->idle_count);
     KheLimitBusyTimesMonitorDebugCost(m, stderr);
   }
-  dev = KheLimitBusyTimesMonitorDev(m, busy_count);
+  dev = KheLimitBusyTimesMonitorDev(m, busy_and_idle->busy_count,
+    busy_and_idle->open_count);
   if( dev != 0 )
   {
     HnAssert(index >= 0 && index < HaArrayCount(m->monitored_time_groups),
       "KheLimitBusyTimesMonitorAddBusyAndIdle internal error 1");
     mtg = HaArray(m->monitored_time_groups, index);
-    /* *** why not have the same time group twice?
-    HnAssert(!HaArrayContains(m->defective_time_groups, mtg, &pos),
-      "KheLimitBusyTimesMonitorAddBusyAndIdle internal error 2");
-    *** */
-    HaArrayAddLast(m->defective_time_groups, mtg);
+    KheLimitBusyTimesMonitorAddDefective(m, mtg);
+    /* HaArrayAddLast(m->defective_time_groups, mtg); */
     m->deviation += dev;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
   if( DEBUG1_R(m->resource_in_soln) )
   {
@@ -659,7 +717,7 @@ void KheLimitBusyTimesMonitorAddBusyAndIdle(
 /*                                                                           */
 /*  void KheLimitBusyTimesMonitorDeleteBusyAndIdle(                          */
 /*    KHE_LIMIT_BUSY_TIMES_MONITOR m, int index,                             */
-/*    int busy_count, int idle_count, float workload)                        */
+/*    KHE_BUSY_AND_IDLE busy_and_idle)                                       */
 /*                                                                           */
 /*  Remove the busy value of one monitored time group.                       */
 /*                                                                           */
@@ -667,30 +725,31 @@ void KheLimitBusyTimesMonitorAddBusyAndIdle(
 
 void KheLimitBusyTimesMonitorDeleteBusyAndIdle(
   KHE_LIMIT_BUSY_TIMES_MONITOR m, int index,
-  int busy_count, int idle_count, float workload)
+  KHE_BUSY_AND_IDLE busy_and_idle)
 {
-  int dev, pos;  KHE_MONITORED_TIME_GROUP mtg;
+  int dev;  KHE_MONITORED_TIME_GROUP mtg;
   if( DEBUG1_R(m->resource_in_soln) )
   {
     fprintf(stderr, "[ KheLimitBusyTimesMonitorDeleteBusyAndIdle(m, %d, %d) ",
-      busy_count, idle_count);
+      busy_and_idle->busy_count, busy_and_idle->idle_count);
     KheLimitBusyTimesMonitorDebugCost(m, stderr);
   }
-  dev = KheLimitBusyTimesMonitorDev(m, busy_count);
+  dev = KheLimitBusyTimesMonitorDev(m, busy_and_idle->busy_count,
+    busy_and_idle->open_count);
   if( dev != 0 )
   {
     HnAssert(index >= 0 && index < HaArrayCount(m->monitored_time_groups),
       "KheLimitBusyTimesMonitorDeleteBusyAndIdle internal error 1");
     mtg = HaArray(m->monitored_time_groups, index);
+    KheLimitBusyTimesMonitorDeleteDefective(m, mtg);
+    /* ***
     HnAssert(HaArrayContains(m->defective_time_groups, mtg, &pos),
       "KheLimitBusyTimesMonitorDeleteBusyAndIdle internal error 2");
     HaArrayDeleteAndShift(m->defective_time_groups, pos);
-    /* tg = KheMonitoredTimeGroupTimeGroup(mtg); */
-    /* HaArrayDeleteAndShift(m->defective_busy_counts, pos); */
-    /* KheLimitBusyTimesMonitorFlush(m); */
+    *** */
     m->deviation -= dev;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
   if( DEBUG1_R(m->resource_in_soln) )
   {
@@ -704,9 +763,8 @@ void KheLimitBusyTimesMonitorDeleteBusyAndIdle(
 /*                                                                           */
 /*  void KheLimitBusyTimesMonitorChangeBusyAndIdle(                          */
 /*    KHE_LIMIT_BUSY_TIMES_MONITOR m, int index,                             */
-/*    int old_busy_count, int new_busy_count,                                */
-/*    int old_idle_count, int new_idle_count,                                */
-/*    float old_workload, float new_workload)                                */
+/*    KHE_BUSY_AND_IDLE old_busy_and_idle,                                   */
+/*    KHE_BUSY_AND_IDLE new_busy_and_idle)                                   */
 /*                                                                           */
 /*  Change the idle value of one monitored time group.                       */
 /*                                                                           */
@@ -717,20 +775,22 @@ void KheLimitBusyTimesMonitorDeleteBusyAndIdle(
 
 void KheLimitBusyTimesMonitorChangeBusyAndIdle(
   KHE_LIMIT_BUSY_TIMES_MONITOR m, int index,
-  int old_busy_count, int new_busy_count,
-  int old_idle_count, int new_idle_count,
-  float old_workload, float new_workload)
+  KHE_BUSY_AND_IDLE old_busy_and_idle,
+  KHE_BUSY_AND_IDLE new_busy_and_idle)
 {
   int old_dev, new_dev, pos;  KHE_MONITORED_TIME_GROUP mtg;
   if( DEBUG1_R(m->resource_in_soln) )
   {
     fprintf(stderr, "[ %s(m, %d, %d, %d, %d), ",
-      "KheLimitBusyTimesMonitorChangeBusyAndIdle", old_busy_count,
-      new_busy_count, old_idle_count, new_idle_count);
+      "KheLimitBusyTimesMonitorChangeBusyAndIdle",
+      old_busy_and_idle->busy_count, new_busy_and_idle->busy_count,
+      old_busy_and_idle->idle_count, new_busy_and_idle->idle_count);
     KheLimitBusyTimesMonitorDebugCost(m, stderr);
   }
-  old_dev = KheLimitBusyTimesMonitorDev(m, old_busy_count);
-  new_dev = KheLimitBusyTimesMonitorDev(m, new_busy_count);
+  old_dev = KheLimitBusyTimesMonitorDev(m, old_busy_and_idle->busy_count,
+    old_busy_and_idle->open_count);
+  new_dev = KheLimitBusyTimesMonitorDev(m, new_busy_and_idle->busy_count,
+    new_busy_and_idle->open_count);
   if( old_dev != new_dev )
   {
     HnAssert(index >= 0 && index < HaArrayCount(m->monitored_time_groups),
@@ -740,13 +800,17 @@ void KheLimitBusyTimesMonitorChangeBusyAndIdle(
     {
       HnAssert(!HaArrayContains(m->defective_time_groups, mtg, &pos),
 	"KheLimitBusyTimesMonitorChangeBusyAndIdle internal error 2");
-      HaArrayAddLast(m->defective_time_groups, mtg);
+      KheLimitBusyTimesMonitorAddDefective(m, mtg);
+      /* HaArrayAddLast(m->defective_time_groups, mtg); */
     }
     else if( new_dev == 0 )
     {
+      KheLimitBusyTimesMonitorDeleteDefective(m, mtg);
+      /* ***
       HnAssert(HaArrayContains(m->defective_time_groups, mtg, &pos),
 	"KheLimitBusyTimesMonitorChangeBusyAndIdle internal error 3");
       HaArrayDeleteAndShift(m->defective_time_groups, pos);
+      *** */
     }
     else
     {
@@ -755,7 +819,7 @@ void KheLimitBusyTimesMonitorChangeBusyAndIdle(
     }
     m->deviation += (new_dev - old_dev);
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
   if( DEBUG1_R(m->resource_in_soln) )
   {
@@ -793,12 +857,15 @@ int KheLimitBusyTimesMonitorDefectiveTimeGroupCount(
 
 void KheLimitBusyTimesMonitorDefectiveTimeGroup(
   KHE_LIMIT_BUSY_TIMES_MONITOR m, int i, KHE_TIME_GROUP *tg,
-  int *busy_count, int *minimum, int *maximum, bool *allow_zero)
+  int *busy_count, int *open_count, int *minimum, int *maximum,
+  bool *allow_zero)
 {
-  KHE_MONITORED_TIME_GROUP mtg;
+  KHE_MONITORED_TIME_GROUP mtg;  KHE_BUSY_AND_IDLE busy_and_idle;
   mtg = HaArray(m->defective_time_groups, i);
   *tg = KheMonitoredTimeGroupTimeGroup(mtg);
-  *busy_count = KheMonitoredTimeGroupBusyCount(mtg);
+  busy_and_idle = KheMonitoredTimeGroupBusyAndIdle(mtg);
+  *busy_count = busy_and_idle->busy_count;
+  *open_count = busy_and_idle->open_count;
   *minimum = m->minimum;
   *maximum = m->maximum;
   *allow_zero = m->allow_zero;
@@ -837,19 +904,26 @@ int KheLimitBusyTimesMonitorDeviation(KHE_LIMIT_BUSY_TIMES_MONITOR m)
 static void KheLimitBusyTimesMonitorDeviationShowOneTimeGroup(
   KHE_LIMIT_BUSY_TIMES_MONITOR m, int i, HA_ARRAY_NCHAR *ac)
 {
-  int busy_count, minimum, maximum;  KHE_TIME_GROUP tg;  char *name;
-  bool allow_zero;
+  int busy_count, open_count, minimum, maximum;  KHE_TIME_GROUP tg;
+  char *name;  bool allow_zero;
   KheLimitBusyTimesMonitorDefectiveTimeGroup(m, i, &tg, &busy_count,
-    &minimum, &maximum, &allow_zero);
+    &open_count, &minimum, &maximum, &allow_zero);
   name = KheTimeGroupName(tg);
-  if( busy_count < minimum )
-    HnStringAdd(ac, "%d - %d %s", minimum, busy_count,
-      name != NULL ? name : KheTimeGroupTimeCount(tg) == 1 ?
-      KheTimeName(KheTimeGroupTime(tg, 0)) : "?");
-  else if( busy_count > maximum )
+  if( busy_count > maximum )
     HnStringAdd(ac, "%d %s - %d", busy_count,
       name != NULL ? name : KheTimeGroupTimeCount(tg) == 1 ?
       KheTimeName(KheTimeGroupTime(tg, 0)) : "?", maximum);
+  else if( busy_count + open_count < minimum )
+  {
+    if( open_count == 0 )
+      HnStringAdd(ac, "%d - %d %s", minimum, busy_count,
+	name != NULL ? name : KheTimeGroupTimeCount(tg) == 1 ?
+	KheTimeName(KheTimeGroupTime(tg, 0)) : "?");
+    else
+      HnStringAdd(ac, "%d - (%d busy + %d open) %s", minimum, busy_count,
+	open_count, name != NULL ? name : KheTimeGroupTimeCount(tg) == 1 ?
+	KheTimeName(KheTimeGroupTime(tg, 0)) : "?");
+  }
 }
 
 
