@@ -127,7 +127,9 @@ KHE_LIMIT_IDLE_TIMES_MONITOR KheLimitIdleTimesMonitorMake(
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_LIMIT_IDLE_TIMES_MONITOR_TAG);
+    KHE_LIMIT_IDLE_TIMES_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->resource_in_soln = rs;
   rtm = KheResourceInSolnTimetableMonitor(rs);
   HaArrayInit(res->monitored_time_groups, a);
@@ -299,7 +301,7 @@ void KheLimitIdleTimesMonitorAttachToSoln(KHE_LIMIT_IDLE_TIMES_MONITOR m)
   m->attached = true;
   if( m->minimum > 0 )
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->minimum));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->minimum));
   HaArrayForEach(m->monitored_time_groups, mtg, i)
     KheMonitoredTimeGroupAttachMonitor(mtg, (KHE_MONITOR) m, i);
   /* KheResourceInSolnAttachMonitor(m->resource_in_soln, (KHE_MONITOR) m); */
@@ -327,6 +329,77 @@ void KheLimitIdleTimesMonitorDetachFromSoln(KHE_LIMIT_IDLE_TIMES_MONITOR m)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLimitIdleTimesMonitorTimeRange(KHE_LIMIT_IDLE_TIMES_MONITOR m,   */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLimitIdleTimesMonitorTimeRange(KHE_LIMIT_IDLE_TIMES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_LIMIT_IDLE_TIMES_CONSTRAINT c;  int i;  KHE_TIME_GROUP tg;
+  KHE_TIME ft, lt;
+  *first_time = *last_time = NULL;
+  c = KheLimitIdleTimesMonitorConstraint(m);
+  for( i = 0;  i < KheLimitIdleTimesConstraintTimeGroupCount(c);  i++ )
+  {
+    tg = KheLimitIdleTimesConstraintTimeGroup(c, i);
+    if( KheTimeGroupTimeCount(tg) > 0 )
+    {
+      ft = KheTimeGroupTime(tg, 0);
+      lt = KheTimeGroupTime(tg, KheTimeGroupTimeCount(tg) - 1);
+      if( *first_time == NULL || KheTimeIndex(ft) < KheTimeIndex(*first_time) )
+	*first_time = ft;
+      if( *last_time == NULL || KheTimeIndex(lt) > KheTimeIndex(*last_time) )
+	*last_time = lt;
+    }
+  }
+  return *first_time != NULL;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheLimitIdleTimesMonitorSetSweepTime(KHE_LIMIT_IDLE_TIMES_MONITOR m,*/
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheLimitIdleTimesMonitorSetSweepTime(KHE_LIMIT_IDLE_TIMES_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLimitIdleTimesMonitorSweepTimeRange(                             */
+/*    KHE_LIMIT_IDLE_TIMES_MONITOR m,                                        */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for m.                                */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLimitIdleTimesMonitorSweepTimeRange(KHE_LIMIT_IDLE_TIMES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return KheLimitIdleTimesMonitorTimeRange(m, first_time, last_time);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -372,7 +445,7 @@ static void KheLimitIdleTimesMonitorFlush(KHE_LIMIT_IDLE_TIMES_MONITOR m,
     new_dev = KheLimitIdleTimesMonitorDev(m, m->idle_count + delta);
     if( old_dev != new_dev )
       KheMonitorChangeCost((KHE_MONITOR) m,
-        KheConstraintCost((KHE_CONSTRAINT) m->constraint, new_dev));
+        KheMonitorDevToCost((KHE_MONITOR) m, new_dev));
     m->idle_count += delta;
     if( DEBUG1(m) )
       fprintf(stderr, "  KheLimitIdleTimesMonitorFlush %s, %d = %d + %d\n",
@@ -386,7 +459,7 @@ static void KheLimitIdleTimesMonitorFlush(KHE_LIMIT_IDLE_TIMES_MONITOR m,
 /*                                                                           */
 /*  void KheLimitIdleTimesMonitorAddBusyAndIdle(                             */
 /*    KHE_LIMIT_IDLE_TIMES_MONITOR m, int index,                             */
-/*    int busy_count, int idle_count, float workload)                        */
+/*    KHE_BUSY_AND_IDLE busy_and_idle)                                       */
 /*                                                                           */
 /*  Add an idle value for one monitored time group.                          */
 /*                                                                           */
@@ -394,13 +467,13 @@ static void KheLimitIdleTimesMonitorFlush(KHE_LIMIT_IDLE_TIMES_MONITOR m,
 
 void KheLimitIdleTimesMonitorAddBusyAndIdle(
   KHE_LIMIT_IDLE_TIMES_MONITOR m, int index,
-  int busy_count, int idle_count, float workload)
+  KHE_BUSY_AND_IDLE busy_and_idle)
 {
   if( DEBUG1(m) )
     fprintf(stderr,
       "  KheLimitIdleTimesMonitorAddBusyAndIdle(%s, index %d, %d)\n",
-      KheMonitorId((KHE_MONITOR) m), index, idle_count);
-  KheLimitIdleTimesMonitorFlush(m, idle_count);
+      KheMonitorId((KHE_MONITOR) m), index, busy_and_idle->idle_count);
+  KheLimitIdleTimesMonitorFlush(m, busy_and_idle->idle_count);
 }
 
 
@@ -416,13 +489,13 @@ void KheLimitIdleTimesMonitorAddBusyAndIdle(
 
 void KheLimitIdleTimesMonitorDeleteBusyAndIdle(
   KHE_LIMIT_IDLE_TIMES_MONITOR m, int index,
-  int busy_count, int idle_count, float workload)
+  KHE_BUSY_AND_IDLE busy_and_idle)
 {
   if( DEBUG1(m) )
     fprintf(stderr,
       "  KheLimitIdleTimesMonitorDeleteBusyAndIdle(%s, index %d, %d)\n",
-      KheMonitorId((KHE_MONITOR) m), index, idle_count);
-  KheLimitIdleTimesMonitorFlush(m, - idle_count);
+      KheMonitorId((KHE_MONITOR) m), index, busy_and_idle->idle_count);
+  KheLimitIdleTimesMonitorFlush(m, - busy_and_idle->idle_count);
 }
 
 
@@ -430,9 +503,8 @@ void KheLimitIdleTimesMonitorDeleteBusyAndIdle(
 /*                                                                           */
 /*  void KheLimitIdleTimesMonitorChangeBusyAndIdle(                          */
 /*    KHE_LIMIT_IDLE_TIMES_MONITOR m, int index,                             */
-/*    int old_busy_count, int new_busy_count,                                */
-/*    int old_idle_count, int new_idle_count,                                */
-/*    float old_workload, float new_workload)                                */
+/*    KHE_BUSY_AND_IDLE old_busy_and_idle,                                   */
+/*    KHE_BUSY_AND_IDLE new_busy_and_idle)                                   */
 /*                                                                           */
 /*  Change the idle value of one monitored time group.                       */
 /*                                                                           */
@@ -440,15 +512,15 @@ void KheLimitIdleTimesMonitorDeleteBusyAndIdle(
 
 void KheLimitIdleTimesMonitorChangeBusyAndIdle(
   KHE_LIMIT_IDLE_TIMES_MONITOR m, int index,
-  int old_busy_count, int new_busy_count,
-  int old_idle_count, int new_idle_count,
-  float old_workload, float new_workload)
+  KHE_BUSY_AND_IDLE old_busy_and_idle,
+  KHE_BUSY_AND_IDLE new_busy_and_idle)
 {
   if( DEBUG1(m) )
     fprintf(stderr, "  KheLimitIdleTimesMonitorChangeBusyAndIdle"
       "(%s, index %d, old %d, new %d)\n", KheMonitorId((KHE_MONITOR) m),
-      index, old_idle_count, new_idle_count);
-  KheLimitIdleTimesMonitorFlush(m, new_idle_count - old_idle_count);
+      index, old_busy_and_idle->idle_count, new_busy_and_idle->idle_count);
+  KheLimitIdleTimesMonitorFlush(m,
+    new_busy_and_idle->idle_count - old_busy_and_idle->idle_count);
 }
 
 
