@@ -72,7 +72,9 @@ KHE_LIMIT_RESOURCES_MONITOR KheLimitResourcesMonitorMake(KHE_SOLN soln,
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_LIMIT_RESOURCES_MONITOR_TAG);
+    KHE_LIMIT_RESOURCES_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->constraint = c;
   res->eg_index = eg_index;
   res->minimum = KheLimitResourcesConstraintMinimum(c);
@@ -265,7 +267,7 @@ static void KheLimitResourcesMonitorFlush(KHE_LIMIT_RESOURCES_MONITOR m)
   new_dev = KheLimitResourcesDev(m, m->active_durn);
   if( m->deviation != new_dev )
   {
-    new_cost = KheConstraintCost((KHE_CONSTRAINT) m->constraint, new_dev);
+    new_cost = KheMonitorDevToCost((KHE_MONITOR) m, new_dev);
     KheMonitorChangeCost((KHE_MONITOR) m, new_cost);
     m->deviation = new_dev;
   }
@@ -672,6 +674,94 @@ char *KheLimitResourcesMonitorId(KHE_LIMIT_RESOURCES_MONITOR m)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLimitResourcesMonitorTimeRange(KHE_LIMIT_RESOURCES_MONITOR m,    */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for limit resources monitor m.             */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLimitResourcesMonitorTimeRange(KHE_LIMIT_RESOURCES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  int i, count;  KHE_EVENT_RESOURCE er;  KHE_TIME ft, lt;
+  *first_time = *last_time = NULL;
+  count = KheLimitResourcesConstraintEventResourceCount(m->constraint,
+    m->eg_index);
+  for( i = 0;  i < count;  i++ )
+  {
+    er = KheLimitResourcesConstraintEventResource(m->constraint, m->eg_index,i);
+    if( !KheEventResourceTimeRange(er, m->soln, &ft, &lt) )
+      return false;
+    if( *first_time == NULL || KheTimeIndex(ft) < KheTimeIndex(*first_time) )
+      *first_time = ft;
+    if( *last_time == NULL || KheTimeIndex(lt) > KheTimeIndex(*last_time) )
+      *last_time = lt;
+  }
+  return *first_time != NULL;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheLimitResourcesMonitorSetSweepTime(                               */
+/*    KHE_LIMIT_RESOURCES_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for limit resources monitor m.          */
+/*                                                                           */
+/*  Implementation note.  Implementing this function by attaching and        */
+/*  detaching is not a great way to do it:  if a monitor is detached for     */
+/*  some other reason, it goes wrong, and there is no recognition that       */
+/*  some of the monitored event resources could be before the cutoff and     */
+/*  others could be after it.  But it's good enough to go on with.           */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheLimitResourcesMonitorSetSweepTime(
+  KHE_LIMIT_RESOURCES_MONITOR m, KHE_TIME t)
+{
+  KHE_TIME first_time, last_time;
+  if( KheLimitResourcesMonitorSweepTimeRange(m, &first_time, &last_time)
+      && (t == NULL || KheTimeIndex(t) < KheTimeIndex(last_time)) )
+  {
+    /* m needs to be cut off */
+    if( m->attached )
+      KheLimitResourcesMonitorDetachFromSoln(m);
+  }
+  else
+  {
+    /* m needs to be not cut off */
+    if( !m->attached )
+      KheLimitResourcesMonitorAttachToSoln(m);
+  }
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLimitResourcesMonitorSweepTimeRange(                             */
+/*    KHE_LIMIT_RESOURCES_MONITOR m, KHE_TIME *first_time,                   */
+/*    KHE_TIME *last_time)                                                   */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for limit resources monitor m.        */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLimitResourcesMonitorSweepTimeRange(
+  KHE_LIMIT_RESOURCES_MONITOR m, KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return KheLimitResourcesMonitorTimeRange(m, first_time, last_time);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "debug"                                                        */
