@@ -47,7 +47,7 @@ struct khe_limit_workload_constraint_rec {
   int				minimum;		/* minimum           */
   int				maximum;		/* maximum           */
   bool				allow_zero;		/* allow_zero        */
-  bool				limits_whole_cycle;	/* allow_zero        */
+  bool				limits_whole_cycle;	/* limits whole cycle*/
   ARRAY_KHE_TIME_GROUP		time_groups;		/* time groups       */
   KHE_TIME_GROUP		domain;			/* time groups union */
 };
@@ -270,7 +270,7 @@ void KheLimitWorkloadConstraintFinalize(KHE_LIMIT_WORKLOAD_CONSTRAINT c)
   HaArrayForEach(c->time_groups, tg, i)
     SSetUnion(ss, *KheTimeGroupTimeSet(tg));
   c->domain = KheTimeGroupMakeAndFinalize(c->instance,
-    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, NULL, false);
+    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, /* NULL, */ false, a);
   if( DEBUG1 )
   {
     fprintf(stderr, "  domain: ");
