@@ -34,6 +34,11 @@
 #define DN "Welfare07"
 #define DO_DEBUG1(m) (DEBUG1 && 0 ==					\
  strcmp(KheResourceName(KheResourceInSolnResource((m)->resource_in_soln)),DN))
+#define bool_show(x) ((x) ? "true" : "false")
+
+#define DEBUG3_ID "Constraint:11/Nurse11"
+#define DEBUG3(m) (false && \
+  strcmp(KheMonitorId((KHE_MONITOR) m), DEBUG3_ID) == 0)
 
 
 /*****************************************************************************/
@@ -50,6 +55,7 @@ struct khe_limit_workload_monitor_rec {
   int				minimum;		/* from constraint   */
   int				maximum;		/* from constraint   */
   int				ceiling;		/* user-defined      */
+  float				max_workload_per_time;	/* max wk per time   */
   ARRAY_KHE_MONITORED_TIME_GROUP monitored_time_groups; /* monitored_tg's    */
   ARRAY_KHE_MONITORED_TIME_GROUP defective_time_groups;	/* defective tg's    */
   KHE_TIME			first_time;		/* time range        */
@@ -96,7 +102,7 @@ static void KheLimitWorkloadMonitorSetLowerBound(KHE_LIMIT_WORKLOAD_MONITOR m)
 
       /* set the lower bound if non-zero */
       if( workload > m->maximum )
-	m->lower_bound = KheConstraintCost((KHE_CONSTRAINT) m->constraint,
+	m->lower_bound = KheMonitorDevToCost((KHE_MONITOR) m,
 	  workload - m->maximum);
     }
   }
@@ -117,14 +123,17 @@ KHE_LIMIT_WORKLOAD_MONITOR KheLimitWorkloadMonitorMake(
 {
   KHE_LIMIT_WORKLOAD_MONITOR res;  KHE_SOLN soln;  HA_ARENA a;
   KHE_RESOURCE_TIMETABLE_MONITOR rtm;  KHE_TIME_GROUP tg;  int i;
+  KHE_RESOURCE_TYPE rt;
 
-  /* make an initialize the monitor object */
+  /* make and initialize the monitor object */
   soln = KheResourceInSolnSoln(rs);
   a = KheSolnArena(soln);
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, KheResourceInSolnSoln(rs),
-    KHE_LIMIT_WORKLOAD_MONITOR_TAG);
+    KHE_LIMIT_WORKLOAD_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   HnAssert(rs != NULL, "KheLimitWorkloadMonitorMake internal error");
   res->resource_in_soln = rs;
   res->constraint = c;
@@ -133,6 +142,8 @@ KHE_LIMIT_WORKLOAD_MONITOR KheLimitWorkloadMonitorMake(
   res->minimum = KheLimitWorkloadConstraintMinimum(c);
   res->maximum = KheLimitWorkloadConstraintMaximum(c);
   res->allow_zero = KheLimitWorkloadConstraintAllowZero(c);
+  rt = KheResourceResourceType(KheResourceInSolnResource(rs));
+  res->max_workload_per_time = KheResourceTypeMaxWorkloadPerTime(rt);
   res->ceiling = INT_MAX;
   /* res->workload = 0.0; */
   res->copy = NULL;
@@ -352,9 +363,10 @@ int KheLimitWorkloadMonitorTimeGroupCount(KHE_LIMIT_WORKLOAD_MONITOR m)
 KHE_TIME_GROUP KheLimitWorkloadMonitorTimeGroup(
   KHE_LIMIT_WORKLOAD_MONITOR m, int i, float *workload)
 {
-  KHE_MONITORED_TIME_GROUP mtg;
+  KHE_MONITORED_TIME_GROUP mtg;  KHE_BUSY_AND_IDLE busy_and_idle;
   mtg = HaArray(m->monitored_time_groups, i);
-  *workload = KheMonitoredTimeGroupWorkload(mtg);
+  busy_and_idle = KheMonitoredTimeGroupBusyAndIdle(mtg);
+  *workload = busy_and_idle->workload;
   return KheMonitoredTimeGroupTimeGroup(mtg);
 }
 
@@ -368,6 +380,7 @@ KHE_TIME_GROUP KheLimitWorkloadMonitorTimeGroup(
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** replaced by KheLimitWorkloadMonitorSweepTimeRange
 void KheLimitWorkloadMonitorRange(KHE_LIMIT_WORKLOAD_MONITOR m,
   KHE_TIME *first_time, KHE_TIME *last_time)
 {
@@ -401,6 +414,7 @@ void KheLimitWorkloadMonitorRange(KHE_LIMIT_WORKLOAD_MONITOR m,
   *first_time = m->first_time;
   *last_time = m->last_time;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -472,24 +486,6 @@ void KheLimitWorkloadMonitorAttachToSoln(KHE_LIMIT_WORKLOAD_MONITOR m)
     KheLimitWorkloadMonitorDebug(m, 1, -1, stderr);
     fprintf(stderr, "  after attach\n");
   }
-  /* ***
-  int new_deviation;
-  m->attached = true;
-  new_deviation = KheLimitWorkloadMonitorDev(m, 0.0);
-  if( new_deviation > 0 )
-  {
-    m->deviation = new_deviation;
-    KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, new_deviation));
-  }
-  KheResourceInSolnAttachMonitor(m->resource_in_soln, (KHE_MONITOR) m);
-  if( DO_DEBUG1(m) )
-  {
-    fprintf(stderr, "  ");
-    KheLimitWorkloadMonitorDebug(m, 1, -1, stderr);
-    fprintf(stderr, "  after attach\n");
-  }
-  *** */
 }
 
 
@@ -509,12 +505,6 @@ void KheLimitWorkloadMonitorDetachFromSoln(KHE_LIMIT_WORKLOAD_MONITOR m)
   HnAssert(m->deviation == 0,
     "KheLimitWorkloadMonitorDetachFromSoln internal error 1");
   m->attached = false;
-  /* ***
-  KheResourceInSolnDetachMonitor(m->resource_in_soln, (KHE_MONITOR) m);
-  if( m->cost > 0 )
-    KheMonitorChangeCost((KHE_MONITOR) m, 0);
-  m->attached = false;
-  *** */
   if( DO_DEBUG1(m) )
   {
     fprintf(stderr, "  ");
@@ -524,6 +514,91 @@ void KheLimitWorkloadMonitorDetachFromSoln(KHE_LIMIT_WORKLOAD_MONITOR m)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLimitWorkloadMonitorTimeRange(KHE_LIMIT_WORKLOAD_MONITOR m,      */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Return the time range of m.                                              */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLimitWorkloadMonitorTimeRange(KHE_LIMIT_WORKLOAD_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  int i, first_index, last_index, index, count;  KHE_TIME_GROUP tg;
+  KHE_INSTANCE ins;  KHE_MONITORED_TIME_GROUP mtg;
+  if( m->first_time == NULL )
+  {
+    first_index = INT_MAX;
+    last_index = 0;
+    HaArrayForEach(m->monitored_time_groups, mtg, i)
+    {
+      tg = KheMonitoredTimeGroupTimeGroup(mtg);
+      count = KheTimeGroupTimeCount(tg);
+      if( count > 0 )
+      {
+	index = KheTimeIndex(KheTimeGroupTime(tg, 0));
+	if( index < first_index )
+          first_index = index;
+	index = KheTimeIndex(KheTimeGroupTime(tg, count - 1));
+	if( index > last_index )
+          last_index = index;
+      }
+    }
+    if( first_index < INT_MAX )
+    {
+      ins = KheSolnInstance(m->soln);
+      m->first_time = KheInstanceTime(ins, first_index);
+      m->last_time = KheInstanceTime(ins, last_index);
+    }
+  }
+  *first_time = m->first_time;
+  *last_time = m->last_time;
+  return *first_time != NULL;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheLimitWorkloadMonitorSetSweepTime(                                */
+/*    KHE_LIMIT_WORKLOAD_MONITOR m, KHE_TIME t)                              */
+/*                                                                           */
+/*  Set the sweep time of m.                                                 */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheLimitWorkloadMonitorSetSweepTime(
+  KHE_LIMIT_WORKLOAD_MONITOR m, KHE_TIME t)
+{
+  KHE_MONITORED_TIME_GROUP mtg;  int i;
+  HaArrayForEach(m->monitored_time_groups, mtg, i)
+    KheMonitoredTimeGroupSetSweepTime(mtg, t);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLimitWorkloadMonitorSweepTimeRange(KHE_LIMIT_WORKLOAD_MONITOR m, */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Return the sweep time range of m.                                        */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLimitWorkloadMonitorSweepTimeRange(KHE_LIMIT_WORKLOAD_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return KheLimitWorkloadMonitorTimeRange(m, first_time, last_time);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "ceiling"                                                      */
@@ -597,7 +672,7 @@ void KheLimitWorkloadMonitorAssignResource(KHE_LIMIT_WORKLOAD_MONITOR m,
     if( new_deviation != m->deviation )
     {
       KheMonitorChangeCost((KHE_MONITOR) m,
-        KheConstraintCost((KHE_CONSTRAINT) m->constraint, new_deviation));
+        KheMonitorDevToCost((KHE_MONITOR) m, new_deviation));
       m->deviation = new_deviation;
     }
     m->workload = new_workload;
@@ -641,7 +716,7 @@ void KheLimitWorkloadMonitorUnAssignResource(KHE_LIMIT_WORKLOAD_MONITOR m,
     if( new_deviation != m->deviation )
     {
       KheMonitorChangeCost((KHE_MONITOR) m,
-        KheConstraintCost((KHE_CONSTRAINT) m->constraint, new_deviation));
+        KheMonitorDevToCost((KHE_MONIOR) m, new_deviation));
       m->deviation = new_deviation;
     }
     m->workload = new_workload;
@@ -666,20 +741,55 @@ void KheLimitWorkloadMonitorUnAssignResource(KHE_LIMIT_WORKLOAD_MONITOR m,
 
 /*****************************************************************************/
 /*                                                                           */
-/*  int KheLimitWorkloadMonitorDev(KHE_LIMIT_WORKLOAD_MONITOR m,             */
-/*    float workload)                                                        */
+/*  void KheLimitWorkloadMonitorDebugOp(KHE_LIMIT_WORKLOAD_MONITOR m,        */
+/*    char *op, KHE_BUSY_AND_IDLE old_busy_and_idle,                         */
+/*    KHE_BUSY_AND_IDLE new_busy_and_idle, KHE_MONITORED_TIME_GROUP mtg)     */
 /*                                                                           */
-/*  Work out the deviation caused by this much workload.                     */
+/*  Debug print of op on mtg.                                                */
+/*                                                                           */
+/*****************************************************************************/
+
+static void KheLimitWorkloadMonitorDebugOp(KHE_LIMIT_WORKLOAD_MONITOR m,
+  char *op, KHE_BUSY_AND_IDLE old_busy_and_idle, int old_dev,
+  KHE_BUSY_AND_IDLE new_busy_and_idle, int new_dev,
+  KHE_MONITORED_TIME_GROUP mtg)
+{
+  if( DEBUG3(m) )
+    fprintf(stderr, "  %s %-6s [%6.1f %2d] %4d -> [%6.1f %2d] %4d %s\n",
+      KheMonitorId((KHE_MONITOR) m), op,
+      old_busy_and_idle == NULL ? 0.0 : old_busy_and_idle->workload,
+      old_busy_and_idle == NULL ? 0   : old_busy_and_idle->open_count,
+      old_dev,
+      new_busy_and_idle == NULL ? 0.0 : new_busy_and_idle->workload,
+      new_busy_and_idle == NULL ? 0   : new_busy_and_idle->open_count,
+      new_dev,
+      KheTimeGroupId(KheMonitoredTimeGroupTimeGroup(mtg)));
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheLimitWorkloadMonitorDev(KHE_LIMIT_WORKLOAD_MONITOR m,             */
+/*    float workload, int open_count)                                        */
+/*                                                                           */
+/*  Work out the deviation caused by this workload and open_count.           */
+/*                                                                           */
+/*  Because there is no clear correspondence between open_count and the      */
+/*  amount of workload that could possibly be accumulated during open        */
+/*  times, all we can say is that if open_count > 0 then violations of       */
+/*  the minimum limit can't be taken as certain.                             */
 /*                                                                           */
 /*****************************************************************************/
 
 static int KheLimitWorkloadMonitorDev(KHE_LIMIT_WORKLOAD_MONITOR m,
-  float workload)
+  float workload, int open_count)
 {
+  float adjusted_workload;
+  adjusted_workload = workload + open_count * m->max_workload_per_time;
   if( (m->allow_zero && workload < 0.001) || workload > m->ceiling + 0.001 )
     return 0;
-  else if( workload < m->minimum - 0.001 )
-    return (int) ceil(m->minimum - 0.001 - workload);
+  else if( adjusted_workload < m->minimum - 0.001 )
+    return (int) ceil(m->minimum - 0.001 - adjusted_workload);
   else if( workload > m->maximum + 0.001 )
     return (int) ceil(workload - m->maximum - 0.001);
   else
@@ -687,11 +797,57 @@ static int KheLimitWorkloadMonitorDev(KHE_LIMIT_WORKLOAD_MONITOR m,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheLimitWorkloadMonitorAddDefective(KHE_LIMIT_WORKLOAD_MONITOR m,   */
+/*    KHE_MONITORED_TIME_GROUP mtg)                                          */
+/*                                                                           */
+/*  Add mtg to m's list of defective time groups, in sorted order.           */
+/*                                                                           */
+/*****************************************************************************/
+
+static void KheLimitWorkloadMonitorAddDefective(KHE_LIMIT_WORKLOAD_MONITOR m,
+  KHE_MONITORED_TIME_GROUP mtg)
+{
+  KHE_MONITORED_TIME_GROUP mtg2;  int i;  KHE_TIME_GROUP tg, tg2;
+  tg = KheMonitoredTimeGroupTimeGroup(mtg);
+  HaArrayAddLast(m->defective_time_groups, NULL);
+  for( i = HaArrayCount(m->defective_time_groups) - 2;  i >= 0;  i-- )
+  {
+    mtg2 = HaArray(m->defective_time_groups, i);
+    tg2 = KheMonitoredTimeGroupTimeGroup(mtg2);
+    if( KheTimeGroupTypedCmp(tg2, tg) )
+      break;
+    HaArrayPut(m->defective_time_groups, i + 1, mtg2);
+  }
+  HaArrayPut(m->defective_time_groups, i + 1, mtg);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheLimitWorkloadMonitorDeleteDefective(KHE_LIMIT_WORKLOAD_MONITOR m,*/
+/*    KHE_MONITORED_TIME_GROUP mtg)                                          */
+/*                                                                           */
+/*  Delete mtg from m's list of defective time groups.  It must be there.    */
+/*                                                                           */
+/*****************************************************************************/
+
+static void KheLimitWorkloadMonitorDeleteDefective(KHE_LIMIT_WORKLOAD_MONITOR m,
+  KHE_MONITORED_TIME_GROUP mtg)
+{
+  int pos;
+  if( !HaArrayContains(m->defective_time_groups, mtg, &pos) )
+    HnAbort("KheLimitWorkloadMonitorDeleteDefective internal error");
+  HaArrayDeleteAndShift(m->defective_time_groups, pos);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheLimitWorkloadMonitorAddBusyAndIdle(                              */
 /*    KHE_LIMIT_WORKLOAD_MONITOR m, int index,                               */
-/*    int busy_count, int idle_count, float workload)                        */
+/*    KHE_BUSY_AND_IDLE busy_and_idle)                                       */
 /*                                                                           */
 /*  Add a workload for one monitored time group.                             */
 /*                                                                           */
@@ -699,21 +855,24 @@ static int KheLimitWorkloadMonitorDev(KHE_LIMIT_WORKLOAD_MONITOR m,
 
 void KheLimitWorkloadMonitorAddBusyAndIdle(
   KHE_LIMIT_WORKLOAD_MONITOR m, int index,
-  int busy_count, int idle_count, float workload)
+  KHE_BUSY_AND_IDLE busy_and_idle)
 {
   int dev, pos;  KHE_MONITORED_TIME_GROUP mtg;
-  dev = KheLimitWorkloadMonitorDev(m, workload);
+  dev = KheLimitWorkloadMonitorDev(m, busy_and_idle->workload,
+    busy_and_idle->open_count);
   if( dev != 0 )
   {
     HnAssert(index >= 0 && index < HaArrayCount(m->monitored_time_groups),
       "KheLimitWorkloadMonitorAddBusyAndIdle internal error 1");
     mtg = HaArray(m->monitored_time_groups, index);
+    KheLimitWorkloadMonitorDebugOp(m, "add", NULL, 0, busy_and_idle, dev, mtg);
     HnAssert(!HaArrayContains(m->defective_time_groups, mtg, &pos),
       "KheLimitWorkloadMonitorAddBusyAndIdle internal error 2");
-    HaArrayAddLast(m->defective_time_groups, mtg);
+    /* HaArrayAddLast(m->defective_time_groups, mtg); */
+    KheLimitWorkloadMonitorAddDefective(m, mtg);
     m->deviation += dev;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -722,7 +881,7 @@ void KheLimitWorkloadMonitorAddBusyAndIdle(
 /*                                                                           */
 /*  void KheLimitWorkloadMonitorDeleteBusyAndIdle(                           */
 /*    KHE_LIMIT_WORKLOAD_MONITOR m, int index,                               */
-/*    int busy_count, int idle_count, float workload)                        */
+/*    KHE_BUSY_AND_IDLE busy_and_idle)                                       */
 /*                                                                           */
 /*  Remove the workload of one monitored time group.                         */
 /*                                                                           */
@@ -730,21 +889,26 @@ void KheLimitWorkloadMonitorAddBusyAndIdle(
 
 void KheLimitWorkloadMonitorDeleteBusyAndIdle(
   KHE_LIMIT_WORKLOAD_MONITOR m, int index,
-  int busy_count, int idle_count, float workload)
+  KHE_BUSY_AND_IDLE busy_and_idle)
 {
-  int dev, pos;  KHE_MONITORED_TIME_GROUP mtg;
-  dev = KheLimitWorkloadMonitorDev(m, workload);
+  int dev;  KHE_MONITORED_TIME_GROUP mtg;
+  dev = KheLimitWorkloadMonitorDev(m, busy_and_idle->workload,
+    busy_and_idle->open_count);
   if( dev != 0 )
   {
     HnAssert(index >= 0 && index < HaArrayCount(m->monitored_time_groups),
       "KheLimitWorkloadMonitorDeleteBusyAndIdle internal error 1");
     mtg = HaArray(m->monitored_time_groups, index);
+    KheLimitWorkloadMonitorDebugOp(m, "del", busy_and_idle, dev, NULL, 0, mtg);
+    KheLimitWorkloadMonitorDeleteDefective(m, mtg);
+    /* ***
     HnAssert(HaArrayContains(m->defective_time_groups, mtg, &pos),
       "KheLimitWorkloadMonitorDeleteBusyAndIdle internal error 2");
     HaArrayDeleteAndShift(m->defective_time_groups, pos);
+    *** */
     m->deviation -= dev;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -753,9 +917,8 @@ void KheLimitWorkloadMonitorDeleteBusyAndIdle(
 /*                                                                           */
 /*  void KheLimitWorkloadMonitorChangeBusyAndIdle(                           */
 /*    KHE_LIMIT_WORKLOAD_MONITOR m, int index,                               */
-/*    int old_busy_count, int new_busy_count,                                */
-/*    int old_idle_count, int new_idle_count,                                */
-/*    float old_workload, float new_workload)                                */
+/*    KHE_BUSY_AND_IDLE old_busy_and_idle,                                   */
+/*    KHE_BUSY_AND_IDLE new_busy_and_idle)                                   */
 /*                                                                           */
 /*  Change the workload of one monitored time group.                         */
 /*                                                                           */
@@ -763,13 +926,14 @@ void KheLimitWorkloadMonitorDeleteBusyAndIdle(
 
 void KheLimitWorkloadMonitorChangeBusyAndIdle(
   KHE_LIMIT_WORKLOAD_MONITOR m, int index,
-  int old_busy_count, int new_busy_count,
-  int old_idle_count, int new_idle_count,
-  float old_workload, float new_workload)
+  KHE_BUSY_AND_IDLE old_busy_and_idle,
+  KHE_BUSY_AND_IDLE new_busy_and_idle)
 {
   int old_dev, new_dev, pos;  KHE_MONITORED_TIME_GROUP mtg;
-  old_dev = KheLimitWorkloadMonitorDev(m, old_workload);
-  new_dev = KheLimitWorkloadMonitorDev(m, new_workload);
+  old_dev = KheLimitWorkloadMonitorDev(m, old_busy_and_idle->workload,
+    old_busy_and_idle->open_count);
+  new_dev = KheLimitWorkloadMonitorDev(m, new_busy_and_idle->workload,
+    new_busy_and_idle->open_count);
   if( old_dev != new_dev )
   {
     HnAssert(index >= 0 && index < HaArrayCount(m->monitored_time_groups),
@@ -777,24 +941,35 @@ void KheLimitWorkloadMonitorChangeBusyAndIdle(
     mtg = HaArray(m->monitored_time_groups, index);
     if( old_dev == 0 )
     {
+      KheLimitWorkloadMonitorDebugOp(m, "cadd", old_busy_and_idle, old_dev,
+	new_busy_and_idle, new_dev, mtg);
       HnAssert(!HaArrayContains(m->defective_time_groups, mtg, &pos),
-	"KheLimitWorkloadMonitorChangeBusyAndIdle internal error 2");
-      HaArrayAddLast(m->defective_time_groups, mtg);
+	"KheLimitWorkloadMonitorChangeBusyAndIdle internal error 2 (%s)",
+	KheMonitorId((KHE_MONITOR) m));
+      KheLimitWorkloadMonitorAddDefective(m, mtg);
+      /* HaArrayAddLast(m->defective_time_groups, mtg); */
     }
     else if( new_dev == 0 )
     {
+      KheLimitWorkloadMonitorDebugOp(m, "cdel", old_busy_and_idle, old_dev,
+	new_busy_and_idle, new_dev, mtg);
+      KheLimitWorkloadMonitorDeleteDefective(m, mtg);
+      /* ***
       HnAssert(HaArrayContains(m->defective_time_groups, mtg, &pos),
 	"KheLimitWorkloadMonitorChangeBusyAndIdle internal error 3");
       HaArrayDeleteAndShift(m->defective_time_groups, pos);
+      *** */
     }
     else
     {
+      KheLimitWorkloadMonitorDebugOp(m, "ckeep", old_busy_and_idle, old_dev,
+	new_busy_and_idle, new_dev, mtg);
       HnAssert(HaArrayContains(m->defective_time_groups, mtg, &pos),
 	"KheLimitWorkloadMonitorChangeBusyAndIdle internal error 4");
     }
     m->deviation += (new_dev - old_dev);
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -827,23 +1002,26 @@ int KheLimitWorkloadMonitorDefectiveTimeGroupCount(
 
 void KheLimitWorkloadMonitorDefectiveTimeGroup(
   KHE_LIMIT_WORKLOAD_MONITOR m, int i, KHE_TIME_GROUP *tg,
-  float *workload, int *minimum, int *maximum, bool *allow_zero)
+  float *workload, int *open_count, int *minimum, int *maximum,
+  bool *allow_zero)
 {
-  KHE_MONITORED_TIME_GROUP mtg;
+  KHE_MONITORED_TIME_GROUP mtg;  KHE_BUSY_AND_IDLE busy_and_idle;
   HnAssert(0 <= i && i < HaArrayCount(m->defective_time_groups),
     "KheLimitWorkloadMonitorDefectiveTimeGroup: i (%d) out of range (0 .. %d)",
     i, HaArrayCount(m->defective_time_groups) - 1);
   mtg = HaArray(m->defective_time_groups, i);
   *tg = KheMonitoredTimeGroupTimeGroup(mtg);
-  *workload = KheMonitoredTimeGroupWorkload(mtg);
+  busy_and_idle = KheMonitoredTimeGroupBusyAndIdle(mtg);
+  *workload = busy_and_idle->workload;
+  *open_count = busy_and_idle->open_count;
   *minimum = m->minimum;
   *maximum = m->maximum;
   *allow_zero = m->allow_zero;
   if( DEBUG2 )
     fprintf(stderr, "  KheLimitWorkloadMonitorDefectiveTimeGroup(%s, %d) = "
-      "tg %s, wk %.1f, min %d, max %d, az %s\n", KheLimitWorkloadMonitorId(m),
-      i, KheTimeGroupId(*tg), *workload, *minimum, *maximum,
-      *allow_zero ? "true" : "false");
+      "tg %s, wk %.1f, op %d, min %d, max %d, az %s\n",
+      KheLimitWorkloadMonitorId(m), i, KheTimeGroupId(*tg), *workload,
+      *open_count, *minimum, *maximum, bool_show(*allow_zero));
 }
 
 
@@ -879,13 +1057,15 @@ int KheLimitWorkloadMonitorDeviation(KHE_LIMIT_WORKLOAD_MONITOR m)
 static void KheLimitWorkloadMonitorDeviationShowOneTimeGroup(
   KHE_LIMIT_WORKLOAD_MONITOR m, int i, HA_ARRAY_NCHAR *ac)
 {
-  float workload;  int minimum, maximum;  KHE_TIME_GROUP tg;  char *name;
-  bool allow_zero;
+  float workload, adjusted_workload;  int minimum, maximum, open_count;
+  KHE_TIME_GROUP tg; char *name; bool allow_zero;
   KheLimitWorkloadMonitorDefectiveTimeGroup(m, i, &tg, &workload,
-    &minimum, &maximum, &allow_zero);
+    &open_count, &minimum, &maximum, &allow_zero);
   name = KheTimeGroupName(tg);
-  if( workload < minimum )
-    HnStringAdd(ac, "%d - %.1f %s", minimum, workload,
+  adjusted_workload = workload + open_count * m->max_workload_per_time;
+  if( adjusted_workload < minimum )
+    HnStringAdd(ac, "%d - %s%.1f %s", minimum,
+      open_count == 0 ? "" : "(adjusted) ", adjusted_workload,
       name == NULL ? "?" : name);
   else if( workload > maximum )
     HnStringAdd(ac, "%.1f %s - %d", workload,
@@ -990,7 +1170,7 @@ char *KheLimitWorkloadMonitorId(KHE_LIMIT_WORKLOAD_MONITOR m)
 void KheLimitWorkloadMonitorDebug(KHE_LIMIT_WORKLOAD_MONITOR m,
   int verbosity, int indent, FILE *fp)
 {
-  char buff[20];  int i, minimum, maximum;  KHE_TIME_GROUP tg;
+  char buff[20];  int i, minimum, maximum, open_count;  KHE_TIME_GROUP tg;
   float workload;  bool allow_zero;
   if( verbosity >= 1 )
   {
@@ -1010,7 +1190,7 @@ void KheLimitWorkloadMonitorDebug(KHE_LIMIT_WORKLOAD_MONITOR m,
     for( i = 0;  i < KheLimitWorkloadMonitorDefectiveTimeGroupCount(m);  i++ )
     {
       KheLimitWorkloadMonitorDefectiveTimeGroup(m, i, &tg,
-	&workload, &minimum, &maximum, &allow_zero);
+	&workload, &open_count, &minimum, &maximum, &allow_zero);
       fprintf(fp, " %.1f", workload);
     }
     KheMonitorDebugEnd((KHE_MONITOR) m, true, indent, fp);
