@@ -80,7 +80,9 @@ KHE_LINK_EVENTS_MONITOR KheLinkEventsMonitorMake(KHE_SOLN soln,
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_LINK_EVENTS_MONITOR_TAG);
+    KHE_LINK_EVENTS_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->constraint = c;
   res->event_group = eg;
   res->wanted_multiplicity = KheEventGroupEventCount(eg);
@@ -395,6 +397,73 @@ void KheLinkEventsMonitorAttachCheck(KHE_LINK_EVENTS_MONITOR m)
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLinkEventsMonitorTimeRange(KHE_LINK_EVENTS_MONITOR m,            */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLinkEventsMonitorTimeRange(KHE_LINK_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_EVENT_GROUP eg;  KHE_EVENT e;  KHE_TIME ft, lt;  int i;
+  eg = KheLinkEventsMonitorEventGroup(m);
+  *first_time = *last_time = NULL;
+  for( i = 0;  i < KheEventGroupEventCount(eg);  i++ )
+  {
+    e = KheEventGroupEvent(eg, i);
+    if( KheEventTimeRange(e, m->soln, &ft, &lt) )
+    {
+      if( *first_time == NULL || KheTimeIndex(ft) < KheTimeIndex(*first_time) )
+	*first_time = ft;
+      if( *last_time == NULL || KheTimeIndex(lt) > KheTimeIndex(*last_time) )
+	*last_time = ft;
+    }
+  }
+  return *first_time != NULL;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheLinkEventsMonitorSetSweepTime(KHE_LINK_EVENTS_MONITOR m,         */
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheLinkEventsMonitorSetSweepTime(KHE_LINK_EVENTS_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheLinkEventsMonitorSweepTimeRange(KHE_LINK_EVENTS_MONITOR m,       */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for m.                                */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheLinkEventsMonitorSweepTimeRange(KHE_LINK_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "fix and unfix"                                                */
@@ -613,7 +682,7 @@ void KheLinkEventsMonitorFlush(KHE_LINK_EVENTS_MONITOR m)
   if( m->new_deviation != m->deviation )
   {
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->new_deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->new_deviation));
     m->deviation = m->new_deviation;
   }
 }
