@@ -28,6 +28,7 @@
 /*                                                                           */
 /*****************************************************************************/
 #include "khe_interns.h"
+#define bool_show(x) ((x) ? "true" : "false")
 
 #define KHE_MAX_DURATION 1000  /* for sanity checking */
 #define DEBUG1 0
@@ -166,7 +167,7 @@ void KheMeetBoundReferenceCountDecrement(KHE_MEET_BOUND mb)
 /*                                                                           */
 /*****************************************************************************/
 
-void KheMeetBoundDoAdd(KHE_MEET_BOUND mb, KHE_SOLN soln,
+static void KheMeetBoundDoAdd(KHE_MEET_BOUND mb, KHE_SOLN soln,
   bool occupancy, KHE_TIME_GROUP dft_tg)
 {
   if( DEBUG1 )
@@ -375,7 +376,7 @@ bool KheMeetBoundDelete(KHE_MEET_BOUND mb)
 
 KHE_MEET_BOUND KheMeetBoundCopyPhase1(KHE_MEET_BOUND mb, HA_ARENA a)
 {
-  KHE_MEET_BOUND copy;  KHE_TIME_GROUP tg;  int i;  KHE_MEET meet;
+  KHE_MEET_BOUND copy;  KHE_TIME_GROUP tg, tg_copy;  int i;  KHE_MEET meet;
   if( mb->copy == NULL )
   {
     HaMake(copy, a);
@@ -384,10 +385,13 @@ KHE_MEET_BOUND KheMeetBoundCopyPhase1(KHE_MEET_BOUND mb, HA_ARENA a)
     /* copy->soln_index = mb->soln_index; */
     copy->reference_count = 1;  /* no paths, and mb is linked in */
     copy->occupancy = mb->occupancy;
-    copy->default_time_group = mb->default_time_group;
+    copy->default_time_group = KheTimeGroupCopy(mb->default_time_group, a);
     HaArrayInit(copy->time_groups, a);
     HaArrayForEach(mb->time_groups, tg, i)
-      HaArrayAddLast(copy->time_groups, tg);
+    {
+      tg_copy = (tg == NULL ? NULL : KheTimeGroupCopy(tg, a));
+      HaArrayAddLast(copy->time_groups, tg_copy);
+    }
     HaArrayInit(copy->meets, a);
     HaArrayForEach(mb->meets, meet, i)
       HaArrayAddLast(copy->meets, KheMeetCopyPhase1(meet, a));
@@ -701,6 +705,52 @@ KHE_TIME_GROUP KheSolnStartingTimeGroup(KHE_SOLN soln, int duration,
   }
   return KheSolnTimeGroupEnd(soln);
 }
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheMeetBoundDebugHeader(KHE_MEET_BOUND mb, FILE *fp)                */
+/*                                                                           */
+/*  Debug print of the header part of the debug of mb onto fp.               */
+/*                                                                           */
+/*****************************************************************************/
+
+static void KheMeetBoundDebugHeader(KHE_MEET_BOUND mb, FILE *fp)
+{
+  fprintf(fp, "MeetBound(oc %s, dtg %s, meets %d)",
+    bool_show(mb->occupancy), KheTimeGroupId(mb->default_time_group),
+    HaArrayCount(mb->meets));
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheMeetBoundDebug(KHE_MEET_BOUND mb, int verbosity,                 */
+/*    int indent, FILE *fp)                                                  */
+/*                                                                           */
+/*  Debug print of mb onto fp with the given verbosity and indent.           */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheMeetBoundDebug(KHE_MEET_BOUND mb, int verbosity, int indent, FILE *fp)
+{
+  KHE_TIME_GROUP tg;  int i;
+  if( indent >= 0 )
+  {
+    fprintf(fp, "%*s[ ", indent, "");
+    KheMeetBoundDebugHeader(mb, fp);
+    fprintf(fp, "\n");
+    if( verbosity >= 2 )
+      HaArrayForEach(mb->time_groups, tg, i)
+      {
+	fprintf(fp, "%*s%3d: ", indent, "", i + 1);
+	KheTimeGroupDebug(tg, 2, 0, fp);
+      }
+    fprintf(fp, "%*s]\n", indent, "");
+  }
+  else
+    KheMeetBoundDebugHeader(mb, fp);
+}
 
 
 /*****************************************************************************/
