@@ -31,6 +31,13 @@
 
 #define DEBUG1(m) (false && strcmp(KheMonitorId((KHE_MONITOR) m), "S4/t24")==0)
 
+#define DEBUG2(mtg)	\
+  (false && strcmp(KheTimeGroupId((mtg)->time_group), "1-3Mon-Sun") == 0 && \
+   strcmp(KheResourceId(KheMonitoredTimeGroupResource(mtg)), "Nurse11") == 0)
+
+#define DEBUG3 0
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  KHE_MONITORED_TIME_GROUP - one time group monitored by a timetable.      */
@@ -49,10 +56,13 @@ struct khe_monitored_time_group_rec {
   ARRAY_KHE_MONITOR		monitors;		/* attached monitors */
   HA_ARRAY_INT			monitor_indexes;	/* id in monitors    */
   KHE_MTG_STATE			state;			/* attach state      */
-  int				busy_count;		/* busy times count  */
-  float				workload;		/* workload in tg    */
-  int				idle_count;		/* idle times count  */
+  struct khe_busy_and_idle_rec	busy_and_idle;		/* busy, idle, etc.  */
+  /* int			busy_count; */		/* busy times count  */
+  /* float			workload; */		/* workload in tg    */
+  /* int			idle_count; */		/* idle times count  */
   SSET				busy_set;		/* busy times lset   */
+  KHE_TIME			sweep_time;		/* sweep time        */
+  int				sweep_index;		/* sweep index       */
   KHE_MONITORED_TIME_GROUP	copy;			/* used when copying */
 };
 
@@ -63,6 +73,20 @@ struct khe_monitored_time_group_rec {
 /*                                                                           */
 /*****************************************************************************/
 
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_RESOURCE KheMonitoredTimeGroupResource(KHE_MONITORED_TIME_GROUP mtg) */
+/*                                                                           */
+/*  Return the resource monitored by mtg.                                    */
+/*                                                                           */
+/*****************************************************************************/
+
+static KHE_RESOURCE KheMonitoredTimeGroupResource(KHE_MONITORED_TIME_GROUP mtg)
+{
+  return KheResourceTimetableMonitorResource(mtg->timetable_monitor);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  KHE_MONITORED_TIME_GROUP KheMonitoredTimeGroupMake(KHE_TIME_GROUP tg,    */
@@ -75,7 +99,7 @@ struct khe_monitored_time_group_rec {
 KHE_MONITORED_TIME_GROUP KheMonitoredTimeGroupMake(KHE_TIME_GROUP tg,
   KHE_RESOURCE_TIMETABLE_MONITOR rtm)
 {
-  KHE_MONITORED_TIME_GROUP res;  HA_ARENA a;
+  KHE_MONITORED_TIME_GROUP res;  HA_ARENA a;  int count;
   a = KheSolnArena(KheMonitorSoln((KHE_MONITOR) rtm));
   HaMake(res, a);
   res->time_group = tg;
@@ -83,10 +107,17 @@ KHE_MONITORED_TIME_GROUP KheMonitoredTimeGroupMake(KHE_TIME_GROUP tg,
   HaArrayInit(res->monitors, a);
   HaArrayInit(res->monitor_indexes, a);
   res->state = KHE_MTG_STATE_UNATTACHED;
-  res->busy_count = 0;
-  res->workload = 0.0;
-  res->idle_count = 0;
+  res->busy_and_idle.busy_count = 0;
+  res->busy_and_idle.workload = 0.0;
+  res->busy_and_idle.idle_count = 0;
+  res->busy_and_idle.open_count = 0;
   SSetInit(res->busy_set, a);
+  count = KheTimeGroupTimeCount(tg);
+  if( count > 0 )
+    res->sweep_time = KheTimeGroupTime(tg, count - 1);
+  else
+    res->sweep_time = NULL;
+  res->sweep_index = count;
   res->copy = NULL;
   return res;
 }
@@ -104,29 +135,55 @@ KHE_MONITORED_TIME_GROUP KheMonitoredTimeGroupMake(KHE_TIME_GROUP tg,
 KHE_MONITORED_TIME_GROUP KheMonitoredTimeGroupCopyPhase1(
   KHE_MONITORED_TIME_GROUP mtg, HA_ARENA a)
 {
-  KHE_MONITORED_TIME_GROUP copy;  int i, mi;  KHE_MONITOR m;
+  KHE_MONITORED_TIME_GROUP copy;  int i, mi;  KHE_MONITOR m, m_copy;
   if( mtg->copy == NULL )
   {
+    if( DEBUG3 )
+      fprintf(stderr, "[ KheMonitoredTimeGroupCopyPhase1(%p, a)\n",
+	(void *) mtg);
     HaMake(copy, a);
     mtg->copy = copy;
-    copy->time_group = mtg->time_group;
+    copy->time_group = KheTimeGroupCopy(mtg->time_group, a);
     copy->timetable_monitor =
       KheResourceTimetableMonitorCopyPhase1(mtg->timetable_monitor, a);
     HaArrayInit(copy->monitors, a);
+    if( DEBUG3 )
+      fprintf(stderr, "  KheMonitoredTimeGroupCopyPhase1(%p) copy = %p, "
+	"&copy->monitors = %p\n", (void *) mtg, (void *) copy,
+	(void *) &copy->monitors);
     HaArrayForEach(mtg->monitors, m, i)
-      HaArrayAddLast(copy->monitors, KheMonitorCopyPhase1(m, a));
+    {
+      if( DEBUG3 )
+      {
+	fprintf(stderr,
+	  "  KheMonitoredTimeGroupCopyPhase1(%p) copying monitor ",
+	  (void *) mtg);
+	KheMonitorDebug(m, 2, 0, stderr);
+      }
+      m_copy = KheMonitorCopyPhase1(m, a);
+      if( DEBUG3 )
+      {
+	fprintf(stderr,
+	  "  KheMonitoredTimeGroupCopyPhase1(%p) copied monitor %p\n",
+	  (void *) mtg, (void *) m_copy);
+	/* *** can't do this, m_copy might not be ready
+	KheMonitorDebug(m_copy, 2, 0, stderr);
+	*** */
+      }
+      HaArrayAddLast(copy->monitors, m_copy);
+    }
     HaArrayInit(copy->monitor_indexes, a);
     HaArrayForEach(mtg->monitor_indexes, mi, i)
       HaArrayAddLast(copy->monitor_indexes, mi);
     copy->state = mtg->state;
-    copy->busy_count = mtg->busy_count;
-    copy->workload = mtg->workload;
-    copy->idle_count = mtg->idle_count;
+    copy->busy_and_idle = mtg->busy_and_idle;
     SSetCopy(copy->busy_set, mtg->busy_set, a);
-    /* ***
-    copy->busy_set = mtg->busy_set == NULL ? NULL : LSetCopy(mtg->busy_set);
-    *** */
+    copy->sweep_time = mtg->sweep_time;
+    copy->sweep_index = mtg->sweep_index;
     copy->copy = NULL;
+    if( DEBUG3 )
+      fprintf(stderr, "] KheMonitoredTimeGroupCopyPhase1 returning %p\n",
+	(void *) mtg->copy);
   }
   return mtg->copy;
 }
@@ -153,26 +210,6 @@ void KheMonitoredTimeGroupCopyPhase2(KHE_MONITORED_TIME_GROUP mtg)
 }
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheMonitoredTimeGroupDelete(KHE_MONITORED_TIME_GROUP mtg)           */
-/*                                                                           */
-/*  Delete mtg.                                                              */
-/*                                                                           */
-/*****************************************************************************/
-
-/* ***
-void KheMonitoredTimeGroupDelete(KHE_MONITORED_TIME_GROUP mtg)
-{
-  MArrayFree(mtg->monitors);
-  MArrayFree(mtg->monitor_indexes);
-  if( mtg->busy_set != NULL )
-    LSetFree(mtg->busy_set);
-  MFree(mtg);
-}
-*** */
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  KHE_TIME_GROUP KheMonitoredTimeGroupTimeGroup(                           */
@@ -222,10 +259,11 @@ static void KheMonitoredTimeGroupUnattachedToBusy(KHE_MONITORED_TIME_GROUP mtg)
 {
   HnAssert(mtg->state == KHE_MTG_STATE_UNATTACHED,
     "KheMonitoredTimeGroupUnattachedToBusy internal error 1");
-  HnAssert(mtg->busy_count == 0,
+  HnAssert(mtg->busy_and_idle.busy_count == 0,
     "KheMonitoredTimeGroupUnattachedToBusy internal error 2");
   KheResourceTimetableMonitorAttachMonitoredTimeGroup(
-    mtg->timetable_monitor, mtg, &mtg->busy_count, &mtg->workload);
+    mtg->timetable_monitor, mtg, &mtg->busy_and_idle.busy_count,
+    &mtg->busy_and_idle.workload);
   mtg->state = KHE_MTG_STATE_ATTACHED_BUSY;
 }
 
@@ -244,8 +282,8 @@ static void KheMonitoredTimeGroupBusyToUnattached(KHE_MONITORED_TIME_GROUP mtg)
     "KheMonitoredTimeGroupBusyToUnattached internal error 1");
   KheResourceTimetableMonitorDetachMonitoredTimeGroup(
     mtg->timetable_monitor, mtg);
-  mtg->busy_count = 0;
-  mtg->workload = 0.0;
+  mtg->busy_and_idle.busy_count = 0;
+  mtg->busy_and_idle.workload = 0.0;
   mtg->state = KHE_MTG_STATE_UNATTACHED;
 }
 
@@ -260,9 +298,9 @@ static void KheMonitoredTimeGroupBusyToUnattached(KHE_MONITORED_TIME_GROUP mtg)
 
 static int KheIdleTimes(KHE_MONITORED_TIME_GROUP mtg)
 {
-  if( mtg->busy_count > 1 )
+  if( mtg->busy_and_idle.busy_count > 1 )
     return SSetMax(mtg->busy_set) - SSetMin(mtg->busy_set) + 1 -
-      mtg->busy_count;
+      mtg->busy_and_idle.busy_count;
   else
     return 0;
 }
@@ -281,12 +319,7 @@ static void KheMonitoredTimeGroupBusyToBusyAndIdle(KHE_MONITORED_TIME_GROUP mtg)
   int i;  KHE_TIME t;  KHE_RESOURCE_TIMETABLE_MONITOR tm;
   HnAssert(mtg->state == KHE_MTG_STATE_ATTACHED_BUSY,
     "KheMonitoredTimeGroupBusyToBusyAndIdle internal error");
-  /* ***
-  HnAssert(mtg->busy_set == NULL,
-    "KheMonitoredTimeGroupBusyToBusyAndIdle internal error 2");
-  mtg->busy_set = LSet New();
-  *** */
-  HnAssert(mtg->idle_count == 0,
+  HnAssert(mtg->busy_and_idle.idle_count == 0,
     "KheMonitoredTimeGroupBusyToBusyAndIdle internal error 2");
   SSetClear(mtg->busy_set);
   tm = mtg->timetable_monitor;
@@ -296,7 +329,7 @@ static void KheMonitoredTimeGroupBusyToBusyAndIdle(KHE_MONITORED_TIME_GROUP mtg)
     if( KheResourceTimetableMonitorTimeTaskCount(tm, t) > 0 )
       SSetInsert(mtg->busy_set, i);  /* NB *position* in time group */
   }
-  mtg->idle_count = KheIdleTimes(mtg);
+  mtg->busy_and_idle.idle_count = KheIdleTimes(mtg);
   mtg->state = KHE_MTG_STATE_ATTACHED_BUSY_AND_IDLE;
 }
 
@@ -314,13 +347,7 @@ static void KheMonitoredTimeGroupBusyAndIdleToBusy(KHE_MONITORED_TIME_GROUP mtg)
   HnAssert(mtg->state == KHE_MTG_STATE_ATTACHED_BUSY_AND_IDLE,
     "KheMonitoredTimeGroupBusyAndIdleToBusy internal error");
   SSetClear(mtg->busy_set);
-  /* ***
-  HnAssert(mtg->busy_set != NULL,
-    "KheMonitoredTimeGroupBusyAndIdleToBusy internal error");
-  LSetFree(mtg->busy_set);
-  *** */
-  mtg->idle_count = 0;
-  /* mtg->busy_set = NULL; */
+  mtg->busy_and_idle.idle_count = 0;
   mtg->state = KHE_MTG_STATE_ATTACHED_BUSY;
 }
 
@@ -358,8 +385,7 @@ void KheMonitoredTimeGroupAttachMonitor(KHE_MONITORED_TIME_GROUP mtg,
   HaArrayAddLast(mtg->monitor_indexes, index);
 
   /* report busy and idle back to m */
-  KheMonitorAddBusyAndIdle(m, index, mtg->busy_count, mtg->idle_count,
-    mtg->workload);
+  KheMonitorAddBusyAndIdle(m, index, &mtg->busy_and_idle);
 }
 
 
@@ -404,8 +430,7 @@ void KheMonitoredTimeGroupDetachMonitor(KHE_MONITORED_TIME_GROUP mtg,
   /* delete busy and idle from m */
   HnAssert(mtg->state != KHE_MTG_STATE_UNATTACHED,
     "KheMonitoredTimeGroupDetachMonitor: internal error");
-  KheMonitorDeleteBusyAndIdle(m, index, mtg->busy_count, mtg->idle_count,
-    mtg->workload);
+  KheMonitorDeleteBusyAndIdle(m, index, &mtg->busy_and_idle);
 
   /* remove m and its index from mtg */
   if( !HaArrayContains(mtg->monitors, m, &pos) )
@@ -433,14 +458,14 @@ void KheMonitoredTimeGroupDetachMonitor(KHE_MONITORED_TIME_GROUP mtg,
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheMonitoredTimeGroupFlush(KHE_MONITORED_TIME_GROUP mtg,            */
-/*    int old_busy_count, int old_idle_count, float old_workload)            */
+/*    KHE_BUSY_AND_IDLE old_busy_and_idle)                                   */
 /*                                                                           */
 /*  Report a change of state in mtg to its attached monitors.                */
 /*                                                                           */
 /*****************************************************************************/
 
 static void KheMonitoredTimeGroupFlush(KHE_MONITORED_TIME_GROUP mtg,
-  int old_busy_count, int old_idle_count, float old_workload)
+  KHE_BUSY_AND_IDLE old_busy_and_idle)
 {
   int i;  KHE_MONITOR m;
   HaArrayForEach(mtg->monitors, m, i)
@@ -448,10 +473,9 @@ static void KheMonitoredTimeGroupFlush(KHE_MONITORED_TIME_GROUP mtg,
     if( DEBUG1(m) )
       fprintf(stderr, "KheMonitoredTimeGroupFlush(%s, %s, idle %d)\n",
 	KheTimeGroupId(mtg->time_group), SSetShow(mtg->busy_set),
-	mtg->idle_count);
+	mtg->busy_and_idle.idle_count);
     KheMonitorChangeBusyAndIdle(m, HaArray(mtg->monitor_indexes, i),
-      old_busy_count, mtg->busy_count, old_idle_count, mtg->idle_count,
-      old_workload, mtg->workload);
+      old_busy_and_idle, &mtg->busy_and_idle);
   }
 }
 
@@ -537,23 +561,20 @@ void KheMonitoredTimeGroupUnAssignNonClash(KHE_MONITORED_TIME_GROUP mtg,
 void KheMonitoredTimeGroupAssign(KHE_MONITORED_TIME_GROUP mtg,
   KHE_TASK task, int assigned_time_index, int busy_count_before)
 {
-  int old_busy_count, old_idle_count;  float old_workload;
-  old_busy_count = mtg->busy_count;
-  old_idle_count = mtg->idle_count;
-  old_workload = mtg->workload;
-  mtg->workload += KheTaskWorkloadPerTime(task);
+  struct khe_busy_and_idle_rec old_busy_and_idle;
+  old_busy_and_idle = mtg->busy_and_idle;
+  mtg->busy_and_idle.workload += KheTaskWorkloadPerTime(task);
   if( busy_count_before == 0 )
   {
-    mtg->busy_count++;
+    mtg->busy_and_idle.busy_count++;
     if( mtg->state == KHE_MTG_STATE_ATTACHED_BUSY_AND_IDLE )
     {
       SSetInsert(mtg->busy_set,
 	KheTimeGroupTimePos(mtg->time_group, assigned_time_index));
-      mtg->idle_count = KheIdleTimes(mtg);
+      mtg->busy_and_idle.idle_count = KheIdleTimes(mtg);
     }
   }
-  KheMonitoredTimeGroupFlush(mtg, old_busy_count, old_idle_count,
-    old_workload);
+  KheMonitoredTimeGroupFlush(mtg, &old_busy_and_idle);
 }
 
 
@@ -576,25 +597,100 @@ void KheMonitoredTimeGroupAssign(KHE_MONITORED_TIME_GROUP mtg,
 void KheMonitoredTimeGroupUnAssign(KHE_MONITORED_TIME_GROUP mtg,
   KHE_TASK task, int assigned_time_index, int busy_count_after)
 {
-  int old_busy_count, old_idle_count;  float old_workload;
-  old_busy_count = mtg->busy_count;
-  old_idle_count = mtg->idle_count;
-  old_workload = mtg->workload;
-  mtg->workload -= KheTaskWorkloadPerTime(task);
+  struct khe_busy_and_idle_rec old_busy_and_idle;
+  old_busy_and_idle = mtg->busy_and_idle;
+  mtg->busy_and_idle.workload -= KheTaskWorkloadPerTime(task);
   if( busy_count_after == 0 )
   {
-    HnAssert(mtg->busy_count > 0,
+    HnAssert(mtg->busy_and_idle.busy_count > 0,
       "KheMonitoredTimeGroupUnAssignNonClash internal error");
-    mtg->busy_count--;
+    mtg->busy_and_idle.busy_count--;
     if( mtg->state == KHE_MTG_STATE_ATTACHED_BUSY_AND_IDLE )
     {
       SSetDelete(mtg->busy_set,
 	KheTimeGroupTimePos(mtg->time_group, assigned_time_index));
-      mtg->idle_count = KheIdleTimes(mtg);
+      mtg->busy_and_idle.idle_count = KheIdleTimes(mtg);
     }
   }
-  KheMonitoredTimeGroupFlush(mtg, old_busy_count, old_idle_count,
-    old_workload);
+  KheMonitoredTimeGroupFlush(mtg, &old_busy_and_idle);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheMonitoredTimeGroupSetSweepTime(KHE_MONITORED_TIME_GROUP mtg,     */
+/*    KHE_TIME t)                                                            */
+/*                                                                           */
+/*  Set the sweep time of mtg to t.  This will cause a flush if it           */
+/*  makes a change.                                                          */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheMonitoredTimeGroupSetSweepTime(KHE_MONITORED_TIME_GROUP mtg,
+  KHE_TIME t)
+{
+  KHE_TIME t2;  int i, count;  struct khe_busy_and_idle_rec old_busy_and_idle;
+  if( t != mtg->sweep_time )
+  {
+    /* find i, the index of the first time after t */
+    count = KheTimeGroupTimeCount(mtg->time_group);
+    if( t == NULL )
+    {
+      /* this means "before first time" */
+      i = 0;
+    }
+    else
+    {
+      /* initialize search spot */
+      if( mtg->sweep_time != NULL &&
+	  KheTimeIndex(t) > KheTimeIndex(mtg->sweep_time) )
+	i = mtg->sweep_index;
+      else
+	i = 0;
+
+      /* search forwards for the right time and index */
+      for( ;  i < count;  i++ )
+      {
+	t2 = KheTimeGroupTime(mtg->time_group, i);
+	if( KheTimeIndex(t2) > KheTimeIndex(t) )
+	  break;
+      }
+    }
+
+    /* update sweep index, sweep time, and open count */
+    old_busy_and_idle = mtg->busy_and_idle;
+    mtg->sweep_index = i;
+    mtg->sweep_time = t;
+    mtg->busy_and_idle.open_count = count - i;
+
+    /* and flush */
+    if( DEBUG2(mtg) )
+      fprintf(stderr, "  KheMonitoredTimeGroupSetSweepTime(%-5s) %s, %s flush "
+	"old(%3d, %6.1f, %3d) -> new(%3d, %6.1f, %3d)\n",
+	t == NULL ? "NULL" : KheTimeId(t),
+	KheResourceId(KheMonitoredTimeGroupResource(mtg)),
+	KheTimeGroupId(mtg->time_group),
+        old_busy_and_idle.busy_count, old_busy_and_idle.workload,
+        old_busy_and_idle.open_count, mtg->busy_and_idle.busy_count,
+        mtg->busy_and_idle.workload, mtg->busy_and_idle.open_count);
+    KheMonitoredTimeGroupFlush(mtg, &old_busy_and_idle);
+  }
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_BUSY_AND_IDLE KheMonitoredTimeGroupBusyAndIdle(                      */
+/*    KHE_MONITORED_TIME_GROUP mtg)                                          */
+/*                                                                           */
+/*  Return the current busy and idle value of mtg.                           */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_BUSY_AND_IDLE KheMonitoredTimeGroupBusyAndIdle(
+  KHE_MONITORED_TIME_GROUP mtg)
+{
+  return &mtg->busy_and_idle;
 }
 
 
@@ -606,10 +702,28 @@ void KheMonitoredTimeGroupUnAssign(KHE_MONITORED_TIME_GROUP mtg,
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** replaced by KheMonitoredTimeGroupBusyAndIdle
 int KheMonitoredTimeGroupBusyCount(KHE_MONITORED_TIME_GROUP mtg)
 {
-  return mtg->busy_count;
+  return mtg->busy_and_idle.busy_count;
 }
+*** */
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheMonitoredTimeGroupOpenCount(KHE_MONITORED_TIME_GROUP mtg)         */
+/*                                                                           */
+/*  Return the current number of open times.                                 */
+/*                                                                           */
+/*****************************************************************************/
+
+/* *** replaced by KheMonitoredTimeGroupBusyAndIdle
+int KheMonitoredTimeGroupOpenCount(KHE_MONITORED_TIME_GROUP mtg)
+{
+  return mtg->busy_and_idle.open_count;
+}
+*** */
 
 
 /*****************************************************************************/
@@ -620,10 +734,12 @@ int KheMonitoredTimeGroupBusyCount(KHE_MONITORED_TIME_GROUP mtg)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** replaced by KheMonitoredTimeGroupBusyAndIdle
 float KheMonitoredTimeGroupWorkload(KHE_MONITORED_TIME_GROUP mtg)
 {
-  return mtg->workload;
+  return mtg->busy_and_idle.workload;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -645,10 +761,10 @@ void KheMonitoredTimeGroupIdleState(KHE_MONITORED_TIME_GROUP mtg,
   KHE_TIME first, last;
   HnAssert( mtg->state == KHE_MTG_STATE_ATTACHED_BUSY_AND_IDLE,
     "KheMonitoredTimeGroupIdleState: mtg not monitoring idle");
-  *busy_count = mtg->busy_count;
-  *idle_count = mtg->idle_count;
+  *busy_count = mtg->busy_and_idle.busy_count;
+  *idle_count = mtg->busy_and_idle.idle_count;
   *extreme_busy_times_count = 0;
-  if( mtg->busy_count > 0 )
+  if( mtg->busy_and_idle.busy_count > 0 )
   {
     first = KheTimeGroupTime(mtg->time_group, SSetMin(mtg->busy_set));
     last  = KheTimeGroupTime(mtg->time_group, SSetMax(mtg->busy_set));
@@ -698,6 +814,24 @@ int KheMonitoredTimeGroupAtMaxLimitCount(KHE_MONITORED_TIME_GROUP mtg)
     }
   return res;
 }
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheMonitoredTimeGroupAvailable(KHE_MONITORED_TIME_GROUP mtg)        */
+/*                                                                           */
+/*  Return true if there are no avoid unavailable times monitors.            */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheMonitoredTimeGroupAvailable(KHE_MONITORED_TIME_GROUP mtg)
+{
+  int i;  KHE_MONITOR m;
+  HaArrayForEach(mtg->monitors, m, i)
+    if( KheMonitorTag(m) == KHE_AVOID_UNAVAILABLE_TIMES_MONITOR_TAG )
+      return false;
+  return true;
+}
 
 
 /*****************************************************************************/
@@ -738,12 +872,12 @@ static bool InRange(KHE_TIME ft, KHE_TIME lt, int first_time_index,
 
 void KheMonitoredTimeGroupAddRange(KHE_MONITORED_TIME_GROUP mtg,
   KHE_RESOURCE_TYPE rt, int first_time_index, int last_time_index,
-  KHE_GROUP_MONITOR gm)
+  bool include_avoid_unavailable_times, KHE_GROUP_MONITOR gm)
 {
   int i, count;  KHE_MONITOR m;  KHE_TIME ft, lt;
   KHE_CLUSTER_BUSY_TIMES_CONSTRAINT cbtc;  KHE_CLUSTER_BUSY_TIMES_MONITOR cbtm;
   KHE_LIMIT_BUSY_TIMES_CONSTRAINT lbtc;  KHE_LIMIT_BUSY_TIMES_MONITOR lbtm;
-  count = KheResourceTypeResourceCount(rt);
+  count = KheResourceTypeResourceCount(rt) / 2;
   HaArrayForEach(mtg->monitors, m, i)
     switch( KheMonitorTag(m) )
     {
@@ -751,26 +885,29 @@ void KheMonitoredTimeGroupAddRange(KHE_MONITORED_TIME_GROUP mtg,
 
 	cbtm = (KHE_CLUSTER_BUSY_TIMES_MONITOR) m;
 	cbtc = KheClusterBusyTimesMonitorConstraint(cbtm);
-	if( KheClusterBusyTimesConstraintResourceOfTypeCount(cbtc,rt) >= count )
-	{
-	  KheClusterBusyTimesMonitorRange(cbtm, &ft, &lt);
-	  if( InRange(ft, lt, first_time_index, last_time_index) &&
-	      !KheGroupMonitorHasChildMonitor(gm, m) )
-	    KheGroupMonitorAddChildMonitor(gm, m);
-	}
+	if( KheClusterBusyTimesConstraintResourceOfTypeCount(cbtc,rt) >= count
+	    && KheClusterBusyTimesMonitorSweepTimeRange(cbtm, &ft, &lt)
+	    && InRange(ft, lt, first_time_index, last_time_index)
+	    && !KheGroupMonitorHasChildMonitor(gm, m) )
+	  KheGroupMonitorAddChildMonitor(gm, m);
 	break;
 
       case KHE_LIMIT_BUSY_TIMES_MONITOR_TAG:
 
 	lbtm = (KHE_LIMIT_BUSY_TIMES_MONITOR) m;
 	lbtc = KheLimitBusyTimesMonitorConstraint(lbtm);
-	if( KheLimitBusyTimesConstraintResourceOfTypeCount(lbtc, rt) >= count )
-	{
-	  KheLimitBusyTimesMonitorRange(lbtm, &ft, &lt);
-	  if( InRange(ft, lt, first_time_index, last_time_index) &&
-	      !KheGroupMonitorHasChildMonitor(gm, m) )
-	    KheGroupMonitorAddChildMonitor(gm, m);
-	}
+	if( KheLimitBusyTimesConstraintResourceOfTypeCount(lbtc, rt) >= count
+	    && KheLimitBusyTimesMonitorSweepTimeRange(lbtm, &ft, &lt)
+	    && InRange(ft, lt, first_time_index, last_time_index)
+	    && !KheGroupMonitorHasChildMonitor(gm, m) )
+	  KheGroupMonitorAddChildMonitor(gm, m);
+	break;
+
+      case KHE_AVOID_UNAVAILABLE_TIMES_MONITOR_TAG:
+
+	if( include_avoid_unavailable_times &&
+	    !KheGroupMonitorHasChildMonitor(gm, m) )
+	  KheGroupMonitorAddChildMonitor(gm, m);
 	break;
 
       default:
@@ -802,9 +939,9 @@ void KheMonitoredTimeGroupDebug(KHE_MONITORED_TIME_GROUP mtg,
   {
     fprintf(fp, "%*s[ ", indent, "");
     KheTimeGroupDebug(mtg->time_group, 1, -1, fp);
-    fprintf(fp, " (%d busy", mtg->busy_count);
+    fprintf(fp, " (%d busy", mtg->busy_and_idle.busy_count);
     if( mtg->state == KHE_MTG_STATE_ATTACHED_BUSY_AND_IDLE )
-      fprintf(fp, ", %d idle", mtg->idle_count);
+      fprintf(fp, ", %d idle", mtg->busy_and_idle.idle_count);
     fprintf(fp, ") ]");
   }
 }
