@@ -77,7 +77,9 @@ KHE_ORDER_EVENTS_MONITOR KheOrderEventsMonitorMake(KHE_SOLN soln,
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_ORDER_EVENTS_MONITOR_TAG);
+    KHE_ORDER_EVENTS_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->constraint = c;
   res->first_event = first_event;
   res->second_event = second_event;
@@ -297,7 +299,7 @@ void KheOrderEventsMonitorAttachToSoln(KHE_ORDER_EVENTS_MONITOR m)
   KheEventInSolnAttachMonitor(es, (KHE_MONITOR) m);
   m->deviation = KheOrderEventsMonitorDev(m);
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
@@ -376,6 +378,77 @@ void KheOrderEventsMonitorAttachCheck(KHE_ORDER_EVENTS_MONITOR m)
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheOrderEventsMonitorTimeRange(KHE_ORDER_EVENTS_MONITOR m,          */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheOrderEventsMonitorTimeRange(KHE_ORDER_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_EVENT e;  KHE_TIME ft, lt;
+  *first_time = *last_time = NULL;
+  e = KheOrderEventsMonitorFirstEvent(m);
+  if( KheEventTimeRange(e, m->soln, &ft, &lt) )
+  {
+    if( *first_time == NULL || KheTimeIndex(ft) < KheTimeIndex(*first_time) )
+      *first_time = ft;
+    if( *last_time == NULL || KheTimeIndex(lt) > KheTimeIndex(*last_time) )
+      *last_time = ft;
+  }
+  e = KheOrderEventsMonitorSecondEvent(m);
+  if( KheEventTimeRange(e, m->soln, &ft, &lt) )
+  {
+    if( *first_time == NULL || KheTimeIndex(ft) < KheTimeIndex(*first_time) )
+      *first_time = ft;
+    if( *last_time == NULL || KheTimeIndex(lt) > KheTimeIndex(*last_time) )
+      *last_time = ft;
+  }
+  return *first_time != NULL;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheOrderEventsMonitorSetSweepTime(KHE_ORDER_EVENTS_MONITOR m,       */
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheOrderEventsMonitorSetSweepTime(KHE_ORDER_EVENTS_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheOrderEventsMonitorSweepTimeRange(KHE_ORDER_TIMES_MONITOR m,      */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for m.                                */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheOrderEventsMonitorSweepTimeRange(KHE_ORDER_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "fix and unfix"                                                */
@@ -472,7 +545,7 @@ void KheOrderEventsMonitorAddMeet(KHE_ORDER_EVENTS_MONITOR m, KHE_MEET meet)
   if( new_dev != m->deviation )
   {
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, new_dev));
+      KheMonitorDevToCost((KHE_MONITOR) m, new_dev));
     m->deviation = new_dev;
   }
 }
@@ -496,7 +569,7 @@ void KheOrderEventsMonitorDeleteMeet(KHE_ORDER_EVENTS_MONITOR m, KHE_MEET meet)
   if( new_dev != m->deviation )
   {
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, new_dev));
+      KheMonitorDevToCost((KHE_MONITOR) m, new_dev));
     m->deviation = new_dev;
   }
 }
@@ -562,7 +635,7 @@ void KheOrderEventsMonitorAssignTime(KHE_ORDER_EVENTS_MONITOR m,
   if( new_dev != m->deviation )
   {
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, new_dev));
+      KheMonitorDevToCost((KHE_MONITOR) m, new_dev));
     m->deviation = new_dev;
   }
 }
