@@ -29,6 +29,12 @@
 /*****************************************************************************/
 #include "khe_interns.h"
 
+#define DEBUG1 0
+#define DEBUG1_INDEX 6910
+#define DEBUG_ME(m)  (DEBUG1 && (m)->soln_index == DEBUG1_INDEX)
+
+#define DEBUG2 0
+
 
 /*****************************************************************************/
 /*                                                                           */
@@ -60,15 +66,63 @@ struct khe_ordinary_demand_monitor_rec {
 /*                                                                           */
 /*****************************************************************************/
 
+KHE_ORDINARY_DEMAND_MONITOR KheOrdinaryDemandMonitorMake(KHE_TASK task,
+  int offset, KHE_MONITOR orig_m)
+{
+  KHE_ORDINARY_DEMAND_MONITOR res;  HA_ARENA a;  KHE_SOLN soln;
+  soln = KheTaskSoln(task);
+  res = KheSolnGetOrdinaryDemandMonitorFromFreeList(soln);
+  if( res == NULL )
+  {
+    a = KheSolnArena(soln);
+    HaMake(res, a);
+    HaArrayInit(res->parent_links, a);
+  }
+  else
+    HaArrayClear(res->parent_links);
+  KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
+    KHE_ORDINARY_DEMAND_MONITOR_TAG, KHE_LINEAR_COST_FUNCTION, KheCost(1, 0));
+  res->demand_chunk = KheTaskDemandChunk(task, offset);
+  res->domain = *KheSolnMatchingZeroDomain(soln);
+  res->demand_asst = NULL;
+  res->demand_asst_index = NO_PREV_ASST;
+  res->unmatched_pos = -1;  /* initially not in unmatched list */
+  res->bfs_next = NULL;
+  res->bfs_parent = NULL;
+  res->hall_set = NULL;
+  res->task = task;
+  res->offset = offset;
+  res->copy = NULL;
+  KheTaskAddDemandMonitor(task, res);
+  if( DEBUG_ME(res) )
+  {
+    fprintf(stderr, "  Make: ");
+    KheMonitorDebug((KHE_MONITOR) res, 1, 2, stderr);
+  }
+  return res;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_ORDINARY_DEMAND_MONITOR KheOrdinaryDemandMonitorMake(KHE_SOLN soln,  */
+/*    KHE_MATCHING_DEMAND_CHUNK dc, KHE_TASK task, int offset)               */
+/*                                                                           */
+/*  Make an ordinary demand monitor with these attributes.                   */
+/*  Because it is unattached initially, its domain does not matter.          */
+/*                                                                           */
+/*****************************************************************************/
+
+/* *** old version
 KHE_ORDINARY_DEMAND_MONITOR KheOrdinaryDemandMonitorMake(KHE_SOLN soln,
   KHE_MATCHING_DEMAND_CHUNK dc, KHE_TASK task, int offset)
 {
   KHE_ORDINARY_DEMAND_MONITOR res;  HA_ARENA a;
-  /* *** silly
+  ** *** silly
   HnAssert(KheMatchingImpl(KheMatchingDemandChunkMatching(dc)) == (void *) soln,
     "KheOrdinaryDemandMonitorMake internal error (%p != %p)",
     KheMatchingImpl(KheMatchingDemandChunkMatching(dc)), (void *) soln);
-  *** */
+  *** **
   res = KheSolnGetOrdinaryDemandMonitorFromFreeList(soln);
   if( res == NULL )
   {
@@ -84,18 +138,19 @@ KHE_ORDINARY_DEMAND_MONITOR KheOrdinaryDemandMonitorMake(KHE_SOLN soln,
   res->domain = *KheSolnMatchingZeroDomain(soln);
   res->demand_asst = NULL;
   res->demand_asst_index = NO_PREV_ASST;
-  res->unmatched_pos = -1;  /* undefined at this point */
+  res->unmatc hed_pos = -1;  ** undefined at this point **
   res->bfs_next = NULL;
   res->bfs_parent = NULL;
   res->hall_set = NULL;
   res->task = task;
   res->offset = offset;
   res->copy = NULL;
-  /* KheGroupMonitorAddMonitor((KHE_GROUP_MONITOR) soln, (KHE_MONITOR) res); */
+  ** KheGroupMonitorAddMonitor((KHE_GROUP_MONITOR) soln, (KHE_MONITOR) res); **
   KheTaskAddDemandMonitor(task, res);
-  /* KheMonitorCheck((KHE_MONITOR) res); */
+  ** KheMonitorCheck((KHE_MONITOR) res); **
   return res;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -140,6 +195,11 @@ int KheOrdinaryDemandMonitorOffset(KHE_ORDINARY_DEMAND_MONITOR m)
 void KheOrdinaryDemandMonitorSetTaskAndOffset(KHE_ORDINARY_DEMAND_MONITOR m,
   KHE_TASK task, int offset)
 {
+  if( DEBUG_ME(m) )
+  {
+    fprintf(stderr, "  SetTaskAndOffset(%s, %d): ", KheTaskId(task), offset);
+    KheMonitorDebug((KHE_MONITOR) m, 1, 2, stderr);
+  }
   m->task = task;
   m->offset = offset;
 }
@@ -164,7 +224,7 @@ KHE_ORDINARY_DEMAND_MONITOR KheOrdinaryDemandMonitorCopyPhase1(
     m->copy = copy;
     KheMonitorCopyCommonFieldsPhase1((KHE_MONITOR) copy, (KHE_MONITOR) m, a);
     copy->demand_chunk = KheMatchingDemandChunkCopyPhase1(m->demand_chunk, a);
-    copy->domain = m->domain;
+    KheSetCopy(copy->domain, m->domain, a);
     copy->demand_asst = (m->demand_asst == NULL ? NULL :
       KheMatchingSupplyNodeCopyPhase1(m->demand_asst, a));
     copy->demand_asst_index = m->demand_asst_index;
@@ -178,6 +238,11 @@ KHE_ORDINARY_DEMAND_MONITOR KheOrdinaryDemandMonitorCopyPhase1(
     copy->task = KheTaskCopyPhase1(m->task, a);
     copy->offset = m->offset;
     copy->copy = NULL;
+    if( DEBUG_ME(m) )
+    {
+      fprintf(stderr, "  Copy: ");
+      KheMonitorDebug((KHE_MONITOR) m, 1, 2, stderr);
+    }
   }
   return m->copy;
 }
@@ -217,6 +282,11 @@ void KheOrdinaryDemandMonitorCopyPhase2(KHE_ORDINARY_DEMAND_MONITOR m)
 
 void KheOrdinaryDemandMonitorDelete(KHE_ORDINARY_DEMAND_MONITOR m)
 {
+  if( DEBUG_ME(m) )
+  {
+    fprintf(stderr, "  Delete: ");
+    KheMonitorDebug((KHE_MONITOR) m, 1, 2, stderr);
+  }
   if( m->attached )
     KheOrdinaryDemandMonitorDetachFromSoln(m);
   KheMonitorDeleteAllParentMonitors((KHE_MONITOR) m);
@@ -252,6 +322,11 @@ void KheOrdinaryDemandMonitorAttachToSoln(KHE_ORDINARY_DEMAND_MONITOR m)
   KheMatchingDemandNodeAdd((KHE_MATCHING_DEMAND_NODE) m);
   m->attached = true;
   /* KheMonitorCheck((KHE_MONITOR) m); */
+  if( DEBUG_ME(m) )
+  {
+    fprintf(stderr, "  Attach: ");
+    KheMonitorDebug((KHE_MONITOR) m, 1, 2, stderr);
+  }
 }
 
 
@@ -269,6 +344,11 @@ void KheOrdinaryDemandMonitorDetachFromSoln(KHE_ORDINARY_DEMAND_MONITOR m)
   KheMatchingDemandNodeDelete((KHE_MATCHING_DEMAND_NODE) m);
   m->attached = false;
   /* KheMonitorCheck((KHE_MONITOR) m); */
+  if( DEBUG_ME(m) )
+  {
+    fprintf(stderr, "  Detach: ");
+    KheMonitorDebug((KHE_MONITOR) m, 1, 2, stderr);
+  }
 }
 
 
@@ -315,6 +395,15 @@ void KheOrdinaryDemandMonitorSetDomain(KHE_ORDINARY_DEMAND_MONITOR m,
     /* KheResourceGroupResourceIndexes(rg), */
     KheResourceGroupKheSet(rg), change_type);
   /* KheMonitorCheck((KHE_MONITOR) m); */
+  if( DEBUG_ME(m) )
+  {
+    fprintf(stderr, "  SetDomain ");
+    /* ***
+    KheResourceGroupDebug(rg, 2, -1, stderr);
+    fprintf(stderr, ": ");
+    *** */
+    KheMonitorDebug((KHE_MONITOR) m, 1, 0, stderr);
+  }
 }
 
 
@@ -334,6 +423,44 @@ void KheOrdinaryDemandMonitorSetWeight(KHE_ORDINARY_DEMAND_MONITOR m,
   if( m->demand_asst == NULL )
     KheMonitorChangeCost((KHE_MONITOR) m, new_weight);
   /* KheMonitorCheck((KHE_MONITOR) m); */
+  if( DEBUG_ME(m) )
+  {
+    fprintf(stderr, "  Weight: ");
+    KheMonitorDebug((KHE_MONITOR) m, 1, 2, stderr);
+  }
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheOrdinaryDemandMonitorCheckConsistency(                           */
+/*    KHE_ORDINARY_DEMAND_MONITOR m, KHE_RESOURCE_GROUP rg)                  */
+/*                                                                           */
+/*  Check the consistency of m's domain with rg.                             */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheOrdinaryDemandMonitorCheckConsistency(
+  KHE_ORDINARY_DEMAND_MONITOR m, KHE_RESOURCE_GROUP rg)
+{
+  KHE_SET rg_set;  KHE_TASK task;  char buff1[200], buff2[200];
+  if( DEBUG2 )
+  {
+    rg_set = *KheResourceGroupKheSet(rg);
+    if( !KheSetEqual(rg_set, m->domain) )
+    {
+      fprintf(stderr, "KheOrdinaryDemandMonitorCheckConsistency"
+	" internal error: ");
+      for( task = m->task;  task != NULL;  task = KheTaskAsst(task) )
+      {
+	if( task != m->task )
+	  fprintf(stderr, " -> ");
+	fprintf(stderr, "%s", KheTaskId(task));
+      }
+      HnAbort(" %s !=  m %s", KheSetShow(rg_set, buff1),
+	KheSetShow(m->domain, buff2));
+    }
+  }
 }
 
 
@@ -350,7 +477,7 @@ void KheOrdinaryDemandMonitorSetWeight(KHE_ORDINARY_DEMAND_MONITOR m,
 void KheOrdinaryDemandMonitorChangeType(KHE_ORDINARY_DEMAND_MONITOR m,
   KHE_MATCHING_TYPE old_mt, KHE_MATCHING_TYPE new_mt)
 {
-  KheMatchingDemandNodeSetDomain((KHE_MATCHING_DEMAND_NODE) m,
+  KheMatchingDeman dNodeSetDomain((KHE_MATCHING_DEMAND_NODE) m,
     KheResourceGroupResourceIndexes(KheTaskDomain(m->task)),
     KHE_MATCHING_DOMAIN_CHANGE_TO_OTHER);
 }
@@ -373,7 +500,8 @@ void KheOrdinaryDemandMonitorChangeType(KHE_ORDINARY_DEMAND_MONITOR m,
 
 int KheOrdinaryDemandMonitorDeviation(KHE_ORDINARY_DEMAND_MONITOR m)
 {
-  return m->demand_asst != NULL ? 1 : 0;
+  return m->demand_asst == NULL ? 1 : 0;
+  /* return m->demand_asst != NULL ? 1 : 0; swapped around 18/12/23 JeffK */
 }
 
 
@@ -389,7 +517,8 @@ int KheOrdinaryDemandMonitorDeviation(KHE_ORDINARY_DEMAND_MONITOR m)
 char *KheOrdinaryDemandMonitorDeviationDescription(
   KHE_ORDINARY_DEMAND_MONITOR m)
 {
-  return m->demand_asst != NULL ? "1" : "0";
+  return m->demand_asst == NULL ? "1" : "0";
+  /* return m->demand_asst != NULL ? "1" : "0"; swapped around 18/12/23 JeffK */
   /* ***
   ARRAY_CHAR ac;
   MStringInit(ac);
@@ -475,13 +604,15 @@ char *KheOrdinaryDemandMonitorDeviationDescription(
 void KheOrdinaryDemandMonitorDebug(KHE_ORDINARY_DEMAND_MONITOR m,
   int verbosity, int indent, FILE *fp)
 {
+  char buff[200];
   if( verbosity >= 1 )
   {
     KheMonitorDebugBegin((KHE_MONITOR) m, indent, fp);
-    fprintf(fp, " ");
+    fprintf(fp, " %p ", (void *) m);
     KheTaskDebug(m->task, 1, -1, fp);
-    fprintf(fp, "+%d:", m->offset);
+    fprintf(fp, "+%d: ", m->offset);
     KheResourceGroupDebug(KheTaskDomain(m->task), 1, -1, fp);
+    fprintf(fp, " : %s", KheSetShow(m->domain, buff));
     KheMonitorDebugEnd((KHE_MONITOR) m, true, indent, fp);
   }
 }
