@@ -43,6 +43,7 @@ struct khe_prefer_resources_constraint_rec {
   ARRAY_KHE_RESOURCE_GROUP	resource_groups;	/* resource groups   */
   ARRAY_KHE_RESOURCE		resources;		/* resources         */
   KHE_RESOURCE_GROUP		domain; 		/* total of above    */
+  KHE_RESOURCE_GROUP		neg_domain; 		/* complement of d.  */
   ARRAY_KHE_EVENT_RESOURCE	event_resources;	/* applies to        */
   ARRAY_KHE_EVENT		events;			/* Events            */
   ARRAY_KHE_EVENT_GROUP		event_groups;		/* EventGroups       */
@@ -87,6 +88,7 @@ bool KhePreferResourcesConstraintMake(KHE_INSTANCE ins, char *id,
   HaArrayInit(res->resource_groups, a);
   HaArrayInit(res->resources, a);
   res->domain = NULL;
+  res->neg_domain = NULL;
   HaArrayInit(res->events, a);
   HaArrayInit(res->event_groups, a);
   HaArrayInit(res->event_resources, a);
@@ -177,6 +179,8 @@ int KhePreferResourcesConstraintAppliesToCount(
 void KhePreferResourcesConstraintFinalize(KHE_PREFER_RESOURCES_CONSTRAINT c)
 {
   int i;  KHE_RESOURCE_GROUP rg;  KHE_RESOURCE r;
+
+  /* set the domain of c */
   if( HaArrayCount(c->resource_groups) == 0 && HaArrayCount(c->resources) == 1 )
     c->domain = KheResourceSingletonResourceGroup(HaArrayFirst(c->resources));
   else if( HaArrayCount(c->resource_groups) == 1 &&
@@ -192,6 +196,14 @@ void KhePreferResourcesConstraintFinalize(KHE_PREFER_RESOURCES_CONSTRAINT c)
       KheResourceGroupAddResourceInternal(c->domain, r);
     KheResourceGroupFinalize(c->domain);
   }
+
+  /* set the complement of the domain of c */
+  c->neg_domain = KheResourceGroupMakeInternal(c->resource_type,
+    KHE_RESOURCE_GROUP_TYPE_CONSTRUCTED, NULL, NULL, NULL);
+  KheResourceGroupUnionInternal(c->neg_domain,
+    KheResourceTypeFullResourceGroup(KheResourceGroupResourceType(c->domain)));
+  KheResourceGroupDifferenceInternal(c->neg_domain, c->domain);
+  KheResourceGroupFinalize(c->neg_domain);
 }
 
 
@@ -363,6 +375,24 @@ KHE_RESOURCE_GROUP KhePreferResourcesConstraintDomain(
     "KhePreferResourcesConstraintDomain called before KheInstanceMakeEnd");
   return c->domain;
 }
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_RESOURCE_GROUP KhePreferResourcesConstraintDomainComplement(         */
+/*    KHE_PREFER_RESOURCES_CONSTRAINT c)                                     */
+/*                                                                           */
+/*  Return the complement of the domain of c.                                */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_RESOURCE_GROUP KhePreferResourcesConstraintDomainComplement(
+  KHE_PREFER_RESOURCES_CONSTRAINT c)
+{
+  HnAssert(c->neg_domain != NULL, "KhePreferResourcesConstraintDomainComplement"
+    " called before KheInstanceMakeEnd");
+  return c->neg_domain;
+}
 
 
 /*****************************************************************************/
