@@ -30,6 +30,7 @@
 #include "khe_interns.h"
 
 #define DEBUG1 0
+#define DEBUG2 0
 
 
 /*****************************************************************************/
@@ -52,6 +53,36 @@ struct khe_prefer_resources_monitor_rec {
 /*                                                                           */
 /*****************************************************************************/
 
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_PREFER_RESOURCES_MONITOR KhePreferResourcesMonitorMakeInternal(      */
+/*    KHE_EVENT_RESOURCE_IN_SOLN ers, KHE_PREFER_RESOURCES_CONSTRAINT c,     */
+/*    KHE_COST_FUNCTION cf, KHE_COST combined_weight)                        */
+/*                                                                           */
+/*  Make a new prefer resources monitor with these attributes.               */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_PREFER_RESOURCES_MONITOR KhePreferResourcesMonitorMakeInternal(
+  KHE_EVENT_RESOURCE_IN_SOLN ers, KHE_PREFER_RESOURCES_CONSTRAINT c,
+  KHE_COST_FUNCTION cf, KHE_COST combined_weight)
+{
+  KHE_PREFER_RESOURCES_MONITOR res;  KHE_SOLN soln;  HA_ARENA a;
+  soln = KheEventInSolnSoln(KheEventResourceInSolnEventInSoln(ers));
+  a = KheSolnArena(soln);
+  HaMake(res, a);
+  HaArrayInit(res->parent_links, a);
+  KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
+    KHE_PREFER_RESOURCES_MONITOR_TAG, cf, combined_weight);
+  res->deviation = 0;
+  res->event_resource_in_soln = ers;
+  res->constraint = c;
+  res->copy = NULL;
+  KheEventResourceInSolnAddMonitor(ers, (KHE_MONITOR) res);
+  return res;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  KHE_PREFER_RESOURCES_MONITOR KhePreferResourcesMonitorMake(              */
@@ -64,19 +95,9 @@ struct khe_prefer_resources_monitor_rec {
 KHE_PREFER_RESOURCES_MONITOR KhePreferResourcesMonitorMake(
   KHE_EVENT_RESOURCE_IN_SOLN ers, KHE_PREFER_RESOURCES_CONSTRAINT c)
 {
-  KHE_PREFER_RESOURCES_MONITOR res;  KHE_SOLN soln;  HA_ARENA a;
-  soln = KheEventInSolnSoln(KheEventResourceInSolnEventInSoln(ers));
-  a = KheSolnArena(soln);
-  HaMake(res, a);
-  HaArrayInit(res->parent_links, a);
-  KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_PREFER_RESOURCES_MONITOR_TAG);
-  res->deviation = 0;
-  res->event_resource_in_soln = ers;
-  res->constraint = c;
-  res->copy = NULL;
-  KheEventResourceInSolnAddMonitor(ers, (KHE_MONITOR) res);
-  return res;
+  return KhePreferResourcesMonitorMakeInternal(ers, c,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
 }
 
 
@@ -122,6 +143,7 @@ void KhePreferResourcesMonitorCopyPhase2(KHE_PREFER_RESOURCES_MONITOR m)
   {
     m->copy = NULL;
     KheMonitorCopyCommonFieldsPhase2((KHE_MONITOR) m);
+    KheEventResourceInSolnCopyPhase2(m->event_resource_in_soln);
   }
 }
 
@@ -178,6 +200,54 @@ KHE_EVENT_RESOURCE KhePreferResourcesMonitorEventResource(
 {
   return KheEventResourceInSolnEventResource(m->event_resource_in_soln);
 }
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_RESOURCE_GROUP KhePreferResourcesMonitorDomain(                      */
+/*    KHE_PREFER_RESOURCES_MONITOR m)                                        */
+/*                                                                           */
+/*  Return the domain of m.                                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_RESOURCE_GROUP KhePreferResourcesMonitorDomain(
+  KHE_PREFER_RESOURCES_MONITOR m)
+{
+  KHE_RESOURCE_TYPE rt;  KHE_EVENT_RESOURCE er;
+  if( m->constraint == NULL )
+  {
+    er = KheEventResourceInSolnEventResource(m->event_resource_in_soln);
+    rt = KheEventResourceResourceType(er);
+    return KheResourceTypeEmptyResourceGroup(rt);
+  }
+  else
+    return KhePreferResourcesConstraintDomain(m->constraint);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_RESOURCE_GROUP KhePreferResourcesMonitorDomainComplement(            */
+/*    KHE_PREFER_RESOURCES_MONITOR m)                                        */
+/*                                                                           */
+/*  Return the complement of the domain of m.                                */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_RESOURCE_GROUP KhePreferResourcesMonitorDomainComplement(
+  KHE_PREFER_RESOURCES_MONITOR m)
+{
+  KHE_RESOURCE_TYPE rt;  KHE_EVENT_RESOURCE er;
+  if( m->constraint == NULL )
+  {
+    er = KheEventResourceInSolnEventResource(m->event_resource_in_soln);
+    rt = KheEventResourceResourceType(er);
+    return KheResourceTypeFullResourceGroup(rt);
+  }
+  else
+    return KhePreferResourcesConstraintDomainComplement(m->constraint);
+}
 
 
 /*****************************************************************************/
@@ -201,6 +271,9 @@ void KhePreferResourcesMonitorAttachToSoln(KHE_PREFER_RESOURCES_MONITOR m)
   m->attached = true;
   KheEventResourceInSolnAttachMonitor(m->event_resource_in_soln,
     (KHE_MONITOR) m);
+  if( DEBUG2 && m->constraint == NULL )
+    fprintf(stderr, "  KhePreferResourcesMonitorAttachToSoln(%s)\n",
+      KheMonitorId((KHE_MONITOR) m));
 }
 
 
@@ -218,6 +291,9 @@ void KhePreferResourcesMonitorDetachFromSoln(KHE_PREFER_RESOURCES_MONITOR m)
   KheEventResourceInSolnDetachMonitor(m->event_resource_in_soln,
     (KHE_MONITOR) m);
   m->attached = false;
+  if( DEBUG2 && m->constraint == NULL )
+    fprintf(stderr, "  KhePreferResourcesMonitorDetachFromSoln(%s)\n",
+      KheMonitorId((KHE_MONITOR) m));
 }
 
 
@@ -235,7 +311,7 @@ static bool KhePreferResourcesMonitorMustHaveZeroCost(
   KHE_PREFER_RESOURCES_MONITOR m)
 {
   KHE_RESOURCE_GROUP domain;  KHE_EVENT_RESOURCE er;  KHE_TASK task;  int i;
-  domain = KhePreferResourcesConstraintDomain(m->constraint);
+  domain = KhePreferResourcesMonitorDomain(m);
   er = KhePreferResourcesMonitorEventResource(m);
   for( i = 0;  i < KheEventResourceTaskCount(m->soln, er);  i++ )
   {
@@ -325,6 +401,72 @@ void KhePreferResourcesMonitorTaskDomainUnFix(KHE_PREFER_RESOURCES_MONITOR m)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KhePreferResourcesMonitorTimeRange(KHE_PREFER_RESOURCES_MONITOR m,  */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for prefer resources monitor m.       */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KhePreferResourcesMonitorTimeRange(KHE_PREFER_RESOURCES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_EVENT_RESOURCE er;
+  er = KhePreferResourcesMonitorEventResource(m);
+  return KheEventResourceTimeRange(er, m->soln, first_time, last_time);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KhePreferResourcesMonitorSetSweepTime(                              */
+/*    KHE_PREFER_RESOURCES_MONITOR m, KHE_TIME t)                            */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for prefer resources monitor m.         */
+/*                                                                           */
+/*  Implementation note.  Installing a cutoff time makes no difference       */
+/*  to this monitor, because an unassigned task has the same effect on       */
+/*  it as an open task, so there is nothing to do here.                      */
+/*                                                                           */
+/*****************************************************************************/
+
+void KhePreferResourcesMonitorSetSweepTime(
+  KHE_PREFER_RESOURCES_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KhePreferResourcesMonitorSweepTimeRange(                            */
+/*    KHE_PREFER_RESOURCES_MONITOR m, KHE_TIME *first_time,                  */
+/*    KHE_TIME *last_time)                                                   */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for prefer resources monitor m.       */
+/*                                                                           */
+/*  Implementation note.  Installing a cutoff time makes no difference       */
+/*  to this monitor, because an unassigned task has the same effect on       */
+/*  it as an open task, so there is nothing to do here.                      */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KhePreferResourcesMonitorSweepTimeRange(
+  KHE_PREFER_RESOURCES_MONITOR m, KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  /* nothing to do */
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -343,8 +485,8 @@ void KhePreferResourcesMonitorTaskDomainUnFix(KHE_PREFER_RESOURCES_MONITOR m)
 static bool KhePreferResourcesMonitorWrongResource(
   KHE_PREFER_RESOURCES_MONITOR m, KHE_RESOURCE r)
 {
-  return r != NULL && !KheResourceGroupContains(
-    KhePreferResourcesConstraintDomain(m->constraint), r);
+  return r != NULL &&
+    !KheResourceGroupContains(KhePreferResourcesMonitorDomain(m), r);
 }
 
 
@@ -364,7 +506,7 @@ void KhePreferResourcesMonitorAddTask(KHE_PREFER_RESOURCES_MONITOR m,
   {
     m->deviation += KheTaskDuration(task);
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -387,7 +529,7 @@ void KhePreferResourcesMonitorDeleteTask(KHE_PREFER_RESOURCES_MONITOR m,
     HnAssert(m->deviation >= 0,
       "KhePreferResourcesMonitorDeleteTask internal error");
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -409,17 +551,16 @@ void KhePreferResourcesMonitorSplitTask(KHE_PREFER_RESOURCES_MONITOR m,
   /* no change in cost is possible */
   /* ***
   KHE_COST new_cost;  int durn1, durn2;
-  if( KheConstraintCostFunction((KHE_CONSTRAINT) m->constraint) !=
-        KHE_SUM_COST_FUNCTION &&
+  if( KheConstraintCostFunction((KHE_CONSTRAINT) m->constr aint) !=
       KhePreferResourcesMonitorWrongResource(m, KheTaskAsstResource(task1)) )
   {
     ** delete 1 old, add 2 new **
     durn1 = KheTaskDuration(task1);
     durn2 = KheTaskDuration(task2);
     new_cost = m->cost
-      + KheConstraintCost((KHE_CONSTRAINT) m->constraint, durn1)
-      + KheConstraintCost((KHE_CONSTRAINT) m->constraint, durn2)
-      - KheConstraintCost((KHE_CONSTRAINT) m->constraint, durn1 + durn2);
+      + KheMonitorDevToCost((KHE_MONITOR) m, durn1)
+      + KheMonitorDevToCost((KHE_MONITOR) m, durn2)
+      - KheMonitorDevToCost((KHE_MONITOR) m, durn1 + durn2);
 
     ** report any change in cost **
     KheMonitorChangeCost((KHE_MONITOR) m, new_cost);
@@ -448,7 +589,7 @@ void KhePreferResourcesMonitorMergeTask(KHE_PREFER_RESOURCES_MONITOR m,
   /* no change in cost is possible */
   /* ***
   KHE_COST new_cost;  int durn1, durn2;
-  if( KheConstraintCostFunction((KHE_CONSTRAINT) m->constraint)
+  if( KheConstraintCostFunction((KHE_CONSTRAINT) m->const raint)
         != KHE_SUM_COST_FUNCTION &&
       KhePreferResourcesMonitorWrongResource(m, KheTaskAsstResource(task1)) )
   {
@@ -456,9 +597,9 @@ void KhePreferResourcesMonitorMergeTask(KHE_PREFER_RESOURCES_MONITOR m,
     durn1 = KheTaskDuration(task1);
     durn2 = KheTaskDuration(task2);
     new_cost = m->cost
-      - KheConstraintCost((KHE_CONSTRAINT) m->constraint, durn1)
-      - KheConstraintCost((KHE_CONSTRAINT) m->constraint, durn2)
-      + KheConstraintCost((KHE_CONSTRAINT) m->constraint, durn1 + durn2);
+      - KheMonitorDevToCost((KHE_MONITOR) m, durn1)
+      - KheMonitorDevToCost((KHE_MONITOR) m, durn2)
+      + KheMonitorDevToCost((KHE_MONITOR) m, durn1 + durn2);
 
     ** report any change in cost **
     KheMonitorChangeCost((KHE_MONITOR) m, new_cost);
@@ -483,7 +624,7 @@ void KhePreferResourcesMonitorAssignResource(KHE_PREFER_RESOURCES_MONITOR m,
   {
     m->deviation += KheTaskDuration(task);
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -504,7 +645,7 @@ void KhePreferResourcesMonitorUnAssignResource(KHE_PREFER_RESOURCES_MONITOR m,
   {
     m->deviation -= KheTaskDuration(task);
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -723,7 +864,10 @@ char *KhePreferResourcesMonitorId(KHE_PREFER_RESOURCES_MONITOR m)
   if( m->id == NULL )
   {
     a = KheSolnArena(m->soln);
-    constraint_id = KheConstraintId((KHE_CONSTRAINT) m->constraint);
+    if( m->constraint == NULL )
+      constraint_id = "PreferResourcesRedundancy";
+    else
+      constraint_id = KheConstraintId((KHE_CONSTRAINT) m->constraint);
     er = KhePreferResourcesMonitorEventResource(m);
     event_id = KheEventId(KheEventResourceEvent(er));
     role = KheEventResourceRole(er);
