@@ -158,7 +158,7 @@ void KhePreferTimesConstraintFinalize(KHE_PREFER_TIMES_CONSTRAINT c)
     HaArrayForEach(c->times, t, i)
       SSetInsert(ss, KheTimeIndex(t));
     c->domain = KheTimeGroupMakeAndFinalize(c->instance,
-      KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, NULL, true);
+      KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, /* NULL, */ true, a);
     /* ***
     c->domain = KheTimeGroup MakeInternal(c->instance,
       ** KHE_TIME_GROUP_TYPE_CONSTRUCTED, ** NULL, KHE_TIME_GROUP_KIND_AUTO,
