@@ -68,7 +68,9 @@ KHE_PREFER_TIMES_MONITOR KhePreferTimesMonitorMake(KHE_EVENT_IN_SOLN es,
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_PREFER_TIMES_MONITOR_TAG);
+    KHE_PREFER_TIMES_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->deviation = 0;
   res->event_in_soln = es;
   res->constraint = c;
@@ -119,6 +121,7 @@ void KhePreferTimesMonitorCopyPhase2(KHE_PREFER_TIMES_MONITOR m)
   {
     m->copy = NULL;
     KheMonitorCopyCommonFieldsPhase2((KHE_MONITOR) m);
+    KheEventInSolnCopyPhase2(m->event_in_soln);
   }
 }
 
@@ -322,6 +325,61 @@ void KhePreferTimesMonitorUnFix(KHE_PREFER_TIMES_MONITOR m)
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KhePreferTimesMonitorTimeRange(KHE_PREFER_TIMES_MONITOR m,          */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KhePreferTimesMonitorTimeRange(KHE_PREFER_TIMES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_EVENT e;
+  e = KhePreferTimesMonitorEvent(m);
+  return KheEventTimeRange(e, m->soln, first_time, last_time);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KhePreferTimesMonitorSetSweepTime(KHE_PREFER_TIMES_MONITOR m,       */
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+void KhePreferTimesMonitorSetSweepTime(KHE_PREFER_TIMES_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KhePreferTimesMonitorSweepTimeRange(KHE_PREFER_TIMES_MONITOR m,     */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for m.                                */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KhePreferTimesMonitorSweepTimeRange(KHE_PREFER_TIMES_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -399,7 +457,7 @@ void KhePreferTimesMonitorAddMeet(KHE_PREFER_TIMES_MONITOR m, KHE_MEET meet)
   {
     m->deviation += durn;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -424,7 +482,7 @@ void KhePreferTimesMonitorDeleteMeet(KHE_PREFER_TIMES_MONITOR m, KHE_MEET meet)
     HnAssert(m->deviation >= 0,
       "KhePreferTimesMonitorDeleteMeet internal error");
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -467,7 +525,7 @@ void KhePreferTimesMonitorSplitMeet(KHE_PREFER_TIMES_MONITOR m,
     HnAssert(m->deviation >= 0,
       "KhePreferTimesMonitorSplitMeet internal error");
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -519,7 +577,7 @@ void KhePreferTimesMonitorMergeMeet(KHE_PREFER_TIMES_MONITOR m,
     HnAssert(m->deviation >= 0,
       "KhePreferTimesMonitorMergeMeet internal error");
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -543,7 +601,7 @@ void KhePreferTimesMonitorAssignTime(KHE_PREFER_TIMES_MONITOR m,
   {
     m->deviation += durn;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -569,7 +627,7 @@ void KhePreferTimesMonitorUnAssignTime(KHE_PREFER_TIMES_MONITOR m,
     HnAssert(m->deviation >= 0,
       "KhePreferTimesMonitorUnAssignTime internal error");
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
