@@ -122,7 +122,7 @@ bool KheResourceMake(KHE_RESOURCE_TYPE rt, char *id, char *name,
   KheResourceGroupAddResourceInternal(KheResourceTypeFullResourceGroup(rt),res);
   *** */
   res->singleton_resource_group = KheResourceGroupMakeInternal(rt,
-    KHE_RESOURCE_GROUP_TYPE_SINGLETON, NULL, NULL, NULL /* , LSetNew() */);
+    KHE_RESOURCE_GROUP_TYPE_SINGLETON, NULL, res->id, res->name /*,LSetNew()*/);
   KheResourceGroupAddResourceInternal(res->singleton_resource_group, res);
   res->hard_unavail = NULL;  /* finalized at instance end */
   res->hard_and_soft_unavail = NULL;  /* finalized at instance end */
@@ -351,7 +351,7 @@ bool KheResourceSimilar(KHE_RESOURCE r1, KHE_RESOURCE r2)
 
   /* scratch memory for this operation */
   ins = KheResourceInstance(r1);
-  a = KheInstanceArenaBegin(ins, false);
+  a = KheInstanceArenaBegin(ins);
 
   /* find the admissible resource groups of the two resources */
   HaArrayInit(resource_groups1, a);
@@ -733,6 +733,49 @@ static void KheResourceMoveToResourceType(KHE_RESOURCE r, KHE_RESOURCE_TYPE rt)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheResourceGroupMatchesPartition(KHE_RESOURCE_GROUP rg,             */
+/*    KHE_RESOURCE r)                                                        */
+/*                                                                           */
+/*  Return true if rg matches r's partition exactly.                         */
+/*                                                                           */
+/*****************************************************************************/
+
+static bool KheResourceGroupMatchesPartition(KHE_RESOURCE_GROUP rg,
+  KHE_RESOURCE r)
+{
+  int i;  KHE_RESOURCE r2;
+  if( KheResourceGroupResourceCount(rg) != HaArrayCount(r->rt_partition) )
+    return false;
+  HaArrayForEach(r->rt_partition, r2, i)
+    if( !KheResourceGroupContains(rg, r2) )
+      return false;
+  return true;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheResourcePartitionEqualsResourceGroup(KHE_RESOURCE r,             */
+/*    KHE_RESOURCE_GROUP *rg)                                                */
+/*                                                                           */
+/*  If r's resource partition equals some user resource group, return true   */
+/*  with *rg set to that resource group.  Otherwise return false.            */
+/*                                                                           */
+/*****************************************************************************/
+
+static bool KheResourcePartitionEqualsResourceGroup(KHE_RESOURCE r,
+  KHE_RESOURCE_GROUP *rg)
+{
+  KHE_RESOURCE_GROUP rg2;  int i;
+  HaArrayForEach(r->user_resource_groups, rg2, i)
+    if( KheResourceGroupMatchesPartition(rg2, r) )
+      return *rg = rg2, true;
+  return *rg = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheResourceResourceTypePartitionMakeResourceType(KHE_RESOURCE r)    */
@@ -740,9 +783,17 @@ static void KheResourceMoveToResourceType(KHE_RESOURCE r, KHE_RESOURCE_TYPE rt)
 /*  Make a resource type based on r, if it is its own root.                  */
 /*                                                                           */
 /*  Implementation note.  This function will not make too many resource      */
+/*  types, because it makes one resource type for each resource r such that  */
+/*  r->rt_partition_parent == NULL, and only when r's partition contains     */
+/*  at most half the number of resources in r's *current* resource type.     */
+/*                                                                           */
+/*  Obsolete below here                                                      */
+/*  -------------------                                                      */
+/*                                                                           */
+/*  Implementation note.  This function will not make too many resource      */
 /*  types, because it makes one resource type for each resource such that    */
 /*  r->rt_partition_parent == NULL, and not when r's current resource type   */
-/*  contains the same number of resources as r's partition.                  */
+/*  contains the same number of resources as r's partition.  (obsolete)      */
 /*                                                                           */
 /*  Implementation note.  This function will not make too many resource      */
 /*  types, because it makes one resource type for each resource such that    */
@@ -754,48 +805,42 @@ static void KheResourceMoveToResourceType(KHE_RESOURCE r, KHE_RESOURCE_TYPE rt)
 void KheResourceResourceTypePartitionMakeResourceType(KHE_RESOURCE r)
 {
   KHE_RESOURCE r2;  int i;  KHE_RESOURCE_TYPE rt, new_rt;
-  char buff[1000];  KHE_INSTANCE ins;
+  char buff[1000], *str;  KHE_INSTANCE ins;  KHE_RESOURCE_GROUP rg;
   rt = r->resource_type;
   if( r->rt_partition_parent == NULL &&
       KheResourceTypeResourceTypePartitioningOn(rt) &&
-      HaArrayCount(r->rt_partition) < KheResourceTypeResourceCount(rt) )
+      HaArrayCount(r->rt_partition) <= KheResourceTypeResourceCount(rt) / 2 )
   {
-    /* make a new resource type, except not if r is rt's first */
+    /* make resource type new_rt */
     HnAssert(KheResourceTypeResourceCount(rt) > 0,
       "KheResourceResourceTypePartitionMakeResourceType internal error 1");
     rt = r->resource_type;
-    /* ***
-    r2 = KheResourceTypeResource(rt, 0);
-    if( r2 != r )
+    ins = KheResourceTypeInstance(rt);
+    str = (KheResourcePartitionEqualsResourceGroup(r, &rg) ?
+      KheResourceGroupId(rg) : KheResourceId(r));
+    snprintf(buff, 1000, "%s:%s", KheResourceTypeId(rt), str);
+    i = 1;
+    while( !KheResourceTypeMakeInternal(ins, buff, buff, false, rt, &new_rt) )
+      snprintf(buff, 1000, "%s:%s:%d", KheResourceTypeId(rt), str, i++);
+
+    /* move r and its partition to new_rt */
+    HaArrayForEach(r->rt_partition, r2, i)
+      KheResourceMoveToResourceType(r2, new_rt);
+
+    /* debug the new resource type */
+    if( DEBUG3 )
     {
-    *** */
-      /* make resource type new_rt */
-      ins = KheResourceTypeInstance(rt);
-      snprintf(buff, 1000, "%s:%s", KheResourceTypeId(rt), KheResourceId(r));
-      i = 1;
-      while( !KheResourceTypeMakeInternal(ins, buff, buff, false, rt, &new_rt) )
-	snprintf(buff, 1000, "%s:%s:%d", KheResourceTypeId(rt),
-	  KheResourceId(r), i++);
-
-      /* move r and its partition to new_rt */
-      HaArrayForEach(r->rt_partition, r2, i)
-	KheResourceMoveToResourceType(r2, new_rt);
-
-      /* debug the new resource type */
-      if( DEBUG3 )
+      fprintf(stderr, "  adding resource type %s = {",
+	KheResourceTypeId(new_rt));
+      for( i = 0;  i < KheResourceTypeResourceCount(new_rt);  i++ )
       {
-	fprintf(stderr, "  adding resource type %s = {",
-	  KheResourceTypeId(new_rt));
-	for( i = 0;  i < KheResourceTypeResourceCount(new_rt);  i++ )
-	{
-	  r2 = KheResourceTypeResource(new_rt, i);
-	  if( i > 0 )
-	    fprintf(stderr, ", ");
-	  fprintf(stderr, "%s", KheResourceId(r2));
-	}
-	fprintf(stderr, "}\n");
+	r2 = KheResourceTypeResource(new_rt, i);
+	if( i > 0 )
+	  fprintf(stderr, ", ");
+	fprintf(stderr, "%s", KheResourceId(r2));
       }
-    /* } */
+      fprintf(stderr, "}\n");
+    }
   }
 }
 
@@ -831,7 +876,7 @@ static KHE_TIME_GROUP KheResourceFindUnavail(KHE_RESOURCE r,
       SSetUnion(ss, *KheTimeGroupTimeSet(tg));
     }
   return KheTimeGroupMakeAndFinalize(KheResourceInstance(r),
-    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, NULL, true);
+    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, /* NULL, */ true, a);
 }
 
 /* *** this version is total rubbish, how did it ever sneak through?
@@ -850,7 +895,7 @@ static KHE_TIME_GROUP KheResourceFindUnavail(KHE_RESOURCE r,
       tg = KhePreferTimesConstraintDomain((KHE_PREFER_TIMES_CONSTRAINT) c);
       SSetUnion(ss, *KheTimeGroupTimeSet(tg));
     }
-  return KheTimeGroupMakeAndFinalize(KheResourceInstance(r),
+  return KheTimeGroupMake AndFinalize(KheResourceInstance(r),
     KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, NULL, true);
 }
 *** */
