@@ -116,6 +116,44 @@ KHE_RESOURCE_GROUP KheResourceGroupMakeInternal(KHE_RESOURCE_TYPE rt,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_RESOURCE_GROUP KheResourceGroupCopy(KHE_RESOURCE_GROUP rg,           */
+/*    HA_ARENA a)                                                            */
+/*                                                                           */
+/*  Make and return a copy of rg in arena a.  There is no need for a         */
+/*  Phase 1 and Phase 2.  If the resource group is not a solution            */
+/*  resource group, just return the current resource group.                  */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_RESOURCE_GROUP KheResourceGroupCopy(KHE_RESOURCE_GROUP rg,
+  HA_ARENA a)
+{
+  KHE_RESOURCE_GROUP res;
+  if( rg->resource_group_type == KHE_RESOURCE_GROUP_TYPE_SOLN )
+  {
+    /* need a real copy */
+    HaMake(res, a);
+    res->back = rg->back;
+    res->resource_group_type = rg->resource_group_type;
+    res->partition_index = -1;
+    res->rs_rec.rt = rg->rs_rec.rt;
+    KheSetCopy(res->rs_rec.resource_indexes, rg->rs_rec.resource_indexes, a);
+    res->id = HnStringCopy(rg->id, a);
+    res->name = HnStringCopy(rg->name, a);
+    res->finalized = true;  /* no need to finalize here */
+    res->partition = NULL;
+    return res;
+  }
+  else
+  {
+    /* OK to share */
+    return rg;
+  }
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheResourceGroupAddResourceInternal(KHE_RESOURCE_GROUP rg,          */
@@ -290,11 +328,13 @@ void KheResourceGroupResetResourceType(KHE_RESOURCE_GROUP rg)
 /*  Also set rg->partition, although it may have to be done again, if        */
 /*  partitions are inferred later on.                                        */
 /*                                                                           */
+/*  Also search for a better Id, if rg->id is currently NULL.                */
+/*                                                                           */
 /*****************************************************************************/
 
 void KheResourceGroupFinalize(KHE_RESOURCE_GROUP rg)
 {
-  KHE_RESOURCE r;  int i;  /* short ix; */
+  KHE_RESOURCE r;  int i;  /* short ix; */  KHE_RESOURCE_GROUP rg2;
   if( DEBUG1 )
     fprintf(stderr, "[ KheResourceGroupFinalize(%s %p)\n",
       rg->id == NULL ? "-" : rg->id, (void *) rg);
@@ -308,6 +348,20 @@ void KheResourceGroupFinalize(KHE_RESOURCE_GROUP rg)
 	KheResourceAddUserResourceGroup(r, rg);
       }
     KheResourceGroupSetPartition(rg);
+
+    if( rg->id == NULL && KheResourceGroupResourceCount(rg) > 0 )
+    {
+      r = KheResourceGroupResource(rg, 0);
+      for( i = 0;  i < KheResourceResourceGroupCount(r);  i++ )
+      {
+	rg2 = KheResourceResourceGroup(r, i);
+	if( rg2 != rg && KheResourceGroupEqual(rg, rg2) && rg2->id != NULL )
+	{
+	  rg->id = rg2->id;
+	  break;
+	}
+      }
+    }
   }
   if( DEBUG1 )
     fprintf(stderr, "] KheResourceGroupFinalize\n");
