@@ -32,6 +32,7 @@
 
 #define DEBUG1 0
 #define DEBUG2 0
+#define DEBUG3 0
 
 
 /*****************************************************************************/
@@ -40,12 +41,14 @@
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_matching.c
 struct khe_workload_requirement_rec {
-  int			count;			/* number required           */
-  KHE_TIME_GROUP	time_group;		/* time group required       */
-  KHE_MONITOR		originating_monitor;	/* originating monitor       */
-  ARRAY_KHE_WORKLOAD_REQUIREMENT children;	/* child requirements        */
+  int			count;			** number required           **
+  KHE_TIME_GROUP	time_group;		** time group required       **
+  KHE_MONITOR		originating_monitor;	** originating monitor       **
+  ARRAY_KHE_WORKLOAD_REQUIREMENT children;	** child requirements        **
 };
+*** */
 
 
 /*****************************************************************************/
@@ -63,6 +66,7 @@ struct khe_resource_in_soln_rec {
   KHE_SOLN			soln;			/* encl. solution    */
   KHE_RESOURCE			resource;		/* monitored resource*/
   ARRAY_KHE_TASK		tasks;			/* assigned resource */
+  KHE_TASK_BOUND		singleton_task_bound;	/* task bound just r */
   int				busy_times;		/* of tasks          */
   float				workload;		/* of tasks          */
   int				max_busy_times;		/* cached            */
@@ -70,9 +74,10 @@ struct khe_resource_in_soln_rec {
   KHE_RESOURCE_TIMETABLE_MONITOR timetable_monitor;	/* its timetable     */
   ARRAY_KHE_MONITOR		all_monitors;		/* all monitors      */
   ARRAY_KHE_MONITOR		attached_monitors;	/* attached monitors */
-  ARRAY_KHE_WORKLOAD_REQUIREMENT	workload_requirements;
+  /* ARRAY_KHE_WORKLOAD_REQUIREMENT	workload_requirements; */
+  ARRAY_KHE_TIME_GROUP			workload_time_groups;
   ARRAY_KHE_MATCHING_DEMAND_CHUNK	workload_demand_chunks;
-  bool				workload_requirements_active;
+  /* bool				workload_requirements_active; */
   KHE_RESOURCE_IN_SOLN		copy;
 };
 
@@ -100,6 +105,7 @@ KHE_RESOURCE_IN_SOLN KheResourceInSolnMake(KHE_SOLN soln, KHE_RESOURCE r)
   res->soln = soln;
   res->resource = r;
   HaArrayInit(res->tasks, a);
+  res->singleton_task_bound = NULL;
   res->busy_times = 0;
   res->workload = 0.0;
   res->max_busy_times = -1;	/* negative signals not calculated yet */
@@ -107,9 +113,10 @@ KHE_RESOURCE_IN_SOLN KheResourceInSolnMake(KHE_SOLN soln, KHE_RESOURCE r)
   res->timetable_monitor = KheResourceTimetableMonitorMake(soln, res);
   HaArrayInit(res->all_monitors, a);
   HaArrayInit(res->attached_monitors, a);
-  HaArrayInit(res->workload_requirements, a);
+  /* HaArrayInit(res->workload_requirements, a); */
+  HaArrayInit(res->workload_time_groups, a);
   HaArrayInit(res->workload_demand_chunks, a);
-  res->workload_requirements_active = false;;
+  /* res->workload_requirements_active = false; */
   res->copy = NULL;
   return res;
 }
@@ -124,24 +131,27 @@ KHE_RESOURCE_IN_SOLN KheResourceInSolnMake(KHE_SOLN soln, KHE_RESOURCE r)
 /*                                                                           */
 /*****************************************************************************/
 
+/* ***
 static KHE_WORKLOAD_REQUIREMENT KheWorkloadRequirementCopy(
   KHE_WORKLOAD_REQUIREMENT wr, HA_ARENA a)
 {
   KHE_WORKLOAD_REQUIREMENT res;
   HaMake(res, a);
   res->count = wr->count;
-  res->time_group = wr->time_group;
+  res->time_group = KheTimeGroupCopy(wr->time_group, a);
   res->originating_monitor = (wr->originating_monitor == NULL ? NULL :
     KheMonitorCopyPhase1(wr->originating_monitor, a));
   HaArrayInit(res->children, a);
   return res;
 }
+*** */
 
 KHE_RESOURCE_IN_SOLN KheResourceInSolnCopyPhase1(KHE_RESOURCE_IN_SOLN rs,
   HA_ARENA a)
 {
   KHE_RESOURCE_IN_SOLN copy;  KHE_MONITOR m;  KHE_TASK task;  int i;
-  KHE_WORKLOAD_REQUIREMENT wr;  KHE_MATCHING_DEMAND_CHUNK dc;
+  /* KHE_WORKLOAD_REQUIREMENT wr; */  KHE_MATCHING_DEMAND_CHUNK dc;
+  KHE_TIME_GROUP tg;
   if( rs->copy == NULL )
   {
     HaMake(copy, a);
@@ -151,6 +161,8 @@ KHE_RESOURCE_IN_SOLN KheResourceInSolnCopyPhase1(KHE_RESOURCE_IN_SOLN rs,
     HaArrayInit(copy->tasks, a);
     HaArrayForEach(rs->tasks, task, i)
       HaArrayAddLast(copy->tasks, KheTaskCopyPhase1(task, a));
+    copy->singleton_task_bound = (rs->singleton_task_bound == NULL ? NULL :
+      KheTaskBoundCopyPhase1(rs->singleton_task_bound, a));
     copy->busy_times = rs->busy_times;
     copy->workload = rs->workload;
     copy->timetable_monitor =
@@ -161,17 +173,25 @@ KHE_RESOURCE_IN_SOLN KheResourceInSolnCopyPhase1(KHE_RESOURCE_IN_SOLN rs,
     HaArrayInit(copy->attached_monitors, a);
     HaArrayForEach(rs->attached_monitors, m, i)
       HaArrayAddLast(copy->attached_monitors, KheMonitorCopyPhase1(m, a));
+    /* ***
     HaArrayInit(copy->workload_requirements, a);
     HaArrayForEach(rs->workload_requirements, wr, i)
       HaArrayAddLast(copy->workload_requirements,
 	KheWorkloadRequirementCopy(wr, a));
+    *** */
+    HaArrayInit(copy->workload_time_groups, a);
+    HaArrayForEach(rs->workload_time_groups, tg, i)
+      HaArrayAddLast(copy->workload_time_groups,
+	KheTimeGroupCopy(tg, a));
     HaArrayInit(copy->workload_demand_chunks, a);
     HaArrayForEach(rs->workload_demand_chunks, dc, i)
       HaArrayAddLast(copy->workload_demand_chunks,
 	KheMatchingDemandChunkCopyPhase1(dc, a));
+    /* ***
     HnAssert(!rs->workload_requirements_active,
       "KheSolnCopy called when currently adding workload requirements");
     copy->workload_requirements_active = false;
+    *** */
     copy->copy = NULL;
   }
   return rs->copy;
@@ -189,16 +209,23 @@ KHE_RESOURCE_IN_SOLN KheResourceInSolnCopyPhase1(KHE_RESOURCE_IN_SOLN rs,
 void KheResourceInSolnCopyPhase2(KHE_RESOURCE_IN_SOLN rs)
 {
   KHE_TASK task;  KHE_MONITOR m;  int i;  KHE_MATCHING_DEMAND_CHUNK dc;
+  /* KHE_TIME_GROUP tg; */
   if( rs->copy != NULL )
   {
     rs->copy = NULL;
     HaArrayForEach(rs->tasks, task, i)
       KheTaskCopyPhase2(task);
+    if( rs->singleton_task_bound != NULL )
+      KheTaskBoundCopyPhase2(rs->singleton_task_bound);
     KheResourceTimetableMonitorCopyPhase2(rs->timetable_monitor);
     HaArrayForEach(rs->all_monitors, m, i)
       KheMonitorCopyPhase2(m);
     HaArrayForEach(rs->attached_monitors, m, i)
       KheMonitorCopyPhase2(m);
+    /* ***
+    HaArrayForEach(rs->workload_time_groups, tg, i)
+      KheTimeGroupCopyPhase2(tg);
+    *** */
     HaArrayForEach(rs->workload_demand_chunks, dc, i)
       KheMatchingDemandChunkCopyPhase2(dc);
   }
@@ -247,6 +274,28 @@ char *KheResourceInSolnId(KHE_RESOURCE_IN_SOLN rs)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_TASK_BOUND KheResourceInSolnSingletonTaskBound(                      */
+/*    KHE_RESOURCE_IN_SOLN rs)                                               */
+/*                                                                           */
+/*  Return a task bound whose resource group contains just rs->resource.     */
+/*  Make this task bound the first time it's needed.                         */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_TASK_BOUND KheResourceInSolnSingletonTaskBound(KHE_RESOURCE_IN_SOLN rs)
+{
+  KHE_RESOURCE_GROUP rg;
+  if( rs->singleton_task_bound == NULL )
+  {
+    rg = KheResourceSingletonResourceGroup(rs->resource);
+    rs->singleton_task_bound = KheTaskBoundMake(rs->soln, rg);
+  }
+  return rs->singleton_task_bound;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheResourceInSolnDelete(KHE_RESOURCE_IN_SOLN rs)                    */
@@ -598,7 +647,7 @@ void KheResourceInSolnUnAssignTime(KHE_RESOURCE_IN_SOLN rs,
 
 /*****************************************************************************/
 /*                                                                           */
-/*  Submodule "workload requirements (for matchings)"                        */
+/*  Submodule "matching"                                                     */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -610,10 +659,12 @@ void KheResourceInSolnUnAssignTime(KHE_RESOURCE_IN_SOLN rs,
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_matching.c
 int KheResourceInSolnWorkloadRequirementCount(KHE_RESOURCE_IN_SOLN rs)
 {
   return HaArrayCount(rs->workload_requirements);
 }
+*** */
 
 
 /*****************************************************************************/
@@ -625,6 +676,7 @@ int KheResourceInSolnWorkloadRequirementCount(KHE_RESOURCE_IN_SOLN rs)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_matching.c
 void KheResourceInSolnWorkloadRequirement(KHE_RESOURCE_IN_SOLN rs,
   int i, int *num, KHE_TIME_GROUP *tg, KHE_MONITOR *m)
 {
@@ -634,6 +686,7 @@ void KheResourceInSolnWorkloadRequirement(KHE_RESOURCE_IN_SOLN rs,
   *tg = wr->time_group;
   *m = wr->originating_monitor;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -663,18 +716,20 @@ static void KheResourceInSolnWorkloadRequirementDelete(
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_matching.c
 void KheResourceInSolnBeginWorkloadRequirements(KHE_RESOURCE_IN_SOLN rs)
 {
   HnAssert(!rs->workload_requirements_active,
     "KheSolnMatchingBeginWorkloadRequirements called out of order");
   KheResourceInSolnDeleteWorkloadRequirements(rs);
-  /* ***
+  ** ***
   while( HaArrayCount(rs->workload_requirements) > 0 )
     KheSolnAddWorkloadRequirementToFreeList(rs->soln,
       HaArrayLastAndDelete(rs->workload_requirements));
-  *** */
+  *** **
   rs->workload_requirements_active = true;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -686,6 +741,7 @@ void KheResourceInSolnBeginWorkloadRequirements(KHE_RESOURCE_IN_SOLN rs)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_matching.c
 void KheResourceInSolnAddWorkloadRequirement(KHE_RESOURCE_IN_SOLN rs,
   int num, KHE_TIME_GROUP tg, KHE_MONITOR m)
 {
@@ -708,6 +764,7 @@ void KheResourceInSolnAddWorkloadRequirement(KHE_RESOURCE_IN_SOLN rs,
   wr->originating_monitor = m;
   HaArrayAddLast(rs->workload_requirements, wr);
 }
+*** */
 
 
 /*****************************************************************************/
@@ -719,12 +776,13 @@ void KheResourceInSolnAddWorkloadRequirement(KHE_RESOURCE_IN_SOLN rs,
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_matching.c
 static void KheWorkloadRequirementInsertIntoForest(
   ARRAY_KHE_WORKLOAD_REQUIREMENT *roots, KHE_WORKLOAD_REQUIREMENT wr)
 {
   int touch_count, superset_count, i;  KHE_WORKLOAD_REQUIREMENT touch_wr, wr2;
 
-  /* find out how many roots wr touches and is a superset of */
+  ** find out how many roots wr touches and is a superset of **
   touch_count = 0;
   touch_wr = NULL;
   superset_count = 0;
@@ -739,7 +797,7 @@ static void KheWorkloadRequirementInsertIntoForest(
 
   if( superset_count == touch_count )
   {
-    /* wr is a superset of everything it touches; move them to below wr */
+    ** wr is a superset of everything it touches; move them to below wr **
     HaArrayForEach(*roots, wr2, i)
       if( KheTimeGroupSubset(wr2->time_group, wr->time_group) )
       {
@@ -752,14 +810,15 @@ static void KheWorkloadRequirementInsertIntoForest(
   else if( touch_count == 1 &&
       KheTimeGroupSubset(wr->time_group, touch_wr->time_group) )
   {
-    /* wr belongs below touch_wr */
+    ** wr belongs below touch_wr **
     KheWorkloadRequirementInsertIntoForest(&touch_wr->children, wr);
   }
   else
   {
-    /* no other legal possibilities, so we abandon wr here */
+    ** no other legal possibilities, so we abandon wr here **
   }
 }
+*** */
 
 
 /*****************************************************************************/
@@ -784,6 +843,7 @@ static void KheWorkloadRequirementInsertIntoForest(
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_matching.c
 static int KheWorkloadRequirementToDemandNodes(KHE_WORKLOAD_REQUIREMENT wr,
   KHE_RESOURCE_IN_SOLN rs)
 {
@@ -791,17 +851,17 @@ static int KheWorkloadRequirementToDemandNodes(KHE_WORKLOAD_REQUIREMENT wr,
   KHE_WORKLOAD_REQUIREMENT child_wr;  KHE_MATCHING_DEMAND_CHUNK dc;
   KHE_WORKLOAD_DEMAND_MONITOR wdm;
 
-  /* build demand nodes in proper descendants and count how many */
+  ** build demand nodes in proper descendants and count how many **
   child_count = 0;
   HaArrayForEach(wr->children, child_wr, i)
     child_count += KheWorkloadRequirementToDemandNodes(child_wr, rs);
 
-  /* find out how many we need here at the root */
+  ** find out how many we need here at the root **
   root_count = KheTimeGroupTimeCount(wr->time_group) - wr->count - child_count;
   if( root_count < 0 )
     root_count = 0;
 
-  /* add one chunk with root_count demand nodes, if root_count > 0 */
+  ** add one chunk with root_count demand nodes, if root_count > 0 **
   if( root_count > 0 )
   {
     m = KheSolnMatching(rs->soln);
@@ -812,7 +872,7 @@ static int KheWorkloadRequirementToDemandNodes(KHE_WORKLOAD_REQUIREMENT wr,
     HaArrayAddLast(rs->workload_demand_chunks, dc);
     for( i = 0;  i < root_count;  i++ )
     {
-      wdm = KheWorkloadDemandMonitorMake(rs->soln, dc, rs, wr->time_group,
+      wdm = KheWorkloadDemandM onitorMake(rs->soln, dc, rs, wr->time_group,
 	wr->originating_monitor);
       KheGroupMonitorAddChildMonitor((KHE_GROUP_MONITOR) rs->soln,
 	(KHE_MONITOR) wdm);
@@ -820,9 +880,10 @@ static int KheWorkloadRequirementToDemandNodes(KHE_WORKLOAD_REQUIREMENT wr,
     }
   }
 
-  /* return the total number of demand nodes added at wr or below it */
+  ** return the total number of demand nodes added at wr or below it **
   return child_count + root_count;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -834,6 +895,7 @@ static int KheWorkloadRequirementToDemandNodes(KHE_WORKLOAD_REQUIREMENT wr,
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_matching.c
 void KheResourceInSolnEndWorkloadRequirements(KHE_RESOURCE_IN_SOLN rs)
 {
   ARRAY_KHE_WORKLOAD_REQUIREMENT roots;  int i;  KHE_WORKLOAD_REQUIREMENT wr;
@@ -859,7 +921,7 @@ void KheResourceInSolnEndWorkloadRequirements(KHE_RESOURCE_IN_SOLN rs)
     "KheSolnMatchingEndWorkloadRequirements called out of order");
   rs->workload_requirements_active = false;
 
-  /* build the forest */
+  ** build the forest **
   a = KheSolnArena(rs->soln);
   HaArrayInit(roots, a);
   HaArrayForEach(rs->workload_requirements, wr, i)
@@ -868,7 +930,7 @@ void KheResourceInSolnEndWorkloadRequirements(KHE_RESOURCE_IN_SOLN rs)
     KheWorkloadRequirementInsertIntoForest(&roots, wr);
   }
 
-  /* traverse the forest, adding workload demand chunks and nodes */
+  ** traverse the forest, adding workload demand chunks and nodes **
   HaArrayForEach(roots, wr, i)
     KheWorkloadRequirementToDemandNodes(wr, rs);
   HaArrayFree(roots);
@@ -880,6 +942,7 @@ void KheResourceInSolnEndWorkloadRequirements(KHE_RESOURCE_IN_SOLN rs)
     fprintf(stderr, "] KheResourceInSolnEndWorkloadRequirements returning\n");
   }
 }
+*** */
 
 
 /*****************************************************************************/
@@ -890,28 +953,83 @@ void KheResourceInSolnEndWorkloadRequirements(KHE_RESOURCE_IN_SOLN rs)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_matching.c
 void KheResourceInSolnDeleteWorkloadRequirements(KHE_RESOURCE_IN_SOLN rs)
 {
   KHE_MONITOR m;  int i;
 
-  /* delete old workload requirements */
+  ** delete old workload requirements **
   while( HaArrayCount(rs->workload_requirements) > 0 )
     KheSolnAddWorkloadRequirementToFreeList(rs->soln,
       HaArrayLastAndDelete(rs->workload_requirements));
 
-  /* delete old workload demand monitors */
+  ** delete old workload demand monitors **
   HaArrayForEach(rs->all_monitors, m, i)
     if( KheMonitorTag(m) == KHE_WORKLOAD_DEMAND_MONITOR_TAG )
     {
-      KheWorkloadDemandMonitorDelete((KHE_WORKLOAD_DEMAND_MONITOR) m);
+      KheWorkload DemandMonitorDelete((KHE_WORKLOAD_DEMAND_MONITOR) m);
       i--;
     }
 
-  /* delete old workload demand chunks */
+  ** delete old workload demand chunks **
   while( HaArrayCount(rs->workload_demand_chunks) > 0 )
     KheMatchingDemandChunkDelete(HaArrayLastAndDelete(rs->workload_demand_chunks));
 
 }
+*** */
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_MATCHING_DEMAND_CHUNK KheResourceInSolnDemandChunk(                  */
+/*    KHE_RESOURCE_IN_SOLN rs, KHE_TIME_GROUP tg)                            */
+/*                                                                           */
+/*  Return a demand chunk for rs and tg.  Make a new one if necessary.       */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_MATCHING_DEMAND_CHUNK KheResourceInSolnDemandChunk(
+  KHE_RESOURCE_IN_SOLN rs, KHE_TIME_GROUP tg)
+{
+  KHE_MATCHING_DEMAND_CHUNK dc;  int i, resource_count;  KHE_TIME_GROUP tg2;
+  KHE_MATCHING m;
+
+  /* make sure all good */
+  m = KheSolnMatching(rs->soln);
+  HnAssert(m != NULL, "KheWorkloadDemandMonitorMake: no matching");
+
+  /* return an existing chunk, if there is one for tg */
+  HaArrayForEach(rs->workload_time_groups, tg2, i)
+    if( KheTimeGroupEqual(tg, tg2) )
+      return HaArray(rs->workload_demand_chunks, i);
+
+  /* no existing chunk, so add one and return it */
+  HaArrayAddLast(rs->workload_time_groups, tg);
+  resource_count = KheInstanceResourceCount(KheSolnInstance(rs->soln));
+  dc = KheMatchingDemandChunkMake(m, NULL, 0, resource_count,
+    KheTimeGroupTimeSet(tg));
+  HaArrayAddLast(rs->workload_demand_chunks, dc);
+  return dc;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheResourceInSolnDeleteDemandChunks(KHE_RESOURCE_IN_SOLN rs)        */
+/*                                                                           */
+/*  Delete the demand chunks of rs.  They should all be empty by now.        */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheResourceInSolnDeleteDemandChunks(KHE_RESOURCE_IN_SOLN rs)
+{
+  KHE_MATCHING_DEMAND_CHUNK wdc;
+  while( HaArrayCount(rs->workload_demand_chunks) > 0 )
+  {
+    wdc = HaArrayLastAndDelete(rs->workload_demand_chunks);
+    KheMatchingDemandChunkDelete(wdc);
+  }
+}
 
 
 /*****************************************************************************/
@@ -999,8 +1117,8 @@ bool KheResourceInSolnMaxBusyTimes(KHE_RESOURCE_IN_SOLN rs,
   if( rs->max_busy_times < 0 )
   {
     /* value not calculated yet, do it now */
-    KheAvailSolverSetResource(as, rs->resource);
-    KheAvailSolverMaxBusyTimes(as, &rs->max_busy_times);
+    /* KheAvailSolverSetResource(as, rs->resource); */
+    KheAvailSolverMaxBusyTimes(as, rs->resource, &rs->max_busy_times);
   }
   *res = rs->max_busy_times;
   return *res < INT_MAX;
@@ -1023,13 +1141,27 @@ bool KheResourceInSolnMaxBusyTimes(KHE_RESOURCE_IN_SOLN rs,
 bool KheResourceInSolnMaxWorkload(KHE_RESOURCE_IN_SOLN rs,
   KHE_AVAIL_SOLVER as, float *res)
 {
+  if( DEBUG3 )
+    fprintf(stderr, "[ KheResourceInSolnMaxWorkload(%s, as, -)\n",
+      KheResourceId(rs->resource));
   if( rs->max_workload < 0.0 )
   {
     /* value not calculated yet, do it now */
-    KheAvailSolverSetResource(as, rs->resource);
-    KheAvailSolverMaxWorkload(as, &rs->max_workload);
+    if( DEBUG3 )
+      fprintf(stderr, "  calling KheAvailSolverSetResource(as, %s)\n",
+        KheResourceId(rs->resource));
+    /* KheAvailSolverSetResource(as, rs->resource); */
+    KheAvailSolverMaxWorkload(as, rs->resource, &rs->max_workload);
   }
   *res = rs->max_workload;
+  if( DEBUG3 )
+  {
+    if( *res < FLT_MAX )
+      fprintf(stderr, "] KheResourceInSolnMaxWorkload returning true (%.1f)\n",
+	*res);
+    else
+      fprintf(stderr, "] KheResourceInSolnMaxWorkload returning false\n");
+  }
   return *res < FLT_MAX;
 }
 
