@@ -277,7 +277,13 @@ int KheResourceSetResourceIndex(KHE_RESOURCE_SET rs, int i)
   return KheSetGet(rs->resource_indexes, i);
 }
 *** */
+
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "union operations"                                             */
+/*                                                                           */
+/*****************************************************************************/
 
 /*****************************************************************************/
 /*                                                                           */
@@ -293,35 +299,6 @@ void KheResourceSetUnion(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
 }
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheResourceSetIntersect(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2) */
-/*                                                                           */
-/*  Update rs1 to contain the intersection of its initial value with rs2.    */
-/*                                                                           */
-/*****************************************************************************/
-
-void KheResourceSetIntersect(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
-{
-  KheSetIntersect(rs1->resource_indexes, rs2->resource_indexes);
-}
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheResourceSetDifference(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)*/
-/*                                                                           */
-/*  Update rs1 to contain the difference of its initial value with rs2,      */
-/*  that is, rs1 - rs2.                                                      */
-/*                                                                           */
-/*****************************************************************************/
-
-void KheResourceSetDifference(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
-{
-  KheSetDifference(rs1->resource_indexes, rs2->resource_indexes);
-}
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheResourceSetUnionGroup(KHE_RESOURCE_SET rs1,                      */
@@ -337,6 +314,57 @@ void KheResourceSetUnionGroup(KHE_RESOURCE_SET rs1, KHE_RESOURCE_GROUP rg2)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheResourceSetUnionCount(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2) */
+/*                                                                           */
+/*  Return the number of elements in the union of rs1 with rs2.              */
+/*                                                                           */
+/*****************************************************************************/
+
+int KheResourceSetUnionCount(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
+{
+  return KheSetUnionCount(rs1->resource_indexes, rs2->resource_indexes);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheResourceSetUnionCountGroup(KHE_RESOURCE_SET rs1,                  */
+/*    KHE_RESOURCE_GROUP rg2)                                                */
+/*                                                                           */
+/*  Like KheResourceSetUnionCount, only the second parameter is a            */
+/*  resource group.                                                          */
+/*                                                                           */
+/*****************************************************************************/
+
+int KheResourceSetUnionCountGroup(KHE_RESOURCE_SET rs1,
+  KHE_RESOURCE_GROUP rg2)
+{
+  return KheResourceSetUnionCount(rs1, KheResourceGroupResourceSet(rg2));
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "intersect operations"                                         */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheResourceSetIntersect(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2) */
+/*                                                                           */
+/*  Update rs1 to contain the intersection of its initial value with rs2.    */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheResourceSetIntersect(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
+{
+  KheSetIntersect(rs1->resource_indexes, rs2->resource_indexes);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheResourceSetIntersectGroup(KHE_RESOURCE_SET rs1,                  */
@@ -353,6 +381,59 @@ void KheResourceSetIntersectGroup(KHE_RESOURCE_SET rs1, KHE_RESOURCE_GROUP rg2)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheResourceSetIntersectCount(KHE_RESOURCE_SET rs1,                   */
+/*    KHE_RESOURCE_SET rs2)                                                  */
+/*                                                                           */
+/*  Return the number of elements in the intersection of rs1 with rs2.       */
+/*                                                                           */
+/*****************************************************************************/
+
+int KheResourceSetIntersectCount(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
+{
+  return KheSetIntersectCount(rs1->resource_indexes, rs2->resource_indexes);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheResourceSetIntersectCountGroup(KHE_RESOURCE_SET rs1,              */
+/*    KHE_RESOURCE_GROUP rg2)                                                */
+/*                                                                           */
+/*  Like KheResourceSetIntersectCount, only the second parameter is a        */
+/*  resource group.                                                          */
+/*                                                                           */
+/*****************************************************************************/
+
+int KheResourceSetIntersectCountGroup(KHE_RESOURCE_SET rs1,
+  KHE_RESOURCE_GROUP rg2)
+{
+  return KheResourceSetIntersectCount(rs1, KheResourceGroupResourceSet(rg2));
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "difference operations"                                        */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheResourceSetDifference(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)*/
+/*                                                                           */
+/*  Update rs1 to contain the difference of its initial value with rs2,      */
+/*  that is, rs1 - rs2.                                                      */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheResourceSetDifference(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
+{
+  KheSetDifference(rs1->resource_indexes, rs2->resource_indexes);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheResourceSetDifferenceGroup(KHE_RESOURCE_SET rs1,                 */
@@ -369,35 +450,6 @@ void KheResourceSetDifferenceGroup(KHE_RESOURCE_SET rs1, KHE_RESOURCE_GROUP rg2)
 }
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  int KheResourceSetUnionCount(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2) */
-/*                                                                           */
-/*  Return the number of elements in the union of rs1 with rs2.              */
-/*                                                                           */
-/*****************************************************************************/
-
-int KheResourceSetUnionCount(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
-{
-  return KheSetUnionCount(rs1->resource_indexes, rs2->resource_indexes);
-}
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  int KheResourceSetIntersectCount(KHE_RESOURCE_SET rs1,                   */
-/*    KHE_RESOURCE_SET rs2)                                                  */
-/*                                                                           */
-/*  Return the number of elements in the intersection of rs1 with rs2.       */
-/*                                                                           */
-/*****************************************************************************/
-
-int KheResourceSetIntersectCount(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
-{
-  return KheSetIntersectCount(rs1->resource_indexes, rs2->resource_indexes);
-}
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  int KheResourceSetDifferenceCount(KHE_RESOURCE_SET rs1,                  */
@@ -413,57 +465,6 @@ int KheResourceSetDifferenceCount(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
 }
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  int KheResourceSetSymmetricDifferenceCount(KHE_RESOURCE_SET rs1,         */
-/*    KHE_RESOURCE_SET rs2)                                                  */
-/*                                                                           */
-/*  Return the number of elements in the symmetric difference of rs1 and rs2.*/
-/*                                                                           */
-/*****************************************************************************/
-
-int KheResourceSetSymmetricDifferenceCount(KHE_RESOURCE_SET rs1,
-  KHE_RESOURCE_SET rs2)
-{
-  return KheSetSymmetricDifferenceCount(rs1->resource_indexes,
-    rs2->resource_indexes);
-}
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  int KheResourceSetUnionCountGroup(KHE_RESOURCE_SET rs1,                  */
-/*    KHE_RESOURCE_GROUP rg2)                                                */
-/*                                                                           */
-/*  Like KheResourceSetUnionCount, only the second parameter is a            */
-/*  resource group.                                                          */
-/*                                                                           */
-/*****************************************************************************/
-
-int KheResourceSetUnionCountGroup(KHE_RESOURCE_SET rs1,
-  KHE_RESOURCE_GROUP rg2)
-{
-  return KheResourceSetUnionCount(rs1, KheResourceGroupResourceSet(rg2));
-}
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  int KheResourceSetIntersectCountGroup(KHE_RESOURCE_SET rs1,              */
-/*    KHE_RESOURCE_GROUP rg2)                                                */
-/*                                                                           */
-/*  Like KheResourceSetIntersectCount, only the second parameter is a        */
-/*  resource group.                                                          */
-/*                                                                           */
-/*****************************************************************************/
-
-int KheResourceSetIntersectCountGroup(KHE_RESOURCE_SET rs1,
-  KHE_RESOURCE_GROUP rg2)
-{
-  return KheResourceSetIntersectCount(rs1, KheResourceGroupResourceSet(rg2));
-}
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  int KheResourceSetDifferenceCountGroup(KHE_RESOURCE_SET rs1,             */
@@ -479,6 +480,29 @@ int KheResourceSetDifferenceCountGroup(KHE_RESOURCE_SET rs1,
 {
   return KheResourceSetDifferenceCount(rs1, KheResourceGroupResourceSet(rg2));
 }
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "symmetric difference operations"                              */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheResourceSetSymmetricDifferenceCount(KHE_RESOURCE_SET rs1,         */
+/*    KHE_RESOURCE_SET rs2)                                                  */
+/*                                                                           */
+/*  Return the number of elements in the symmetric difference of rs1 and rs2.*/
+/*                                                                           */
+/*****************************************************************************/
+
+int KheResourceSetSymmetricDifferenceCount(KHE_RESOURCE_SET rs1,
+  KHE_RESOURCE_SET rs2)
+{
+  return KheSetSymmetricDifferenceCount(rs1->resource_indexes,
+    rs2->resource_indexes);
+}
 
 
 /*****************************************************************************/
@@ -497,7 +521,13 @@ int KheResourceSetSymmetricDifferenceCountGroup(KHE_RESOURCE_SET rs1,
   return KheResourceSetSymmetricDifferenceCount(rs1,
     KheResourceGroupResourceSet(rg2));
 }
+
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "equality operations"                                          */
+/*                                                                           */
+/*****************************************************************************/
 
 /*****************************************************************************/
 /*                                                                           */
@@ -513,6 +543,28 @@ bool KheResourceSetEqual(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheResourceSetEqualGroup(KHE_RESOURCE_SET rs1,                      */
+/*    KHE_RESOURCE_GROUP rg2)                                                */
+/*                                                                           */
+/*  Like KheResourceSetEqual, only the second parameter is a resource group. */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheResourceSetEqualGroup(KHE_RESOURCE_SET rs1,
+  KHE_RESOURCE_GROUP rg2)
+{
+  return KheResourceSetEqual(rs1, KheResourceGroupResourceSet(rg2));
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "subset operations"                                            */
+/*                                                                           */
+/*****************************************************************************/
+
 /*****************************************************************************/
 /*                                                                           */
 /*  bool KheResourceSetSubset(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)    */
@@ -527,6 +579,28 @@ bool KheResourceSetSubset(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheResourceSetSubsetGroup(KHE_RESOURCE_SET rs1,                     */
+/*    KHE_RESOURCE_GROUP rg2)                                                */
+/*                                                                           */
+/*  Like KheResourceSetSubset, only the second parameter is a resource group.*/
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheResourceSetSubsetGroup(KHE_RESOURCE_SET rs1,
+  KHE_RESOURCE_GROUP rg2)
+{
+  return KheResourceSetSubset(rs1, KheResourceGroupResourceSet(rg2));
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "disjoint operations"                                          */
+/*                                                                           */
+/*****************************************************************************/
+
 /*****************************************************************************/
 /*                                                                           */
 /*  bool KheResourceSetDisjoint(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)  */
@@ -541,6 +615,29 @@ bool KheResourceSetDisjoint(KHE_RESOURCE_SET rs1, KHE_RESOURCE_SET rs2)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheResourceSetDisjointGroup(KHE_RESOURCE_SET rs1,                   */
+/*    KHE_RESOURCE_GROUP rg2)                                                */
+/*                                                                           */
+/*  Like KheResourceSetDisjoint, only the second parameter is a resource     */
+/*  group.                                                                   */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheResourceSetDisjointGroup(KHE_RESOURCE_SET rs1,
+  KHE_RESOURCE_GROUP rg2)
+{
+  return KheResourceSetDisjoint(rs1, KheResourceGroupResourceSet(rg2));
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "other operations"                                             */
+/*                                                                           */
+/*****************************************************************************/
+
 /*****************************************************************************/
 /*                                                                           */
 /*  bool KheResourceSetContainsResource(KHE_RESOURCE_SET rs, KHE_RESOURCE r) */
@@ -601,7 +698,7 @@ void KheResourceSetDebug(KHE_RESOURCE_SET rs, int verbosity, int indent,
   if( indent >= 0 )
     fprintf(fp, "%*s", indent, "");
   fprintf(fp, "{");
-  if( indent >= 0 && verbosity >= 2 )
+  if( verbosity >= 2 && (indent >= 0 || KheResourceSetResourceCount(rs) < 8) )
   {
     /* full print */
     for( i = 0;  i < KheResourceSetResourceCount(rs);  i++ )
