@@ -31,6 +31,7 @@
 
 #define DEBUG1 0
 #define DEBUG2 0		/* resource type partitioning */
+#define DEBUG3 0
 
 /*****************************************************************************/
 /*                                                                           */
@@ -48,6 +49,7 @@ struct khe_resource_type_rec {
   bool				demand_all_preassigned;	/* all preassigned   */
   int				avoid_split_assts_count; /* asacount         */
   int				limit_resources_count;	/* no of limit r.    */
+  float				max_workload_per_time;	/* max workload per t*/
   bool				rt_partitioning_on;	/* rt partitioning   */
   KHE_RESOURCE_TYPE		original_rt;		/* before rt part.   */
   ARRAY_KHE_RESOURCE_GROUP	resource_group_array;	/* resource groups   */
@@ -95,6 +97,7 @@ bool KheResourceTypeMakeInternal(KHE_INSTANCE ins, char *id, char *name,
   res->demand_all_preassigned = true;  /* until proved false */
   res->avoid_split_assts_count = 0;
   res->limit_resources_count = 0;
+  res->max_workload_per_time = 0.0;
   res->rt_partitioning_on = false;
   res->original_rt = (orig_rt == NULL ? res : orig_rt);
   HaArrayInit(res->resource_group_array, a);
@@ -238,7 +241,7 @@ int KheResourceTypeIndex(KHE_RESOURCE_TYPE rt)
 
 char *KheResourceTypeId(KHE_RESOURCE_TYPE rt)
 {
-  return rt->id;
+  return rt == NULL ? "(NULL)" : rt->id;
 }
 
 
@@ -696,7 +699,7 @@ void KheResourceTypeInferPartitions(KHE_RESOURCE_TYPE rt)
       rt->id == NULL ? "-" : rt->id);
 
   /* arena for temporary memory, all deleted at the end */
-  a = KheInstanceArenaBegin(rt->instance, false);
+  a = KheInstanceArenaBegin(rt->instance);
 
   /* initialize to one cluster per resource */
   HaArrayInit(all_clusters, a);
@@ -1023,3 +1026,38 @@ int KheResourceTypeLimitResourcesCount(KHE_RESOURCE_TYPE rt)
 {
   return rt->limit_resources_count;
 }
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheResourceTypeAddMaxWorkloadPerTime(KHE_RESOURCE_TYPE rt, float x) */
+/*                                                                           */
+/*  Report to rt an event resource with max workload per time x.             */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheResourceTypeAddMaxWorkloadPerTime(KHE_RESOURCE_TYPE rt, float x)
+{
+  if( x > rt->max_workload_per_time )
+  {
+    rt->max_workload_per_time = x;
+    if( DEBUG3 )
+      fprintf(stderr, "  KheResourceTypeAddMaxWorkloadPerTime(%s, %.1f)\n",
+	KheResourceTypeId(rt), x);
+  }
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  float KheResourceTypeMaxWorkloadPerTime(KHE_RESOURCE_TYPE rt)            */
+/*                                                                           */
+/*  Return the maximum, over all event resources er of type rt, of the       */
+/*  workload per time of er.                                                 */
+/*                                                                           */
+/*****************************************************************************/
+
+float KheResourceTypeMaxWorkloadPerTime(KHE_RESOURCE_TYPE rt)
+{
+  return rt->max_workload_per_time;
+}
