@@ -35,9 +35,11 @@
 #include "howard_n.h"
 #define value(sp, i) HaArray((sp)->ss_array, (i))
 #define max(a, b) ((a) > (b) ? (a) : (b))
+#define bool_show(x) ((x) ? "true" : "false")
 
 #define DEBUG1 0
 #define DEBUG2 0
+#define DEBUG3 0
 
 /*****************************************************************************/
 /*                                                                           */
@@ -58,16 +60,17 @@
 
 static bool KheSetImplBinarySearch(KHE_SET *sp, int item, int *pos)
 {
-  int l, r, mid;
+  int l, r, mid, mid_el;
   l = 0;
   r = HaArrayCount(sp->ss_array) - 1;
   while( l <= r )
   {
     /* invariant:  item is in ss_items[l..r] or nowhere */
     mid = (l + r) / 2;
-    if( item < HaArray(sp->ss_array, mid) )
+    mid_el = HaArray(sp->ss_array, mid);
+    if( item < mid_el )
       r = mid - 1;
-    else if( item > HaArray(sp->ss_array, mid) )
+    else if( item > mid_el )
       l = mid + 1;
     else
       return *pos = mid, true;
@@ -227,11 +230,12 @@ void KheSetImplDelete(KHE_SET *sp, int item)
 void KheSetImplUnion(KHE_SET *to_sp, KHE_SET *from_sp)
 {
   int target_i, to_i, from_i, to_val, from_val;
+  char buff1[200], buff2[200];
   KheSetImplCheck(from_sp, "KheSetImplUnion from_sp initial value");
   KheSetImplCheck(to_sp, "KheSetImplUnion to_sp initial value");
   if( DEBUG2 )
     fprintf(stderr, "KheSetImplUnion(%s, %s) = ",
-      KheSetImplShow(to_sp), KheSetImplShow(from_sp));
+      KheSetImplShow(to_sp, buff1), KheSetImplShow(from_sp, buff2));
 
   /* if to_sp and from_sp are the exact same object, do nothing */
   if( to_sp == from_sp )
@@ -248,6 +252,9 @@ void KheSetImplUnion(KHE_SET *to_sp, KHE_SET *from_sp)
   to_i = HaArrayCount(to_sp->ss_array) - 1;
   from_i = HaArrayCount(from_sp->ss_array) - 1;
   target_i = HaArrayCount(to_sp->ss_array) + HaArrayCount(from_sp->ss_array) -1;
+  if( DEBUG3 && target_i > 10000 )
+    fprintf(stderr, "  KheSetImplUnion target_i = %d + %d - 1\n",
+      HaArrayCount(to_sp->ss_array), HaArrayCount(from_sp->ss_array));
   HaArrayFill(to_sp->ss_array, target_i + 1, 0);
   while( from_i >= 0 && to_i >= 0 )
   {
@@ -275,7 +282,7 @@ void KheSetImplUnion(KHE_SET *to_sp, KHE_SET *from_sp)
     from_i--;
   }
   if( DEBUG2 )
-    fprintf(stderr, "%s\n", KheSetImplShow(to_sp));
+    fprintf(stderr, "%s\n", KheSetImplShow(to_sp, buff1));
   KheSetImplCheck(from_sp, "KheSetImplUnion from_sp final value");
   KheSetImplCheck(to_sp, "KheSetImplUnion to_sp final value");
 }
@@ -332,11 +339,12 @@ void KheSetImplIntersect(KHE_SET *to_sp, KHE_SET *from_sp)
 void KheSetImplDifference(KHE_SET *to_sp, KHE_SET *from_sp)
 {
   int target_i, to_i, from_i, to_val, from_val;
+  char buff1[200], buff2[200];
   KheSetImplCheck(from_sp, "KheSetImplDifference from_sp initial value");
   KheSetImplCheck(to_sp, "KheSetImplDifference to_sp initial value");
   if( DEBUG2 )
     fprintf(stderr, "KheSetImplDifference(%s, %s) = ",
-      KheSetImplShow(to_sp), KheSetImplShow(from_sp));
+      KheSetImplShow(to_sp, buff1), KheSetImplShow(from_sp, buff2));
   from_i = to_i = target_i = 0;
   while( from_i < HaArrayCount(from_sp->ss_array) &&
          to_i < HaArrayCount(to_sp->ss_array) )
@@ -367,7 +375,7 @@ void KheSetImplDifference(KHE_SET *to_sp, KHE_SET *from_sp)
   KheSetImplCheck(from_sp, "KheSetImplDifference from_sp final value");
   KheSetImplCheck(to_sp, "KheSetImplDifference to_sp final value");
   if( DEBUG2 )
-    fprintf(stderr, "%s\n", KheSetImplShow(to_sp));
+    fprintf(stderr, "%s\n", KheSetImplShow(to_sp, buff1));
 }
 
 
@@ -388,11 +396,12 @@ void KheSetImplDifference(KHE_SET *to_sp, KHE_SET *from_sp)
 int KheSetImplUnionCount(KHE_SET *to_sp, KHE_SET *from_sp)
 {
   int to_i, from_i, to_val, from_val, res;
+  char buff1[200], buff2[200];
   KheSetImplCheck(from_sp, "KheSetImplUnionCount from_sp initial value");
   KheSetImplCheck(to_sp, "KheSetImplUnionCount to_sp initial value");
   if( DEBUG2 )
     fprintf(stderr, "KheSetImplUnionCount(%s, %s) = ",
-      KheSetImplShow(to_sp), KheSetImplShow(from_sp));
+      KheSetImplShow(to_sp, buff1), KheSetImplShow(from_sp, buff2));
   from_i = to_i = res = 0;
   while( from_i < HaArrayCount(from_sp->ss_array) &&
          to_i < HaArrayCount(to_sp->ss_array) )
@@ -432,11 +441,12 @@ int KheSetImplUnionCount(KHE_SET *to_sp, KHE_SET *from_sp)
 int KheSetImplIntersectCount(KHE_SET *to_sp, KHE_SET *from_sp)
 {
   int to_i, from_i, to_val, from_val, res, first_pos, last_pos; KHE_SET *tmp_sp;
+  char buff1[200], buff2[200];
   KheSetImplCheck(from_sp, "KheSetImplIntersectCount from_sp initial value");
   KheSetImplCheck(to_sp, "KheSetImplIntersectCount to_sp initial value");
   if( DEBUG2 )
     fprintf(stderr, "KheSetImplIntersectCount(%s, %s) = ",
-      KheSetImplShow(to_sp), KheSetImplShow(from_sp));
+      KheSetImplShow(to_sp, buff1), KheSetImplShow(from_sp, buff2));
 
   /* make from_sp into the smaller of the two sets */
   if( HaArrayCount(from_sp->ss_array) > HaArrayCount(to_sp->ss_array) )
@@ -482,11 +492,12 @@ int KheSetImplIntersectCount(KHE_SET *to_sp, KHE_SET *from_sp)
 int KheSetImplDifferenceCount(KHE_SET *to_sp, KHE_SET *from_sp)
 {
   int to_i, from_i, to_val, from_val, res;
+  char buff1[200], buff2[200];
   KheSetImplCheck(from_sp, "KheSetImplDifferenceCount from_sp initial value");
   KheSetImplCheck(to_sp, "KheSetImplDifferenceCount to_sp initial value");
   if( DEBUG2 )
     fprintf(stderr, "KheSetImplDifferenceCount(%s, %s) = ",
-      KheSetImplShow(to_sp), KheSetImplShow(from_sp));
+      KheSetImplShow(to_sp, buff1), KheSetImplShow(from_sp, buff2));
   from_i = to_i = res = 0;
   while( from_i < HaArrayCount(from_sp->ss_array) &&
          to_i < HaArrayCount(to_sp->ss_array) )
@@ -519,13 +530,14 @@ int KheSetImplDifferenceCount(KHE_SET *to_sp, KHE_SET *from_sp)
 int KheSetImplSymmetricDifferenceCount(KHE_SET *to_sp, KHE_SET *from_sp)
 {
   int to_i, from_i, to_val, from_val, res;
+  char buff1[200], buff2[200];
   KheSetImplCheck(from_sp,
     "KheSetImplSymmetricDifferenceCount from_sp initial value");
   KheSetImplCheck(to_sp,
     "KheSetImplSymmetricDifferenceCount to_sp initial value");
   if( DEBUG2 )
     fprintf(stderr, "KheSetImplSymmetricDifferenceCount(%s, %s) = ",
-      KheSetImplShow(to_sp), KheSetImplShow(from_sp));
+      KheSetImplShow(to_sp, buff1), KheSetImplShow(from_sp, buff2));
   from_i = to_i = res = 0;
   while( from_i < HaArrayCount(from_sp->ss_array) &&
          to_i < HaArrayCount(to_sp->ss_array) )
@@ -681,6 +693,39 @@ bool KheSetImplContains(KHE_SET *sp, int item)
 /*  Return a string representation of ss in static memory.                   */
 /*                                                                           */
 /*****************************************************************************/
+
+char *KheSetImplShow(KHE_SET *sp, char buff[200])
+{
+  char result_so_far[170], one_range[20], tmp[200];  int i, j, k;
+  snprintf(result_so_far, 170, "{");
+  for( i = 0;  i < HaArrayCount(sp->ss_array);  i = j )
+  {
+    /* find one range of groupable elements sp->ss_items[i .. j-1] */
+    for( j = i + 1;  j < HaArrayCount(sp->ss_array);  j++ )
+      if( HaArray(sp->ss_array, j) != HaArray(sp->ss_array, j-1) + 1 )
+	break;
+
+    /* print the range */
+    if( j - 1 == i )
+      snprintf(one_range, 20, "%u", HaArray(sp->ss_array, i));
+    else
+      snprintf(one_range, 20, "%u-%u", HaArray(sp->ss_array, i),
+	HaArray(sp->ss_array, j-1));
+
+    /* update result_so_far to contain the range */
+    snprintf(tmp, 200, "%s%s%s", result_so_far, i > 0 ? ", " : "", one_range);
+    for( k = 0;  k < 169 && tmp[k] != '\0';  k++ )
+      result_so_far[k] = tmp[k];
+    result_so_far[k] = '\0';
+  }
+
+  /* print the closing brace and return */
+  snprintf(buff, 200, "%s%s", result_so_far, "}");
+  return buff;
+}
+
+
+/* *** this code is perfectly safe, but the C compiler can't see that
 #define do_add(str) (strcpy(&buff[bpos], str), bpos += strlen(str))
 
 char *KheSetImplShow(KHE_SET *sp)
@@ -693,19 +738,19 @@ char *KheSetImplShow(KHE_SET *sp)
   do_add("{");
   for( i = 0;  i < HaArrayCount(sp->ss_array);  i = j )
   {
-    /* if running out of space, abbreviate and exit */
+    ** if running out of space, abbreviate and exit **
     if( bpos > 180 )
     {
       do_add(", ...");
       break;
     }
 
-    /* find range of groupable elements sp->ss_items[i .. j-1] */
+    ** find range of groupable elements sp->ss_items[i .. j-1] **
     for( j = i + 1;  j < HaArrayCount(sp->ss_array);  j++ )
       if( HaArray(sp->ss_array, j) != HaArray(sp->ss_array, j-1) + 1 )
 	break;
 
-    /* print the range, optionally preceded by a command and space */
+    ** print the range, optionally preceded by a command and space **
     if( i > 0 )
       do_add(", ");
     if( j - 1 == i )
@@ -718,6 +763,7 @@ char *KheSetImplShow(KHE_SET *sp)
   do_add("}");
   return buff;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -753,40 +799,44 @@ static void KheSetBuild(KHE_SET *sp, HA_ARENA a, int count, ...)
 
 void KheSetTest(FILE *fp)
 {
-  KHE_SET ss[5];  int i, j;  HA_ARENA a;
+  KHE_SET ss[5];  int i, j;  HA_ARENA a;  HA_ARENA_SET as;
+  char buff1[200], buff2[200];
   fprintf(fp, "[ KheSetTest\n");
 
   /* initialize ss[0] */
-  a = HaArenaMake(false);  /* this arena is used only for testing */
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);  /* this arena is used only for testing */
   KheSetBuild(&ss[0], a, 5, 2, 4, 6, 3, 1);
-  fprintf(fp, "  ss[0] = %s\n", KheSetShow(ss[0]));
+  fprintf(fp, "  ss[0] = %s\n", KheSetShow(ss[0], buff1));
 
   /* initialize ss[1] */
   KheSetBuild(&ss[1], a, 4, 8, 6, 4, 2);
-  fprintf(fp, "  ss[1] = %s\n", KheSetShow(ss[1]));
+  fprintf(fp, "  ss[1] = %s\n", KheSetShow(ss[1], buff1));
 
   /* initialize ss[2] */
   KheSetInit(ss[2], a);
-  fprintf(fp, "  ss[2] = %s\n", KheSetShow(ss[2]));
+  fprintf(fp, "  ss[2] = %s\n", KheSetShow(ss[2], buff1));
 
   /* initialize ss[3] */
   KheSetBuild(&ss[3], a, 1, 3);
-  fprintf(fp, "  ss[3] = %s\n", KheSetShow(ss[3]));
+  fprintf(fp, "  ss[3] = %s\n", KheSetShow(ss[3], buff1));
 
   /* initialize ss[4] */
   KheSetBuild(&ss[4], a, 5, 2, 4, 6, 3, 1);
-  fprintf(fp, "  ss[4] = %s\n", KheSetShow(ss[4]));
+  fprintf(fp, "  ss[4] = %s\n", KheSetShow(ss[4], buff1));
 
   /* testing count and the others */
   fprintf(fp, "\n");
   for( i = 0;  i < 5;  i++ )
   {
-    fprintf(fp, "  KheSetCount(%s) = %d\n", KheSetShow(ss[i]),
+    fprintf(fp, "  KheSetCount(%s) = %d\n", KheSetShow(ss[i], buff1),
       KheSetCount(ss[i]));
     if( KheSetCount(ss[i]) > 0 )
     {
-      fprintf(fp, "  KheSetMin(%s) = %d\n", KheSetShow(ss[i]),KheSetMin(ss[i]));
-      fprintf(fp, "  KheSetMax(%s) = %d\n", KheSetShow(ss[i]),KheSetMax(ss[i]));
+      fprintf(fp, "  KheSetMin(%s) = %d\n", KheSetShow(ss[i], buff1),
+	KheSetMin(ss[i]));
+      fprintf(fp, "  KheSetMax(%s) = %d\n", KheSetShow(ss[i], buff1),
+	KheSetMax(ss[i]));
     }
   }
 
@@ -795,67 +845,66 @@ void KheSetTest(FILE *fp)
   for( i = 0;  i < 4;  i++ )
     for( j = 0;  j <= 8;  j += 2 )
       fprintf(fp, "  KheSetContains(%s, %d) = %s\n",
-	KheSetShow(ss[i]), j, KheSetContains(ss[i], j) ? "true" : "false");
+	KheSetShow(ss[i], buff1), j, bool_show(KheSetContains(ss[i], j)));
 
   /* testing KheSetEquals */
   fprintf(fp, "\n");
   for( i = 0;  i < 5;  i++ )
     for( j = 0;  j < 5;  j++ )
-      fprintf(fp, "  KheSetEqual(%s, %s) = %s\n", KheSetShow(ss[i]),
-	KheSetShow(ss[j]), KheSetEqual(ss[i], ss[j]) ? "true" : "false");
+      fprintf(fp, "  KheSetEqual(%s, %s) = %s\n", KheSetShow(ss[i], buff1),
+	KheSetShow(ss[j], buff2), bool_show(KheSetEqual(ss[i], ss[j])));
 
   /* testing KheSetSubset */
   fprintf(fp, "\n");
   for( i = 0;  i < 5;  i++ )
     for( j = 0;  j < 5;  j++ )
-      fprintf(fp, "  KheSetSubset(%s, %s) = %s\n", KheSetShow(ss[i]),
-	KheSetShow(ss[j]), KheSetSubset(ss[i], ss[j]) ? "true" : "false");
+      fprintf(fp, "  KheSetSubset(%s, %s) = %s\n", KheSetShow(ss[i], buff1),
+	KheSetShow(ss[j], buff2), bool_show(KheSetSubset(ss[i], ss[j])));
 
   /* testing KheSetDisjoint */
   fprintf(fp, "\n");
   for( i = 0;  i < 5;  i++ )
     for( j = 0;  j < 5;  j++ )
-      fprintf(fp, "  KheSetDisjoint(%s, %s) = %s\n", KheSetShow(ss[i]),
-	KheSetShow(ss[j]), KheSetDisjoint(ss[i], ss[j]) ? "true" : "false");
+      fprintf(fp, "  KheSetDisjoint(%s, %s) = %s\n", KheSetShow(ss[i], buff1),
+	KheSetShow(ss[j], buff2), bool_show(KheSetDisjoint(ss[i], ss[j])));
 
   /* testing KheSetUnion */
   fprintf(fp, "\n");
   for( i = 0;  i < 5;  i++ )
   {
-    fprintf(fp, "  KheSetUnion(%s, %s) = ",KheSetShow(ss[3]),KheSetShow(ss[i]));
+    fprintf(fp, "  KheSetUnion(%s, %s) = ", KheSetShow(ss[3], buff1),
+      KheSetShow(ss[i], buff2));
     KheSetUnion(ss[3], ss[i]);
-    fprintf(fp, "%s\n", KheSetShow(ss[3]));
+    fprintf(fp, "%s\n", KheSetShow(ss[3], buff1));
   }
 
   /* testing KheSetDifference */
   fprintf(fp, "\n");
   for( i = 0;  i < 5;  i++ )  if( i != 3 )
   {
-    fprintf(fp, "  KheSetDifference(%s, %s) = ", KheSetShow(ss[3]),
-      KheSetShow(ss[i]));
+    fprintf(fp, "  KheSetDifference(%s, %s) = ", KheSetShow(ss[3], buff1),
+      KheSetShow(ss[i], buff2));
     KheSetDifference(ss[3], ss[i]);
-    fprintf(fp, "%s\n", KheSetShow(ss[3]));
+    fprintf(fp, "%s\n", KheSetShow(ss[3], buff1));
   }
 
   /* testing KheSetIntersect */
   fprintf(fp, "\n");
   for( i = 0;  i < 5;  i++ )
   {
-    fprintf(fp, "  KheSetIntersect(%s, %s) = ", KheSetShow(ss[0]),
-      KheSetShow(ss[i]));
+    fprintf(fp, "  KheSetIntersect(%s, %s) = ", KheSetShow(ss[0], buff1),
+      KheSetShow(ss[i], buff2));
     KheSetIntersect(ss[0], ss[i]);
-    fprintf(fp, "%s\n", KheSetShow(ss[0]));
+    fprintf(fp, "%s\n", KheSetShow(ss[0], buff1));
   }
 
   fprintf(fp, "] KheSetTest\n");
 }
-
-
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  Submodule "tables indexed by KheSets"                                      */
+/*  Submodule "tables indexed by KheSets"                                    */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -907,7 +956,7 @@ KHE_SET_TABLE KheSetTableMake(HA_ARENA a)
 
 void KheSetImplTableInsert(KHE_SET_TABLE st, KHE_SET *sp, void *val)
 {
-  int i, key;  KHE_SET_TABLE st2;
+  int i, key;  KHE_SET_TABLE st2;  char buff[200];
   HnAssert(val != NULL, "KheSetImplTableInsert:  NULL val");
   for( i = 0;  i < HaArrayCount(sp->ss_array);  i++ )
   {
@@ -923,7 +972,7 @@ void KheSetImplTableInsert(KHE_SET_TABLE st, KHE_SET *sp, void *val)
     st = st2;
   }
   HnAssert(st->entry.key == NULL, "KheSetTableInsert: key %s already present",
-    KheSetImplShow(sp));
+    KheSetImplShow(sp, buff));
   st->entry.key = sp;
   st->entry.value = val;
 }
@@ -967,13 +1016,14 @@ bool KheSetImplTableRetrieve(KHE_SET_TABLE st, KHE_SET *sp, void **val)
 
 void KheSetTableDoDebug(KHE_SET_TABLE st, int encl_index, int indent, FILE *fp)
 {
-  KHE_SET_TABLE st2;  int i;
+  KHE_SET_TABLE st2;  int i;  char buff[200];
   if( encl_index == -1 )
     fprintf(fp, "%*s[", indent, "");
   else
     fprintf(fp, "%*s%d [", indent, "", encl_index);
   if( st->entry.key != NULL )
-    fprintf(fp, " %s -> %p", KheSetImplShow(st->entry.key), st->entry.value);
+    fprintf(fp, " %s -> %p", KheSetImplShow(st->entry.key, buff),
+      st->entry.value);
   if( HaArrayCount(st->sub_tables) == 0 )
     fprintf(fp, " ]\n");
   else
@@ -1001,8 +1051,8 @@ void KheSetTableDebug(KHE_SET_TABLE st, int indent, FILE *fp)
 
 static void KheSetDebugRetrieve(KHE_SET_TABLE st, KHE_SET *sp, FILE *fp)
 {
-  void *val;
-  fprintf(fp, "KheSetTableRetrieve(st, %s, &val) = ", KheSetShow(*sp));
+  void *val;  char buff[200];
+  fprintf(fp, "KheSetTableRetrieve(st, %s, &val) = ", KheSetShow(*sp, buff));
   if( KheSetTableRetrieve(st, *sp, &val) )
     fprintf(fp, "true, val = %p\n", val);
   else
@@ -1020,10 +1070,12 @@ static void KheSetDebugRetrieve(KHE_SET_TABLE st, KHE_SET *sp, FILE *fp)
 
 void KheSetTableTest(FILE *fp)
 {
-  KHE_SET_TABLE st;  KHE_SET ss[5];  int i;  HA_ARENA a;
+  KHE_SET_TABLE st;  KHE_SET ss[5];  int i;  HA_ARENA a;  HA_ARENA_SET as;
+  char buff[200];
 
   /* initialize */
-  a = HaArenaMake(false);  /* this arena is used only for testing */
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);  /* this arena is used only for testing */
   st = KheSetTableMake(a);
   fprintf(fp, "\nafter initializing, table is:\n");
   KheSetTableDebug(st, 0, fp);
@@ -1031,19 +1083,19 @@ void KheSetTableTest(FILE *fp)
   /* first insertion */
   KheSetBuild(&ss[0], a, 4, 1, 2, 3, 5);
   KheSetTableInsert(st, ss[0], (void *) 0x1);
-  fprintf(fp, "\nafter inserting %s, table is:\n", KheSetShow(ss[0]));
+  fprintf(fp, "\nafter inserting %s, table is:\n", KheSetShow(ss[0], buff));
   KheSetTableDebug(st, 0, fp);
 
   /* second insertion */
   KheSetBuild(&ss[1], a, 3, 1, 2, 3);
   KheSetTableInsert(st, ss[1], (void *) 0x2);
-  fprintf(fp, "\nafter inserting %s, table is:\n", KheSetShow(ss[1]));
+  fprintf(fp, "\nafter inserting %s, table is:\n", KheSetShow(ss[1], buff));
   KheSetTableDebug(st, 0, fp);
 
   /* third insertion */
   KheSetBuild(&ss[2], a, 3, 1, 3, 5);
   KheSetTableInsert(st, ss[2], (void *) 0x3);
-  fprintf(fp, "\nafter inserting %s, table is:\n", KheSetShow(ss[2]));
+  fprintf(fp, "\nafter inserting %s, table is:\n", KheSetShow(ss[2], buff));
   KheSetTableDebug(st, 0, fp);
 
   /* two more, not in the table */
