@@ -85,7 +85,7 @@ bool KheSolnGroupMake(KHE_ARCHIVE archive, char *id,
   }
 
   /* OK, so go ahead and make the soln group, in its own arena */
-  a = KheArchiveArenaBegin(archive, false);
+  a = KheArchiveArenaBegin(archive);
   HaMake(res, a);
   res->arena = a;
   res->back = NULL;
@@ -109,33 +109,30 @@ bool KheSolnGroupMake(KHE_ARCHIVE archive, char *id,
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheSolnGroupDelete(KHE_SOLN_GROUP soln_group)                       */
+/*  void KheSolnGroupDelete(KHE_SOLN_GROUP soln_group, bool delete_solns)    */
 /*                                                                           */
-/*  Delete soln_group.  At present this does not delete its solutions,       */
-/*  altogether, because they could lie in multiple solution groups.          */
+/*  Delete soln_group.  If delete_solns is true, also delete its solutions.  */
 /*                                                                           */
 /*****************************************************************************/
 
-void KheSolnGroupDelete(KHE_SOLN_GROUP soln_group)
+void KheSolnGroupDelete(KHE_SOLN_GROUP soln_group, bool delete_solns)
 {
-  /* KHE_SOLN_SET ss;  int i; */
+  KHE_SOLN soln;
 
-  /* remove soln_group's solutions from soln_group but do not delete them */
+  /* remove soln_group's solutions from soln_group; optionally delete them */
   while( HaArrayCount(soln_group->solutions) > 0 )
-    KheSolnGroupDeleteSoln(soln_group, HaArrayFirst(soln_group->solutions));
+  {
+    soln = HaArrayFirst(soln_group->solutions);
+    KheSolnGroupDeleteSoln(soln_group, soln);
+    if( delete_solns )
+      KheSolnDelete(soln);
+  }
 
   /* remove soln_group from its archive */
   KheArchiveDeleteSolnGroup(soln_group->archive, soln_group);
 
   /* free memory */
   KheArchiveArenaEnd(soln_group->archive, soln_group->arena);
-  /* ***
-  MArrayFree(soln_group->solutions);
-  HaArrayForEach(soln_group->soln_sets, &ss, &i)
-    KheSolnSetDelete(ss);
-  MArrayFree(soln_group->soln_sets);
-  MFree(soln_group);
-  *** */
 }
 
 
@@ -558,8 +555,7 @@ KHE_SOLN_SET KheSolnGroupInstanceSolnSet(KHE_SOLN_GROUP soln_group,
 {
   int index;
   if( !KheArchiveContainsInstance(soln_group->archive, ins, &index) )
-    HnAbort("KheSolnGroupInstanceSolnSet: soln's instance not in soln_group's"
-      " archive");
+    HnAbort("KheSolnGroupInstanceSolnSet: ins not in soln_group's archive");
   return KheSolnGroupInstanceSolnSetByIndex(soln_group, index);
 }
 
