@@ -74,7 +74,9 @@ KHE_SPLIT_EVENTS_MONITOR KheSplitEventsMonitorMake(KHE_EVENT_IN_SOLN es,
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res,
-    soln, KHE_SPLIT_EVENTS_MONITOR_TAG);
+    soln, KHE_SPLIT_EVENTS_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->event_in_soln = es;
   res->constraint = c;
   res->min_duration = KheSplitEventsConstraintMinDuration(c);
@@ -138,6 +140,7 @@ void KheSplitEventsMonitorCopyPhase2(KHE_SPLIT_EVENTS_MONITOR m)
   {
     m->copy = NULL;
     KheMonitorCopyCommonFieldsPhase2((KHE_MONITOR) m);
+    KheEventInSolnCopyPhase2(m->event_in_soln);
   }
 }
 
@@ -235,7 +238,7 @@ void KheSplitEventsMonitorAttachToSoln(KHE_SPLIT_EVENTS_MONITOR m)
   m->deviation = m->min_amount;
   if( m->deviation > 0 )
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   KheEventInSolnAttachMonitor(m->event_in_soln, (KHE_MONITOR) m);
 }
 
@@ -281,6 +284,62 @@ void KheSplitEventsMonitorAttachCheck(KHE_SPLIT_EVENTS_MONITOR m)
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheSplitEventsMonitorTimeRange(KHE_SPLIT_EVENTS_MONITOR m,          */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheSplitEventsMonitorTimeRange(KHE_SPLIT_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_EVENT e;
+  e = KheSplitEventsMonitorEvent(m);
+  return KheEventTimeRange(e, m->soln, first_time, last_time);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheSplitEventsMonitorSetSweepTime(KHE_SPLIT_EVENTS_MONITOR m,       */
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheSplitEventsMonitorSetSweepTime(KHE_SPLIT_EVENTS_MONITOR m,
+  KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheSplitEventsMonitorSweepTimeRange(KHE_SPLIT_EVENTS_MONITOR m,     */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for m.                                */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheSplitEventsMonitorSweepTimeRange(KHE_SPLIT_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -314,7 +373,7 @@ void KheSplitEventsMonitorAddMeet(KHE_SPLIT_EVENTS_MONITOR m, KHE_MEET meet)
 
   /* update the cost */
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
@@ -346,7 +405,7 @@ void KheSplitEventsMonitorDeleteMeet(KHE_SPLIT_EVENTS_MONITOR m,
 
   /* update the cost */
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
@@ -384,7 +443,7 @@ void KheSplitEventsMonitorSplitMeet(KHE_SPLIT_EVENTS_MONITOR m,
 
   /* update the cost */
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
@@ -431,7 +490,7 @@ void KheSplitEventsMonitorMergeMeet(KHE_SPLIT_EVENTS_MONITOR m,
 
   /* do the update if deviations have changed */
   KheMonitorChangeCost((KHE_MONITOR) m,
-    KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+    KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 }
 
 
