@@ -131,7 +131,7 @@ static KHE_SPREAD_TIME_GROUP KheSpreadTimeGroupCopyPhase1(
   {
     HaMake(copy, a);
     stg->copy = copy;
-    copy->time_group = stg->time_group;
+    copy->time_group = KheTimeGroupCopy(stg->time_group, a);
     copy->minimum = stg->minimum;
     copy->maximum = stg->maximum;
     copy->incidences = stg->incidences;
@@ -257,7 +257,9 @@ KHE_SPREAD_EVENTS_MONITOR KheSpreadEventsMonitorMake(KHE_SOLN soln,
   HaMake(res, a);
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
-    KHE_SPREAD_EVENTS_MONITOR_TAG);
+    KHE_SPREAD_EVENTS_MONITOR_TAG,
+    KheConstraintCostFunction((KHE_CONSTRAINT) c),
+    KheConstraintCombinedWeight((KHE_CONSTRAINT) c));
   res->constraint = c;
   res->event_group = eg;
   res->deviation = 0;
@@ -495,7 +497,7 @@ void KheSpreadEventsMonitorAttachToSoln(KHE_SPREAD_EVENTS_MONITOR m)
   }
   if( m->deviation != 0 )
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
 
   /* attach */
   for( i = 0;  i < KheEventGroupEventCount(m->event_group);  i++ )
@@ -525,7 +527,7 @@ void KheSpreadEventsMonitorAttachToSoln(KHE_SPREAD_EVENTS_MONITOR m)
     if( KheDevMonitorHasChanged(&m->dev_monitor) )
     {
       KheMonitorChangeCost((KHE_MONITOR) m,
-	KheConstraintCostMulti((KHE_CONSTRAINT) m->constraint,
+	KheCons traintCostMulti((KHE_CONSTRAINT) m->constraint,
 	  KheDevMonitorDevs(&m->dev_monitor)));
       KheDevMonitorFlush(&m->dev_monitor);
     }
@@ -542,7 +544,7 @@ void KheSpreadEventsMonitorAttachToSoln(KHE_SPREAD_EVENTS_MONITOR m)
     if( m->deviation != 0 )
     {
       KheMonitorChangeCost((KHE_MONITOR) m,
-        KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+        KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
     }
   }
 
@@ -596,6 +598,73 @@ void KheSpreadEventsMonitorAttachCheck(KHE_SPREAD_EVENTS_MONITOR m)
 *** */
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "time ranges and sweep times"                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheSpreadEventsMonitorTimeRange(KHE_SPREAD_EVENTS_MONITOR m,        */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorTimeRange for m.                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheSpreadEventsMonitorTimeRange(KHE_SPREAD_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  KHE_EVENT_GROUP eg;  KHE_EVENT e;  KHE_TIME ft, lt;  int i;
+  eg = KheSpreadEventsMonitorEventGroup(m);
+  *first_time = *last_time = NULL;
+  for( i = 0;  i < KheEventGroupEventCount(eg);  i++ )
+  {
+    e = KheEventGroupEvent(eg, i);
+    if( KheEventTimeRange(e, m->soln, &ft, &lt) )
+    {
+      if( *first_time == NULL || KheTimeIndex(ft) < KheTimeIndex(*first_time) )
+	*first_time = ft;
+      if( *last_time == NULL || KheTimeIndex(lt) > KheTimeIndex(*last_time) )
+	*last_time = ft;
+    }
+  }
+  return *first_time != NULL;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheSpreadEventsMonitorSetSweepTime(KHE_SPREAD_EVENTS_MONITOR m,     */
+/*    KHE_ASSIGN_RESOURCE_MONITOR m, KHE_TIME t)                             */
+/*                                                                           */
+/*  Implement KheMonitorSetSweepTime for m.                                  */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheSpreadEventsMonitorSetSweepTime(KHE_SPREAD_EVENTS_MONITOR m, KHE_TIME t)
+{
+  /* nothing to do */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheSpreadEventsMonitorSweepTimeRange(KHE_SPREAD_EVENTS_MONITOR m,   */
+/*    KHE_TIME *first_time, KHE_TIME *last_time)                             */
+/*                                                                           */
+/*  Implement KheMonitorSweepTimeRange for m.                                */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheSpreadEventsMonitorSweepTimeRange(KHE_SPREAD_EVENTS_MONITOR m,
+  KHE_TIME *first_time, KHE_TIME *last_time)
+{
+  return *first_time = *last_time = NULL, false;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "monitoring calls"                                             */
@@ -698,7 +767,7 @@ void KheSpreadEventsMonitorAssignTime(KHE_SPREAD_EVENTS_MONITOR m,
   {
     m->deviation += delta_devs;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
   if( DEBUG1 )
     fprintf(stderr, "] KheSpreadEventsMonitorAssignTime returning\n");
@@ -739,7 +808,7 @@ void KheSpreadEventsMonitorAssignTime(KHE_SPREAD_EVENTS_MONITOR m,
     if( KheDevMonitorHasChanged(&m->dev_monitor) )
     {
       KheMonitorChangeCost((KHE_MONITOR) m,
-        KheConstraintCostMulti((KHE_CONSTRAINT) m->constraint,
+        KheConstrai ntCostMulti((KHE_CONSTRAINT) m->constraint,
 	  KheDevMonitorDevs(&m->dev_monitor)));
       KheDevMonitorFlush(&m->dev_monitor);
     }
@@ -759,7 +828,7 @@ void KheSpreadEventsMonitorAssignTime(KHE_SPREAD_EVENTS_MONITOR m,
     {
       m->deviation += delta_devs;
       KheMonitorChangeCost((KHE_MONITOR) m,
-        KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+        KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
     }
   }
   if( DEBUG1 )
@@ -796,7 +865,7 @@ void KheSpreadEventsMonitorUnAssignTime(KHE_SPREAD_EVENTS_MONITOR m,
   {
     m->deviation += delta_devs;
     KheMonitorChangeCost((KHE_MONITOR) m,
-      KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+      KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
   }
 }
 
@@ -832,7 +901,7 @@ void KheSpreadEventsMonitorUnAssignTime(KHE_SPREAD_EVENTS_MONITOR m,
     if( KheDevMonitorHasChanged(&m->dev_monitor) )
     {
       KheMonitorChangeCost((KHE_MONITOR) m,
-	KheConstraintCostMulti((KHE_CONSTRAINT) m->constraint,
+	KheConstra intCostMulti((KHE_CONSTRAINT) m->constraint,
 	  KheDevMonitorDevs(&m->dev_monitor)));
       KheDevMonitorFlush(&m->dev_monitor);
     }
@@ -852,7 +921,7 @@ void KheSpreadEventsMonitorUnAssignTime(KHE_SPREAD_EVENTS_MONITOR m,
     {
       m->deviation += delta_devs;
       KheMonitorChangeCost((KHE_MONITOR) m,
-        KheConstraintCost((KHE_CONSTRAINT) m->constraint, m->deviation));
+        KheMonitorDevToCost((KHE_MONITOR) m, m->deviation));
     }
   }
 }
