@@ -366,7 +366,7 @@ KHE_TASK_BOUND KheTaskBoundCopyPhase1(KHE_TASK_BOUND tb, HA_ARENA a)
     copy->soln = KheSolnCopyPhase1(tb->soln);
     /* copy->soln_index = tb->soln_index; */
     copy->reference_count = 1;  /* no paths, and tb is linked in */
-    copy->resource_group = tb->resource_group;
+    copy->resource_group = KheResourceGroupCopy(tb->resource_group, a);
     HaArrayInit(copy->tasks, a);
     HaArrayForEach(tb->tasks, task, i)
       HaArrayAddLast(copy->tasks, KheTaskCopyPhase1(task, a));
@@ -386,9 +386,13 @@ KHE_TASK_BOUND KheTaskBoundCopyPhase1(KHE_TASK_BOUND tb, HA_ARENA a)
 
 void KheTaskBoundCopyPhase2(KHE_TASK_BOUND tb)
 {
+  int i;  KHE_TASK task;
   if( tb->copy != NULL )
   {
     tb->copy = NULL;
+    KheSolnCopyPhase2(tb->soln);
+    HaArrayForEach(tb->tasks, task, i)
+      KheTaskCopyPhase2(task);
   }
 }
 
