@@ -403,6 +403,7 @@ float KheTaskSetTotalWorkload(KHE_TASK_SET ts)
 /*                                                                           */
 /*****************************************************************************/
 
+/* ***
 void KheTaskSetUnGroup(KHE_TASK_SET ts)
 {
   KHE_TASK task;  int i;  KHE_RESOURCE r;
@@ -413,6 +414,7 @@ void KheTaskSetUnGroup(KHE_TASK_SET ts)
       KheTaskAssignResource(task, r);
   }
 }
+*** */
 
 
 /*****************************************************************************/
@@ -729,6 +731,60 @@ bool KheTaskSetPartMoveResource(KHE_TASK_SET ts,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheTaskSetAssignFix(KHE_TASK_SET ts)                                */
+/*                                                                           */
+/*  Fix the tasks of ts.                                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheTaskSetAssignFix(KHE_TASK_SET ts)
+{
+  KHE_TASK task;  int i;
+  HaArrayForEach(ts->tasks, task, i)
+    KheTaskAssignFix(task);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheTaskSetAssignUnFix(KHE_TASK_SET ts)                              */
+/*                                                                           */
+/*  Unfix the tasks of ts.                                                   */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheTaskSetAssignUnFix(KHE_TASK_SET ts)
+{
+  KHE_TASK task;  int i;
+  HaArrayForEach(ts->tasks, task, i)
+    KheTaskAssignUnFix(task);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheTaskSetOneLineDebug(KHE_TASK_SET ts, int verbosity, FILE *fp)    */
+/*                                                                           */
+/*  Print ts in full onto a single line of fp.                               */
+/*                                                                           */
+/*****************************************************************************/
+
+static void KheTaskSetOneLineDebug(KHE_TASK_SET ts, int verbosity, FILE *fp)
+{
+  KHE_TASK task;  int i;
+  fprintf(fp, "{");
+  HaArrayForEach(ts->tasks, task, i)
+  {
+    if( i > 0 )
+      fprintf(fp, ", ");
+    KheTaskDebug(task, verbosity, -1, fp);
+  }
+  fprintf(fp, "}");
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /* void KheTaskSetDebug(KHE_TASK_SET ts, int verbosity, int indent, FILE *fp)*/
@@ -740,28 +796,53 @@ bool KheTaskSetPartMoveResource(KHE_TASK_SET ts,
 void KheTaskSetDebug(KHE_TASK_SET ts, int verbosity, int indent, FILE *fp)
 {
   KHE_TASK task;  int i;
-  if( verbosity >= 1 )
+  if( verbosity < 1 )
   {
-    if( indent >= 0 )
-      fprintf(fp, "%*s", indent, "");
-    fprintf(fp, "{");
-    HaArrayForEach(ts->tasks, task, i)
+    /* print nothing */
+  }
+  else if( indent < 0 )
+  {
+    if( HaArrayCount(ts->tasks) <= 3 )
     {
-      if( i > 0 )
-	fprintf(fp, ", ");
+      /* complete one-line print */
+      KheTaskSetOneLineDebug(ts, verbosity, fp);
+    }
+    else
+    {
+      /* incomplete one-line print */
+      fprintf(fp, "{");
+      task = HaArrayFirst(ts->tasks);
       KheTaskDebug(task, verbosity, -1, fp);
+      fprintf(fp, ", ... (%d tasks)", HaArrayCount(ts->tasks));
+      fprintf(fp, "}");
     }
-    fprintf(fp, "}");
-    /* *** correct but currently unused
-    if( verbosity >= 2 && HaArrayCount(ts->tasks) > 0 )
+  }
+  else
+  {
+    fprintf(fp, "%*s", indent, "");
+    if( HaArrayCount(ts->tasks) <= 5 )
     {
-      if( KheTaskSetAllVisited(ts, 0) )
-	fprintf(fp, "V");
-      else if( KheTaskSetAnyVisited(ts, 0) )
-	fprintf(fp, "v");
+      /* complete one-line print */
+      KheTaskSetOneLineDebug(ts, verbosity, fp);
     }
-    *** */
-    if( indent >= 0 )
-      fprintf(fp, "\n");
+    else
+    {
+      /* complete multi-line print */
+      fprintf(fp, "{ ");
+      HaArrayForEach(ts->tasks, task, i)
+      {
+	if( i > 0 )
+	{
+	  if( i % 5 == 0 )
+	    fprintf(fp, ",\n%*s", indent + 2, "");
+	  else
+	    fprintf(fp, ", ");
+	}
+	KheTaskDebug(task, verbosity, -1, fp);
+      }
+      /* fprintf(fp, "\n%*s", indent, ""); */
+      fprintf(fp, " }");
+    }
+    fprintf(fp, "\n");
   }
 }
