@@ -114,7 +114,7 @@ void KheTimeFinalize(KHE_TIME t)
   SSetInit(ss, a);
   SSetInsert(ss, KheTimeIndex(t));
   t->singleton_time_group = KheTimeGroupMakeAndFinalize(t->instance,
-    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, NULL, true);
+    KHE_TIME_GROUP_KIND_AUTO, NULL, NULL, &ss, /* NULL, */ true, a);
 }
 
 
