@@ -89,15 +89,15 @@ struct khe_time_group_rec {
 /*****************************************************************************/
 
 static KHE_TIME_GROUP_NHOOD KheTimeGroupNHoodMakeEmpty(SSET *base_set,
-  KHE_INSTANCE ins, KHE_SOLN soln)
+  KHE_INSTANCE ins, KHE_TIME_GROUP_KIND kind, /* KHE_SOLN soln */ HA_ARENA a)
 {
-  KHE_TIME_GROUP_NHOOD res;  HA_ARENA a;
-  a = (soln != NULL ? KheSolnArena(soln) : KheInstanceArena(ins));
+  KHE_TIME_GROUP_NHOOD res;
+  /* a = (soln != NULL ? KheSolnArena(soln) : KheInstanceArena(ins)); */
   HaMake(res, a);
   res->base_set = *base_set;
   res->instance = ins;
   HaArrayInit(res->time_groups, a);
-  if( soln == NULL )
+  if( kind != KHE_TIME_GROUP_KIND_SOLN )
     KheInstanceTimeGroupNeighbourhoodTableInsert(ins, &res->base_set, res);
   /* *** arenas take care of this now
   else
@@ -123,7 +123,8 @@ static KHE_TIME_GROUP_NHOOD KheTimeGroupNHoodMakeEmpty(SSET *base_set,
 /*****************************************************************************/
 
 static KHE_TIME_GROUP_NHOOD KheTimeGroupNhoodMake(KHE_TIME_GROUP tg,
-  KHE_SOLN soln, int *index_in_nhood)
+  /* KHE_TIME_GROUP_KIND kind, KHE_SOLN soln, */ HA_ARENA arena,
+  int *index_in_nhood)
 {
   KHE_TIME_GROUP_NHOOD res;  int a, b, count, max_index, delta;
   SSET *tg_set, shifted_set, base_set;  KHE_TIME ta, tb;
@@ -145,7 +146,7 @@ static KHE_TIME_GROUP_NHOOD KheTimeGroupNhoodMake(KHE_TIME_GROUP tg,
     /* tg is empty; retrieve or make empty neighbourhood (special case) */
     *index_in_nhood = 0;
     if( !KheInstanceTimeGroupNeighbourhoodTableRetrieve(ins, tg_set, &res) )
-      res = KheTimeGroupNHoodMakeEmpty(tg_set, ins, soln);
+      res = KheTimeGroupNHoodMakeEmpty(tg_set, ins, tg->kind, /* soln */ arena);
   }
   else
   {
@@ -160,7 +161,7 @@ static KHE_TIME_GROUP_NHOOD KheTimeGroupNhoodMake(KHE_TIME_GROUP tg,
     if( !KheInstanceTimeGroupNeighbourhoodTableRetrieve(ins, &base_set, &res) )
     {
       /* no existing neighbourhood; make one, and add non-empty time groups */
-      res = KheTimeGroupNHoodMakeEmpty(&base_set, ins, soln);
+      res = KheTimeGroupNHoodMakeEmpty(&base_set, ins, tg->kind, /*soln*/arena);
       max_index = KheInstanceTimeCount(ins) - 1;
       b = max_index - KheTimeIndex(tb);
       if( DEBUG1 )
@@ -174,7 +175,8 @@ static KHE_TIME_GROUP_NHOOD KheTimeGroupNhoodMake(KHE_TIME_GROUP tg,
 	  fprintf(stderr, "  delta = %d:\n", delta);
 	SSetInitShiftedAndSliced(shifted_set, *tg_set, delta, 0, max_index);
 	shifted_tg = KheTimeGroupMakeAndFinalize(ins, KHE_TIME_GROUP_KIND_AUTO,
-	  KheTimeGroupId(tg), KheTimeGroupName(tg), &shifted_set, soln, false);
+	  KheTimeGroupId(tg), KheTimeGroupName(tg), &shifted_set, false,
+	  /* soln */ arena);
 	if( shifted_tg->neighbourhood == NULL )
 	{
 	  shifted_tg->neighbourhood = res;
@@ -248,10 +250,11 @@ static KHE_TIME_GROUP KheTimeGroupNHoodNeighbour(KHE_TIME_GROUP_NHOOD tgn,
 /*****************************************************************************/
 
 static KHE_TIME_GROUP KheTimeGroupMakeInternal(KHE_INSTANCE ins,
-  KHE_SOLN soln, KHE_TIME_GROUP_KIND kind, char *id, char *name)
+  /* KHE_SOLN soln, */ KHE_TIME_GROUP_KIND kind, char *id, char *name,
+  HA_ARENA a)
 {
-  KHE_TIME_GROUP res;  static int count = 0;  HA_ARENA a;
-  a = (soln != NULL ? KheSolnArena(soln) : KheInstanceArena(ins));
+  KHE_TIME_GROUP res;  static int count = 0;  /* HA_ARENA a; */
+  /* a = (soln != NULL ? KheSolnArena(soln) : KheInstanceArena(ins)); */
   HaMake(res, a);
   res->back = NULL;
   res->instance = ins;
@@ -271,6 +274,42 @@ static KHE_TIME_GROUP KheTimeGroupMakeInternal(KHE_INSTANCE ins,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_TIME_GROUP KheTimeGroupCopy(KHE_TIME_GROUP tg, HA_ARENA a)           */
+/*                                                                           */
+/*  Make a copy of tg, or share it if it is safe to do so.                   */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_TIME_GROUP KheTimeGroupCopy(KHE_TIME_GROUP tg, HA_ARENA a)
+{
+  KHE_TIME_GROUP res;
+  if( tg->kind == KHE_TIME_GROUP_KIND_SOLN )
+  {
+    /* need a true copy */
+    HaMake(res, a);
+    res->back = tg->back;
+    res->instance = tg->instance;
+    res->kind = tg->kind;
+    res->id = HnStringCopy(tg->id, a);
+    res->name = HnStringCopy(tg->name, a);
+    SSetCopy(res->times, tg->times, a);
+    if( tg->neighbourhood != NULL )
+      res->neighbourhood = KheTimeGroupNhoodMake(res, a, /*soln,*/
+	&res->pos_in_nhood);
+    res->neighbourhood = NULL;  /* still to do */
+    res->pos_in_nhood = -1;	/* still to do */
+    return res;
+  }
+  else
+  {
+    /* sharing is safe */
+    return tg;
+  }
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  char *KheTimeGroupFinalName(KHE_TIME_GROUP tg, HA_ARENA a)               */
@@ -301,11 +340,13 @@ static char *KheTimeGroupFinalId(KHE_TIME_GROUP tg, HA_ARENA a)
   if( tg->id == NULL )
   {
     if( KheTimeGroupTimeCount(tg) == 0 )
-      return "no-times";
+      return "TG:Empty";
     else if( KheTimeGroupTimeCount(tg) == KheInstanceTimeCount(tg->instance) )
-      return "all-times";
+      return "TG:All";
+    else if( KheTimeGroupTimeCount(tg) == 1 )
+      return HnStringMake(a, "TG:%s", KheTimeId(KheTimeGroupTime(tg, 0)));
     else
-      return "no-id";
+      return "TG:NoId";
   }
   else if( SSetShift(tg->times) == 0 )
     return tg->id;
@@ -323,19 +364,21 @@ static char *KheTimeGroupFinalId(KHE_TIME_GROUP tg, HA_ARENA a)
 /*                                                                           */
 /*****************************************************************************/
 
-void KheTimeGroupFinalize(KHE_TIME_GROUP tg, KHE_SOLN soln, bool with_nhood)
+void KheTimeGroupFinalize(KHE_TIME_GROUP tg, /* KHE_SOLN soln, */ HA_ARENA a,
+  bool with_nhood)
 {
-  HA_ARENA a;
+  /* HA_ARENA a; */
   if( DEBUG4 )
     fprintf(stderr, "[ KheTimeGroupFinalize(%p %s)\n", (void *) tg,
       tg->id != NULL ? tg->id : "-");
-  a = (soln != NULL ? KheSolnArena(soln) : KheInstanceArena(tg->instance));
+  /* a = (soln!=NULL ? KheSolnArena(soln) : KheInstanceArena(tg->instance)); */
   tg->id = KheTimeGroupFinalId(tg, a);
   tg->name = KheTimeGroupFinalName(tg, a);
   if( !SSetIsFinalized(tg->times) )
     SSetFinalize(tg->times);
   if( with_nhood )
-    tg->neighbourhood = KheTimeGroupNhoodMake(tg, soln, &tg->pos_in_nhood);
+    tg->neighbourhood = KheTimeGroupNhoodMake(tg, a, /*soln,*/
+      &tg->pos_in_nhood);
   if( DEBUG4 )
   {
     KheTimeGroupDebug(tg, 3, 0, stderr);
@@ -359,7 +402,7 @@ void KheTimeGroupFinalize(KHE_TIME_GROUP tg, KHE_SOLN soln, bool with_nhood)
 
 KHE_TIME_GROUP KheTimeGroupMakeAndFinalize(KHE_INSTANCE ins,
   KHE_TIME_GROUP_KIND kind, char *id, char *name, SSET *times,
-  KHE_SOLN soln, bool with_neighbourhood)
+  /* KHE_SOLN soln, */ bool with_neighbourhood, HA_ARENA a)
 {
   KHE_TIME_GROUP res;
   if( KheInstanceUserTimeGroupTableRetrieve(ins, times, &res) )
@@ -369,9 +412,9 @@ KHE_TIME_GROUP KheTimeGroupMakeAndFinalize(KHE_INSTANCE ins,
   }
   else
   {
-    res = KheTimeGroupMakeInternal(ins, soln, kind, id, name);
+    res = KheTimeGroupMakeInternal(ins, /* soln, */ kind, id, name, a);
     res->times = *times;
-    KheTimeGroupFinalize(res, soln, with_neighbourhood);
+    KheTimeGroupFinalize(res, a, /* soln, */ with_neighbourhood);
   }
   return res;
 }
@@ -436,7 +479,8 @@ bool KheTimeGroupMake(KHE_INSTANCE ins, KHE_TIME_GROUP_KIND kind,
     *tg = NULL;
     return false;
   }
-  *tg = KheTimeGroupMakeInternal(ins, NULL, kind, id, name);
+  *tg = KheTimeGroupMakeInternal(ins, /* NULL, */ kind, id, name,
+    KheInstanceArena(ins));
   KheInstanceAddTimeGroup(ins, *tg);
   return true;
 }
