@@ -46,6 +46,8 @@ typedef HA_ARRAY(KHE_MONITOR_INFO) ARRAY_KHE_MONITOR_INFO;
 struct khe_trace_rec {
   KHE_GROUP_MONITOR		group_monitor;		/* tracing this      */
   KHE_COST			gm_init_cost;		/* init cost of gm   */
+  int				init_matching_defect_count;
+  /* int			soln_defect_count; */	/* soln defect count */
   bool				active;			/* tracing now       */
   ARRAY_KHE_MONITOR_INFO	monitor_info;		/* monitor info      */
   /* ARRAY_KHE_MONITOR		monitors; */		/* changed ones      */
@@ -93,6 +95,8 @@ KHE_TRACE KheTraceMake(KHE_GROUP_MONITOR gm)
   }
   res->group_monitor = gm;
   res->gm_init_cost = 0;
+  res->init_matching_defect_count = 0;
+  /* res->soln_defect_count = 0; */
   res->active = false;
   return res;
 }
@@ -141,9 +145,13 @@ void KheTraceDelete(KHE_TRACE t)
 
 void KheTraceBegin(KHE_TRACE t)
 {
+  KHE_MONITOR m;
   HnAssert(!t->active,
     "KheTraceBegin called twice with no intervening KheTraceEnd");
-  t->gm_init_cost = KheMonitorCost((KHE_MONITOR) t->group_monitor);
+  m = (KHE_MONITOR) t->group_monitor;
+  t->gm_init_cost = KheMonitorCost(m);
+  t->init_matching_defect_count = KheSolnMatchingDefectCount(KheMonitorSoln(m));
+  /* t->soln_defect_count = KheSolnDefectCount(KheMonitorSoln(m)); */
   t->active = true;
   HaArrayClear(t->monitor_info);
   /* ***
@@ -374,14 +382,27 @@ void KheTraceReduceByCostIncrease(KHE_TRACE t, int max_num)
 
 void KheTraceDebug(KHE_TRACE t, int verbosity, int indent, FILE *fp)
 {
-  KHE_MONITOR_INFO mi;  int i;
-  fprintf(fp, "%*s[ trace (%s)\n", indent, "",
-    t->active ? "active" : "inactive");
-  HaArrayForEach(t->monitor_info, mi, i)
+  KHE_MONITOR_INFO mi;  int i;  KHE_MONITOR m;  KHE_SOLN soln;
+  m = (KHE_MONITOR) t->group_monitor;
+  soln = KheMonitorSoln(m);
+  fprintf(fp, "%*s[ trace (%s, cost %.5f -> %.5f, matching %d -> %d)\n",
+    indent, "", t->active ? "active" : "inactive",
+    KheCostShow(t->gm_init_cost), KheCostShow(KheMonitorCost(m)),
+    t->init_matching_defect_count, KheSolnMatchingDefectCount(soln));
+  fprintf(fp, "%*s  [ %d matching defects according to trace:\n", indent, "",
+    KheSolnMatchingDefectCount(soln));
+  for( i = 0;  i < KheSolnMatchingDefectCount(soln);  i++ )
   {
-    fprintf(fp, "%*s  %.5f -> %.5f ", indent, "", KheCostShow(mi.init_cost),
-      KheCostShow(KheMonitorCost(mi.monitor)));
-    KheMonitorDebug(mi.monitor, verbosity, 0, fp);
+    m = KheSolnMatchingDefect(soln, i);
+    KheMonitorDebug(m, 2, 4, stderr);
   }
+  fprintf(fp, "%*s  ]\n", indent, "");
+  HaArrayForEach(t->monitor_info, mi, i)
+    if( verbosity >= 4 || mi.init_cost != KheMonitorCost(mi.monitor) )
+    {
+      fprintf(fp, "%*s  %.5f -> %.5f ", indent, "", KheCostShow(mi.init_cost),
+	KheCostShow(KheMonitorCost(mi.monitor)));
+      KheMonitorDebug(mi.monitor, verbosity, 0, fp);
+    }
   fprintf(fp, "%*s]\n", indent, "");
 }
