@@ -53,7 +53,45 @@ struct khe_workload_demand_monitor_rec {
 
 /*****************************************************************************/
 /*                                                                           */
-/*  KHE_WORKLOAD_DEMAND_MONITOR KheWorkloadDemandMonitorMake(                */
+/*  KHE_WORKLOAD_DEMAND_MONITOR KheWorkloadDemandMonitorMake(KHE_SOLN soln,  */
+/*    KHE_RESOURCE r, KHE_TIME_GROUP tg, KHE_MONITOR orig_m)                 */
+/*                                                                           */
+/*  Make a new workload demand monitor with these attributes.                */
+/*  Because it is unattached initially, its domain does not matter.          */
+/*                                                                           */
+/*****************************************************************************/
+
+KHE_WORKLOAD_DEMAND_MONITOR KheWorkloadDemandMonitorMake(KHE_SOLN soln,
+  KHE_RESOURCE r, KHE_TIME_GROUP tg, KHE_MONITOR orig_m)
+{
+  KHE_WORKLOAD_DEMAND_MONITOR res;  HA_ARENA a;  KHE_RESOURCE_IN_SOLN rs;
+  rs = KheSolnResourceInSoln(soln, r);
+  a = KheSolnArena(soln);
+  HaMake(res, a);
+  HaArrayInit(res->parent_links, a);
+  KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
+    KHE_WORKLOAD_DEMAND_MONITOR_TAG, KHE_LINEAR_COST_FUNCTION, KheCost(1, 0));
+  res->demand_chunk = KheResourceInSolnDemandChunk(rs, tg);
+  res->domain = *KheResourceGroupKheSet(
+    KheResourceSingletonResourceGroup(KheResourceInSolnResource(rs)));
+  res->demand_asst = NULL;
+  res->demand_asst_index = NO_PREV_ASST;
+  res->unmatched_pos = -1;  /* initially not in unmatched list */
+  res->bfs_next = NULL;
+  res->bfs_parent = NULL;
+  res->hall_set = NULL;
+  res->resource_in_soln = rs;
+  res->time_group = tg;
+  res->originating_monitor = orig_m;
+  res->copy = NULL;
+  KheResourceInSolnAddMonitor(rs, (KHE_MONITOR) res);
+  return res;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_WORKLOAD_DEMAND_MONITOR KheWorklo dDemandMonitorMake(                */
 /*    KHE_MATCHING_DEMAND_CHUNK dc, KHE_RESOURCE_IN_SOLN rs,                 */
 /*    KHE_TIME_GROUP tg, KHE_MONITOR originating_monitor)                    */
 /*                                                                           */
@@ -62,7 +100,8 @@ struct khe_workload_demand_monitor_rec {
 /*                                                                           */
 /*****************************************************************************/
 
-KHE_WORKLOAD_DEMAND_MONITOR KheWorkloadDemandMonitorMake(KHE_SOLN soln,
+/* *** old version
+KHE_WORKLOAD_DEMAND_MONITOR KheWorkloa DemandMonitorMake(KHE_SOLN soln,
   KHE_MATCHING_DEMAND_CHUNK dc, KHE_RESOURCE_IN_SOLN rs,
   KHE_TIME_GROUP tg, KHE_MONITOR originating_monitor)
 {
@@ -72,13 +111,13 @@ KHE_WORKLOAD_DEMAND_MONITOR KheWorkloadDemandMonitorMake(KHE_SOLN soln,
   HaArrayInit(res->parent_links, a);
   KheMonitorInitCommonFields((KHE_MONITOR) res, soln,
     KHE_WORKLOAD_DEMAND_MONITOR_TAG);
-  HnAssert(dc != NULL, "KheWorkloadDemandMonitorMake internal error");
+  HnAssert(dc != NULL, "KheWorkloadDem ndMonitorMake internal error");
   res->demand_chunk = dc;
   res->domain = *KheResourceGroupKheSet(
     KheResourceSingletonResourceGroup(KheResourceInSolnResource(rs)));
   res->demand_asst = NULL;
   res->demand_asst_index = NO_PREV_ASST;
-  res->unmatched_pos = -1;  /* undefined at this point */
+  res->unmat ched_pos = -1;  ** undefined at this point **
   res->bfs_next = NULL;
   res->bfs_parent = NULL;
   res->hall_set = NULL;
@@ -86,10 +125,11 @@ KHE_WORKLOAD_DEMAND_MONITOR KheWorkloadDemandMonitorMake(KHE_SOLN soln,
   res->time_group = tg;
   res->originating_monitor = originating_monitor;
   res->copy = NULL;
-  /* KheGroupMonitorAddMonitor((KHE_GROUP_MONITOR) soln, (KHE_MONITOR) res); */
+  ** KheGroupMonitorAddMonitor((KHE_GROUP_MONITOR) soln, (KHE_MONITOR) res); **
   KheResourceInSolnAddMonitor(rs, (KHE_MONITOR) res);
   return res;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -158,7 +198,7 @@ KHE_WORKLOAD_DEMAND_MONITOR KheWorkloadDemandMonitorCopyPhase1(
     m->copy = copy;
     KheMonitorCopyCommonFieldsPhase1((KHE_MONITOR) copy, (KHE_MONITOR) m, a);
     copy->demand_chunk = KheMatchingDemandChunkCopyPhase1(m->demand_chunk, a);
-    copy->domain = m->domain;
+    KheSetCopy(copy->domain, m->domain, a);
     copy->demand_asst = (m->demand_asst == NULL ? NULL :
       KheMatchingSupplyNodeCopyPhase1(m->demand_asst, a));
     copy->demand_asst_index = m->demand_asst_index;
@@ -170,7 +210,7 @@ KHE_WORKLOAD_DEMAND_MONITOR KheWorkloadDemandMonitorCopyPhase1(
     copy->hall_set = (m->hall_set == NULL ? NULL :
       KheMatchingHallSetCopyPhase1(m->hall_set, a));
     copy->resource_in_soln = KheResourceInSolnCopyPhase1(m->resource_in_soln,a);
-    copy->time_group = m->time_group;
+    copy->time_group = KheTimeGroupCopy(m->time_group, a);
     copy->originating_monitor = (m->originating_monitor == NULL ? NULL :
       KheMonitorCopyPhase1(m->originating_monitor, a));
     copy->copy = NULL;
@@ -317,7 +357,8 @@ void KheWorkloadDemandMonitorSetWeight(KHE_WORKLOAD_DEMAND_MONITOR m,
 
 int KheWorkloadDemandMonitorDeviation(KHE_WORKLOAD_DEMAND_MONITOR m)
 {
-  return m->demand_asst != NULL ? 1 : 0;
+  return m->demand_asst == NULL ? 1 : 0;
+  /* return m->demand_asst != NULL ? 1 : 0; swapped around 18/12/23 JeffK */
 }
 
 
@@ -333,7 +374,8 @@ int KheWorkloadDemandMonitorDeviation(KHE_WORKLOAD_DEMAND_MONITOR m)
 char *KheWorkloadDemandMonitorDeviationDescription(
   KHE_WORKLOAD_DEMAND_MONITOR m)
 {
-  return m->demand_asst != NULL ? "1" : "0";
+  return m->demand_asst == NULL ? "1" : "0";
+  /* return m->demand_asst != NULL ? "1" : "0"; swapped around 18/12/23 JeffK */
   /* ***
   ARRAY_CHAR ac;
   MStringInit(ac);
