@@ -1095,14 +1095,20 @@ bool KmlCheck(KML_ELT elt, char *fmt, KML_ERROR *ke)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  KML_FILE KmlMakeFile(FILE *fp, int initial_indent, int indent_step)      */
+/*  KML_FILE KmlMakeFile(FILE *fp, int initial_indent, int indent_step,      */
+/*    HA_ARENA a)                                                            */
+/*                                                                           */
+/*  Make a KML file object with these attributes.  The arena parameter       */
+/*  was added by JeffK in 11/25, following a suggestion by Ortiz.            */
 /*                                                                           */
 /*****************************************************************************/
 
-KML_FILE KmlMakeFile(FILE *fp, int initial_indent, int indent_step)
+KML_FILE KmlMakeFile(FILE *fp, int initial_indent, int indent_step,
+  HA_ARENA a)
 {
   KML_FILE res;
-  res = (KML_FILE) malloc(sizeof(struct kml_file_rec));
+  HaMake(res, a);
+  /* res = (KML_FILE) malloc(sizeof(struct kml_file_rec)); */
   res->fp = fp;
   res->curr_indent = initial_indent;
   res->indent_step = indent_step;
