@@ -298,7 +298,7 @@ static KML_SEGMENT KmlSegmentMake(KML_READER ri,
   HA_ARENA a;  KML_SEGMENT res;
   HnAssert(KmlSegmentDeclGivesRiseToSegments(decl),
     "KmlSegmentMake internal error 1");
-  a = (parent == NULL ? ri->arena : HaArenaSetArenaBegin(ri->arena_set, false));
+  a = (parent == NULL ? ri->arena : HaArenaMake(ri->arena_set));
   HaMake(res, a);
   res->arena = a;
   res->reader = ri;
@@ -395,7 +395,7 @@ HA_ARENA KmlSegmentArena(KML_SEGMENT ks)
 /* *** withdrawn
 void KmlSegmentFree(KML_SEGMENT ks)
 {
-  HaArenaSetArenaEnd(ks->reader->arena_set, ks->arena);
+  HaArenaDelete(ks->arena);
 }
 *** */
 
@@ -694,7 +694,7 @@ static void KmlReaderEndSegment(KML_READER ri)
 
     /* free seg if it is not the root */
     if( seg->parent != NULL )
-      HaArenaSetArenaEnd(ri->arena_set, seg->arena);
+      HaArenaDelete(seg->arena);
 
     /* the current seg is now seg's parent */
     ri->curr_seg = encl_seg;
