@@ -200,7 +200,8 @@ void SSetImplCopy(SSET *to_sp, SSET *from_sp, HA_ARENA a)
   HaArrayAppend(to_sp->ss_array, from_sp->ss_array, i);
   to_sp->ss_shift = from_sp->ss_shift;
   /* to_sp->ss_slice = false; */
-  to_sp->ss_finalized = false;
+  to_sp->ss_finalized = true;
+  /* to_sp->ss_finalized = false; bug fix JeffK 14/11/25 */
   SSetImplCheck(from_sp, "SSetImplCopy from_sp final value");
   SSetImplCheck(to_sp, "SSetImplCopy to_sp final value");
 }
@@ -710,9 +711,48 @@ bool SSetImplContains(SSET *sp, int item, int *pos)
 /*                                                                           */
 /*  char *SSetImplShow(SSET *sp)                                             */
 /*                                                                           */
-/*  Return a string representation of ss in static memory.                   */
+/*  Return a string representation of ss in static memory.  This is stupid   */
+/*  code, but it passes the C compiler's static array bound checks.          */
+/*                                                                           */
+/*  Loop invariant:  at the start of each iteration of the loop, tmp_buff    */
+/*  is defined and contains everything we've done so far; buff is undefined. */
 /*                                                                           */
 /*****************************************************************************/
+
+char *SSetImplShow(SSET *sp)
+{
+  static char buff[200], tmp_buff[170], tmp[20];  int i, j, k;
+  snprintf(tmp_buff, 170, "{");
+  for( i = 0;  i < HaArrayCount(sp->ss_array);  i = j )
+  {
+    /* find range of groupable elements sp->ss_items[i .. j-1] */
+    for( j = i + 1;  j < HaArrayCount(sp->ss_array);  j++ )
+      if( HaArray(sp->ss_array, j) != HaArray(sp->ss_array, j-1) + 1 )
+	break;
+
+    /* print the range, optionally preceded by a comma and space */
+    if( j - 1 == i )
+      snprintf(tmp, 20, "%s%u", i > 0 ? ", " : "",
+	HaArray(sp->ss_array, i) + sp->ss_shift);
+    else
+      snprintf(tmp, 20, "%s%u-%u", i > 0 ? ", " : "",
+	HaArray(sp->ss_array, i) + sp->ss_shift,
+	HaArray(sp->ss_array, j-1) + sp->ss_shift);
+    snprintf(buff, 200, "%s%s", tmp_buff, tmp);
+    for( k = 0;  k < 170;  k++ )
+    {
+      tmp_buff[k] = buff[k];
+      if( tmp_buff[k] == '\0' )
+	break;
+    }
+  }
+
+  /* print the closing brace and return */
+  snprintf(buff, 200, "%s%s", tmp_buff, sp->ss_finalized ? "}*" : "}");
+  return buff;
+}
+
+/* *** this code is perfectly safe, but the C compiler can't see that
 #define do_add(str) (strcpy(&buff[bpos], str), bpos += strlen(str))
 
 char *SSetImplShow(SSET *sp)
@@ -725,19 +765,19 @@ char *SSetImplShow(SSET *sp)
   do_add("{");
   for( i = 0;  i < HaArrayCount(sp->ss_array);  i = j )
   {
-    /* if running out of space, abbreviate and exit */
+    ** if running out of space, abbreviate and exit **
     if( bpos > 180 )
     {
       do_add(", ...");
       break;
     }
 
-    /* find range of groupable elements sp->ss_items[i .. j-1] */
+    ** find range of groupable elements sp->ss_items[i .. j-1] **
     for( j = i + 1;  j < HaArrayCount(sp->ss_array);  j++ )
       if( HaArray(sp->ss_array, j) != HaArray(sp->ss_array, j-1) + 1 )
 	break;
 
-    /* print the range, optionally preceded by a command and space */
+    ** print the range, optionally preceded by a command and space **
     if( i > 0 )
       do_add(", ");
     if( j - 1 == i )
@@ -750,6 +790,7 @@ char *SSetImplShow(SSET *sp)
   do_add(sp->ss_finalized ? "}*" : "}");
   return buff;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -785,11 +826,12 @@ static void SSetBuild(SSET *sp, HA_ARENA a, int count, ...)
 
 void SSetTest(FILE *fp)
 {
-  SSET ss[5];  int i, j, pos;  HA_ARENA a;
+  SSET ss[5];  int i, j, pos;  HA_ARENA a;  HA_ARENA_SET as;
   fprintf(fp, "[ SSetTest\n");
 
   /* initialize ss[0] */
-  a = HaArenaMake(false);  /* this arena is used only for testing */
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);  /* this arena is used only for testing */
   SSetBuild(&ss[0], a, 5, 2, 4, 6, 3, 1);
   fprintf(fp, "  ss[0] = %s\n", SSetShow(ss[0]));
 
@@ -1084,10 +1126,11 @@ static void SSetDebugRetrieve(SSET_TABLE st, SSET *sp, FILE *fp)
 
 void SSetTableTest(FILE *fp)
 {
-  SSET_TABLE st;  SSET ss[5];  int i;  HA_ARENA a;
+  SSET_TABLE st;  SSET ss[5];  int i;  HA_ARENA a;  HA_ARENA_SET as;
 
   /* initialize */
-  a = HaArenaMake(false);  /* this arena is used only for testing */
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);  /* this arena is used only for testing */
   st = SSetTableMake(a);
   fprintf(fp, "\nafter initializing, table is:\n");
   SSetTableDebug(st, 0, fp);
