@@ -1517,13 +1517,11 @@ static bool FindMinCostAugmentingPath(KHE_MMATCH m)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void FindMinCostMMatch(KHE_MMATCH m, int indent)                         */
+/*  bool KheMMatchSolve(KHE_MMATCH m)                                        */
 /*                                                                           */
-/*  Run a min-cost max-flow algorithm.  Parameter indent is for debugging.   */
-/*  The algorithm is the standard one of starting from an empty flow and     */
-/*  repeatedly augmenting along a path of minimum cost.  Edge cost           */
-/*  adjustment is used to ensure that edges have non-negative cost,          */
-/*  permitting Dijkstra's algorithm to be used.                              */
+/*  Run a min-cost max-flow algorithm.  Return true if every node was        */
+/*  matched; but a false value is not a problem, it just means that some     */
+/*  calls to KheMMatchResultEdge will return false with *sn == NULL.         */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -1999,8 +1997,9 @@ static bool edge_fn(void *demand, void *supply, int64_t *cost)
 
 static void KheMMatchTest1(FILE *fp)
 {
-  HA_ARENA a;  KHE_MMATCH m;
-  a = HaArenaMake(false);  /* this arena is used for testing only */
+  HA_ARENA a;  KHE_MMATCH m;  HA_ARENA_SET as;
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);  /* this arena is used for testing only */
   m = KheMMatchMake(NULL, NULL, &node_debug, &node_debug, NULL, a);
   /* TEST_NODE ta, tb, tc, td */;
   MakeTestDemandNode(m, "A1234");
