@@ -1053,12 +1053,12 @@ void KhePartitionsUpTo(int n, int m, ARRAY_KHE_PARTITION *res, HA_ARENA a)
 /*                                                                           */
 /*****************************************************************************/
 
-bool KhePartitionBinPack(KHE_PARTITION p1, KHE_PARTITION p2)
+bool KhePartitionDoBinPack(KHE_PARTITION p1, KHE_PARTITION p2)
 {
   bool res;  int m, i;
   if( DEBUG4 )
   {
-    fprintf(stderr, "[ KhePartitionBinPack(");
+    fprintf(stderr, "[ KhePartitionDoBinPack(");
     KhePartitionDebug(p1, stderr);
     fprintf(stderr, ", ");
     KhePartitionDebug(p2, stderr);
@@ -1071,12 +1071,14 @@ bool KhePartitionBinPack(KHE_PARTITION p1, KHE_PARTITION p2)
     res = false;
   else if( KhePartitionMax(p1) > KhePartitionMax(p2) )
     res = false;
+  else if( KhePartitionMax(p1) == 1 )
+    res = true;
   else if( KhePartitionContains(p2, KhePartitionMax(p1)) )
   {
     m = KhePartitionMax(p1);
     KhePartitionSub(p1, m);
     KhePartitionSub(p2, m);
-    res = KhePartitionBinPack(p1, p2);
+    res = KhePartitionDoBinPack(p1, p2);
     KhePartitionAdd(p2, m);
     KhePartitionAdd(p1, m);
   }
@@ -1091,13 +1093,42 @@ bool KhePartitionBinPack(KHE_PARTITION p1, KHE_PARTITION p2)
       {
 	KhePartitionSub(p2, i);
 	KhePartitionAdd(p2, i - m);
-	res = KhePartitionBinPack(p1, p2);
+	res = KhePartitionDoBinPack(p1, p2);
 	KhePartitionSub(p2, i - m);
 	KhePartitionAdd(p2, i);
       }
     }
     KhePartitionAdd(p1, m);
   }
+  if( DEBUG4 )
+    fprintf(stderr, "] KhePartitionDoBinPack returning %s\n",
+      res ? "true" : "false");
+  return res;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KhePartitionBinPack(KHE_PARTITION p1, KHE_PARTITION p2)             */
+/*                                                                           */
+/*  Return true if p1 is packable into p2.                                   */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KhePartitionBinPack(KHE_PARTITION p1, KHE_PARTITION p2)
+{
+  bool res;
+  if( DEBUG4 )
+  {
+    fprintf(stderr, "[ KhePartitionBinPack(");
+    KhePartitionDebug(p1, stderr);
+    fprintf(stderr, ", ");
+    KhePartitionDebug(p2, stderr);
+    fprintf(stderr, ")\n");
+  }
+
+  res = KhePartitionSize(p1) <= KhePartitionSize(p2) &&
+    KhePartitionDoBinPack(p1, p2);
   if( DEBUG4 )
     fprintf(stderr, "] KhePartitionBinPack returning %s\n",
       res ? "true" : "false");
@@ -1364,7 +1395,9 @@ void KhePartitionTest2(FILE *fp)
 
 void KhePartitionTest(FILE *fp)
 {
-  HA_ARENA a = HaArenaMake(false);  /* this arena is used for testing only */
+  HA_ARENA_SET as;  HA_ARENA a;
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);  /* this arena is used for testing only */
   KHE_PARTITION items1 = KhePartitionMake4(0, 3, 0, 0, a);
   KHE_PARTITION items2 = KhePartitionMake4(25, 6, 1, 0, a);
   KHE_PARTITION items3 = KhePartitionMake4(50, 12, 2, 0, a);
