@@ -265,6 +265,21 @@ void KhePriQueueDeleteEntry(KHE_PRIQUEUE p, void *entry)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  void KhePriQueueClear(KHE_PRIQUEUE p)                                    */
+/*                                                                           */
+/*  Clear p.                                                                 */
+/*                                                                           */
+/*****************************************************************************/
+
+void KhePriQueueClear(KHE_PRIQUEUE p)
+{
+  while( HaArrayCount(p->entries) > 0 )
+    KhePriQueueDeleteEntry(p, HaArrayLast(p->entries));
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KhePriQueueNotifyKeyChange(KHE_PRIQUEUE p, void *entry)             */
@@ -377,9 +392,10 @@ static KHE_PRIQUEUE_TEST_ENTRY KhePriQueueTestEntryMake(int64_t key,
 
 void KhePriQueueTest(FILE *fp)
 {
-  KHE_PRIQUEUE p;  KHE_PRIQUEUE_TEST_ENTRY pte;  HA_ARENA a;
+  KHE_PRIQUEUE p;  KHE_PRIQUEUE_TEST_ENTRY pte;  HA_ARENA a;  HA_ARENA_SET as;
   fprintf(fp, "[ KhePriQueueTest()\n");
-  a = HaArenaMake(false);  /* this arena is used only for testing */
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);  /* this arena is used only for testing */
   p = KhePriQueueMake((KHE_PRIQUEUE_KEY_FN) &KhePriQueueTestKey,
     (KHE_PRIQUEUE_INDEX_GET_FN) &KhePriQueueTestIndex,
     (KHE_PRIQUEUE_INDEX_SET_FN) &KhePriQueueTestIndexSet, a);
