@@ -40,7 +40,7 @@
 /*  void KheSplitAddConstraintJobs(KHE_SOLN soln,                            */
 /*    ARRAY_KHE_SPLIT_JOB *split_jobs)                                       */
 /*                                                                           */
-/*  Add to *split_jobs those jobs of ins that derived from constraints.      */
+/*  Add to *split_jobs those jobs of ins that are derived from constraints.  */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -238,7 +238,7 @@ static void KheSplitAddAssignedSplitJobs(KHE_SOLN soln,
 /*  void KheSplitEventHomogenizeAssignments(KHE_SOLN soln, KHE_EVENT e)      */
 /*                                                                           */
 /*  Homogenize the assignments of e, so that they are all either to          */
-/*  meets of the same event, or to cycle meets.                              */
+/*  meets of the same event, or to cycle meets.  Here e is not preassigned.  */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -262,7 +262,7 @@ static void KheSplitEventHomogenizeAssignments(KHE_SOLN soln, KHE_EVENT e)
 	    fprintf(stderr, "  unassigning meet of %s (not cycle)\n",
 	      KheEventId(e) != NULL ? KheEventId(e) : "-");
 	  KheMeetAssignUnFix(meet);
-	  KheMeetUnAssign(meet);
+	  KheMeetUnAssign(meet);  /* meet known to be not preassigned */
 	}
       }
       else if( KheMeetEvent(first_asst) != NULL )
@@ -275,7 +275,7 @@ static void KheSplitEventHomogenizeAssignments(KHE_SOLN soln, KHE_EVENT e)
 	    fprintf(stderr, "  unassigning meet of %s (not event)\n",
 	      KheEventId(e) != NULL ? KheEventId(e) : "-");
 	  KheMeetAssignUnFix(meet);
-	  KheMeetUnAssign(meet);
+	  KheMeetUnAssign(meet);  /* meet known to be not preassigned */
 	}
       }
     }
@@ -509,7 +509,7 @@ static bool KheLayerTreePreferResourcesMonitorMustHaveZeroCost(KHE_SOLN soln,
 {
   int i;  KHE_JOB job;  KHE_RESOURCE_GROUP rg;
   rg =
-    KhePreferResourcesConstraintDomain(KhePreferResourcesMonitorConstraint(m));
+    KhePreferResource sConstraintDomain(KhePreferResourcesMonitorConstraint(m));
   for( i = 0;  i < KheEventResourceJobCount(soln, er);  i++ )
   {
     job = KheEventResourceJob(soln, er, i);
@@ -803,12 +803,12 @@ KHE_NODE KheLayerTreeMake(KHE_SOLN soln)
   KHE_INSTANCE ins;  KHE_SPLIT_FOREST sf;  KHE_SPLIT_JOB sj, sj2;
   int i, j, max_duration;  KHE_MEET meet, junk;  /* KHE_LAYER layer; */
   ARRAY_KHE_SPLIT_JOB split_jobs;  KHE_NODE res;  KHE_CONSTRAINT c;
-  ARRAY_KHE_RESOURCE ra1, ra2;  HA_ARENA a;
+  ARRAY_KHE_RESOURCE ra1, ra2;  HA_ARENA a;  KHE_EVENT e;
   KHE_TIMER st = NULL;
   /* KHE_STATS_TIMER st = NULL; */
   /* struct timeval tv; */
 
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaArena Make(); */
 
   ins = KheSolnInstance(soln);
@@ -826,7 +826,11 @@ KHE_NODE KheLayerTreeMake(KHE_SOLN soln)
 
   /* ensure that all events are assigned to at most one distinct thing */
   for( i = 0;  i < KheInstanceEventCount(ins);  i++ )
-    KheSplitEventHomogenizeAssignments(soln, KheInstanceEvent(ins, i));
+  {
+    e = KheInstanceEvent(ins, i);
+    if( KheEventPreassignedTime(e) == NULL )
+      KheSplitEventHomogenizeAssignments(soln, e);
+  }
 
   /* ensure that all meets have durations within the cycle size */
   max_duration = KheInstanceTimeCount(ins);
