@@ -37,6 +37,8 @@
 #define DEBUG6 0
 #define DEBUG7 0
 #define DEBUG8 0
+#define DEBUG9 0
+#define DEBUG10 0
 
 
 /*****************************************************************************/
@@ -758,15 +760,25 @@ void KheSplitClassTryAddToResource(KHE_SPLIT_CLASS sc, KHE_RESOURCE r,
 {
   KHE_SPLIT_LAYER split_layer;  int i;
 
+  if( DEBUG9 )
+    fprintf(stderr, "[ KheSplitClassTryAddToResource(sc, %s, a)\n",
+      KheResourceId(r));
+
   /* fail now if there is an existing child layer of sc for r */
   if( KheSplitClassContainsResource(sc, r, &i) )
+  {
+    if( DEBUG9 )
+      fprintf(stderr, "] KheSplitClassTryAddToResource returning early\n");
     return;
+  }
 
   /* add a new split layer and recurse */
   split_layer = KheSplitLayerMake(r, KhePartitionUnitary(sc->duration, a), a);
   HaArrayAddLast(sc->child_layers, split_layer);
   if( sc->parent_class != NULL )
     KheSplitClassAddChildToResource(sc->parent_class, sc, r, a);
+  if( DEBUG9 )
+    fprintf(stderr, "] KheSplitClassTryAddToResource returning\n");
 }
 
 
@@ -1171,18 +1183,33 @@ static void KheSplitClassGenPartitions(KHE_SPLIT_CLASS sc,
 {
   int min_of_size, max_of_size, parts, i;  KHE_PARTITION p;
   KHE_SPLIT_LAYER sl;  KHE_SPLIT_CLASS child_sc;
+  if( DEBUG10 )
+    fprintf(stderr, "[ KheSplitClassGenPartitions(sc, %d, %d, %d, %d, a)\n",
+      stem_part_size, stem_durn, stem_parts, max_results);
 
   /* quit now if enough results */
   if( HaArrayCount(sc->tmp_partitions) >= max_results )
+  {
+    if( DEBUG10 )
+      fprintf(stderr, "] KheSplitClassGenPartition returning (max_results)\n");
     return;
+  }
 
   /* quit now if sum of all parts is too great */
   if( stem_durn > sc->duration )
+  {
+    if( DEBUG10 )
+      fprintf(stderr, "] KheSplitClassGenPartition returning (stem_durn)\n");
     return;
+  }
 
   /* quit now if too many parts */
   if( stem_parts > sc->max_total_amount )
+  {
+    if( DEBUG10 )
+      fprintf(stderr, "] KheSplitClassGenPartition returning (stem_parts)\n");
     return;
+  }
 
   /* quit now if all parts are done, possibly saving an acceptable stem */
   if( stem_part_size == 0 )
@@ -1192,28 +1219,47 @@ static void KheSplitClassGenPartitions(KHE_SPLIT_CLASS sc,
       /* make sure that tmp_stem is packable into each upper partition */
       HaArrayForEach(sc->upper_partitions, p, i)
 	if( !KheDoBinPack(sc->tmp_stem, p, a) )
+	{
+	  if( DEBUG10 )
+	    fprintf(stderr, "] KheSplitClassGenPartition returning "
+	      "(KheDoBinPack 1)\n");
 	  return;
+	}
 
       /* make sure that each child layer is packable into tmp_stem */
       HaArrayForEach(sc->child_layers, sl, i)
 	if( !KheDoBinPack(KheSplitLayerMinPartition(sl), sc->tmp_stem, a) )
+	{
+	  if( DEBUG10 )
+	    fprintf(stderr, "] KheSplitClassGenPartition returning "
+	      "(KheDoBinPack 2)\n");
 	  return;
+	}
 
       /* make sure that each child class is packable into tmp_stem */
       HaArrayForEach(sc->child_classes, child_sc, i)
       {
-	if( DEBUG4 )
+	if( DEBUG4 || DEBUG10 )
 	  fprintf(stderr, " KheSplitClassGenPartitions testing %s into %s\n",
 	    KhePartitionShow(child_sc->min_partition, a),
 	    KhePartitionShow(sc->tmp_stem, a));
 	if( /* HaArrayCount(child_sc->parent_layers) == 0 && */
 	    !KheDoBinPack(child_sc->min_partition, sc->tmp_stem, a) )
+	{
+	  if( DEBUG10 )
+	    fprintf(stderr, "] KheSplitClassGenPartition returning "
+	      "(KheDoBinPack 3)\n");
 	  return;
+	}
       }
 
       /* save tmp_stem; it is an acceptable partition */
       HaArrayAddLast(sc->tmp_partitions, KhePartitionCopy(sc->tmp_stem, a));
     }
+
+    if( DEBUG10 )
+      fprintf(stderr, "] KheSplitClassGenPartition returning "
+	"(stem_part_size == 0)\n");
     return;
   }
 
@@ -1238,6 +1284,8 @@ static void KheSplitClassGenPartitions(KHE_SPLIT_CLASS sc,
       stem_parts -= 1;
     }
   }
+  if( DEBUG10 )
+    fprintf(stderr, "] KheSplitClassGenPartition returning (at end)\n");
 }
 
 
@@ -1254,12 +1302,14 @@ bool KheSplitClassTryAlive(KHE_SPLIT_CLASS sc, HA_ARENA a)
 {
   int min_by_duration, max_by_duration, min_amt, max_amt, i, j;
   int stem_durn, stem_parts;  KHE_PARTITION p;  KHE_SPLIT_LAYER sl;
+  if( DEBUG1 )
+    fprintf(stderr, "[ KheSplitClassTryAlive(sc, a)\n");
 
   /* no leader test */
   if( HaArrayCount(sc->events) > 0 && sc->leader == NULL )
   {
     if( DEBUG1 )
-      fprintf(stderr, "    dead: no leader\n");
+      fprintf(stderr, "] KheSplitClassTryAlive returning dead: no leader\n");
     return false;
   }
 
@@ -1268,7 +1318,7 @@ bool KheSplitClassTryAlive(KHE_SPLIT_CLASS sc, HA_ARENA a)
       sc->min_total_amount > sc->duration )
   {
     if( DEBUG1 )
-      fprintf(stderr, "    dead: total amount\n");
+      fprintf(stderr, "] KheSplitClassTryAlive returning dead: total amount\n");
     return false;
   }
   min_by_duration = max_by_duration = 0;
@@ -1279,7 +1329,7 @@ bool KheSplitClassTryAlive(KHE_SPLIT_CLASS sc, HA_ARENA a)
     if( min_amt > max_amt )
     {
       if( DEBUG1 )
-	fprintf(stderr, "    dead: amount for durn %d\n", i + 1);
+	fprintf(stderr, "] KheSplitClassTryAlive returning dead: amount for durn %d\n", i + 1);
       return false;
     }
     min_by_duration += min_amt;
@@ -1290,7 +1340,7 @@ bool KheSplitClassTryAlive(KHE_SPLIT_CLASS sc, HA_ARENA a)
       max_by_duration < sc->min_total_amount )
   {
     if( DEBUG1 )
-      fprintf(stderr, "    dead: total amount (2)\n");
+      fprintf(stderr, "] KheSplitClassTryAlive ret. dead: total amount (2)\n");
     return false;
   }
 
@@ -1312,13 +1362,15 @@ bool KheSplitClassTryAlive(KHE_SPLIT_CLASS sc, HA_ARENA a)
   while( HaArrayCount(sc->tmp_partitions) > 0 )
     KhePartitionFree(HaArrayLastAndDelete(sc->tmp_partitions));
   *** */
+  if( DEBUG1 )
+    fprintf(stderr,"  KheSplitClassTryAlive call KheSplitClassGenPartitions\n");
   KheSplitClassGenPartitions(sc, sc->duration, stem_durn, stem_parts, 1, a);
 
   /* fail and return if no partitions */
   if( HaArrayCount(sc->tmp_partitions) == 0 )
   {
     if( DEBUG1 )
-      fprintf(stderr, "    dead: did not generate\n");
+      fprintf(stderr, "] KheSplitClassTryAlive . dead: did not generate\n");
     return false;
   }
 
@@ -1340,14 +1392,14 @@ bool KheSplitClassTryAlive(KHE_SPLIT_CLASS sc, HA_ARENA a)
   if( sc->parent_class == NULL || KheSplitClassTryAlive(sc->parent_class, a) )
   {
     if( DEBUG1 )
-      fprintf(stderr, "    alive: %s\n",
+      fprintf(stderr, "] KheSplitClassTryAlive returning alive: %s\n",
 	KhePartitionShow(sc->min_partition, a));
     return true;
   }
   else
   {
     if( DEBUG1 )
-      fprintf(stderr, "    dead: parent class\n");
+      fprintf(stderr, "] KheSplitClassTryAlive returning dead: parent class\n");
     return false;
   }
 }
@@ -1462,7 +1514,9 @@ static void KheSplitSplitEvent(KHE_SOLN soln, KHE_EVENT e,
     for( i = 0;  i < KheEventMeetCount(soln, e);  i++ )
     {
       meet = KheEventMeet(soln, e, i);
-      fprintf(stderr, "  meet durn %d\n", KheMeetDuration(meet));
+      KheMeetCheck(meet);
+      fprintf(stderr, "  meet %s durn %d\n",
+	KheMeetId(meet), KheMeetDuration(meet));
     }
   }
   initial_count = KheEventMeetCount(soln, e);
@@ -1475,9 +1529,12 @@ static void KheSplitSplitEvent(KHE_SOLN soln, KHE_EVENT e,
     HaArrayForEach(*partitions, p, j)
       if( KhePartitionSize(p) == KheMeetDuration(meet) )
 	break;
-    HnAssert(j < HaArrayCount(*partitions), "KheSplitSplitEvent internal error");
+    HnAssert(j < HaArrayCount(*partitions),"KheSplitSplitEvent internal error");
 
     /* split meet according to the instructions in p */
+    if( DEBUG2 )
+      fprintf(stderr, "  splitting meet %s with durn %d by partition %s\n",
+	KheMeetId(meet), KheMeetDuration(meet), KhePartitionShow(p, a));
     while( KhePartitionParts(p) > 1 )
     {
       m = KhePartitionMax(p);
@@ -1495,7 +1552,8 @@ static void KheSplitSplitEvent(KHE_SOLN soln, KHE_EVENT e,
     for( i = 0;  i < KheEventMeetCount(soln, e);  i++ )
     {
       meet = KheEventMeet(soln, e, i);
-      fprintf(stderr, " %d", KheMeetDuration(meet));
+      KheMeetCheck(meet);
+      fprintf(stderr, " %s %d", KheMeetId(meet), KheMeetDuration(meet));
     }
     fprintf(stderr, "\n");
   }
@@ -1600,6 +1658,7 @@ KHE_NODE KheSplitClassFinalize(KHE_SPLIT_CLASS sc, KHE_NODE parent_node,
       meet = KheSolnMeet(soln, i);
       if( KheMeetIsCycleMeet(meet) )
       {
+	KheMeetCheck(meet);
 	if( !KheNodeAddMeet(leader_node, meet) )
 	  HnAbort("KheLayerTreeMake internal error 3");
 	sz += KheMeetDuration(meet);
@@ -1727,7 +1786,11 @@ KHE_NODE KheSplitClassFinalize(KHE_SPLIT_CLASS sc, KHE_NODE parent_node,
     KheSplitClassFinalize(child_sc, leader_node, a);
 
   if( DEBUG7 )
+  {
+    fprintf(stderr, "  KheSplitClassFinalize final node:\n");
+    KheNodeDebug(leader_node, 4, 2, stderr);
     fprintf(stderr, "] KheSplitClassFinalize returning\n");
+  }
   return leader_node;
 }
 
