@@ -32,6 +32,7 @@
 #define DEBUG1 0
 #define DEBUG2 0
 #define DEBUG3 0
+#define DEBUG4 0
 
 
 /*****************************************************************************/
@@ -210,7 +211,7 @@ bool KheSplitForestTryAddToLayer(KHE_SPLIT_FOREST sf,
 bool KheSplitForestTryAddToResource(KHE_SPLIT_FOREST sf, KHE_EVENT e,
   KHE_RESOURCE r, HA_ARENA a)
 {
-  KHE_SPLIT_CLASS sc;  bool res;
+  KHE_SPLIT_CLASS sc;  bool res, success;
   sc = KheSplitForestSplitClass(sf, e);
   if( DEBUG2 )
   {
@@ -220,8 +221,18 @@ bool KheSplitForestTryAddToResource(KHE_SPLIT_FOREST sf, KHE_EVENT e,
     KheSplitClassDebug(sc, 2, stderr, a);
   }
   KheSplitClassTryBegin(sc);
+  if( DEBUG2 )
+    fprintf(stderr, "  KheSplitForestTryAddToResource calling "
+      "KheSplitClassTryAddToResource:\n");
   KheSplitClassTryAddToResource(sc, r, a);
-  res = KheSplitClassTryEnd(sc, KheSplitClassTryAlive(sc, a));
+  if( DEBUG2 )
+    fprintf(stderr, "  KheSplitForestTryAddToResource calling "
+      "KheSplitClassTryAlive:\n");
+  success = KheSplitClassTryAlive(sc, a);
+  if( DEBUG2 )
+    fprintf(stderr, "  KheSplitForestTryAddToResource calling "
+      "KheSplitClassTryEnd:\n");
+  res = KheSplitClassTryEnd(sc, success);
   if( DEBUG2 )
   {
     KheSplitClassDebug(sc, 2, stderr, a);
@@ -818,6 +829,11 @@ static void KheNodeMeetsTriviallyAssign(KHE_NODE child_node,
   {
     child_meet = KheNodeMeet(child_node, i);
     parent_meet = KheNodeMeet(parent_node, i);
+    KheMeetCheck(child_meet);
+    KheMeetCheck(parent_meet);
+    if( DEBUG4 )
+      fprintf(stderr, "KheNodeMeetsTriviallyAssign calling KheMeetAssign"
+	"(%s, %s, 0)\n", KheMeetId(child_meet), KheMeetId(parent_meet));
     if( !KheMeetAssign(child_meet, parent_meet, 0) )
       HnAbort("KheLayerTreeMake internal error A");
     KheMeetAssignFix(child_meet);
@@ -952,7 +968,8 @@ static void KheSplitForestMakeTrivialAssignmentsOrLayers(KHE_NODE parent_node)
       KheNodeMeetsMakeCommonLayer(child_node, tmp_layers);
     }
     *** */
-    KheSplitForestMakeTrivialAssignmentsOrLayers(child_node /* , tmp_layers */);
+    else /* this else is a 15/12/23 bug fix; why was it missing? */
+      KheSplitForestMakeTrivialAssignmentsOrLayers(child_node /*,tmp_layers*/);
   }
 }
 
