@@ -37,7 +37,8 @@
 #define DEBUG4 0
 #define DEBUG5 0
 #define DEBUG6 0
-#define DEBUG7 1
+#define DEBUG7 0
+#define DEBUG8 0
 /* ***
 #define DEBUG4 1		** malloc_stats **
 #include <malloc.h>
@@ -62,7 +63,7 @@ static void KheDebugStage(KHE_SOLN soln, KHE_OPTIONS options,
   HnAssert(soln != NULL, "KheDebugStage internal error");
 
   ** sort out time usage **
-  if( !KheOptionsContainsTimer(options, "global", &timer) )
+  if( !KheOptionsContainsTimer(options, "gs_time_limit", &timer) )
     sprintf(time_buff, "no time limit");
   else
   {
@@ -95,12 +96,14 @@ static void KheDebugStage(KHE_SOLN soln, KHE_OPTIONS options,
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** currently unused
 typedef struct khe_resource_outcome_rec {
   KHE_RESOURCE		resource;
   KHE_COST		cost;
 } RESOURCE_OUTCOME;
 
 typedef HA_ARRAY(RESOURCE_OUTCOME) ARRAY_RESOURCE_OUTCOME;
+*** */
 
 
 /*****************************************************************************/
@@ -111,12 +114,14 @@ typedef HA_ARRAY(RESOURCE_OUTCOME) ARRAY_RESOURCE_OUTCOME;
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** currently unused
 static int ResourceOutcomeCmp(const void *t1, const void *t2)
 {
   RESOURCE_OUTCOME rc1 = * (RESOURCE_OUTCOME *) t1;
   RESOURCE_OUTCOME rc2 = * (RESOURCE_OUTCOME *) t2;
   return KheCostCmp(rc2.cost, rc1.cost);
 }
+*** */
 
 
 /*****************************************************************************/
@@ -128,14 +133,15 @@ static int ResourceOutcomeCmp(const void *t1, const void *t2)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** currently unused
 static void KheSolnDebugPoorResources(KHE_SOLN soln, int k,
   int indent, FILE *fp)
 {
   HA_ARENA a;  ARRAY_RESOURCE_OUTCOME outcomes;  KHE_INSTANCE ins;
   RESOURCE_OUTCOME oc;  int i, j;  KHE_MONITOR m;
 
-  /* build resource outcome array */
-  a = KheSolnArenaBegin(soln, false);
+  ** build resource outcome array **
+  a = KheSolnArenaBegin(soln);
   ins = KheSolnInstance(soln);
   HaArrayInit(outcomes, a);
   for( i = 0;  i < KheInstanceResourceCount(ins);  i++ )
@@ -145,10 +151,10 @@ static void KheSolnDebugPoorResources(KHE_SOLN soln, int k,
     HaArrayAddLast(outcomes, oc);
   }
 
-  /* sort the array */
+  ** sort the array **
   HaArraySort(outcomes, &ResourceOutcomeCmp);
 
-  /* print the resources */
+  ** print the resources **
   fprintf(stderr, "%*s[ resources of %s:\n", indent, "", KheInstanceId(ins));
   HaArrayForEach(outcomes, oc, i)
   {
@@ -166,9 +172,10 @@ static void KheSolnDebugPoorResources(KHE_SOLN soln, int k,
   }
   fprintf(stderr, "%*s]\n", indent, "");
 
-  /* recycle memory */
+  ** recycle memory **
   KheSolnArenaEnd(soln, a);
 }
+*** */
 
 
 /*****************************************************************************/
@@ -265,7 +272,7 @@ static void KheTryDoubleMove(KHE_SOLN soln, KHE_OPTIONS options,
   if( DEBUG7 )
   {
     ** convert resource names into resources **
-    a = KheSolnArenaBegin(soln, false);
+    a = KheSolnArenaBegin(soln);
     frame = KheOptionsFrame(options, "gs_common_frame", soln);
     ins = KheSolnInstance(soln);
     trace = KheTraceMake((KHE_GROUP_MONITOR) soln);
@@ -292,7 +299,7 @@ static void KheTryDoubleMove(KHE_SOLN soln, KHE_OPTIONS options,
         first_index2_name, last_index2_name);
 
     ** get the task sets and carry out the double moves **
-    tf = KheTaskFinderMake(soln, options, a);
+    tf = KheTaskFinde rMake(soln, options, a);
     ts1 = KheTaskSetMake(soln);
     ts2 = KheTaskSetMake(soln);
     if( KheFindT asksInInterval(tf, r1, first_index1, last_index1, ts1) &&
@@ -417,54 +424,56 @@ static void HandleDebugRTM(KHE_SOLN soln, char *debug_rtm)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)         */
+/*  KHE_SOLN KheGeneralSolve2024(KHE_SOLN soln, KHE_OPTIONS options)         */
 /*                                                                           */
 /*  Solve soln, assuming that it has just emerged from KheSolnMake.          */
 /*                                                                           */
 /****************************************************************************/
 /* extern void KheEjectionChainNodeGroupForTimingTest(KHE_NODE parent_node); */
 
-KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
+/* *** replaced by do-it-yourself version below
+KHE_SOLN KheGeneralSolve2024(KHE_SOLN soln, KHE_OPTIONS options)
 {
   int i, diversifier;  KHE_EVENT junk;  KHE_NODE cycle_node;  float time_limit;
   char *monitor_id, *debug_rtm;  KHE_MONITOR m;  KHE_TIMER global_timer;
-  KHE_EVENT_TIMETABLE_MONITOR etm;  KHE_INSTANCE ins;  KHE_TASKING tasking;
-  bool matching_off, monitor_evenness, time_assignment_only, unassign_off;
+  KHE_EVENT_TIMETABLE_MONITOR etm;  KHE_INSTANCE ins;
+  bool matching_off, monitor_evenness, time_assignment_only, unas sign_off;
   bool propagate_unavailable_times_off;  KHE_GROUP_MONITOR low_cost_gm;
-  /* KHE_RESOURCE_TYPE rt; */
+  ** char *rs;  KHE_TASKING tasking; **
+  ** KHE_RESOURCE_TYPE rt; **
 
 
-  /***************************************************************************/
-  /*                                                                         */
-  /*  initialization phase                                                   */
-  /*                                                                         */
-  /***************************************************************************/
+  *****************************************************************************
+  **                                                                         **
+  **  initialization phase                                                   **
+  **                                                                         **
+  *****************************************************************************
 
-  /* set soft time limit */
+  ** set soft time limit **
   time_limit = KheTimeFromString(KheOptionsGet(options, "gs_time_limit", "-"));
-  global_timer = KheOptionsAddTimer(options, "global", time_limit);
+  global_timer = KheOptionsAddTimer(options, "gs_time_limit", time_limit);
 
   ins = KheSolnInstance(soln);
-  matching_off = KheOptionsGetBool(options, "gs_matching_off", false);
+  matching_off = KheOptionsGetBool(options, "gs_matching_ off", false);
   monitor_evenness = KheOptionsGetBool(options, "gs_monitor_evenness", false);
   propagate_unavailable_times_off = KheOptionsGetBool(options,
     "gs_propagate_unavailable_times_off", false);
   time_assignment_only = KheOptionsGetBool(options, "gs_time_assignment_only",
     false);
-  unassign_off = KheOptionsGetBool(options, "gs_unassignment_off", false);
+  unas sign_off = KheOptionsGetBool(options, "gs_unassi gnment_off", false);
   diversifier = KheOptionsGetInt(options, "gs_diversifier", -1);
   if( diversifier != -1 )
     KheSolnSetDiversifier(soln, diversifier);
   if( DEBUG1 )
   {
-    fprintf(stderr, "[ KheGeneralSolve2020(%p %s) div %d\n", (void *) soln,
+    fprintf(stderr, "[ KheGeneralSolve2024(%p %s) div %d\n", (void *) soln,
       KheInstanceId(ins) == NULL ? "-" : KheInstanceId(ins),
       KheSolnDiversifier(soln));
-    /* KheDebugStage(soln, options, "at start", 2); */
+    ** KheDebugStage(soln, options, "at start", 2); **
     KheSolveDebug(soln, options, "at start");
   }
 
-  /* set debug_monitor option if debug_monitor_id is present */
+  ** set debug_monitor option if debug_monitor_id is present **
   monitor_id = KheOptionsGet(options, "gs_debug_monitor_id", NULL);
   if( monitor_id != NULL )
   {
@@ -485,12 +494,12 @@ KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
 	monitor_id);
   }
 
-  /* handle gs_debug_rtm option */
+  ** handle gs_debug_rtm option **
   debug_rtm = KheOptionsGet(options, "gs_debug_rtm", NULL);
   if( debug_rtm != NULL )
     HandleDebugRTM(soln, debug_rtm);
 
-  /* detach low-cost monitors if requested */
+  ** detach low-cost monitors if requested **
   low_cost_gm = NULL;
   if( KheOptionsGetBool(options, "gs_hard_constraints_only", false) )
   {
@@ -499,20 +508,20 @@ KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
   }
 
 
-  /***************************************************************************/
-  /*                                                                         */
-  /*  structural phase                                                       */
-  /*                                                                         */
-  /***************************************************************************/
+  *****************************************************************************
+  **                                                                         **
+  **  time structural phase                                                  **
+  **                                                                         **
+  *****************************************************************************
 
-  /* split initial cycle meet and make complete representation */
+  ** split initial cycle meet and make complete representation **
   KheSolnSplitCycleMeet(soln);
   if( DEBUG1 )
     KheSolveDebug(soln, options, "before KheSolnMakeCompleteRepresentation");
   if( !KheSolnMakeCompleteRepresentation(soln, &junk) )
     HnAbort("KheGeneralSolve2020:  !KheSolnMakeCompleteRepresentation");
 
-  /* build layer tree */
+  ** build layer tree **
   cycle_node = KheLayerTreeMake(soln);
   if( DEBUG2 )
   {
@@ -522,25 +531,26 @@ KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
   KheOptionsSetBool(options, "es_split_moves",
     KheInstanceContainsSoftSplitConstraint(ins));
 
-  /* optionally install the global tixel matching */
+  ** +++ gs starts here +++ **
+  ** optionally install the global tixel matching **
   if( !matching_off )
   {
     if( DEBUG1 )
-      KheSolveDebug(soln, options, "before KheSolnMatchingBegin");
-    KheSolnMatchingBegin(soln);
+      KheSolveDebug(soln, options, "before KheS olnMatchingBegin");
+    KheSoln MatchingBegin(soln);
     KheSolnMatchingSetWeight(soln, KheCost(1, 0));
     KheSolnMatchingAddAllWorkloadRequirements(soln);
     KheSolnMatchingAttachAllOrdinaryDemandMonitors(soln);
     if( DEBUG2 )
     {
-      /* KheDebugStage(soln, options, "after installing the matching", 2); */
+      ** KheDebugStage(soln, options, "after installing the matching", 2); **
       KheSolveDebug(soln, options, "after installing the matching");
       if( KheSolnMatchingDefectCount(soln) > 0 )
 	KheGroupMonitorDefectDebug((KHE_GROUP_MONITOR) soln, 2, 2, stderr);
     }
   }
 
-  /* optionally install evenness monitors */
+  ** optionally install evenness monitors **
   if( monitor_evenness )
   {
     KheSolnEvennessBegin(soln);
@@ -548,18 +558,18 @@ KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
     KheSolnAttachAllEvennessMonitors(soln);
   }
 
-  /* optionally propagate resource monitor adjustments */
+  ** optionally propagate resource monitor adjustments **
   if( !propagate_unavailable_times_off )
     KhePropagateUnavailableTimes(soln, NULL);
 
-  /* build task tree, including assigning preassigned resources */
+  ** build task tree, including assigning preassigned resources **
   KheOptionsSetBool(options, "rs_invariant", true);
   KheTaskTreeMake(soln, options);
 
-  /* find time-equivalent events and resources */
-  /* done on first use now KheTimeEquivSolve(time_eq uiv, soln); */
+  ** find time-equivalent events and resources **
+  ** done on first use now KheTimeEquivSolve(time_eq uiv, soln); **
 
-  /* coordinate layers and build runarounds */
+  ** coordinate layers and build runarounds **
   if( DEBUG1 )
     KheSolveDebug(soln, options, "before KheCoordinateLayers");
   KheCoordinateLayers(cycle_node, true);
@@ -570,7 +580,7 @@ KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
   }
   if( DEBUG1 )
     KheSolveDebug(soln, options, "before KheBuildRunarounds");
-  KheBuildRunarounds(cycle_node, &KheNodeSimpleAssignTimes, options,
+  KheBuildRunarounds(cycle_node, &KheNodeSimple AssignTimes, options,
     &KheRunaroundNodeAssignTimes, options);
   if( DEBUG2 )
   {
@@ -579,25 +589,25 @@ KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
   }
 
 
-  /***************************************************************************/
-  /*                                                                         */
-  /*  time assignment phase                                                  */
-  /*                                                                         */
-  /***************************************************************************/
+  *****************************************************************************
+  **                                                                         **
+  **  time assignment phase                                                  **
+  **                                                                         **
+  *****************************************************************************
 
-  /* assign to nodes below the cycle node */
+  ** assign to nodes below the cycle node **
   if( DEBUG1 )
-    KheSolveDebug(soln, options, "before %d KheNodeRecursiveAssignTimes",
+    KheSolveDebug(soln, options, "before %d KheNodeRecur siveAssignTimes",
       KheNodeChildCount(cycle_node));
   for( i = 0;  i < KheNodeChildCount(cycle_node);  i++ )
-    KheNodeRecursiveAssignTimes(KheNodeChild(cycle_node, i),
+    KheNodeRecursiveAssignT imes(KheNodeChild(cycle_node, i),
       &KheRunaroundNodeAssignTimes, options);
 
-  /* assign to the cycle node */
+  ** assign to the cycle node **
   if( DEBUG1 )
     KheSolveDebug(soln, options, "before KheCycleNodeAssignTimes");
   KheCycleNodeAssignTimes(cycle_node, options);
-    /* KheDebugStage(soln, options, "after time assignment", 2); */
+    ** KheDebugStage(soln, options, "after time assignment", 2); **
   if( time_assignment_only )
   {
     if( low_cost_gm != NULL )
@@ -610,70 +620,78 @@ KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
   }
 
 
-  /***************************************************************************/
-  /*                                                                         */
-  /*  resource assignment phases                                             */
-  /*                                                                         */
-  /***************************************************************************/
+  *****************************************************************************
+  **                                                                         **
+  **  resource assignment phases                                             **
+  **                                                                         **
+  *****************************************************************************
 
-  /* end the inclusion of demand cost in the solution cost */
+  ** end the inclusion of demand cost in the solution cost **
   KheDisconnectAllDemandMonitors(soln, NULL);
 
-  /* add an event timetable monitor to options */
-  etm = KheEventTimetableMonitorMake(soln, KheInstanceFullEventGroup(ins));
+  ** add an event timetable monitor to options **
+  e tm = KheEventTimetableMonitorMake(soln, KheInstanceFullEventGroup(ins));
   KheMonitorAttachToSoln((KHE_MONITOR) etm);
   KheOptionsSetObject(options, "gs_event_timetable_monitor", (void *) etm);
 
-  /* assign taskings (parts 1 and 2) */
-  for( i = 0;  i < KheSolnTaskingCount(soln);  i++ )
+  ** ***
+  rs = KheOptionsGet(options, "r s", "auto");
+  if( strcmp(rs, "auto") == 0 )
   {
-    tasking = KheSolnTasking(soln, i);
-    if( DEBUG1 )
-      KheSolveDebug(soln, options, "before %s assignment (stages 1 and 2)",
-	KheResourceTypeName(KheTaskingResourceType(tasking)));
-    KheTaskingAssignResourcesStage1(tasking, options);
-    KheTaskingAssignResourcesStage2(tasking, options);
+    ** assign taskings (parts 1 and 2) **
+    for( i = 0;  i < KheSolnTaskingCount(soln);  i++ )
+    {
+      tasking = KheSolnTasking(soln, i);
+      if( DEBUG1 )
+	KheSolveDebug(soln, options, "before %s assignment (stages 1 and 2)",
+	  KheResourceTypeName(KheTaskingResourceType(tasking)));
+      KheTaskingAssignResourcesStage1(tasking, options);
+      KheTaskingAssignResourcesStage2(tasking, options);
+    }
+
+    ** assign taskings (part 3) **
+    for( i = 0;  i < KheSolnTaskingCount(soln);  i++ )
+    {
+      tasking = KheSolnTasking(soln, i);
+      if( DEBUG1 )
+	KheSolveDebug(soln, options, "before %s assignment (part 3)",
+	  KheResourceTypeName(KheTaskingResourceType(tasking)));
+      KheTaskingAssignResourcesStage3(tasking, options);
+    }
+
+    ** try a tree search wherever there are still problems **
+    ** ***
+    KheTreeSearchRepa irTimes(soln, NULL, true);
+    *** **
   }
-
-  /* assign taskings (part 3) */
-  for( i = 0;  i < KheSolnTaskingCount(soln);  i++ )
-  {
-    tasking = KheSolnTasking(soln, i);
-    if( DEBUG1 )
-      KheSolveDebug(soln, options, "before %s assignment (part 3)",
-	KheResourceTypeName(KheTaskingResourceType(tasking)));
-    KheTaskingAssignResourcesStage3(tasking, options);
-  }
-
-  /* try a tree search wherever there are still problems */
-  /* ***
-  KheTreeSearchRepairTimes(soln, NULL, true);
-  *** */
+  else
+  *** **
+  KheCombinedResourceAssign(soln, options);
 
 
-  /***************************************************************************/
-  /*                                                                         */
-  /*  cleanup phase                                                          */
-  /*                                                                         */
-  /***************************************************************************/
+  *****************************************************************************
+  **                                                                         **
+  **  cleanup phase                                                          **
+  **                                                                         **
+  *****************************************************************************
 
-  /* ensure that the solution cost is the official cost */
+  ** ensure that the solution cost is the official cost **
   if( low_cost_gm != NULL )
     KheAttachLowCostMonitors(low_cost_gm);
   KheSolnEnsureOfficialCost(soln);
 
-  /* merge meets (requires split events monitors, hence this placement) */
+  ** merge meets (requires split events monitors, hence this placement) **
   KheMergeMeets(soln);
 
-  /* try task and meet unassignments */
-  if( !unassign_off )
+  ** try task and meet unassignments **
+  if( !unas sign_off )
   {
     KheSolnTryTaskUnAssignments(soln, options);
     KheSolnTryMeetUnAssignments(soln);
   }
 
-  /* last-ditch check of swap possibilities (NB requires etm) */
-  /* ***
+  ** last-ditch check of swap possibilities (NB requires etm) **
+  ** ***
   if( DEBUG6 )
     for( i = 0;  i < KheInstanceResourceTypeCount(ins);  i++ )
     {
@@ -682,15 +700,15 @@ KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
     }
   KheTryDoubleMove(soln, options,
     "TR_27", "4Sat", "4Sun", "TR_26", "1Sat", "2Mon");
-  *** */
+  *** **
 
-  /* remove the event timetable monitor from options and delete it */
+  ** remove the event timetable monitor from options and delete it **
   HnAssert(KheOptionsGetObject(options, "gs_event_timetable_monitor", NULL)
     == (void *) etm, "KheGeneralSolve2020 internal error (etm)");
   KheOptionsSetObject(options, "gs_event_timetable_monitor", NULL);
-  /* KheEventTimetableMonitorDelete(etm); */
+  ** KheEventTimetableMonitorDelete(etm); **
 
-  /* wrapup */
+  ** wrapup **
   if( DEBUG5 )
     KheSolnDebugPoorResources(soln, 8, 2, stderr);
   if( DEBUG1 )
@@ -713,6 +731,162 @@ KHE_SOLN KheGeneralSolve2020(KHE_SOLN soln, KHE_OPTIONS options)
   KheOptionsDeleteTimer(options, global_timer);
   return soln;
 }
+*** */
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_SOLN KheGeneralSolve2024(KHE_SOLN soln, KHE_OPTIONS options)         */
+/*                                                                           */
+/*  Solve soln, assuming that it has just emerged from KheSolnMake.          */
+/*                                                                           */
+/****************************************************************************/
+/* extern void KheEjectionChainNodeGroupForTimingTest(KHE_NODE parent_node); */
+
+KHE_SOLN KheGeneralSolve2024(KHE_SOLN soln, KHE_OPTIONS options)
+{
+  int diversifier, i;  KHE_EVENT junk;  KHE_NODE cycle_node;  float time_limit;
+  char *monitor_id, *debug_rtm, *gs;  KHE_MONITOR m;  KHE_TIMER gs_timer;
+  KHE_INSTANCE ins;  bool unassign_off, time_limit_consistency;
+  KHE_RESOURCE_TYPE rt;
+
+
+  /***************************************************************************/
+  /*                                                                         */
+  /*  handle options                                                         */
+  /*                                                                         */
+  /***************************************************************************/
+
+  ins = KheSolnInstance(soln);
+  if( DEBUG1 )
+    fprintf(stderr, "[ KheGeneralSolve2024(soln of %s, options)\n",
+      KheInstanceId(ins));
+
+  /* gs_diversifier */
+  diversifier = KheOptionsGetInt(options, "gs_diversifier", -1);
+  if( diversifier != -1 )
+    KheSolnSetDiversifier(soln, diversifier);
+
+  /* gs_time_limit */
+  time_limit = KheTimeFromString(KheOptionsGet(options, "gs_time_limit", "-"));
+  gs_timer = KheOptionsAddTimer(options, "gs_time_limit", time_limit);
+
+  /* gs_time_limit_consistency */
+  time_limit_consistency = KheOptionsGetBool(options,
+    "gs_time_limit_consistency", false);
+
+  /* gs_unassignment_off */
+  unassign_off = KheOptionsGetBool(options, "gs_unassignment_off", false);
+
+  /* gs_debug_monitor_id and gs_debug_monitor */
+  monitor_id = KheOptionsGet(options, "gs_debug_monitor_id", NULL);
+  if( monitor_id != NULL )
+  {
+    if( KheSolnRetrieveMonitor(soln, monitor_id, &m) )
+      KheOptionsSetObject(options, "gs_debug_monitor", (void *) m);
+    else
+      HnAbort("KheGeneralSolve2020: cannot find gs_debug_monitor_id \"%s\"\n",
+	monitor_id);
+  }
+
+  /* gs_debug_rtm */
+  debug_rtm = KheOptionsGet(options, "gs_debug_rtm", NULL);
+  if( debug_rtm != NULL )
+    HandleDebugRTM(soln, debug_rtm);
+
+
+  /***************************************************************************/
+  /*                                                                         */
+  /*  functions more or less unavoidable when starting a solve               */
+  /*                                                                         */
+  /***************************************************************************/
+
+  /* begin time limit consistency, if requested */
+  if( DEBUG1 )
+    fprintf(stderr, "  KheGeneralSolve2024 KheOptionsTimeLimitConsistency:\n");
+  if( time_limit_consistency )
+    KheOptionsTimeLimitConsistencyBegin(options, soln);
+
+  /* split initial cycle meet */
+  if( DEBUG1 )
+    fprintf(stderr, "  KheGeneralSolve2024 KheSolnSplitCycleMeet:\n");
+  KheSolnSplitCycleMeet(soln);
+
+  /* make complete representation */
+  if( DEBUG1 )
+    fprintf(stderr, "  KheGeneralSolve2024 MakeCompleteRepresentation:\n");
+  if( !KheSolnMakeCompleteRepresentation(soln, &junk) )
+    HnAbort("KheGeneralSolve2020:  !KheSolnMakeCompleteRepresentation");
+
+  /* layer tree */
+  if( DEBUG1 )
+    fprintf(stderr, "  KheGeneralSolve2024 KheLayerTreeMake:\n");
+  cycle_node = KheLayerTreeMake(soln);
+
+  /* es_split_moves */
+  KheOptionsSetBool(options, "es_split_moves",
+    KheInstanceContainsSoftSplitConstraint(ins));
+
+  /* build task tree, including assigning preassigned resources */
+  if( DEBUG1 )
+    fprintf(stderr, "  KheGeneralSolve2024 KheTaskTreeMake:\n");
+  KheOptionsSetBool(options, "rs_invariant", true);
+  KheTaskTreeMake(soln, NULL, options);
+  KheOptionsSetBool(options, "rs_invariant", false);
+
+
+  /***************************************************************************/
+  /*                                                                         */
+  /*  do-it-yourself part                                                    */
+  /*                                                                         */
+  /***************************************************************************/
+
+  if( DEBUG1 )
+    fprintf(stderr, "  KheGeneralSolve2024 do-it-yourself part\n");
+  gs = KheOptionsGet(options, "gs", "(gpu, ts, rs)");
+  KheDoItYourselfSolverParseAndRun(soln, cycle_node, options, "gs", gs);
+
+
+  /***************************************************************************/
+  /*                                                                         */
+  /*  functions more or less unavoidable when ending a solve                 */
+  /*                                                                         */
+  /***************************************************************************/
+
+  /* ensure that the solution cost is the official cost */
+  if( DEBUG1 )
+    fprintf(stderr, "  KheGeneralSolve2024 ending solve part\n");
+  KheSolnEnsureOfficialCost(soln);
+
+  /* merge meets (requires split events monitors, hence this placement) */
+  KheMergeMeets(soln);
+
+  /* try task and meet unassignments */
+  if( !unassign_off )
+  {
+    if( DEBUG8 )
+      fprintf(stderr, "  KheGeneralSolve2024 calling "
+	"KheSolnTryTaskUnAssignments (etc.)\n");
+    for( i = 0;  i < KheInstanceResourceTypeCount(ins);  i++ )
+    {
+      rt = KheInstanceResourceType(ins, i);
+      KheEjectionChainRepairResources(soln, rt, options, "2+");
+    }
+    /* KheSolnTryTaskUnAssignments(soln, false, options); */
+    KheSolnTryMeetUnAssignments(soln);
+  }
+
+  /* end time limit consistency, if requested */
+  if( time_limit_consistency )
+    KheOptionsTimeLimitConsistencyEnd(options, soln);
+
+  /* set running time and delete timer */
+  KheSolnSetRunningTime(soln, KheTimerElapsedTime(gs_timer));
+  KheOptionsDeleteTimer(options, gs_timer);
+  if( DEBUG1 )
+    fprintf(stderr, "] KheGeneralSolve2024 returning soln\n");
+  return soln;
+}
 
 
 /*****************************************************************************/
@@ -731,7 +905,7 @@ void KheSolveDebug(KHE_SOLN soln, KHE_OPTIONS options, char *fmt, ...)
   ins = KheSolnInstance(soln);
   fprintf(stderr, "%*s%s#%d: %.5f", 2, "", KheInstanceId(ins),
     KheSolnDiversifier(soln), KheCostShow(KheSolnCost(soln)));
-  if( KheOptionsContainsTimer(options, "global", &timer) )
+  if( KheOptionsContainsTimer(options, "gs_time_limit", &timer) )
     fprintf(stderr, ", %9s", KheTimeShow(KheTimerElapsedTime(timer), buff));
   if( fmt != NULL )
   {
