@@ -316,8 +316,8 @@ void KheOptionsSetInt(KHE_OPTIONS options, char *key, int value)
 /*                                                                           */
 /*  int KheOptionsGetInt(KHE_OPTIONS options, char *key, int dft)            */
 /*                                                                           */
-/*  Get the value of key in options, assumed to represent an integer,        */
-/*  Return dft if the option is absent or "auto".                            */
+/*  Get the value of key in options, assumed to represent an integer.        */
+/*  Return dft if the option is absent.                                      */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -330,13 +330,52 @@ int KheOptionsGetInt(KHE_OPTIONS options, char *key, int dft)
     return res;
   else
   {
-    HnAbort("KheOptionsGetBool: key \"%s\" has non-int value \"%s\"",
+    HnAbort("KheOptionsGetInt: key \"%s\" has non-int value \"%s\"",
       key, value);
     return 0;  /* keep compiler happy */
   }
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheOptionsSetFloat(KHE_OPTIONS options, char *key, float value)     */
+/*                                                                           */
+/*  Set the value of key in options to the string equivalent of value.       */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheOptionsSetFloat(KHE_OPTIONS options, char *key, float value)
+{
+  KheOptionsSet(options, key, HnStringMake(options->arena, "%f", value));
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  float KheOptionsGetFloat(KHE_OPTIONS options, char *key, float dft)      */
+/*                                                                           */
+/*  Get the value of key in options, assumed to represent a float.           */
+/*  Return dft if the option is absent.                                      */
+/*                                                                           */
+/*****************************************************************************/
+
+float KheOptionsGetFloat(KHE_OPTIONS options, char *key, float dft)
+{
+  char *value;  int pos;  float res;
+  if( !HnTableRetrieve(options->string_table, key, value, pos) )
+    return dft;
+  else if( sscanf(value, "%f", &res) == 1 )
+    return res;
+  else
+  {
+    HnAbort("KheOptionsGetFloat: key \"%s\" has non-float value \"%s\"",
+      key, value);
+    return 0.0;  /* keep compiler happy */
+  }
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheOptionsSetObject(KHE_OPTIONS options, char *key, void *value,    */
@@ -458,6 +497,20 @@ bool KheOptionsContainsTimer(KHE_OPTIONS options, char *tag, KHE_TIMER *timer)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  float KheOptionsRemainingTime(KHE_OPTIONS options)                       */
+/*                                                                           */
+/*  Return the remaining time of the time set of options.                    */
+/*                                                                           */
+/*****************************************************************************/
+
+float KheOptionsRemainingTime(KHE_OPTIONS options)
+{
+  return KheTimerSetRemainingTime(options->timer_set);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  bool KheOptionsTimeLimitReached(KHE_OPTIONS options)                     */
@@ -472,6 +525,53 @@ bool KheOptionsTimeLimitReached(KHE_OPTIONS options)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheOptionsTimeLimitReachedQueryCount(KHE_OPTIONS options)            */
+/*                                                                           */
+/*  Return the number of calls to KheOptionsTimeLimitReached since this      */
+/*  function was last called.                                                */
+/*                                                                           */
+/*****************************************************************************/
+
+/* ***
+int KheOptionsTimeLimitReachedQueryCount(KHE_OPTIONS options)
+{
+  return KheTimeSetTimeLimitReachedQueryCount(options->timer_set);
+}
+*** */
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheOptionsTimeLimitConsistencyBegin(KHE_OPTIONS options,            */
+/*    KHE_SOLN soln)                                                         */
+/*                                                                           */
+/*  Begin run consistency when solving soln.                                 */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheOptionsTimeLimitConsistencyBegin(KHE_OPTIONS options, KHE_SOLN soln)
+{
+  KheTimerSetTimeLimitConsistencyBegin(options->timer_set, soln);
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheOptionsTimeLimitConsistencyEnd(KHE_OPTIONS options,              */
+/*    KHE_SOLN soln)                                                         */
+/*                                                                           */
+/*  End run consistency when solving soln.                                   */
+/*                                                                           */
+/*****************************************************************************/
+
+void KheOptionsTimeLimitConsistencyEnd(KHE_OPTIONS options, KHE_SOLN soln)
+{
+  KheTimerSetTimeLimitConsistencyEnd(options->timer_set, soln);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheOptionsTimerSetSaveLog(KHE_OPTIONS options, KHE_SOLN soln)       */
@@ -907,9 +1007,9 @@ KHE_FRAME KheOptionsFrame(KHE_OPTIONS options, char *key, KHE_SOLN soln)
   res = (KHE_FRAME) KheOptionsGetObject(options, key, NULL);
   if( res == NULL )
   {
-    res = KheFrameMakeCommon(soln);
+    res = KheFrameMakeCommon(KheSolnInstance(soln), options->arena);
     if( res == NULL )
-      res = KheFrameMakeSingletons(soln);
+      res = KheFrameMakeSingletons(KheSolnInstance(soln), options->arena);
     KheOptionsSetObject(options, key, (void *) res);
   }
   return res;
