@@ -144,7 +144,8 @@ struct khe_graph_rec {
   char				*left_caption;	/* left caption of graph     */
   char				*left_gap;	/* left_gap option of graph  */
   char				*right_caption;	/* right caption of graph    */
-  char				*right_gap;	/* right_gap option of graph  */
+  char				*right_gap;	/* right_gap option of graph */
+  char				*key_label;	/* at start of graph key     */
   ARRAY_KHE_DATASET		datasets;	/* the datasets              */
   HA_ARRAY_NSTRING		caption_lines;	/* the lines of the caption  */
 };
@@ -1218,8 +1219,9 @@ static void KheTableWrite(KHE_TABLE tbl)
 
 KHE_FILE KheFileBegin(char *file_name, KHE_FILE_FORMAT fmt)
 {
-  KHE_FILE res;  char buff[200];  HA_ARENA a;
-  a = HaArenaMake(false);  /* we've decided not to remove this call */
+  KHE_FILE res;  char buff[200];  HA_ARENA a;  HA_ARENA_SET as;
+  as = HaArenaSetMake();
+  a = HaArenaMake(as);  /* we've decided not to remove this call */
   HaMake(res, a);
   res->arena = a;
   res->file_name = HnStringCopy(file_name, a);
@@ -1868,6 +1870,12 @@ static void KheGraphWrite(KHE_GRAPH graph, KHE_FILE file)
 
   /* print objects option, if there are any non-NULL labels */
   objects_started = false;
+  if( graph->key_label != NULL )
+  {
+    fprintf(file->fp, "  objects { @SE at {0 ymax} @OneRow {\n");
+    fprintf(file->fp, "    %s\n", graph->key_label);
+    objects_started = true;
+  }
   HaArrayForEach(graph->datasets, ds, i)
     if( ds->label != NULL )
     {
@@ -1927,6 +1935,7 @@ KHE_GRAPH KheGraphBegin(KHE_FILE kf)
   res->left_gap = NULL;
   res->right_caption = NULL;
   res->right_gap = NULL;
+  res->key_label = NULL;
   HaArrayInit(res->datasets, kf->arena);
   HaArrayInit(res->caption_lines, kf->arena);
   kf->thing_count++;
@@ -1959,6 +1968,7 @@ void KheGraphEnd(KHE_GRAPH kg)
 /*  void KheStatsGraphSetLeftCaption(char *file_name, char *val)             */
 /*  void KheStatsGraphSetRightCaption(char *file_name, char *val)            */
 /*                                                                           */
+/*                                                                           */
 /*  Set these various global attributes of the graph.                        */
 /*                                                                           */
 /*****************************************************************************/
@@ -2017,6 +2027,12 @@ void KheGraphSetRightCaptionAndGap(KHE_GRAPH kg, char *val, char *gap)
   kg->right_gap = HnStringCopy(gap, kg->file->arena);
 }
 
+void KheGraphSetKeyLabel(KHE_GRAPH kg, char *val)
+{
+  HnAssert(kg->key_label == NULL, "KheGraphSetKeyLabel: key_label already set");
+  kg->key_label = HnStringCopy(val, kg->file->arena);
+}
+
 
 /*****************************************************************************/
 /*                                                                           */
