@@ -40,20 +40,6 @@
 #define DEBUG2	0
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  KHE_TIMER_LOG                                                            */
-/*                                                                           */
-/*****************************************************************************/
-
-typedef struct khe_timer_log_rec {
-  char				*tag;		/* identifying tag           */
-  HA_ARRAY_INT			log;		/* log of what happened      */
-} *KHE_TIMER_LOG;
-
-typedef HA_ARRAY(KHE_TIMER_LOG) ARRAY_KHE_TIMER_LOG;
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  KHE_TIMER                                                                */
@@ -66,7 +52,6 @@ struct khe_timer_rec {
   struct timeval		start_time;	/* start time                */
 #endif
   float				time_limit;	/* time limit                */
-  /* KHE_TIMER_LOG		timer_log; */	/* the log                   */
 };
 
 typedef HA_ARRAY(KHE_TIMER) ARRAY_KHE_TIMER;
@@ -74,20 +59,63 @@ typedef HA_ARRAY(KHE_TIMER) ARRAY_KHE_TIMER;
 
 /*****************************************************************************/
 /*                                                                           */
-/*  KHE_TIMER_SET                                                            */
+/*  KHE_CONSISTENCY_STATE                                                    */
+/*                                                                           */
+/*  What a timer set is currently doing about time limit consistency:        */
+/*                                                                           */
+/*    KHE_CONSISTENCY_NONE                                                   */
+/*      There has been no call to KheTimerSetTimeLimitConsistencyBegin,      */
+/*      so the timer set is using timers to implement time limits, in        */
+/*      the usual way.                                                       */
+/*                                                                           */
+/*    KHE_CONSISTENCY_STARTING                                               */
+/*      There has been a call to KheTimerSetTimeLimitConsistencyBegin,       */
+/*      but there is no .tlc file, so the timer set is using timers to       */
+/*      implement time limits, but it will be writing query counts to        */
+/*      a .tlc file so that the next run can use them.                       */
+/*                                                                           */
+/*    KHE_CONSISTENCY_ACTIVE                                                 */
+/*      There has been a call to KheTimerSetTimeLimitConsistencyBegin,       */
+/*      and there is a .tlc file, so the timer set is using query            */
+/*      counts to determine time limits, not timers.                         */
 /*                                                                           */
 /*****************************************************************************/
 
 typedef enum {
-  TIMER_SET_LOG_NONE,
-  TIMER_SET_LOG_READ,
-  TIMER_SET_LOG_WRITE
-} TIMER_SET_LOG_STATE;
+  KHE_CONSISTENCY_NONE,
+  KHE_CONSISTENCY_STARTING,
+  KHE_CONSISTENCY_ACTIVE
+} KHE_CONSISTENCY_STATE;
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_TIMER_SET                                                            */
+/*                                                                           */
+/*  old_query_counts                                                         */
+/*    If state == KHE_CONSISTENCY_ACTIVE, the query counts from a previous   */
+/*    run as read from a .tlc file.                                          */
+/*                                                                           */
+/*  curr_query_counts                                                        */
+/*    The query counts from the current run.  These are recorded whatever    */
+/*    the state, but only saved to a file if state != KHE_CONSISTENCY_NONE.  */
+/*                                                                           */
+/*  curr_timer_tags                                                          */
+/*    One for each query count except the last, identifying the most         */
+/*    recently added timer when the queries were made.  This is a rough      */
+/*    but ready way to work out which solvers are making the calls.          */
+/*    These values are written to .tlc files, but only as an aid to          */
+/*    testing; they could be omitted completely without harm.                */
+/*                                                                           */
+/*****************************************************************************/
 
 struct khe_timer_set_rec {
-  TIMER_SET_LOG_STATE		log_state;	/* timer set log state       */
-  ARRAY_KHE_TIMER		timers;		/* the timers                */
-  /* ARRAY_KHE_TIMER_LOG	timer_logs; */	/* the timer logs            */
+  HA_ARENA			arena;
+  ARRAY_KHE_TIMER		timers;
+  KHE_CONSISTENCY_STATE		state;
+  HA_ARRAY_INT			old_query_counts;
+  HA_ARRAY_INT			curr_query_counts;
+  HA_ARRAY_NSTRING		curr_timer_tags;
 };
 
 
@@ -147,13 +175,22 @@ char *KheDateToday(void)
 
 float KheTimeFromString(char *str)
 {
+  float res;
+  if( KheStringContainsTime(str, &res) )
+    return res;
+  else
+  {
+    HnAbort("KheTimeFromString:  format error in time limit %s", str);
+    return KHE_NO_TIME;  /* keep compiler happy */
+  }
+  /* ***
   char *p;  float hrs, mins, secs;
 
-  /* "-" special case */
+  ** "-" special case **
   if( strcmp(str, "-") == 0 )
     return KHE_NO_TIME;
 
-  /* secs */
+  ** secs **
   p = strstr(str, ":");
   if( p == NULL )
   {
@@ -162,7 +199,7 @@ float KheTimeFromString(char *str)
     return secs;
   }
 
-  /* mins:secs */
+  ** mins:secs **
   p = strstr(p + 1, ":");
   if( p == NULL )
   {
@@ -171,10 +208,57 @@ float KheTimeFromString(char *str)
     return mins * 60.0 + secs;
   }
 
-  /* hrs:mins:secs */
+  ** hrs:mins:secs **
   if( sscanf(str, "%f:%f:%f", &hrs, &mins, &secs) != 3 )
     HnAbort("KheTimeFromString:  format error in time limit %s", str);
   return hrs * 60.0 * 60.0 + mins * 60.0 + secs;
+  *** */
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheStringContainsTime(char *str, float *time)                       */
+/*                                                                           */
+/*  If str contains a time (possibly -, denoting KHE_NO_TIME), set *time     */
+/*  to the time denoted and return true.  Otherwise set *time to             */
+/*  KHE_NO_TIME and return false.                                            */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheStringContainsTime(char *str, float *time)
+{
+  char *p;  float hrs, mins, secs;
+
+  /* "-" special case */
+  if( strcmp(str, "-") == 0 )
+    return *time = KHE_NO_TIME, true;
+
+  /* secs */
+  p = strstr(str, ":");
+  if( p == NULL )
+  {
+    if( sscanf(str, "%f", &secs) != 1 )
+      return *time = KHE_NO_TIME, false;
+    else
+      return *time = secs, true;
+  }
+
+  /* mins:secs */
+  p = strstr(p + 1, ":");
+  if( p == NULL )
+  {
+    if( sscanf(str, "%f:%f", &mins, &secs) != 2 )
+      return *time = KHE_NO_TIME, false;
+    else
+      return *time = mins * 60.0 + secs, true;
+  }
+
+  /* hrs:mins:secs */
+  if( sscanf(str, "%f:%f:%f", &hrs, &mins, &secs) != 3 )
+    return *time = KHE_NO_TIME, false;
+  else
+    return *time = hrs * 60.0 * 60.0 + mins * 60.0 + secs, true;
 }
 
 
@@ -188,7 +272,9 @@ float KheTimeFromString(char *str)
 
 char *KheTimeShow(float secs, char buff[20])
 {
-  if( secs > 300.0 )
+  if( secs == KHE_NO_TIME )
+    snprintf(buff, 20, "(no time)");
+  else if( secs > 300.0 )
     snprintf(buff, 20, "%.1f mins", secs / 60.0);
   else
     snprintf(buff, 20, "%.1f secs", secs);
@@ -196,73 +282,6 @@ char *KheTimeShow(float secs, char buff[20])
 }
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  Submodule "timer logs"                                                   */
-/*                                                                           */
-/*****************************************************************************/
-
-/*****************************************************************************/
-/*                                                                           */
-/*  KHE_TIMER_LOG KheTimerLogMake(char *tag, HA_ARENA a)                     */
-/*                                                                           */
-/*  Make a new, empty timer log with these attributes.                       */
-/*                                                                           */
-/*****************************************************************************/
-
-/* ***
-static KHE_TIMER_LOG KheTimerLogMake(char *tag, HA_ARENA a)
-{
-  KHE_TIMER_LOG res;
-  HaMake(res, a);
-  res->tag = tag;
-  HaArrayInit(res->log, a);
-  HaArrayAddLast(res->log, 0);
-  return res;
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheTimerLogUpdate(KHE_TIMER_LOG timer_log, bool reached)            */
-/*                                                                           */
-/*  Log the occurrence of a call to KheTimerTimeLimitReached whose result    */
-/*  is reached.                                                              */
-/*                                                                           */
-/*****************************************************************************/
-
-/* ***
-static void KheTimerLogUpdate(KHE_TIMER_LOG timer_log, bool reached)
-{
-  if( reached )
-    HaArrayAddLast(timer_log->log, 0);
-  else
-    HaArrayLast(timer_log->log)++;
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheTimerLogSave(KHE_TIMER_LOG timer_log, FILE *fp)                  */
-/*                                                                           */
-/*  Save timer_log on fp.                                                    */
-/*                                                                           */
-/*****************************************************************************/
-
-/* ***
-static void KheTimerLogSave(KHE_TIMER_LOG timer_log, FILE *fp)
-{
-  int val, i;
-  fprintf(fp, "  %s {", timer_log->tag);
-  HaArrayForEach(timer_log->log, val, i)
-    fprintf(fp, "%s%d", i % 8 == 0 ? "\n    " : " ", val);
-  fprintf(fp, "\n  }\n");
-}
-*** */
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "timers"                                                       */
@@ -284,7 +303,6 @@ KHE_TIMER KheTimerMake(char *tag, float limit_in_seconds, HA_ARENA a)
   res->tag = tag;
   KheTimerResetStartTime(res);
   res->time_limit = limit_in_seconds;
-  /* res->timer_log = NULL; */
   return res;
 }
 
@@ -306,7 +324,6 @@ KHE_TIMER KheTimerCopy(KHE_TIMER timer, HA_ARENA a)
   res->start_time = timer->start_time;
 #endif
   res->time_limit = timer->time_limit;
-  /* res->timer_log = NULL; */
   return res;
 }
 
@@ -373,7 +390,7 @@ void KheTimerResetTimeLimit(KHE_TIMER timer, float limit_in_seconds)
 /*                                                                           */
 /*  float KheTimeDiff(struct timeval *earlier, struct timeval *later)        */
 /*                                                                           */
-/*  Return the time difference between earlier and later.                    */
+/*  Return the time difference between earlier and later, in seconds.        */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -406,6 +423,29 @@ float KheTimerElapsedTime(KHE_TIMER timer)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  float KheTimerRemainingTime(KHE_TIMER timer)                             */
+/*                                                                           */
+/*  Return the remaining time until timer reaches its time limit, or 0.0     */
+/*  if it is already at or over its time limit.                              */
+/*                                                                           */
+/*****************************************************************************/
+
+float KheTimerRemainingTime(KHE_TIMER timer)
+{
+  float remaining_time;
+#if KHE_USE_TIMING
+  if( timer->time_limit == KHE_NO_TIME )
+    return KHE_NO_TIME;
+  remaining_time = timer->time_limit - KheTimerElapsedTime(timer);
+  return remaining_time >= 0.0 ? remaining_time : 0.0;
+#else
+  return KHE_NO_TIME;
+#endif
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  bool KheTimerReached(KHE_TIMER timer, bool *curr_time_known,             */
@@ -423,7 +463,6 @@ float KheTimerElapsedTime(KHE_TIMER timer)
 static bool KheTimerReached(KHE_TIMER timer, bool *curr_time_known,
   struct timeval *curr_time)
 {
-  bool reached;
   if( timer->time_limit != KHE_NO_TIME )
   {
     /* get current time, if not done yet */
@@ -434,16 +473,9 @@ static bool KheTimerReached(KHE_TIMER timer, bool *curr_time_known,
     }
 
     /* work out whether this timer's limit has been reached */
-    reached = KheTimeDiff(&timer->start_time, curr_time) >= timer->time_limit;
     if( DEBUG2 )
       KheTimerDebug(timer, 2, 2, stderr);
-
-    /* update the log and return */
-    /* ***
-    if( timer->timer_log != NULL )
-      KheTimerLogUpdate(timer->timer_log, reached);
-    *** */
-    return reached;
+    return KheTimeDiff(&timer->start_time, curr_time) >= timer->time_limit;
   }
   else
     return false;
@@ -504,9 +536,13 @@ KHE_TIMER_SET KheTimerSetMake(HA_ARENA a)
 {
   KHE_TIMER_SET res;
   HaMake(res, a);
-  res->log_state = TIMER_SET_LOG_NONE;
+  res->arena = a;
   HaArrayInit(res->timers, a);
-  /* HaArrayInit(res->timer_logs, a); */
+  res->state = KHE_CONSISTENCY_NONE;
+  HaArrayInit(res->old_query_counts, a);
+  HaArrayInit(res->curr_query_counts, a);
+  HaArrayAddLast(res->curr_query_counts, 0);
+  HaArrayInit(res->curr_timer_tags, a);
   if( DEBUG1 )
   {
     fprintf(stderr, "timer set at start:\n");
@@ -527,11 +563,15 @@ KHE_TIMER_SET KheTimerSetMake(HA_ARENA a)
 KHE_TIMER_SET KheTimerSetCopy(KHE_TIMER_SET timer_set, HA_ARENA a)
 {
   KHE_TIMER_SET res;  KHE_TIMER timer;  int i;
-  HaMake(res, a);
-  HaArrayInit(res->timers, a);
+
+  /* make sure there are no query counts around */
+  HnAssert(HaArrayCount(timer_set->old_query_counts) == 0,
+    "KheTimerSetCopy: old query counts are present");
+
+  /* make a new, empty timer set object and add copies of timer_sets's timers */
+  res = KheTimerSetMake(a);
   HaArrayForEach(timer_set->timers, timer, i)
     HaArrayAddLast(timer_set->timers, KheTimerCopy(timer, a));
-  /* HaArrayInit(res->timer_logs, a); */
   return res;
 }
 
@@ -543,7 +583,6 @@ KHE_TIMER_SET KheTimerSetCopy(KHE_TIMER_SET timer_set, HA_ARENA a)
 /*  Add timer to timer_set.                                                  */
 /*                                                                           */
 /*****************************************************************************/
-/* static KHE_TIMER_LOG KheTimerSetGetTimerLog(KHE_TIMER_SET ts, char *tag); */
 
 void KheTimerSetAddTimer(KHE_TIMER_SET timer_set, KHE_TIMER timer)
 {
@@ -556,7 +595,6 @@ void KheTimerSetAddTimer(KHE_TIMER_SET timer_set, KHE_TIMER timer)
   HnAssert(!HaArrayContains(timer_set->timers, timer, &pos),
     "KheTimerSetAddTimer: timer_set already contains timer");
   HaArrayAddLast(timer_set->timers, timer);
-  /* timer->timer_log = KheTimerSetGetTimerLog(timer_set, timer->tag); */
   if( DEBUG1 )
   {
     KheTimerSetDebug(timer_set, 2, 2, stderr);
@@ -580,7 +618,6 @@ void KheTimerSetDeleteTimer(KHE_TIMER_SET timer_set, KHE_TIMER timer)
     HnAbort("KheTimerSetDeleteTimer: time_set does not contain timer \"%s\"",
       timer->tag);
   HaArrayDeleteAndShift(timer_set->timers, pos);
-  /* timer->timer_log = NULL; */
 }
 
 
@@ -605,6 +642,38 @@ bool KheTimerSetContainsTimer(KHE_TIMER_SET timer_set, char *tag,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  float KheTimerSetRemainingTime(KHE_TIMER_SET timer_set)                  */
+/*                                                                           */
+/*  Return the remaining time, or KHE_NO_TIME if unlimited or time limits    */
+/*  are irrelevant because time limit consistency is active.                 */
+/*                                                                           */
+/*****************************************************************************/
+
+float KheTimerSetRemainingTime(KHE_TIMER_SET timer_set)
+{
+#if KHE_USE_TIMING
+  KHE_TIMER timer;  int i;
+  float min_remaining_time, remaining_time;
+  if( timer_set->state == KHE_CONSISTENCY_ACTIVE )
+    return KHE_NO_TIME;
+  min_remaining_time = KHE_NO_TIME;
+  HaArrayForEach(timer_set->timers, timer, i)
+    if( timer->time_limit != KHE_NO_TIME )
+    {
+      remaining_time = KheTimerRemainingTime(timer);
+      if( min_remaining_time == KHE_NO_TIME ||
+	  remaining_time < min_remaining_time )
+        min_remaining_time = remaining_time;
+    }
+  return min_remaining_time;
+#else
+  return KHE_NO_TIME;
+#endif
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  bool KheTimerSetTimeLimitReached(KHE_TIMER_SET timer_set)                */
@@ -612,100 +681,178 @@ bool KheTimerSetContainsTimer(KHE_TIMER_SET timer_set, char *tag,
 /*  Return true if any of the time limits of timer_set have been reached.    */
 /*                                                                           */
 /*  Implementation note.  This does not call KheTimerTimeLimitReached,       */
-/*  because it it optimized to call gettimeofday only once.                  */
+/*  because it is optimized to call gettimeofday only once.                  */
 /*                                                                           */
 /*****************************************************************************/
 
 bool KheTimerSetTimeLimitReached(KHE_TIMER_SET timer_set)
 {
 #if KHE_USE_TIMING
-  struct timeval curr_time;  KHE_TIMER timer;  int i;  bool curr_time_known;
-  curr_time_known = false;
-  HaArrayForEach(timer_set->timers, timer, i)
-    if( KheTimerReached(timer, &curr_time_known, &curr_time) )
-      return true;
-#endif
+  struct timeval curr_time;  KHE_TIMER timer;  int i, curr_index, curr_count;
+  bool curr_time_known;
+  switch( timer_set->state )
+  {
+    case KHE_CONSISTENCY_NONE:
+
+      /* no consistency checking, just use timers */
+      curr_time_known = false;
+      HaArrayForEach(timer_set->timers, timer, i)
+	if( KheTimerReached(timer, &curr_time_known, &curr_time) )
+	  return true;
+      return false;
+
+    case KHE_CONSISTENCY_STARTING:
+
+      /* use timers, but save query counts for the next run to use */
+      curr_index = HaArrayCount(timer_set->curr_query_counts) - 1;
+      curr_count = ++HaArray(timer_set->curr_query_counts, curr_index);
+      curr_time_known = false;
+      HaArrayForEach(timer_set->timers, timer, i)
+	if( KheTimerReached(timer, &curr_time_known, &curr_time) )
+	{
+	  HaArrayAddLast(timer_set->curr_query_counts, 0);
+	  timer = HaArrayLast(timer_set->timers);  /* most specific */
+	  HaArrayAddLast(timer_set->curr_timer_tags,
+	    HnStringCopy(timer->tag, timer_set->arena));
+	  return true;
+	}
+      return false;
+
+    case KHE_CONSISTENCY_ACTIVE:
+
+      /* bypass timers and use old query counts to determine the result; */
+      /* but don't save timer tags, there may be no timers at all */
+      curr_index = HaArrayCount(timer_set->curr_query_counts) - 1;
+      curr_count = ++HaArray(timer_set->curr_query_counts, curr_index);
+      HnAssert(curr_index < HaArrayCount(timer_set->old_query_counts),
+	"KheTimerSetTimeLimitReached: inconsistency detected "
+	"(did the options change from the previous run?)");
+      if( curr_count >= HaArray(timer_set->old_query_counts, curr_index) )
+      {
+	HaArrayAddLast(timer_set->curr_query_counts, 0);
+	return true;
+      }
+      return false;
+
+    default:
+
+      HnAbort("KheTimerSetTimeLimitReached internal error");
+      return false;  /* keep compiler happy */
+  }
+#else
   return false;
+#endif
 }
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  KHE_OPTIONS_TIME_LIMIT KheOptionsTimeMake(char *tag, float time_limit,   */
-/*    HA_ARENA a)                                                            */
+/*  void KheTimerSetTimeLimitConsistencyBegin(KHE_TIMER_SET timer_set,       */
+/*    KHE_SOLN soln)                                                         */
 /*                                                                           */
-/*  Make an options time limit object with these attributes.                 */
+/*  Begin a time limit consistency run.                                      */
 /*                                                                           */
 /*****************************************************************************/
 
-/* ***
-static KHE_OPTIONS_TIME_LIMIT KheOptionsTimeMake(char *tag, float time_limit,
-  HA_ARENA a)
+void KheTimerSetTimeLimitConsistencyBegin(KHE_TIMER_SET timer_set,
+  KHE_SOLN soln)
 {
-  KHE_OPTIONS_TIME_LIMIT res;
-  HaMake(res, a);
-  res->tag = tag;
-  res->timer = KheStatsTimerMake(a);
-  res->time_limit = time_limit;
-  return res;
-}
-*** */
+#if KHE_USE_TIMING
+  char file_name[200], buff[200], header[200];  FILE *fp;  int line_num, val;
 
+  /* ensure not called twice */
+  HnAssert(timer_set->state == KHE_CONSISTENCY_NONE,
+    "KheTimerSetTimeLimitConsistencyBegin: called twice");
 
-/*****************************************************************************/
-/*                                                                           */
-/*  KHE_TIMER_LOG KheTimerSetGetTimerLog(KHE_TIMER_SET ts, char *tag)        */
-/*                                                                           */
-/*  Find or create a timer log with this tag.                                */
-/*                                                                           */
-/*****************************************************************************/
+  /* ensure not called too late */
+  HnAssert(HaArrayCount(timer_set->curr_query_counts) == 1 &&
+    HaArrayFirst(timer_set->curr_query_counts) == 0,
+    "KheTimerSetTimeLimitConsistencyBegin: called too late");
 
-/* ***
-static KHE_TIMER_LOG KheTimerSetGetTimerLog(KHE_TIMER_SET ts, char *tag)
-{
-  KHE_TIMER_LOG timer_log;  int i;
-  if( DEBUG1 )
+  /* construct the file name and open the file for reading */
+  snprintf(file_name, 200, "khe_%s_%d.tlc",
+    KheInstanceId(KheSolnInstance(soln)), KheSolnDiversifier(soln));
+  fp = fopen(file_name, "r");
+
+  /* read old queries (from previous run) if file opened */
+  if( fp != NULL )
   {
-    fprintf(stderr, "[ KheTimerSetGetTimerLog(timer_set, \"%s\"):\n", tag);
-    KheTimerSetDebug(ts, 2, 2, stderr);
+    /* read the header line of the file and make sure it's OK */
+    if( fgets(buff, 200, fp) == NULL || sscanf(buff, "%199s", header) != 1 ||
+	strcmp(header, file_name) != 0 )
+      HnAbort("KheTimerSetTimeLimitConsistencyBegin:  missing or erroneous "
+	"header line in file %s\n", file_name);
+
+    /* read the old query counts into timer_set->old_query_counts */
+    line_num = 1;
+    while( fgets(buff, 200, fp) != NULL )
+    {
+      line_num++;
+      if( sscanf(buff, "%d", &val) != 1 )
+	HnAbort("KheTimerSetTimeLimitConsistencyBegin:  error in file "
+	  "%s (line %d)\n", file_name, line_num);
+      HaArrayAddLast(timer_set->old_query_counts, val);
+    }
+    fclose(fp);
+
+    /* record that old queries were found */
+    timer_set->state = KHE_CONSISTENCY_ACTIVE;
   }
-  HaArrayForEach(ts->timer_logs, timer_log, i)
-    if( strcmp(timer_log->tag, tag) == 0 )
-      return timer_log;
-  timer_log = KheTimerLogMake(tag, HaArrayArena(ts->timers));
-  HaArrayAddLast(ts->timer_logs, timer_log);
-  if( DEBUG1 )
+  else
   {
-    KheTimerSetDebug(ts, 2, 2, stderr);
-    fprintf(stderr, "] KheTimerSetGetTimerLog returning\n");
+    /* no file, so just starting time limit consistency on this run */
+    timer_set->state = KHE_CONSISTENCY_STARTING;
   }
-  return timer_log;
+#endif
 }
-*** */
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheTimerSetSaveLog(KHE_TIMER_SET ts, KHE_SOLN soln)                 */
+/*  void KheTimerSetTimeLimitConsistencyEnd(KHE_TIMER_SET timer_set,         */
+/*    KHE_SOLN soln)                                                         */
 /*                                                                           */
-/*  Save a log of ts in a file whose name is based on soln's instance        */
-/*  and diversifier.                                                         */
+/*  End a time limit consistency run.  This writes out the current query     */
+/*  counts to the .tlc file.                                                 */
+/*                                                                           */
+/*  Implementation note.  When state == KHE_CONSISTENCY_ACTIVE, a            */
+/*  concluding 0 is written to the file that is not written when             */
+/*  state == KHE_CONSISTENCY_STARTING.  This is because a time limit         */
+/*  is reached at the end of the run in the first case which is not          */
+/*  reached in the second case.   This extra 0 is harmless.                  */
 /*                                                                           */
 /*****************************************************************************/
 
-/* ***
-void KheTimerSetSaveLog(KHE_TIMER_SET ts, KHE_SOLN soln)
+void KheTimerSetTimeLimitConsistencyEnd(KHE_TIMER_SET timer_set, KHE_SOLN soln)
 {
-  char buff[200];  FILE *fp;  KHE_TIMER_LOG timer_log;  int i;
-  snprintf(buff, 200, "log-%s-%d.tsl", KheInstanceId(KheSolnInstance(soln)),
-    KheSolnDiversifier(soln));
-  fp = fopen(buff, "w");
-  HnAssert(fp != NULL, "KheTimerSetSaveLog:  cannot write file \"%s\"", buff);
-  HaArrayForEach(ts->timer_logs, timer_log, i)
-    KheTimerLogSave(timer_log, fp);
+#if KHE_USE_TIMING
+  char file_name[200];  FILE *fp;  int val, i;
+
+  /* ensure matching consistency begin */
+  HnAssert(timer_set->state != KHE_CONSISTENCY_NONE,
+    "KheTimerSetTimeLimitConsistencyEnd: no matching "
+    "KheTimerSetTimeLimitConsistencyBegin");
+
+  /* construct the file name and open the file for writing */
+  snprintf(file_name, 200, "khe_%s_%d.tlc",
+    KheInstanceId(KheSolnInstance(soln)), KheSolnDiversifier(soln));
+  fp = fopen(file_name, "w");
+  if( fp == NULL )
+    HnAbort("KheTimerSetTimeLimitConsistencyEnd: cannot open file %s"
+      " for writing\n", file_name);
+
+  /* write the header line and query counts to the file */
+  fprintf(fp, "%s\n", file_name);
+  HaArrayForEach(timer_set->curr_query_counts, val, i)
+  {
+    fprintf(fp, "%d", val);
+    if( i < HaArrayCount(timer_set->curr_timer_tags) )
+      fprintf(fp, " %s", HaArray(timer_set->curr_timer_tags, i));
+    fprintf(fp, "\n");
+  }
   fclose(fp);
+#endif
 }
-*** */
 
 
 /*****************************************************************************/
@@ -720,7 +867,7 @@ void KheTimerSetSaveLog(KHE_TIMER_SET ts, KHE_SOLN soln)
 void KheTimerSetDebug(KHE_TIMER_SET timer_set, int verbosity,
   int indent, FILE *fp)
 {
-  int i;  KHE_TIMER timer;  /* KHE_TIMER_LOG timer_log; */
+  int i;  KHE_TIMER timer;
   if( indent >= 0 )
     fprintf(fp, "%*s", indent, "");
   if( verbosity >= 2 )
@@ -736,11 +883,6 @@ void KheTimerSetDebug(KHE_TIMER_SET timer_set, int verbosity,
       KheTimerDebug(timer, verbosity, -1, fp);
     }
   }
-  /* ***
-  if( verbosity >= 2 )
-    HaArrayForEach(timer_set->timer_logs, timer_log, i)
-      KheTimerLogSave(timer_log, fp);
-  *** */
   if( indent >= 0 )
     fprintf(fp, "\n");
 }
