@@ -33,8 +33,7 @@
 #define max(a, b) ((a) > (b) ? (a) : (b))
 #define min(a, b) ((a) < (b) ? (a) : (b))
 
-#define DEBUG1	0
-
+#define	DEBUG1	0
 
 /*****************************************************************************/
 /*                                                                           */
@@ -59,9 +58,10 @@ typedef HA_ARRAY(KHE_CONSEC_LIMITS) ARRAY_KHE_CONSEC_LIMITS;
 
 typedef struct khe_consec_resource_rec {
   KHE_RESOURCE					resource;	/* resource */
-  KHE_CONSEC_LIMITS				free_days_limits;
-  KHE_CONSEC_LIMITS				busy_days_limits;
-  ARRAY_KHE_CONSEC_LIMITS			busy_times_limits;
+  KHE_CONSEC_LIMITS				free_days;
+  KHE_CONSEC_LIMITS				busy_days;
+  ARRAY_KHE_CONSEC_LIMITS			busy_times;
+  int						non_rigidity;
 } *KHE_CONSEC_RESOURCE;
 
 typedef HA_ARRAY(KHE_CONSEC_RESOURCE) ARRAY_KHE_CONSEC_RESOURCE;
@@ -137,12 +137,7 @@ static void KheConsecLimitsDebug(KHE_CONSEC_LIMITS cl, int indent, FILE *fp)
 {
   if( indent > 0 )
     fprintf(fp, "%*s", indent, "");
-  /* ***
-  if( cl.history == 0 )
-    fprintf(fp, "%d-%d", cl.min_limit, cl.max_limit);
-  else
-  *** */
-    fprintf(fp, "%d|%d-%d", cl.history, cl.min_limit, cl.max_limit);
+  fprintf(fp, "%d|%d-%d", cl.history, cl.min_limit, cl.max_limit);
   if( indent >= 0 )
     fprintf(fp, "\n");
 }
@@ -150,7 +145,7 @@ static void KheConsecLimitsDebug(KHE_CONSEC_LIMITS cl, int indent, FILE *fp)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  Submodule "KHE_CONSEC_RESOURCE objects (private)"                        */
+/*  Submodule "KHE_CONSEC_RESOURCE"                                          */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -164,17 +159,17 @@ static void KheConsecLimitsDebug(KHE_CONSEC_LIMITS cl, int indent, FILE *fp)
 /*****************************************************************************/
 
 static KHE_CONSEC_RESOURCE KheConsecResourceMake(KHE_RESOURCE r,
-  int max_offset, HA_ARENA a)
+  int max_offset, int frame_len, HA_ARENA a)
 {
   KHE_CONSEC_RESOURCE res;  int i;  KHE_CONSEC_LIMITS init_cl;
-  init_cl = KheConsecLimitsMake(0, 0, INT_MAX);
+  init_cl = KheConsecLimitsMake(0, 1, frame_len);
   HaMake(res, a);
   res->resource = r;
-  res->free_days_limits = init_cl;
-  res->busy_days_limits = init_cl;
-  HaArrayInit(res->busy_times_limits, a);
+  res->free_days = init_cl;
+  res->busy_days = init_cl;
+  HaArrayInit(res->busy_times, a);
   for( i = 0;  i <= max_offset;  i++ )
-    HaArrayAddLast(res->busy_times_limits, init_cl);
+    HaArrayAddLast(res->busy_times, init_cl);
   return res;
 }
 
@@ -193,17 +188,18 @@ static void KheConsecResourceDebug(KHE_CONSEC_RESOURCE cr, int indent, FILE *fp)
   if( indent > 0 )
     fprintf(fp, "%*s", indent, "");
   fprintf(fp, "[ %5s: free days ", KheResourceId(cr->resource));
-  KheConsecLimitsDebug(cr->free_days_limits, -1, fp);
+  KheConsecLimitsDebug(cr->free_days, -1, fp);
   fprintf(fp, "; busy days ");
-  KheConsecLimitsDebug(cr->busy_days_limits, -1, fp);
+  KheConsecLimitsDebug(cr->busy_days, -1, fp);
   fprintf(fp, "; busy times ");
-  HaArrayForEach(cr->busy_times_limits, cl, i)
+  HaArrayForEach(cr->busy_times, cl, i)
   {
     if( i > 0 )
       fprintf(fp, ", ");
     fprintf(fp, "%d:", i);
     KheConsecLimitsDebug(cl, -1, fp);
   }
+  fprintf(fp, "; non_rigidity %d", cr->non_rigidity);
   fprintf(fp, " ]");
   if( indent >= 0 )
     fprintf(fp, "\n");
@@ -329,7 +325,7 @@ static void KheAddFreeDaysLimits(KHE_CONSEC_SOLVER cs, KHE_RESOURCE r,
 {
   KHE_CONSEC_RESOURCE cr;
   cr = KheConsecSolverGetResource(cs, r);
-  cr->free_days_limits = KheConsecLimitsMerge(cr->free_days_limits,
+  cr->free_days = KheConsecLimitsMerge(cr->free_days,
     history, min_limit, max_limit);
 }
 
@@ -348,7 +344,7 @@ static void KheAddBusyDaysLimits(KHE_CONSEC_SOLVER cs, KHE_RESOURCE r,
 {
   KHE_CONSEC_RESOURCE cr;
   cr = KheConsecSolverGetResource(cs, r);
-  cr->busy_days_limits = KheConsecLimitsMerge(cr->busy_days_limits,
+  cr->busy_days = KheConsecLimitsMerge(cr->busy_days,
     history, min_limit, max_limit);
 }
 
@@ -367,11 +363,11 @@ static void KheAddBusyTimesLimits(KHE_CONSEC_SOLVER cs, KHE_RESOURCE r,
 {
   KHE_CONSEC_RESOURCE cr;  KHE_CONSEC_LIMITS cl;
   cr = KheConsecSolverGetResource(cs, r);
-  HnAssert(offset >= 0 && offset < HaArrayCount(cr->busy_times_limits),
+  HnAssert(offset >= 0 && offset < HaArrayCount(cr->busy_times),
     "KheAddBusyTimesLimits internal error");
-  cl = KheConsecLimitsMerge(HaArray(cr->busy_times_limits, offset),
+  cl = KheConsecLimitsMerge(HaArray(cr->busy_times, offset),
     history, min_limit, max_limit);
-  HaArrayPut(cr->busy_times_limits, offset, cl);
+  HaArrayPut(cr->busy_times, offset, cl);
 }
 
 
@@ -463,6 +459,202 @@ static void KheConsecSolverAddConstraint(KHE_CONSEC_SOLVER cs,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  Submodule "non_rigidity"                                                 */
+/*                                                                           */
+/*****************************************************************************/
+
+/*****************************************************************************/
+/*                                                                           */
+/*  int ceil_div(int a, int b)                                               */
+/*                                                                           */
+/*  Return ceiling(a/b).                                                     */
+/*                                                                           */
+/*****************************************************************************/
+
+int ceil_div(int a, int b)
+{
+  return (a + b - 1) / b;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  bool KheFindBusyRuns(int frame_len, int total_busy, int busy_run_len,    */
+/*    int free_run_len, int *busy_runs)                                      */
+/*                                                                           */
+/*  If the parameters lead to a valid timetable, return true and set         */
+/*  *busy_runs to the number of busy runs in that timetable.  If not,        */
+/*  return false with *busy_runs undefined.  The parameters are:             */
+/*                                                                           */
+/*  frame_len                                                                */
+/*    The number of days in the cycle.                                       */
+/*                                                                           */
+/*  total_busy                                                               */
+/*    The total number of times that we want the resource to be busy.        */
+/*                                                                           */
+/*  busy_run_len                                                             */
+/*    The length of one busy run.  One busy run may be smaller.              */
+/*                                                                           */
+/*  free_run_len                                                             */
+/*    The length of one free run.                                            */
+/*                                                                           */
+/*****************************************************************************/
+
+static bool KheFindBusyRuns(int frame_len, int total_busy, int busy_run_len,
+  int free_run_len, int *busy_runs)
+{
+  int min_total_free, max_total_free;
+
+  /* find the number of busy runs */
+  *busy_runs = ceil_div(total_busy, busy_run_len);
+
+  /* min free times when all free runs lie between busy runs */
+  min_total_free = free_run_len * (*busy_runs - 1);
+
+  /* max free times when free runs lie before, between and after busy runs */
+  max_total_free = free_run_len * (*busy_runs + 1);
+
+  /* success if free times plus total_busy can add up to frame_len */
+  return min_total_free + total_busy <= frame_len &&
+    max_total_free + total_busy >= frame_len;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheConsecSolverNonRigidity1(KHE_CONSEC_RESOURCE cr,                  */
+/*    int frame_len, KHE_SOLN soln)                                          */
+/*                                                                           */
+/*  Return the value of the first measure of the non-rigidity of cr.  If     */
+/*  there are any problems, return 0.                                        */
+/*                                                                           */
+/*****************************************************************************/
+
+static int KheConsecSolverNonRigidity1(KHE_CONSEC_RESOURCE cr,
+  int frame_len, KHE_SOLN soln)
+{
+  int x, y, min_busy_runs, max_busy_runs, busy_runs, total_busy, res;
+  if( DEBUG1 )
+    fprintf(stderr, "[ KheConsecSolverSetRigidity(%s, %d, soln)\n",
+      KheResourceId(cr->resource), frame_len);
+  res = 0;
+  if( KheResourceMaxBusyTimes(soln, cr->resource, &total_busy) )
+  {
+    min_busy_runs = INT_MAX;  max_busy_runs = 0;
+    for( x = cr->busy_days.min_limit;  x <= cr->busy_days.max_limit;  x++ )
+      for( y = cr->free_days.min_limit;  y <= cr->free_days.max_limit;  y++ )
+      {
+	if( KheFindBusyRuns(frame_len, total_busy, x, y, &busy_runs) )
+	{
+	  if( busy_runs < min_busy_runs )
+	    min_busy_runs = busy_runs;
+	  if( busy_runs > max_busy_runs )
+	    max_busy_runs = busy_runs;
+	}
+      }
+    if( min_busy_runs < INT_MAX )
+    {
+      res = max_busy_runs - min_busy_runs;
+      if( DEBUG1 )
+	fprintf(stderr, "  max_busy_runs %d, min_busy_runs %d\n",
+          max_busy_runs, min_busy_runs);
+    }
+  }
+  if( DEBUG1 )
+    fprintf(stderr, "] KheConsecSolverSetRigidity returning %d\n", res);
+  return res;
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheConsecSolverNonRigidity2(KHE_CONSEC_RESOURCE cr, KHE_SOLN soln)   */
+/*                                                                           */
+/*  Return the value of the second measure of the non-rigidity of cr.        */
+/*                                                                           */
+/*****************************************************************************/
+
+static int KheConsecSolverNonRigidity2(KHE_CONSEC_RESOURCE cr, KHE_SOLN soln)
+{
+  int b_min, b_max, f_min, f_max, b, f;
+
+  /* get b_min, b_max, and b, and make sure they have consistent values */
+  b_min = cr->busy_days.min_limit;
+  b_max = cr->busy_days.max_limit;
+  b     = cr->busy_days.history;
+  if( b_min < 1 ) b_min = 1;
+  if( b_max < b_min ) b_max = b_min;
+  if( b < 0 ) b = 0;
+
+  /* get f_min, f_max, and f, and make sure they have consistent values */
+  f_min = cr->free_days.min_limit;
+  f_max = cr->free_days.max_limit;
+  f     = cr->free_days.history;
+  if( f_min < 1 ) f_min = 1;
+  if( f_max < f_min ) f_max = f_min;
+  if( f < 0 ) f = 0;
+
+  /* apply the formula from the Guide */
+  if( b == 0 )
+  {
+    if( f == 0 )
+    {
+      /* b == 0 and f == 0 */
+      return f_max - f_min + 2;
+    }
+    else
+    {
+      /* b == 0 and f > 0 */
+      if( f < f_min )
+	return f_max - f_min + 1;
+      else if( f < f_max )
+	return 1 + f_max - f;
+      else
+	return 1;
+    }
+  }
+  else
+  {
+    if( f == 0 )
+    {
+      /* b > 0 and f == 0 */
+      if( b < b_min )
+	return 1;
+      else if( b < b_max )
+	return f_max - f_min + 2;
+      else
+	return f_max - f_min + 1;
+    }
+    else
+    {
+      /* b > 0 and f > 0 */
+      /* should never happen, but we are at the mercy of the history */
+      /* values stored in the instance, and we don't want to crash */
+      return 0;
+    }
+  }
+}
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheConsecSolverSetNonRigidity(KHE_CONSEC_RESOURCE cr,               */
+/*    int frame_len, KHE_SOLN soln)                                          */
+/*                                                                           */
+/*  Set cr->non_rigidity following the formula given in the Guide.           */
+/*                                                                           */
+/*****************************************************************************/
+
+static void KheConsecSolverSetNonRigidity(KHE_CONSEC_RESOURCE cr,
+  int frame_len, KHE_SOLN soln)
+{
+  cr->non_rigidity = 10 * KheConsecSolverNonRigidity1(cr, frame_len, soln) +
+    1 * KheConsecSolverNonRigidity2(cr, soln);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "main functions"                                               */
@@ -480,15 +672,16 @@ static void KheConsecSolverAddConstraint(KHE_CONSEC_SOLVER cs,
 
 KHE_CONSEC_SOLVER KheConsecSolverMake(KHE_SOLN soln, KHE_FRAME frame)
 {
-  HA_ARENA a;  KHE_CONSEC_SOLVER res;  int i, max_offset;
+  HA_ARENA a;  KHE_CONSEC_SOLVER res;  int i, max_offset, frame_len;
   KHE_RESOURCE r;  KHE_TIME_GROUP tg;  KHE_INSTANCE ins;  KHE_CONSTRAINT c;
+  KHE_CONSEC_RESOURCE cr;
 
   /* frame must be non-empty */
   HnAssert(KheFrameTimeGroupCount(frame) > 0,
     "KheConsecSolverMake: frame has no time groups");
 
   /* make the initial object */
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   HaMake(res, a);
   res->arena = a;
   res->soln = soln;
@@ -507,12 +700,13 @@ KHE_CONSEC_SOLVER KheConsecSolverMake(KHE_SOLN soln, KHE_FRAME frame)
   }
 
   /* add one KHE_CONSEC_RESOURCE for each resource of the instance */
+  frame_len = KheFrameTimeGroupCount(frame);
   ins = KheSolnInstance(soln);
   for( i = 0;  i < KheInstanceResourceCount(ins);  i++ )
   {
     r = KheInstanceResource(ins, i);
     HaArrayAddLast(res->resources,
-      KheConsecResourceMake(r, res->max_offset, a));
+      KheConsecResourceMake(r, res->max_offset, frame_len, a));
   }
 
   /* find all relevant constraints and add them in */
@@ -525,6 +719,10 @@ KHE_CONSEC_SOLVER KheConsecSolverMake(KHE_SOLN soln, KHE_FRAME frame)
 	(KHE_LIMIT_ACTIVE_INTERVALS_CONSTRAINT) c);
   }
 
+  /* set the rigidities */
+  HaArrayForEach(res->resources, cr, i)
+    KheConsecSolverSetNonRigidity(cr, frame_len, soln);
+
   /* all done */
   return res;
 }
@@ -586,7 +784,7 @@ void KheConsecSolverFreeDayLimits(KHE_CONSEC_SOLVER cs, KHE_RESOURCE r,
 {
   KHE_CONSEC_RESOURCE cr;  KHE_CONSEC_LIMITS cl;
   cr = KheConsecSolverGetResource(cs, r);
-  cl = cr->free_days_limits;
+  cl = cr->free_days;
   *history = cl.history;
   *min_limit = cl.min_limit;
   *max_limit = cl.max_limit;
@@ -607,7 +805,7 @@ void KheConsecSolverBusyDaysLimits(KHE_CONSEC_SOLVER cs, KHE_RESOURCE r,
 {
   KHE_CONSEC_RESOURCE cr;  KHE_CONSEC_LIMITS cl;
   cr = KheConsecSolverGetResource(cs, r);
-  cl = cr->busy_days_limits;
+  cl = cr->busy_days;
   *history = cl.history;
   *min_limit = cl.min_limit;
   *max_limit = cl.max_limit;
@@ -628,10 +826,10 @@ void KheConsecSolverBusyTimesLimits(KHE_CONSEC_SOLVER cs, KHE_RESOURCE r,
 {
   KHE_CONSEC_RESOURCE cr;  KHE_CONSEC_LIMITS cl;
   cr = KheConsecSolverGetResource(cs, r);
-  HnAssert(offset >= 0 && offset < HaArrayCount(cr->busy_times_limits),
+  HnAssert(offset >= 0 && offset < HaArrayCount(cr->busy_times),
     "KheConsecSolverBusyTimeLimits: offset (%d) out of range (0 .. %d)",
-    offset, HaArrayCount(cr->busy_times_limits) - 1);
-  cl = HaArray(cr->busy_times_limits, offset);
+    offset, HaArrayCount(cr->busy_times) - 1);
+  cl = HaArray(cr->busy_times, offset);
   *history = cl.history;
   *min_limit = cl.min_limit;
   *max_limit = cl.max_limit;
@@ -652,6 +850,22 @@ int KheConsecSolverMaxOffset(KHE_CONSEC_SOLVER cs)
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  int KheConsecSolverNonRigidity(KHE_CONSEC_SOLVER cs, KHE_RESOURCE r)     */
+/*                                                                           */
+/*  Return the non_rigidity of r.                                            */
+/*                                                                           */
+/*****************************************************************************/
+
+int KheConsecSolverNonRigidity(KHE_CONSEC_SOLVER cs, KHE_RESOURCE r)
+{
+  KHE_CONSEC_RESOURCE cr;
+  cr = KheConsecSolverGetResource(cs, r);
+  return cr->non_rigidity;
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheConsecSolverDebug(KHE_CONSEC_SOLVER cs, int verbosity,           */
