@@ -35,52 +35,52 @@
 
 /*****************************************************************************/
 /*                                                                           */
-/*  KHE_TASK_GROUP_NODE - a node representing one task group                 */
-/*  KHE_RESOURCE_NODE - a node representing one resource.                    */
+/*  KHE_MTASK_NODE - a node representing one mtask                           */
+/*  KHE_RESOURCE_NODE - a node representing one resource                     */
 /*                                                                           */
 /*****************************************************************************/
 
-typedef struct khe_task_group_node_rec *KHE_TASK_GROUP_NODE;
-typedef HA_ARRAY(KHE_TASK_GROUP_NODE) ARRAY_KHE_TASK_GROUP_NODE;
+typedef struct khe_mtask_node_rec *KHE_MTASK_NODE;
+typedef HA_ARRAY(KHE_MTASK_NODE) ARRAY_KHE_MTASK_NODE;
 
 typedef struct khe_resource_node_rec *KHE_RESOURCE_NODE;
 typedef HA_ARRAY(KHE_RESOURCE_NODE) ARRAY_KHE_RESOURCE_NODE;
 
-struct khe_task_group_node_rec {
+struct khe_mtask_node_rec {
   int				priqueue_index;		/* for priqueue      */
-  KHE_TASK_GROUP		task_group;		/* tasks to do       */
-  int				unassigned_tasks;	/* no unassigned     */
+  KHE_MTASK			mtask;			/* tasks to do       */
+  int				unassigned_tasks;	/* no. unassigned    */
   ARRAY_KHE_RESOURCE_NODE	resource_nodes;		/* avail resources   */
 };
 
 struct khe_resource_node_rec {
   KHE_RESOURCE			resource;		/* the resource      */
-  ARRAY_KHE_TASK_GROUP_NODE	task_group_nodes;	/* suitable tasks    */
+  ARRAY_KHE_MTASK_NODE		mtask_nodes;		/* suitable mtasks   */
 };
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  Submodule "task group nodes" (private)                                   */
+/*  Submodule "mtask nodes" (private)                                        */
 /*                                                                           */
 /*****************************************************************************/
 
 /*****************************************************************************/
 /*                                                                           */
-/*  KHE_TASK_GROUP_NODE KheTaskGroupNodeMake(KHE_TASK_GROUP task_group,      */
-/*    int unassigned_tasks)                                                  */
+/*  KHE_MTASK_NODE KheMTaskNodeMake(KHE_MTASK mtask,                         */
+/*    int unassigned_tasks, HA_ARENA a)                                      */
 /*                                                                           */
 /*  Create a new task set node with these attributes.                        */
 /*                                                                           */
 /*****************************************************************************/
 
-static KHE_TASK_GROUP_NODE KheTaskGroupNodeMake(KHE_TASK_GROUP task_group,
+static KHE_MTASK_NODE KheMTaskNodeMake(KHE_MTASK mtask,
   int unassigned_tasks, HA_ARENA a)
 {
-  KHE_TASK_GROUP_NODE res;
+  KHE_MTASK_NODE res;
   HaMake(res, a);
-  res->priqueue_index = 0;  /* actually undefined */
-  res->task_group = task_group;
+  res->priqueue_index = -1;  /* undefined here */
+  res->mtask = mtask;
   res->unassigned_tasks = unassigned_tasks;
   HaArrayInit(res->resource_nodes, a);
   return res;
@@ -89,87 +89,87 @@ static KHE_TASK_GROUP_NODE KheTaskGroupNodeMake(KHE_TASK_GROUP task_group,
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheTaskGroupNodeDelete(KHE_TASK_GROUP_NODE tgn)                     */
+/*  void KheMTaskNodeDelete(KHE_MTASK_NODE mtn)                              */
 /*                                                                           */
-/*  Delete tgn.                                                              */
+/*  Delete mtn.                                                              */
 /*                                                                           */
 /*****************************************************************************/
 
 /* ***
-static void KheTaskGroupNodeDelete(KHE_TASK_GROUP_NODE tgn)
+static void KheMTaskNodeDelete(KHE_MTASK_NODE mtn)
 {
-  MArrayFree(tgn->resource_nodes);
-  MFree(tgn);
+  MArrayFree(mtn->resource_nodes);
+  MFree(mtn);
 }
 *** */
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  int64_t KheTaskGroupNodeKey(void *tgn)                                   */
+/*  int64_t KheMTaskNodeKey(void *mtn)                                       */
 /*                                                                           */
-/*  PriQueue callback function which returns the priority of tgn.            */
+/*  PriQueue callback function which returns the priority of mtn.            */
 /*                                                                           */
 /*****************************************************************************/
 
-static int64_t KheTaskGroupNodeKey(void *tgn)
+static int64_t KheMTaskNodeKey(void *mtn)
 {
-  return HaArrayCount(((KHE_TASK_GROUP_NODE) tgn)->resource_nodes) -
-    ((KHE_TASK_GROUP_NODE) tgn)->unassigned_tasks;
+  KHE_MTASK_NODE mt_node = (KHE_MTASK_NODE) mtn;
+  return HaArrayCount(mt_node->resource_nodes) - mt_node->unassigned_tasks;
 }
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  int KheTaskGroupNodeIndex(void *tgn)                                     */
+/*  int KheMTaskNodeIndex(void *mtn)                                         */
 /*                                                                           */
-/*  PriQueue callback function which returns the index of tgn.               */
+/*  PriQueue callback function which returns the index of mtn.               */
 /*                                                                           */
 /*****************************************************************************/
 
-static int KheTaskGroupNodeIndex(void *tgn)
+static int KheMTaskNodeIndex(void *mtn)
 {
-  return ((KHE_TASK_GROUP_NODE) tgn)->priqueue_index;
+  return ((KHE_MTASK_NODE) mtn)->priqueue_index;
 }
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheTaskGroupNodeSetIndex(void *tgn, int index)                      */
+/*  void KheMTaskNodeSetIndex(void *mtn, int index)                          */
 /*                                                                           */
-/*  PriQueue callback function which sets the index of tgn.                  */
+/*  PriQueue callback function which sets the index of mtn.                  */
 /*                                                                           */
 /*****************************************************************************/
 
-static void KheTaskGroupNodeSetIndex(void *tgn, int index)
+static void KheMTaskNodeSetIndex(void *mtn, int index)
 {
-  ((KHE_TASK_GROUP_NODE) tgn)->priqueue_index = index;
+  ((KHE_MTASK_NODE) mtn)->priqueue_index = index;
 }
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheTaskGroupNodeDebug(KHE_TASK_GROUP_NODE tgn, int verbosity,       */
+/*  void KheMTaskNodeDebug(KHE_MTASK_NODE mtn, int verbosity,                */
 /*    int indent, FILE *fp)                                                  */
 /*                                                                           */
-/*  Debug print of tgn onto fp with the given verbosity and indent.          */
+/*  Debug print of mtn onto fp with the given verbosity and indent.          */
 /*                                                                           */
 /*****************************************************************************/
 
-static void KheTaskGroupNodeDebug(KHE_TASK_GROUP_NODE tgn, int verbosity,
+static void KheMTaskNodeDebug(KHE_MTASK_NODE mtn, int verbosity,
   int indent, FILE *fp)
 {
   if( verbosity >= 1 )
   {
     if( indent >= 0 )
       fprintf(fp, "%*s", indent, "");
-    fprintf(fp, "[ TaskGroupNode %ld (%d resources, %d unassigned tasks)",
-      KheTaskGroupNodeKey(tgn), HaArrayCount(tgn->resource_nodes),
-      tgn->unassigned_tasks);
+    fprintf(fp, "[ MTaskNode %ld (%d resources, %d unassigned tasks)",
+      KheMTaskNodeKey(mtn), HaArrayCount(mtn->resource_nodes),
+      mtn->unassigned_tasks);
     if( indent >= 0 )
     {
       fprintf(fp, ":\n");
-      KheTaskGroupDebug(tgn->task_group, 2, 4, stderr);
+      KheMTaskDebug(mtn->mtask, 2, 4, stderr);
       fprintf(fp, "%*s]\n", indent, "");
     }
     else
@@ -197,7 +197,7 @@ static KHE_RESOURCE_NODE KheResourceNodeMake(KHE_RESOURCE r, HA_ARENA a)
   KHE_RESOURCE_NODE res;
   HaMake(res, a);
   res->resource = r;
-  HaArrayInit(res->task_group_nodes, a);
+  HaArrayInit(res->mtask_nodes, a);
   return res;
 }
 
@@ -221,64 +221,64 @@ static void KheResourceNodeDelete(KHE_RESOURCE_NODE rn)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheAddEdge(KHE_TASK_GROUP_NODE tgn, KHE_RESOURCE_NODE rn)           */
+/*  void KheAddEdge(KHE_MTASK_NODE mtn, KHE_RESOURCE_NODE rn)                */
 /*                                                                           */
-/*  Add an edge from tgn to rn.                                              */
+/*  Add an edge from mtn to rn.                                              */
 /*                                                                           */
 /*****************************************************************************/
 
-static void KheAddEdge(KHE_TASK_GROUP_NODE tgn, KHE_RESOURCE_NODE rn)
+static void KheAddEdge(KHE_MTASK_NODE mtn, KHE_RESOURCE_NODE rn)
 {
-  HaArrayAddLast(tgn->resource_nodes, rn);
-  HaArrayAddLast(rn->task_group_nodes, tgn);
+  HaArrayAddLast(mtn->resource_nodes, rn);
+  HaArrayAddLast(rn->mtask_nodes, mtn);
 }
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheDeleteEdge(KHE_TASK_GROUP_NODE tgn, KHE_RESOURCE_NODE rn)        */
+/*  void KheDeleteEdge(KHE_MTASK_NODE mtn, KHE_RESOURCE_NODE rn)             */
 /*                                                                           */
-/*  Delete the edge from tgn to rn.                                          */
+/*  Delete the edge from mtn to rn.                                          */
 /*                                                                           */
 /*****************************************************************************/
 
-static void KheDeleteEdge(KHE_TASK_GROUP_NODE tgn, KHE_RESOURCE_NODE rn)
+static void KheDeleteEdge(KHE_MTASK_NODE mtn, KHE_RESOURCE_NODE rn)
 {
   int pos;
-  if( !HaArrayContains(tgn->resource_nodes, rn, &pos) )
+  if( !HaArrayContains(mtn->resource_nodes, rn, &pos) )
     HnAbort("KheDeleteEdge internal error 1");
-  HaArrayDeleteAndShift(tgn->resource_nodes, pos);
-  if( !HaArrayContains(rn->task_group_nodes, tgn, &pos) )
+  HaArrayDeleteAndShift(mtn->resource_nodes, pos);
+  if( !HaArrayContains(rn->mtask_nodes, mtn, &pos) )
     HnAbort("KheDeleteEdge internal error 2");
-  HaArrayDeleteAndShift(rn->task_group_nodes, pos);
+  HaArrayDeleteAndShift(rn->mtask_nodes, pos);
 }
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  bool KheEdgeAssignable(KHE_TASK_GROUP_NODE tgn, KHE_RESOURCE_NODE rn,    */
+/*  bool KheEdgeAssignable(KHE_MTASK_NODE mtn, KHE_RESOURCE_NODE rn,         */
 /*    KHE_SOLN soln, KHE_COST *cost)                                         */
 /*                                                                           */
-/*  If rn can be assigned to tgn without increasing the solution cost or     */
+/*  If rn can be assigned to mtn without increasing the solution cost or     */
 /*  the number of unassignable tixels in soln's matching, return true with   */
 /*  *cost set to the solution cost after the assignment (but don't leave     */
 /*  the assignment there).  Otherwise return false.                          */
 /*                                                                           */
 /*****************************************************************************/
 
-static bool KheEdgeAssignable(KHE_TASK_GROUP_NODE tgn, KHE_RESOURCE_NODE rn,
+static bool KheEdgeAssignable(KHE_MTASK_NODE mtn, KHE_RESOURCE_NODE rn,
   KHE_SOLN soln, KHE_COST *cost)
 {
   int unmatched_before, unmatched_after;  bool res;  KHE_COST cost_before;
   KheSolnMatchingMarkBegin(soln);
   cost_before = KheSolnCost(soln);
   unmatched_before = KheSolnMatchingDefectCount(soln);
-  if( KheTaskGroupAssign(tgn->task_group, rn->resource) )
+  if( KheMTaskMoveResource(mtn->mtask, NULL, rn->resource, true) )
   {
     *cost = KheSolnCost(soln);
     unmatched_after = KheSolnMatchingDefectCount(soln);
-    KheTaskGroupUnAssign(tgn->task_group, rn->resource);
-    res = unmatched_after <= unmatched_before && *cost < cost_before;
+    KheMTaskMoveResource(mtn->mtask, rn->resource, NULL, true);
+    res = (unmatched_after <= unmatched_before && *cost < cost_before);
   }
   else
     res = false;
@@ -296,11 +296,12 @@ static bool KheEdgeAssignable(KHE_TASK_GROUP_NODE tgn, KHE_RESOURCE_NODE rn,
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** no longer used
 static bool KheTaskWantsResource(KHE_TASK task, KHE_RESOURCE r)
 {
   int i;  KHE_EVENT_RESOURCE er;  KHE_SOLN soln;
 
-  /* see whether task itself wants r */
+  ** see whether task itself wants r **
   soln = KheTaskSoln(task);
   er = KheTaskEventResource(task);
   if( er != NULL )
@@ -308,17 +309,18 @@ static bool KheTaskWantsResource(KHE_TASK task, KHE_RESOURCE r)
       if( KheTaskAsstResource(KheEventResourceTask(soln, er, i)) == r )
 	return true;
 
-  /* see whether any tasks assigned to task want r */
+  ** see whether any tasks assigned to task want r **
   for( i = 0;  i < KheTaskAssignedToCount(task);  i++ )
     if( KheTaskWantsResource(KheTaskAssignedTo(task, i), r) )
       return true;
   return false;
 }
+*** */
 
 
 /*****************************************************************************/
 /*                                                                           */
-/*  Submodule "most constraint first resource assignment"                    */
+/*  Submodule "most constrained first resource assignment"                   */
 /*                                                                           */
 /*****************************************************************************/
 
@@ -331,48 +333,54 @@ static bool KheTaskWantsResource(KHE_TASK task, KHE_RESOURCE r)
 /*                                                                           */
 /*****************************************************************************/
 
-bool KheMostConstrainedFirstAssignResources(KHE_TASKING tasking,
-  KHE_OPTIONS options)
+bool KheMostConstrainedFirstAssignResources(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, KHE_OPTIONS options)
 {
-  ARRAY_KHE_TASK_GROUP_NODE task_group_nodes;  KHE_TASK_GROUP_NODE tgn, tgn2;
+  ARRAY_KHE_MTASK_NODE mtask_nodes;  KHE_MTASK_NODE mtn, mtn2;
   ARRAY_KHE_RESOURCE_NODE resource_nodes, best_resource_nodes;
-  KHE_RESOURCE_NODE rn;  KHE_PRIQUEUE task_queue;  KHE_SOLN soln;
+  KHE_RESOURCE_NODE rn;  KHE_PRIQUEUE mtask_queue;
   KHE_INSTANCE ins;  int i, j, index, unassigned_tasks;
-  KHE_COST best_cost, cost;  HA_ARENA a;
-  KHE_TASK_GROUP task_group;  KHE_RESOURCE r;  KHE_RESOURCE_GROUP rg;
-  KHE_TASK task;  KHE_TASK_GROUPS task_groups;
+  KHE_COST best_cost, cost;  HA_ARENA a;  KHE_FRAME days_frame;
+  KHE_MTASK_FINDER mtf;  KHE_MTASK mtask;
+  KHE_RESOURCE r;  KHE_RESOURCE_GROUP rg;
+  KHE_TASK task;  /* KHE_TASK_GROUPS task_groups; */
+  /* KHE_COST asst_cost, non_asst_cost; */
+  /* KHE_EVENT_TIMETABLE_MONITOR etm; */
   if( DEBUG1 )
-  {
-    fprintf(stderr, "[ KheMostConstrainedFirstAssignResources(tasking):\n");
-    KheTaskingDebug(tasking, 2, 2, stderr);
-  }
+    fprintf(stderr, "[ KheMostConstrainedFirstAssignResources(soln, %s)\n",
+      KheResourceTypeId(rt));
 
-  /* make task groups */
-  soln = KheTaskingSoln(tasking);
-  a = KheSolnArenaBegin(soln, false);
+  /* make mtasks */
+  a = KheSolnArenaBegin(soln);
   /* a = HaArena Make(); */
+  days_frame = KheOptionsFrame(options, "gs_common_frame", soln);
+  /* ***
+  e tm = (KHE_EVENT_TIMETABLE_MONITOR)
+    KheOptionsGetObject(options, "gs_event_timetable_monitor", NULL);
+  *** */
   ins = KheSolnInstance(soln);
-  task_groups = KheTaskGroupsMakeFromTasking(tasking);
+  mtf = KheMTaskFinderMake(soln, rt, days_frame, /* etm, */ true, a);
+  /* task_groups = KheTaskGroupsMakeFromTasking(tasking); */
 
-  /* build one task set node for each incompletely assigned task group */
-  HaArrayInit(task_group_nodes, a);
-  for( i = 0;  i < KheTaskGroupsTaskGroupCount(task_groups);  i++ )
+  /* build one mtask node for each incompletely assigned mtask */
+  HaArrayInit(mtask_nodes, a);
+  for( i = 0;  i < KheMTaskFinderMTaskCount(mtf);  i++ )
   {
-    task_group = KheTaskGroupsTaskGroup(task_groups, i);
-    unassigned_tasks = KheTaskGroupUnassignedTaskCount(task_group);
+    mtask = KheMTaskFinderMTask(mtf, i);
+    unassigned_tasks = KheMTaskUnassignedTaskCount(mtask);
     if( unassigned_tasks > 0 )
     {
-      tgn = KheTaskGroupNodeMake(task_group, unassigned_tasks, a);
-      HaArrayAddLast(task_group_nodes, tgn);
+      mtn = KheMTaskNodeMake(mtask, unassigned_tasks, a);
+      HaArrayAddLast(mtask_nodes, mtn);
     }
   }
 
   /* build one resource node for each resource in any domain, plus edges */
   HaArrayInit(resource_nodes, a);
   HaArrayFill(resource_nodes, KheInstanceResourceCount(ins), NULL);
-  HaArrayForEach(task_group_nodes, tgn, i)
+  HaArrayForEach(mtask_nodes, mtn, i)
   {
-    rg = KheTaskGroupDomain(tgn->task_group);
+    rg = KheMTaskDomain(mtn->mtask);
     for( j = 0;  j < KheResourceGroupResourceCount(rg);  j++ )
     {
       r = KheResourceGroupResource(rg, j);
@@ -380,34 +388,34 @@ bool KheMostConstrainedFirstAssignResources(KHE_TASKING tasking,
       if( HaArray(resource_nodes, index) == NULL )
 	HaArrayPut(resource_nodes, index, KheResourceNodeMake(r, a));
       rn = HaArray(resource_nodes, index);
-      if( KheEdgeAssignable(tgn, rn, soln, &cost) )
-	KheAddEdge(tgn, rn);
+      if( KheEdgeAssignable(mtn, rn, soln, &cost) )
+	KheAddEdge(mtn, rn);
     }
   }
 
   /* create and populate the priority queue */
-  task_queue = KhePriQueueMake(&KheTaskGroupNodeKey, &KheTaskGroupNodeIndex,
-    &KheTaskGroupNodeSetIndex, a);
-  HaArrayForEach(task_group_nodes, tgn, i)
-    KhePriQueueInsert(task_queue, tgn);
+  mtask_queue = KhePriQueueMake(&KheMTaskNodeKey, &KheMTaskNodeIndex,
+    &KheMTaskNodeSetIndex, a);
+  HaArrayForEach(mtask_nodes, mtn, i)
+    KhePriQueueInsert(mtask_queue, mtn);
 
   /* do the assigning */
   HaArrayInit(best_resource_nodes, a);
-  while( !KhePriQueueEmpty(task_queue) )
+  while( !KhePriQueueEmpty(mtask_queue) )
   {
     /* find the task set node that needs assigning next */
-    tgn = (KHE_TASK_GROUP_NODE) KhePriQueueDeleteMin(task_queue);
+    mtn = (KHE_MTASK_NODE) KhePriQueueDeleteMin(mtask_queue);
     if( DEBUG1 )
-      KheTaskGroupNodeDebug(tgn, 2, 2, stderr);
+      KheMTaskNodeDebug(mtn, 2, 2, stderr);
 
     /* find the best resources to assign to it */
     HaArrayClear(best_resource_nodes);
     best_cost = KheCostMax;
-    HaArrayForEach(tgn->resource_nodes, rn, i)
-      if( !KheEdgeAssignable(tgn, rn, soln, &cost) )
+    HaArrayForEach(mtn->resource_nodes, rn, i)
+      if( !KheEdgeAssignable(mtn, rn, soln, &cost) )
       {
 	/* rn has gone bad, remove it permanently */
-        KheDeleteEdge(tgn, rn);
+        KheDeleteEdge(mtn, rn);
 	i--;
       }
       else if( cost <= best_cost )
@@ -424,23 +432,48 @@ bool KheMostConstrainedFirstAssignResources(KHE_TASKING tasking,
     /* if there is a suitable resource, assign one (else do nothing) */
     if( HaArrayCount(best_resource_nodes) > 0 )
     {
-      /* find task, an unassigned task of tgn */
-      task = NULL;  /* keep compiler happy */
-      for( i = 0;  i < KheTaskGroupTaskCount(tgn->task_group);  i++ )
+      /* find one of the best_resource_nodes to assign to mtn */
+      if( KheMTaskResourceAssignSuggestion(mtn->mtask, &r) )
       {
-	task = KheTaskGroupTask(tgn->task_group, i);
+	HaArrayForEach(best_resource_nodes, rn, i)
+	  if( rn->resource == r )
+	    break;
+	if( i < HaArrayCount(best_resource_nodes) )
+	{
+	  /* we have a suggestion and it is one of the best resource nodes */
+	  rn = HaArray(best_resource_nodes, i);
+	}
+	else
+	{
+	  /* suggestion is not one of the best resource nodes */
+	  rn = HaArrayFirst(best_resource_nodes);
+	}
+      }
+      else
+      {
+	/* no suggestion, just use first resource node */
+	rn = HaArrayFirst(best_resource_nodes);
+      }
+
+      /* ***
+      ** find task, an unassigned task of mtn **
+      task = NULL;  ** keep compiler happy **
+      for( i = 0;  i < KheMTaskTaskCount(mtn->mtask);  i++ )
+      {
+	task = KheMTaskT ask(mtn->mtask, i, &asst_cost, &non_asst_cost);
 	if( KheTaskAsst(task) == NULL )
 	  break;
       }
-      HnAssert(i < KheTaskGroupTaskCount(tgn->task_group),
+      HnAssert(i < KheMTaskTaskCount(mtn->mtask),
         "KheMostConstrainedFirstAssignResources internal error 1");
 
-      /* find rn, the best of best_resource_nodes to assign to task */
+      ** find rn, the best of best_resource_nodes to assign to task **
       HaArrayForEach(best_resource_nodes, rn, i)
 	if( KheTaskWantsResource(task, rn->resource) )
 	  break;
       if( i >= HaArrayCount(best_resource_nodes) )
 	rn = HaArrayFirst(best_resource_nodes);
+      *** */
 
       /* assign rn's resource to task */
       if( DEBUG1 )
@@ -450,29 +483,29 @@ bool KheMostConstrainedFirstAssignResources(KHE_TASKING tasking,
 	fprintf(stderr, " to ");
 	KheTaskDebug(task, 1, 0, stderr);
       }
-      if( !KheTaskAssignResource(task, rn->resource) )
+      if( !KheMTaskMoveResource(mtn->mtask, NULL, rn->resource, true) )
 	HnAbort("KheMostConstrainedFirstAssignResources internal err 2");
-      KheDeleteEdge(tgn, rn);
+      KheDeleteEdge(mtn, rn);
 
-      /* if tgn still has unassigned tasks, reinsert, else delete all edges */
-      tgn->unassigned_tasks--;
-      if( tgn->unassigned_tasks > 0 )
-	KhePriQueueInsert(task_queue, tgn);
+      /* if mtn still has unassigned tasks, reinsert, else delete all edges */
+      mtn->unassigned_tasks--;
+      if( mtn->unassigned_tasks > 0 )
+	KhePriQueueInsert(mtask_queue, mtn);
       else
       {
-	while( HaArrayCount(tgn->resource_nodes) > 0 )
-          KheDeleteEdge(tgn, HaArrayFirst(tgn->resource_nodes));
+	while( HaArrayCount(mtn->resource_nodes) > 0 )
+          KheDeleteEdge(mtn, HaArrayFirst(mtn->resource_nodes));
       }
 
       /* update rn's availability over all its edges */
-      HaArrayForEach(rn->task_group_nodes, tgn2, i)
+      HaArrayForEach(rn->mtask_nodes, mtn2, i)
       {
-	HnAssert(tgn2 != tgn,
+	HnAssert(mtn2 != mtn,
 	  "KheMostConstrainedFirstAssignResources internal error 3");
-	if( !KheEdgeAssignable(tgn2, rn, soln, &cost) )
+	if( !KheEdgeAssignable(mtn2, rn, soln, &cost) )
 	{
-	  KheDeleteEdge(tgn2, rn);
-	  KhePriQueueNotifyKeyChange(task_queue, tgn2);
+	  KheDeleteEdge(mtn2, rn);
+	  KhePriQueueNotifyKeyChange(mtask_queue, mtn2);
 	  i--;
 	}
       }
@@ -485,8 +518,8 @@ bool KheMostConstrainedFirstAssignResources(KHE_TASKING tasking,
   /* HaArenaDel te(a); */
   /* ***
   MArrayFree(best_resource_nodes);
-  HaArrayForEach(task_group_nodes, tgn, i)
-    KheTaskGroupNodeDelete(tgn);
+  HaArrayForEach(task_group_nodes, mtn, i)
+    KheMTaskNodeDelete(mtn);
   MArrayFree(task_group_nodes);
   HaArrayForEach(resource_nodes, rn, i)
     if( rn != NULL )
