@@ -81,7 +81,9 @@ typedef HA_ARRAY(KHE_MULTI_TASK) ARRAY_KHE_MULTI_TASK;
 
 typedef struct khe_partition_solver_rec {
   HA_ARENA			arena;			/* arena             */
-  KHE_TASKING			tasking;		/* the tasking       */
+  KHE_SOLN			soln;			/* solution          */
+  KHE_RESOURCE_TYPE		resource_type;		/* resource type     */
+  /* KHE_TASKING		tasking; */		/* the tasking       */
   bool				resource_invariant;	/* flag from caller  */
   ARRAY_KHE_PARTITION		partitions;		/* the partitions    */
   ARRAY_KHE_MULTI_TASK		multi_tasks;		/* multi-partn tasks */
@@ -230,7 +232,7 @@ static void KhePartitionDelete(KHE_PARTITION p)
 /*****************************************************************************/
 
 static bool KhePartitionAcceptsTask(KHE_PARTITION p, KHE_TASK task,
-  KHE_TASK_BOUND_GROUP tbg, bool resource_invariant)
+  /* KHE_TASK_BOUND_GROUP tbg, */ KHE_SOLN_ADJUSTER sa, bool resource_invariant)
 {
   int init_count;  bool success;  KHE_MARK mark;  KHE_SOLN soln;
   KHE_TASK_BOUND tb;
@@ -240,12 +242,15 @@ static bool KhePartitionAcceptsTask(KHE_PARTITION p, KHE_TASK task,
   soln = KheTaskSoln(task);
   KheAtomicOperationBegin(soln, &mark, &init_count, resource_invariant);
   /* success = KheTaskBoundMake(tbg, task, p->resource_group, &tb); */
-  tb = KheTaskBoundMake(soln, p->resource_group);
+  tb = KheSolnAdjusterTaskBoundMake(sa, soln, p->resource_group);
+  /* tb = KheTaskBoundMake(soln, p->resource_group); */
   success = KheTaskAddTaskBound(task, tb);
   success = KheAtomicOperationEnd(soln, &mark, &init_count,
     resource_invariant, success);
+  /* *** already added to sa now
   if( success && tbg != NULL )
     KheTaskBoundGroupAddTaskBound(tbg, tb);
+  *** */
   if( DEBUG6 )
     fprintf(stderr, "] KhePartitionAcceptsTask returning %s\n",
       success ? "true" : "false");
@@ -459,19 +464,18 @@ static bool KhePartitionSolverRetrieve(KHE_PARTITION_SOLVER ps,
 /*                                                                           */
 /*****************************************************************************/
 
-static KHE_PARTITION_SOLVER KhePartitionSolverMake(KHE_TASKING tasking,
-  bool resource_invariant, HA_ARENA a)
+static KHE_PARTITION_SOLVER KhePartitionSolverMake(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, bool resource_invariant, HA_ARENA a)
 {
   int i;  KHE_RESOURCE_GROUP rg;  KHE_PARTITION_SOLVER res;
-  KHE_RESOURCE_TYPE rt;
 
   /* do the basic initialization */
-  rt = KheTaskingResourceType(tasking);
   HnAssert(rt != NULL, "KhePartitionSolverMake internal error");
-  /* a = HaAren aMake(); */
   HaMake(res, a);
   res->arena = a;
-  res->tasking = tasking;
+  res->soln = soln;
+  res->resource_type = rt;
+  /* res->tasking = tasking; */
   res->resource_invariant = resource_invariant;
   HaArrayInit(res->partitions, a);
   HaArrayInit(res->multi_tasks, a);
@@ -480,8 +484,7 @@ static KHE_PARTITION_SOLVER KhePartitionSolverMake(KHE_TASKING tasking,
   for( i = 0;  i < KheResourceTypePartitionCount(rt);  i++ )
   {
     rg = KheResourceTypePartition(rt, i);
-    HaArrayAddLast(res->partitions,
-      KhePartitionMake(rg, KheTaskingSoln(tasking), a));
+    HaArrayAddLast(res->partitions, KhePartitionMake(rg, soln, a));
   }
   return res;
 }
@@ -550,23 +553,26 @@ static void KhePartitionSolverClassifyTask(KHE_PARTITION_SOLVER ps,
 static void KhePartitionSolverSetDemandsAndTasks(KHE_PARTITION_SOLVER ps)
 {
   int i;  KHE_TASK task;  KHE_PARTITION p;  KHE_MULTI_TASK mt;
-  for( i = 0;  i < KheTaskingTaskCount(ps->tasking);  i++ )
+  for( i = 0;  i < KheSolnTaskCount(ps->soln);  i++ )
   {
-    task = KheTaskingTask(ps->tasking, i);
-    HnAssert(!KheTaskAssignIsFixed(task),
-      "KhePartitionSolverSetDemandsAndTasks internal error");
-    KhePartitionSolverClassifyTask(ps, task, &p, &mt);
-    if( mt != NULL )
-      HaArrayAddLast(ps->multi_tasks, mt);
-    else if( p != NULL )
-      p->duration_demand += KheTaskTotalDuration(task);
-    else
+    task = KheSolnTask(ps->soln, i);
+    if( KheTaskResourceType(task) == ps->resource_type )
     {
-      /* task is unassignable; ignore it */
-      if( DEBUG1 )
+      HnAssert(!KheTaskAssignIsFixed(task),
+	"KhePartitionSolverSetDemandsAndTasks internal error");
+      KhePartitionSolverClassifyTask(ps, task, &p, &mt);
+      if( mt != NULL )
+	HaArrayAddLast(ps->multi_tasks, mt);
+      else if( p != NULL )
+	p->duration_demand += KheTaskTotalDuration(task);
+      else
       {
-	fprintf(stderr, "  unclassified task ");
-	KheTaskDebug(task, 1, 0, stderr);
+	/* task is unassignable; ignore it */
+	if( DEBUG1 )
+	{
+	  fprintf(stderr, "  unclassified task ");
+	  KheTaskDebug(task, 1, 0, stderr);
+	}
       }
     }
   }
@@ -613,7 +619,7 @@ static void KhePartitionSolverDebug(KHE_PARTITION_SOLVER ps, int verbosity,
   KHE_PARTITION p;  KHE_MULTI_TASK mt;  int i;  KHE_RESOURCE_TYPE rt;
   if( verbosity >= 1 && indent >= 0 )
   {
-    rt = KheTaskingResourceType(ps->tasking);
+    rt = ps->resource_type;
     fprintf(fp, "%*s[ PartitionSolver (type %s)\n", indent, "",
       KheResourceTypeId(rt) == NULL ? "-" : KheResourceTypeId(rt));
     HaArrayForEach(ps->partitions, p, i)
@@ -636,13 +642,12 @@ static void KhePartitionSolverDebug(KHE_PARTITION_SOLVER ps, int verbosity,
 /*                                                                           */
 /*****************************************************************************/
 
-bool KheTaskingTightenToPartition(KHE_TASKING tasking,
-  KHE_TASK_BOUND_GROUP tbg, KHE_OPTIONS options)
+bool KheTightenToPartition(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
+  KHE_SOLN_ADJUSTER sa, KHE_OPTIONS options)
 {
-  KHE_RESOURCE_TYPE rt;  bool resource_invariant;  KHE_SOLN soln;  HA_ARENA a;
-  int i, j;  KHE_PARTITION_SOLVER ps;  KHE_PARTITION p;  KHE_MULTI_TASK mt;
+  bool resource_invariant;  HA_ARENA a;  int i, j;
+  KHE_PARTITION_SOLVER ps;  KHE_PARTITION p;  KHE_MULTI_TASK mt;
 
-  rt = KheTaskingResourceType(tasking);
   resource_invariant = KheOptionsGetBool(options, "rs_invariant", false);
   if( DEBUG1 )
     fprintf(stderr, "[ KheDoPartitionJob(%s, %s)\n",
@@ -690,9 +695,8 @@ bool KheTaskingTightenToPartition(KHE_TASKING tasking,
   }
 
   /* initialize a partition solver */
-  soln = KheTaskingSoln(tasking);
-  a = KheSolnArenaBegin(soln, false);
-  ps = KhePartitionSolverMake(tasking, resource_invariant, a);
+  a = KheSolnArenaBegin(soln);
+  ps = KhePartitionSolverMake(soln, rt, resource_invariant, a);
 
   /* set duration demand and accumulate general tasks */
   KhePartitionSolverSetDemandsAndTasks(ps);
@@ -710,7 +714,8 @@ bool KheTaskingTightenToPartition(KHE_TASKING tasking,
     {
       /* if there is a dominant partition, try it only */
       p = mt->dominant_partition;
-      if( KhePartitionAcceptsTask(p, mt->task, tbg, ps->resource_invariant) )
+      if( KhePartitionAcceptsTask(p, mt->task, sa, /* tbg, */
+	    ps->resource_invariant) )
       {
 	p->duration_demand += KheTaskTotalDuration(mt->task);
 	if( DEBUG1 )
@@ -745,7 +750,8 @@ bool KheTaskingTightenToPartition(KHE_TASKING tasking,
 	  fprintf(stderr, "  trying:\n");
 	  KhePartitionDebug(p, 2, 4, stderr);
 	}
-	if( KhePartitionAcceptsTask(p, mt->task, tbg, ps->resource_invariant) )
+	if( KhePartitionAcceptsTask(p, mt->task, sa, /* tbg, */
+	      ps->resource_invariant) )
 	{
 	  p->duration_demand += KheTaskTotalDuration(mt->task);
 	  if( DEBUG1 )
