@@ -52,10 +52,12 @@
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_solvers.h
 typedef struct khe_interval_rec {
   int first;
   int last;
 } KHE_INTERVAL;
+*** */
 
 typedef HA_ARRAY(KHE_INTERVAL) ARRAY_KHE_INTERVAL;
 
@@ -97,7 +99,7 @@ typedef enum {
 typedef struct reassign_attributes_rec {
   KHE_INTERVAL		interval;
   KHE_REASSIGN_GROUPING	grouping;
-  bool			ignore_partial;
+  bool			allow_partial;
 } REASSIGN_ATTRIBUTES;
 
 
@@ -186,6 +188,7 @@ struct khe_reassign_solver_rec {
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_interval.c
 static KHE_INTERVAL KheInterval(int first, int last)
 {
   KHE_INTERVAL res;
@@ -193,6 +196,7 @@ static KHE_INTERVAL KheInterval(int first, int last)
   res.last = last;
   return res;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -203,10 +207,12 @@ static KHE_INTERVAL KheInterval(int first, int last)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_interval.c
 static bool KheIntervalEmpty(KHE_INTERVAL in)
 {
   return in.first > in.last;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -217,6 +223,7 @@ static bool KheIntervalEmpty(KHE_INTERVAL in)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_interval.c
 static bool KheIntervalEqual(KHE_INTERVAL in1, KHE_INTERVAL in2)
 {
   if( KheIntervalEmpty(in1) )
@@ -224,6 +231,7 @@ static bool KheIntervalEqual(KHE_INTERVAL in1, KHE_INTERVAL in2)
   else
     return in1.first == in2.first && in1.last == in2.last;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -234,11 +242,13 @@ static bool KheIntervalEqual(KHE_INTERVAL in1, KHE_INTERVAL in2)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_interval.c
 static bool KheIntervalDisjoint(KHE_INTERVAL in1, KHE_INTERVAL in2)
 {
   return KheIntervalEmpty(in1) || KheIntervalEmpty(in2) ||
     in1.last < in2.first || in2.last < in1.first;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -249,6 +259,7 @@ static bool KheIntervalDisjoint(KHE_INTERVAL in1, KHE_INTERVAL in2)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_interval.c
 static int KheIntervalTypedCmp(KHE_INTERVAL i1, KHE_INTERVAL i2)
 {
   if( i1.first != i2.first )
@@ -256,6 +267,7 @@ static int KheIntervalTypedCmp(KHE_INTERVAL i1, KHE_INTERVAL i2)
   else
     return i1.last - i2.last;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -266,12 +278,14 @@ static int KheIntervalTypedCmp(KHE_INTERVAL i1, KHE_INTERVAL i2)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_interval.c
 static int KheIntervalCmp(const void *t1, const void *t2)
 {
   KHE_INTERVAL i1 = * (KHE_INTERVAL *) t1;
   KHE_INTERVAL i2 = * (KHE_INTERVAL *) t2;
   return KheIntervalTypedCmp(i1, i2);
 }
+*** */
 
 
 /*****************************************************************************/
@@ -282,6 +296,7 @@ static int KheIntervalCmp(const void *t1, const void *t2)
 /*                                                                           */
 /*****************************************************************************/
 
+/* *** moved to khe_sm_interval.c
 static char *KheIntervalShow(KHE_INTERVAL in, KHE_FRAME frame)
 {
   static char buff[500];
@@ -295,6 +310,7 @@ static char *KheIntervalShow(KHE_INTERVAL in, KHE_FRAME frame)
     sprintf(buff, "%d-%d", in.first, in.last);
   return buff;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -514,7 +530,7 @@ static char *ResourceDefectiveShow(RESOURCE_DEFECTIVE defective)
 /*****************************************************************************/
 /*                                                                           */
 /*  REASSIGN_ATTRIBUTES ReassignAttributesMake(int first_index,              */
-/*    int last_index, KHE_REASSIGN_GROUPING grouping, bool ignore_partial)   */
+/*    int last_index, KHE_REASSIGN_GROUPING grouping, bool allow_partial)   */
 /*                                                                           */
 /*  Make a reassign attributes object.  These are the attributes that        */
 /*  affect which groups are needed for some resource when solving.           */
@@ -522,12 +538,12 @@ static char *ResourceDefectiveShow(RESOURCE_DEFECTIVE defective)
 /*****************************************************************************/
 
 static REASSIGN_ATTRIBUTES ReassignAttributesMake(KHE_INTERVAL in,
-  KHE_REASSIGN_GROUPING grouping, bool ignore_partial)
+  KHE_REASSIGN_GROUPING grouping, bool allow_partial)
 {
   REASSIGN_ATTRIBUTES res;
   res.interval = in;
   res.grouping = grouping;
-  res.ignore_partial = ignore_partial;
+  res.allow_partial = allow_partial;
   return res;
 }
 
@@ -546,7 +562,7 @@ static bool ReassignAttributesEqual(REASSIGN_ATTRIBUTES ra1,
 {
   return KheIntervalEqual(ra1.interval, ra2.interval) &&
     ra1.grouping == ra2.grouping &&
-    ra1.ignore_partial == ra2.ignore_partial;
+    ra1.allow_partial == ra2.allow_partial;
 }
 
 
@@ -565,7 +581,7 @@ static char *ReassignAttributesShow(REASSIGN_ATTRIBUTES ra,
   static char buff[200];  KHE_FRAME frame;
   frame = rs == NULL ? NULL : KheTaskFinderFrame(rs->task_finder);
   snprintf(buff, 200, "%s:%s:%s", KheIntervalShow(ra.interval, frame),
-    ReassignGroupingShow(ra.grouping), bool_show(ra.ignore_partial));
+    ReassignGroupingShow(ra.grouping), bool_show(ra.allow_partial));
   return buff;
 }
 
@@ -813,7 +829,7 @@ static RESOURCE_INFO ResourceInfoMake(KHE_REASSIGN_SOLVER rs, KHE_RESOURCE r)
   RESOURCE_INFO res;
   HaMake(res, rs->arena);
   res->resource = r;
-  res->group_attributes = ReassignAttributesMake(KheInterval(-1, -1),
+  res->group_attributes = ReassignAttributesMake(KheIntervalMake(-1, -1),
     KHE_REASSIGN_MINIMAL, false);
   HaArrayInit(res->groups, rs->arena);
   res->defective = RESOURCE_DEFECTIVE_UNKNOWN;
@@ -833,7 +849,7 @@ static RESOURCE_INFO ResourceInfoMake(KHE_REASSIGN_SOLVER rs, KHE_RESOURCE r)
 
 static void ResourceInfoSetGroupsOutOfDate(RESOURCE_INFO ri)
 {
-  ri->group_attributes = ReassignAttributesMake(KheInterval(-1, -1),
+  ri->group_attributes = ReassignAttributesMake(KheIntervalMake(-1, -1),
     KHE_REASSIGN_MINIMAL, false);
 }
 
@@ -878,17 +894,15 @@ static void ResourceInfoEnsureGroupsUpToDate(RESOURCE_INFO ri,
 
 	/* one group for each task in the interval */
 	tmp_group = KheGroupMake(rs);
-	KheFindTasksInInterval(rs->task_finder, ra.interval.first,
-	  ra.interval.last, rs->resource_type, ri->resource, false,
-	  ra.ignore_partial, tmp_group->task_set, &tmp_group->interval.first,
-	  &tmp_group->interval.last);
+	KheFindTasksInInterval(rs->task_finder, ra.interval,
+	  rs->resource_type, ri->resource, false,
+	  ra.allow_partial, tmp_group->task_set, &tmp_group->interval);
 	for( i = 0;  i < KheTaskSetTaskCount(tmp_group->task_set);  i++ )
 	{
 	  task = KheTaskSetTask(tmp_group->task_set, i);
 	  group = KheGroupMake(rs);
 	  KheTaskSetAddTask(group->task_set, task);
-	  KheTaskFinderTaskInterval(rs->task_finder, task,
-	    &group->interval.first, &group->interval.last);
+	  group->interval = KheTaskFinderTaskInterval(rs->task_finder, task);
 	  group->unassignable = ri->resource != NULL &&
 	    !KheTaskSetNeedsAssignment(group->task_set);
 	  HaArrayAddLast(ri->groups, group);
@@ -900,10 +914,9 @@ static void ResourceInfoEnsureGroupsUpToDate(RESOURCE_INFO ri,
 
 	/* one group for each run of tasks overlapping the interval */
 	group = KheGroupMake(rs);
-	while( KheFindFirstRunInInterval(rs->task_finder, ra.interval.first,
-	  ra.interval.last, rs->resource_type, ri->resource, false,
-	  ra.ignore_partial, true, group->task_set, &group->interval.first,
-	  &group->interval.last) )
+	while( KheFindFirstRunInInterval(rs->task_finder, ra.interval,
+	  rs->resource_type, ri->resource, false, ra.allow_partial,
+	  true, group->task_set, &group->interval) )
 	{
 	  HnAssert(group->interval.first >= 0,
 	    "ResourceInfoEnsureGroupsUpToDate internal error 1");
@@ -922,10 +935,9 @@ static void ResourceInfoEnsureGroupsUpToDate(RESOURCE_INFO ri,
 
 	/* a single group containing all the tasks in the interval */
 	group = KheGroupMake(rs);
-	KheFindTasksInInterval(rs->task_finder, ra.interval.first,
-	  ra.interval.last, rs->resource_type, ri->resource, false,
-	  ra.ignore_partial, group->task_set, &group->interval.first,
-	  &group->interval.last);
+	KheFindTasksInInterval(rs->task_finder, ra.interval,
+	  rs->resource_type, ri->resource, false, ra.allow_partial,
+	  group->task_set, &group->interval);
 	group->unassignable = ri->resource != NULL &&
 	  !KheTaskSetNeedsAssignment(group->task_set);
 	HaArrayAddLast(ri->groups, group);
@@ -1433,7 +1445,7 @@ static bool KheReassignSolverDoSolve(KHE_REASSIGN_SOLVER rs,
 /*****************************************************************************/
 /*                                                                           */
 /*  bool KheReassignSolverSolve(KHE_REASSIGN_SOLVER rs, int first_index,     */
-/*    int last_index, KHE_REASSIGN_GROUPING grouping, bool ignore_partial,   */
+/*    int last_index, KHE_REASSIGN_GROUPING grouping, bool allow_partial,    */
 /*    KHE_REASSIGN_METHOD method, int max_assignments)                       */
 /*                                                                           */
 /*  Solve, and return true if any assignments changed.  At the start, make   */
@@ -1443,16 +1455,16 @@ static bool KheReassignSolverDoSolve(KHE_REASSIGN_SOLVER rs,
 /*****************************************************************************/
 
 bool KheReassignSolverSolve(KHE_REASSIGN_SOLVER rs, int first_index,
-  int last_index, KHE_REASSIGN_GROUPING grouping, bool ignore_partial,
+  int last_index, KHE_REASSIGN_GROUPING grouping, bool allow_partial,
   KHE_REASSIGN_METHOD method, int max_assignments)
 {
   bool res;  REASSIGN_ATTRIBUTES ra;
   if( DEBUG2 )
     fprintf(stderr, "[ KheReassignSolverSolve(rs, %d, %d, %s, %s)\n",
       first_index, last_index, ReassignGroupingShow(grouping),
-      bool_show(ignore_partial));
-  ra = ReassignAttributesMake(KheInterval(first_index, last_index),
-    grouping, ignore_partial);
+      bool_show(allow_partial));
+  ra = ReassignAttributesMake(KheIntervalMake(first_index, last_index),
+    grouping, allow_partial);
   rs->method = method;
   rs->max_assignments = max_assignments;
   res = KheReassignSolverDoSolve(rs, ra, 2);
@@ -1482,7 +1494,7 @@ KHE_REASSIGN_SOLVER KheReassignSolverMake(KHE_SOLN soln,
   KHE_RESOURCE_TYPE rt, KHE_OPTIONS options)
 {
   KHE_REASSIGN_SOLVER res;  HA_ARENA a;
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   HaMake(res, a);
 
   /* used throughout the lifetime of the solver */
@@ -1877,7 +1889,7 @@ static bool KheReassignSolveConstraint(KHE_REASSIGN_SOLVER rs,
 /*  bool KheReassignSolveForResources(KHE_REASSIGN_SOLVER rs,                */
 /*    int reassign_resources, REASSIGN_SELECT_TYPE reassign_select,          */
 /*    bool reassign_null, KHE_REASSIGN_GROUPING reassign_grouping,           */
-/*    bool reassign_ignore_partial, int indent)                              */
+/*    bool reassign_allow_partial, int indent)                               */
 /*                                                                           */
 /*  Solve with these parameters.  Return true if any solves succeeded.       */
 /*                                                                           */
@@ -1886,7 +1898,7 @@ static bool KheReassignSolveConstraint(KHE_REASSIGN_SOLVER rs,
 static bool KheReassignSolveForResources(KHE_REASSIGN_SOLVER rs,
   int reassign_resources, REASSIGN_SELECT_TYPE reassign_select,
   bool reassign_null, KHE_REASSIGN_GROUPING reassign_grouping,
-  bool reassign_ignore_partial, int indent)
+  bool reassign_allow_partial, int indent)
 {
   bool res;  int i;  REASSIGN_ATTRIBUTES ra;  KHE_INTERVAL in;
   if( DEBUG2 )
@@ -1900,11 +1912,11 @@ static bool KheReassignSolveForResources(KHE_REASSIGN_SOLVER rs,
   final_index = KheTaskFinderLastIndex(rs->task_finder);
   ra = ReassignAttributesMake(rs->reassign_start,
     rs->reassign_start + rs->reassign_parts - 1, rs->reassign_grouping,
-    rs->reassign_ignore_partial);
+    rs->reassign_allow_partial);
   while( ra.last_index <= final_index )
   *** */
-  ra = ReassignAttributesMake(KheInterval(-1, -1), reassign_grouping,
-    reassign_ignore_partial);
+  ra = ReassignAttributesMake(KheIntervalMake(-1, -1), reassign_grouping,
+    reassign_allow_partial);
   HaArrayForEach(rs->intervals, in, i)
   {
     ra.interval = in;
@@ -2064,8 +2076,9 @@ static void KheReassignSolverSetSelectConstraints(KHE_REASSIGN_SOLVER rs,
 static void KheReassignSolverSetConstraintIntervals(KHE_REASSIGN_SOLVER rs,
   char *str)
 {
-  KHE_INSTANCE ins;  int i, j, k, count, offset, fi, li;  KHE_CONSTRAINT c;
-  KHE_CLUSTER_BUSY_TIMES_CONSTRAINT cbtc;  KHE_TIME_GROUP tg;  KHE_POLARITY po;
+  KHE_INSTANCE ins;  int i, j, k, count, offset /* , fi, li */;
+  KHE_CLUSTER_BUSY_TIMES_CONSTRAINT cbtc;  KHE_TIME_GROUP tg;
+  KHE_POLARITY po;  KHE_CONSTRAINT c;  KHE_INTERVAL in;
   HaArrayClear(rs->intervals);
   ins = KheSolnInstance(rs->soln);
   for( i = 0;  i < KheInstanceConstraintCount(ins);  i++ )
@@ -2084,8 +2097,8 @@ static void KheReassignSolverSetConstraintIntervals(KHE_REASSIGN_SOLVER rs,
 	for( k = 0; k < KheClusterBusyTimesConstraintTimeGroupCount(cbtc); k++ )
 	{
 	  tg = KheClusterBusyTimesConstraintTimeGroup(cbtc, k, offset, &po);
-	  KheTaskFinderTimeGroupInterval(rs->task_finder, tg, &fi, &li);
-	  HaArrayAddLast(rs->intervals, KheInterval(fi, li));
+	  in = KheTaskFinderTimeGroupInterval(rs->task_finder, tg);
+	  HaArrayAddLast(rs->intervals, in);
 	}
       }
     }
@@ -2109,7 +2122,7 @@ static void KheReassignSolverSetPartsIntervals(KHE_REASSIGN_SOLVER rs,
   final = KheTaskFinderLastIndex(rs->task_finder);
   HaArrayClear(rs->intervals);
   for( fi = start, li = start + parts - 1;  li <= final;  fi += inc, li += inc )
-    HaArrayAddLast(rs->intervals, KheInterval(fi, li));
+    HaArrayAddLast(rs->intervals, KheIntervalMake(fi, li));
 }
 
 
@@ -2120,7 +2133,7 @@ static void KheReassignSolverSetPartsIntervals(KHE_REASSIGN_SOLVER rs,
 /*    char *rs_reassign_select, char *rs_reassign_null,                      */
 /*    char *rs_reassign_parts, char *rs_reassign_start,                      */
 /*    char *rs_reassign_increment, char *rs_reassign_grouping,               */
-/*    char *rs_reassign_ignore_partial, char *rs_reassign_method,            */
+/*    char *rs_reassign_allow_partial, char *rs_reassign_method,             */
 /*    char *rs_reassign_max_assignments)                                     */
 /*                                                                           */
 /*  Call KheReassignSolverSolve repeatedly on the resources of type rt,      */
@@ -2133,13 +2146,13 @@ bool KheReassignDoRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
   char *rs_reassign_select, char *rs_reassign_null,
   char *rs_reassign_parts, char *rs_reassign_start,
   char *rs_reassign_increment, char *rs_reassign_grouping,
-  char *rs_reassign_ignore_partial, char *rs_reassign_method,
+  char *rs_reassign_allow_partial, char *rs_reassign_method,
   char *rs_reassign_max_assignments)
 {
   KHE_REASSIGN_SOLVER rs;  KHE_INSTANCE ins;
   char *select_constraints_str, *str;
   int reassign_resources, parts, start, inc;
-  bool reassign_null, reassign_ignore_partial;
+  bool reassign_null, reassign_allow_partial;
   REASSIGN_SELECT_TYPE reassign_select;  KHE_COST init_cost;
   KHE_RESOURCE_TIMETABLE_MONITOR rtm;  KHE_RESOURCE r;
   KHE_REASSIGN_GROUPING reassign_grouping;
@@ -2169,8 +2182,8 @@ bool KheReassignDoRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
   rs->reassign_null = reassign_null;
   *** */
   reassign_grouping = ReassignGroupingGet(options, rs_reassign_grouping);
-  reassign_ignore_partial = KheOptionsGetBool(options,
-    rs_reassign_ignore_partial, false);
+  reassign_allow_partial = KheOptionsGetBool(options,
+    rs_reassign_allow_partial, false);
   rs->method = ReassignMethodGet(options, rs_reassign_method);
   rs->max_assignments = KheOptionsGetInt(options,
     rs_reassign_max_assignments, 1000000);
@@ -2202,7 +2215,7 @@ bool KheReassignDoRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
     fprintf(stderr, "[ %s(select %d %s, partial %s)\n", fn_name,
       reassign_resources,
       ReassignSelectTypeShow(reassign_select, select_constraints_str),
-      bool_show(reassign_ignore_partial));
+      bool_show(reassign_allow_partial));
     KheReassignSolverDebug(rs, 2, 2, stderr);
   }
   /* ***
@@ -2211,13 +2224,13 @@ bool KheReassignDoRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
       rs->reassign_null ? " plus NULL" : "",
       parts, start, inc,
       ReassignGroupingShow(reassign_grouping),
-      bool_show(reassign_ignore_partial),
+      bool_show(reassign_allow_partial),
       ReassignMethodShow(reassign_method));
   *** */
   if( DEBUG1 && DEBUG8 && KheInstanceRetrieveResource(ins, DEBUG8_ID, &r) )
   {
     rtm = KheResourceTimetableMonitor(soln, r);
-    KheResourceTimetableMonitorPrintTimetable(rtm, 10, 2, stderr);
+    KheResourceTimetableMonitorPrintTimetable(rtm, NULL, 10, 2, stderr);
   }
 
   /* quit now if rs_reassign_resources (possibly plus NULL) is less than 2 */
@@ -2245,7 +2258,7 @@ bool KheReassignDoRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
   /* solve for rt, debug result and return */
   init_cost = KheSolnCost(soln);
   if( KheReassignSolveForResources(rs, reassign_resources, reassign_select,
-	reassign_null, reassign_grouping, reassign_ignore_partial, 2) )
+	reassign_null, reassign_grouping, reassign_allow_partial, 2) )
   {
     KheReassignSolverDelete(rs);
     if( DEBUG1 )
@@ -2265,7 +2278,7 @@ bool KheReassignDoRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
 
 /*****************************************************************************/
 /*                                                                           */
-/*  bool KheReassignRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,              */
+/*  bool KheReassign1Repair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,             */
 /*    KHE_OPTIONS options)                                                   */
 /*                                                                           */
 /*  Call KheReassignSolverSolve repeatedly on the resources of type rt,      */
@@ -2273,13 +2286,13 @@ bool KheReassignDoRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
 /*                                                                           */
 /*****************************************************************************/
 
-bool KheReassignRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
+bool KheReassign1Repair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
   KHE_OPTIONS options)
 {
   return KheReassignDoRepair(soln, rt, options, "KheReassignRepair",
     "rs_reassign_resources", "rs_reassign_select", "rs_reassign_null",
     "rs_reassign_parts", "rs_reassign_start", "rs_reassign_increment",
-    "rs_reassign_grouping", "rs_reassign_ignore_partial",
+    "rs_reassign_grouping", "rs_reassign_allow_partial",
     "rs_reassign_method", "rs_reassign_max_assignments");
 }
 
@@ -2299,7 +2312,7 @@ bool KheReassign2Repair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
   return KheReassignDoRepair(soln, rt, options, "KheReassign2Repair",
     "rs_reassign2_resources", "rs_reassign2_select", "rs_reassign2_null",
     "rs_reassign2_parts", "rs_reassign2_start", "rs_reassign2_increment",
-    "rs_reassign2_grouping", "rs_reassign2_ignore_partial",
+    "rs_reassign2_grouping", "rs_reassign2_allow_partial",
     "rs_reassign2_method", "rs_reassign2_max_assignments");
 }
 
@@ -2319,6 +2332,6 @@ bool KheReassign3Repair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
   return KheReassignDoRepair(soln, rt, options, "KheReassign3Repair",
     "rs_reassign3_resources", "rs_reassign3_select", "rs_reassign3_null",
     "rs_reassign3_parts", "rs_reassign3_start", "rs_reassign3_increment",
-    "rs_reassign3_grouping", "rs_reassign3_ignore_partial",
+    "rs_reassign3_grouping", "rs_reassign3_allow_partial",
     "rs_reassign3_method", "rs_reassign3_max_assignments");
 }
