@@ -46,234 +46,9 @@ typedef enum {
   KHE_REMATCH_INTERVALS
 } KHE_REMATCH_TYPE;
 
-
-/*****************************************************************************/
-/*                                                                           */
-/*  KHE_TIME_SET                                                             */
-/*                                                                           */
-/*  A set of times.                                                          */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** moved to platform, khe_time_set.c
-typedef HA_ARRAY(KHE_TIME) ARRAY_KHE_TIME;
-
-typedef struct khe_rematch_timeset_rec {
-  ARRAY_KHE_TIME		times;		** the times                 **
-} *KHE_TIME_SET;
-*** */
-
 typedef HA_ARRAY(KHE_TIME_SET) ARRAY_KHE_TIME_SET;
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  Submodule "rematch time sets"                                            */
-/*                                                                           */
-/*****************************************************************************/
-
-/*****************************************************************************/
-/*                                                                           */
-/*  int KheTimeSetTypedCmp(KHE_TIME_SET ts1, KHE_TIME_SET ts2)               */
-/*                                                                           */
-/*  Typed comparison function for sorting an array of time sets.  It doesn't */
-/*  matter, really, what the order is; it's just for uniqueifying.           */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** moved to platform, khe_time_set.c
-static int KheTimeSetTypedCmp(KHE_TIME_SET ts1, KHE_TIME_SET ts2)
-{
-  KHE_TIME time1, time2;  int i;
-  if( HaArrayCount(ts1->times) != HaArrayCount(ts2->times) )
-    return HaArrayCount(ts1->times) - HaArrayCount(ts2->times);
-  for( i = 0;  i < HaArrayCount(ts1->times);  i++ )
-  {
-    time1 = HaArray(ts1->times, i);
-    time2 = HaArray(ts2->times, i);
-    if( time1 != time2 )
-      return KheTimeIndex(time1) - KheTimeIndex(time2);
-  }
-  return 0;
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  int KheTimeSetCmp(const void *p1, const void *p2)                        */
-/*                                                                           */
-/*  Comparison function for sorting an array of time sets.  It doesn't       */
-/*  matter, really, what the order is; it's just for uniqueifying.           */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** moved to platform, khe_time_set.c
-static int KheTimeSetCmp(const void *p1, const void *p2)
-{
-  KHE_TIME_SET ts1 = * (KHE_TIME_SET *) p1;
-  KHE_TIME_SET ts2 = * (KHE_TIME_SET *) p2;
-  return KheTimeSetTypedCmp(ts1, ts2);
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  KHE_TIME_SET KheTimeSetBegin(void)                                       */
-/*                                                                           */
-/*  Begin the construction of a new rematch time set.                        */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** moved to platform, khe_time_set.c
-static KHE_TIME_SET KheTimeSetBegin(HA_ARENA a)
-{
-  KHE_TIME_SET res;
-  HaMake(res, a);
-  HaArrayInit(res->times, a);
-  return res;
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheTimeSetAddTaskTimes(KHE_TIME_SET ts, KHE_TASK task)              */
-/*                                                                           */
-/*  Add to ts the times that task is running.                                */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** moved to platform, khe_time_set.c
-static void KheTimeSetAddTaskTimes(KHE_TIME_SET ts, KHE_TASK task)
-{
-  KHE_MEET meet;  KHE_TASK sub_task;  int i;  KHE_TIME starting_time;
-
-  ** add the times of task **
-  meet = KheTaskMeet(task);
-  if( meet != NULL )
-  {
-    starting_time = KheMeetAsstTime(meet);
-    if( starting_time != NULL )
-    {
-      for( i = 0;  i < KheMeetDuration(meet);  i++ )
-      {
-	HnAssert(KheTimeHasNeighbour(starting_time, i),
-	  "KheTimeSetAddTaskTimes: time off end");
-	HaArrayAddLast(ts->times, KheTimeNeighbour(starting_time, i));
-      }
-    }
-  }
-
-  ** recursively add the times of tasks assigned to task **
-  for( i = 0;  i < KheTaskAssignedToCount(task);  i++ )
-  {
-    sub_task = KheTaskAssignedTo(task, i);
-    KheTimeSetAddTaskTimes(ts, sub_task);
-  }
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheTimeSetAddTimeGroupTimes(KHE_TIME_SET ts, KHE_TIME_GROUP tg)     */
-/*                                                                           */
-/*  Add the times of tg to ts.                                               */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** moved to platform, khe_time_set.c
-static void KheTimeSetAddTimeGroupTimes(KHE_TIME_SET ts, KHE_TIME_GROUP tg)
-{
-  int i;
-  for( i = 0;  i < KheTimeGroupTimeCount(tg);  i++ )
-    HaArrayAddLast(ts->times, KheTimeGroupTime(tg, i));
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  int KheTimeIncreasingIndexCmp(const void *p1, const void *p2)            */
-/*                                                                           */
-/*  Comparison function for sorting an array of times into increasing        */
-/*  index order.                                                             */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** moved to platform, khe_time_set.c (in fact, not needed there)
-static int KheTimeIncreasingIndexCmp(const void *p1, const void *p2)
-{
-  KHE_TIME time1 = * (KHE_TIME *) p1;
-  KHE_TIME time2 = * (KHE_TIME *) p2;
-  return KheTimeIndex(time1) - KheTimeIndex(time2);
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheTimeSetEnd(KHE_TIME_SET ts)                                      */
-/*                                                                           */
-/*  End the construction of a rematch time set ts.                           */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** moved to platform, khe_time_set.c
-void KheTimeSetEnd(KHE_TIME_SET ts)
-{
-  HaArraySortUnique(ts->times, &KheTimeIncreasingIndexCmp);
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheTimeSetDelete(KHE_TIME_SET ts)                                   */
-/*                                                                           */
-/*  Delete ts.                                                               */
-/*                                                                           */
-/*****************************************************************************/
-
-/* ***
-static void KheTimeSetDelete(KHE_TIME_SET ts)
-{
-  MArrayFree(ts->times);
-  MFree(ts);
-}
-*** */
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheTimeSetDebug(KHE_TIME_SET ts, int indent, FILE *fp)              */
-/*                                                                           */
-/*  Debug print of ts onto fp.                                               */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** not currently used
-static void KheTimeSetDebug(KHE_TIME_SET ts, int indent, FILE *fp)
-{
-  KHE_TIME time;  int i;
-  if( indent >= 0 )
-    fprintf(fp, "%*s", indent, "");
-  fprintf(fp, "[");
-  HaArrayForEach(ts->times, time, i)
-  {
-    if( i > 0 )
-      fprintf(fp, ", ");
-    fprintf(fp, "%s", KheTimeName(time));
-  }
-  fprintf(fp, "]");
-  if( indent >= 0 )
-    fprintf(fp, "\n");
-}
-*** */
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  Submodule "main function"                                                */
@@ -303,12 +78,15 @@ static bool TaskAssignmentIsDefective(KHE_TASK task)
 
   /* check whether this task has a defective event resource monitor */
   er = KheTaskEventResource(task);
-  soln = KheTaskSoln(task);
-  for( i = 0;  i < KheSolnEventResourceMonitorCount(soln, er);  i++ )
+  if( er != NULL )
   {
-    m = KheSolnEventResourceMonitor(soln, er, i);
-    if( KheMonitorCost(m) > 0 )
-      return true;
+    soln = KheTaskSoln(task);
+    for( i = 0;  i < KheSolnEventResourceMonitorCount(soln, er);  i++ )
+    {
+      m = KheSolnEventResourceMonitor(soln, er, i);
+      if( KheMonitorCost(m) > 0 )
+	return true;
+    }
   }
 
   /* check for defective sub-tasks */
@@ -337,7 +115,7 @@ static bool TaskAssignmentIsDefective(KHE_TASK task)
 static void KheRematchAddDefectiveTimeSets(KHE_SOLN soln,
   KHE_RESOURCE_GROUP rg, ARRAY_KHE_TIME_SET *time_sets, HA_ARENA a)
 {
-  KHE_TASK task;  int i;  KHE_TIME_SET ts;  KHE_TASKING tasking;
+  KHE_TASK task;  int i;  KHE_TIME_SET ts;
   KHE_RESOURCE_TYPE rt;  bool group_is_all;  KHE_RESOURCE r;
   KHE_INSTANCE ins;
 
@@ -346,7 +124,8 @@ static void KheRematchAddDefectiveTimeSets(KHE_SOLN soln,
     fprintf(stderr, "[ KheRematchAddDefectiveTimeSets(soln, rg, *time_sets)\n");
   ins = KheSolnInstance(soln);
   rt = KheResourceGroupResourceType(rg);
-  tasking = NULL;  /* keep compiler happy */
+  /* ***
+  tasking = NULL;  ** keep compiler happy **
   for( i = 0;  i < KheSolnTaskingCount(soln);  i++ )
   {
     tasking = KheSolnTasking(soln, i);
@@ -355,20 +134,24 @@ static void KheRematchAddDefectiveTimeSets(KHE_SOLN soln,
   }
   HnAssert(i < KheSolnTaskingCount(soln),
     "KheResourceRematch: no tasking with type %s", KheResourceTypeId(rt));
+  *** */
 
   /* add one time set for each defective task */
   group_is_all = (KheResourceTypeResourceCount(rt) ==
     KheResourceGroupResourceCount(rg));
-  for( i = 0;  i < KheTaskingTaskCount(tasking);  i++ )
+  for( i = 0;  i < KheSolnTaskCount(soln);  i++ )
   {
-    task = KheTaskingTask(tasking, i);
-    r = KheTaskAsstResource(task);
-    if( (r == NULL || group_is_all || KheResourceGroupContains(rg, r)) &&
-        TaskAssignmentIsDefective(task) )
+    task = KheSolnTask(soln, i);
+    if( KheTaskResourceType(task) == rt )
     {
-      ts = KheTimeSetMake(ins, a);
-      KheTimeSetAddTaskTimes(ts, task);
-      HaArrayAddLast(*time_sets, ts);
+      r = KheTaskAsstResource(task);
+      if( (r == NULL || group_is_all || KheResourceGroupContains(rg, r)) &&
+	  TaskAssignmentIsDefective(task) )
+      {
+	ts = KheTimeSetMake(ins, a);
+	KheTimeSetAddTaskTimes(ts, task);
+	HaArrayAddLast(*time_sets, ts);
+      }
     }
   }
   if( DEBUG2 )
@@ -414,7 +197,7 @@ static void KheRematchAddFrameTimeSets(KHE_FRAME frame, int max_groups,
   ARRAY_KHE_TIME_SET *time_sets, HA_ARENA a)
 {
   int p, i, j;  KHE_TIME_SET ts;  KHE_TIME_GROUP tg;  KHE_INSTANCE ins;
-  ins = KheSolnInstance(KheFrameSoln(frame));
+  ins = KheFrameInstance(frame);
   for( p = KheSmallestGroupSize(frame, max_groups);  p <= max_groups;  p++ )
   {
     for( i = 0;  i <= KheFrameTimeGroupCount(frame) - p;  i += p )
@@ -561,9 +344,6 @@ bool KheResourceRematch(KHE_SOLN soln, KHE_RESOURCE_GROUP rg,
       KheResourceGroupId(rg), KheCostShow(init_cost));
 
   /* get matching_off and edge_adjust123_off options */
-  /* ***
-  matching_off = KheOptionsGetBool(options, "rs_rematch_matching_off", false);
-  *** */
   edge_adjust1_off = KheOptionsGetBool(options, "rs_rematch_edge_adjust1_off",
     false);
   edge_adjust2_off = KheOptionsGetBool(options, "rs_rematch_edge_adjust2_off",
@@ -576,8 +356,8 @@ bool KheResourceRematch(KHE_SOLN soln, KHE_RESOURCE_GROUP rg,
   /* nocost_off = KheOptionsGetBool(options, "rs_rematch_nocost_off", false); */
 
   /* build a solver */
-  a = KheSolnArenaBegin(soln, false);
-  rms = KheResourceMatchingSolverMake(soln, rg, a);
+  a = KheSolnArenaBegin(soln);
+  rms = KheResourceMatchingSolverMake(soln, rg, options, a);
 
   /* get the time sets, depending on the rematch type */
   HaArrayInit(time_sets, a);
@@ -634,7 +414,7 @@ bool KheResourceRematch(KHE_SOLN soln, KHE_RESOURCE_GROUP rg,
 
     if( KheTimeSetTimeCount(ts) > 0 )
     {
-      demand_set = KheResourceMatchingDemandSetMake(rms, false);
+      demand_set = KheResourceMatchingDemandSetMake(rms /* , false */);
       /* HaArrayForEach(ts->times, t, j) */
       for( j = 0;  j < KheTimeSetTimeCount(ts);  j++ )
       {
@@ -666,7 +446,7 @@ bool KheResourceRematch(KHE_SOLN soln, KHE_RESOURCE_GROUP rg,
     KheOptionsDeleteTimer(options, timer);
 
   /* free the time sets and solver, and return */
-  KheResourceMatchingSolverDelete(rms);
+  /* KheResourceMatchingSolverDelete(rms); */
   KheSolnArenaEnd(soln, a);
   if( DEBUG1 )
   {
