@@ -158,7 +158,10 @@ static void KheRequestSolverTryTime(KHE_REQUEST_SOLVER rs, KHE_TIME t)
   /* if r is already busy at time t, return immediately */
   index = KheFrameTimeIndex(rs->days_frame, t);
   tg = KheFrameTimeGroup(rs->days_frame, index);
-  if( !KheResourceTimetableMonitorTimeGroupAvailable(rs->rtm, tg, false) )
+  if( !KheResourceTimetableMonitorFreeForTimeGroup(rs->rtm,tg,NULL,NULL,false) )
+  /* ***
+  if( !KheResourceTimetableMonitorTimeGroupAvailable(rs->rtm, tg, false, true) )
+  *** */
     return;
 
   /* try a selection of tasks running at time t */
@@ -184,8 +187,12 @@ static void KheRequestSolverTryTime(KHE_REQUEST_SOLVER rs, KHE_TIME t)
 	    KheResourceId(KheTaskAsstResource(task)));
 	}
       }
+      else if( !KheResourceTimetableMonitorFreeForTask(rs->rtm, task,
+	    rs->days_frame, NULL, false) )
+      /* ***
       else if( !KheResourceTimetableMonitorTaskAvailableInFrame(rs->rtm, task,
 	    rs->days_frame, NULL) )
+      *** */
       {
 	/* resource not available at frame time groups covered by task */
 	if( DEBUG6_ON(rs->resource) )
@@ -306,12 +313,14 @@ static KHE_RESOURCE KheMonitorResource(KHE_MONITOR m)
 
 static int KheMonitorDecreasingWeightTypedCmp(KHE_MONITOR m1, KHE_MONITOR m2)
 {
-  KHE_COST weight1, weight2;  int cmp;
-  KHE_CONSTRAINT c1, c2;  KHE_RESOURCE r1, r2;
-  c1 = KheMonitorConstraint(m1);
-  c2 = KheMonitorConstraint(m2);
-  weight1 = KheConstraintCombinedWeight(c1);
-  weight2 = KheConstraintCombinedWeight(c2);
+  KHE_COST weight1, weight2;  int cmp;  KHE_RESOURCE r1, r2;
+  /* ***
+  KHE_CONSTRAINT c1, c2;
+  c1 = KheMonitor Constraint(m1);
+  c2 = KheMonitorC onstraint(m2);
+  *** */
+  weight1 = KheMonitorCombinedWeight(m1);
+  weight2 = KheMonitorCombinedWeight(m2);
   cmp = KheCostCmp(weight2, weight1);  /* reverse order, decreasing weight */
   if( cmp != 0 )  return cmp;
   r1 = KheMonitorResource(m1);
@@ -538,7 +547,7 @@ static bool KheHandleLimitForced(KHE_LIMIT_BUSY_TIMES_MONITOR lbtm,
   KHE_SOLN soln, KHE_GROUP_MONITOR gm, KHE_FRAME days_frame,
   KHE_EVENT_TIMETABLE_MONITOR etm)
 {
-  int min_limit, max_limit, i, k, busy_count;  KHE_RESOURCE r;
+  int min_limit, max_limit, i, k, busy_count, open_count;  KHE_RESOURCE r;
   bool allow_zero, res, progressing;  KHE_TIME_GROUP tg;  KHE_TIME t;
   r = KheLimitBusyTimesMonitorResource(lbtm);
   res = false;
@@ -548,8 +557,9 @@ static bool KheHandleLimitForced(KHE_LIMIT_BUSY_TIMES_MONITOR lbtm,
     for( i = 0; i < KheLimitBusyTimesMonitorDefectiveTimeGroupCount(lbtm); i++ )
     {
       KheLimitBusyTimesMonitorDefectiveTimeGroup(lbtm, i, &tg,
-	&busy_count, &min_limit, &max_limit, &allow_zero);
-      if( busy_count < min_limit && KheTimeGroupTimeCount(tg) <= min_limit )
+	&busy_count, &open_count, &min_limit, &max_limit, &allow_zero);
+      if( busy_count + open_count < min_limit &&
+	  KheTimeGroupTimeCount(tg) <= min_limit )
       {
 	/* every time of tg is requested to be busy */
 	for( k = 0;  k < KheTimeGroupTimeCount(tg);  k++ )
@@ -578,9 +588,9 @@ static bool KheHandleLimitForced(KHE_LIMIT_BUSY_TIMES_MONITOR lbtm,
 static bool KheHandleLimitNotForced(KHE_LIMIT_BUSY_TIMES_MONITOR lbtm,
   KHE_SOLN soln, KHE_FRAME days_frame, KHE_EVENT_TIMETABLE_MONITOR etm)
 {
-  int min_limit, max_limit, i, k, busy_count; struct khe_request_solver_rec rsr;
+  int min_limit, max_limit, i, k, busy_count, open_count;
+  KHE_RESOURCE r;  struct khe_request_solver_rec rsr;
   bool allow_zero, res, progressing;  KHE_TIME_GROUP tg;  KHE_TIME t;
-  KHE_RESOURCE r;
   r = KheLimitBusyTimesMonitorResource(lbtm);
   res = false;
   do
@@ -589,8 +599,9 @@ static bool KheHandleLimitNotForced(KHE_LIMIT_BUSY_TIMES_MONITOR lbtm,
     for( i = 0; i < KheLimitBusyTimesMonitorDefectiveTimeGroupCount(lbtm); i++ )
     {
       KheLimitBusyTimesMonitorDefectiveTimeGroup(lbtm, i, &tg,
-	&busy_count, &min_limit, &max_limit, &allow_zero);
-      if( busy_count < min_limit && KheTimeGroupTimeCount(tg) > min_limit )
+	&busy_count, &open_count, &min_limit, &max_limit, &allow_zero);
+      if( busy_count + open_count < min_limit &&
+	  KheTimeGroupTimeCount(tg) > min_limit )
       {
 	/* not every time of tg is requested to be busy */
 	KheRequestSolverBegin(&rsr, r, soln, NULL, days_frame, etm);
@@ -639,7 +650,7 @@ bool KheSolnAssignRequestedResources(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
   }
   days_frame = KheOptionsFrame(options, "gs_common_frame", soln);
   if( DEBUG5 )
-    KheFrameAssertNoClashes(days_frame);
+    KheFrameAssertNoClashes(days_frame, soln);
   etm = (KHE_EVENT_TIMETABLE_MONITOR)
     KheOptionsGetObject(options, "gs_event_timetable_monitor", NULL);
   HnAssert(etm != NULL, "KheSolnAssignRequestedResources: "
@@ -650,7 +661,7 @@ bool KheSolnAssignRequestedResources(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
   /* rt, those limit busy times and cluster busy times monitors with  */
   /* non-zero cost whose allow_zero attributes are false              */
   init_cost = KheSolnCost(soln);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   HaArrayInit(monitors, a);
   for( i = 0;  i < KheResourceTypeResourceCount(rt);  i++ )
   {
@@ -710,7 +721,7 @@ bool KheSolnAssignRequestedResources(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
     {
       fprintf(stderr, "  monitors %d .. %d (%s, gm %.5f):\n", i, j - 1,
 	KheResourceId(r), KheCostShow(KheMonitorCost((KHE_MONITOR) gm)));
-	/* KheCostShow(KheConstraintCombinedWeight(KheMonitorConstraint(m))));*/
+      /*KheCostShow(KheConst raintCombinedWeight(KheMonitor Constraint(m))));*/
       for( k = i;  k < j;  k++ )
       {
 	m = HaArray(monitors, k);
@@ -723,7 +734,7 @@ bool KheSolnAssignRequestedResources(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
       if( DEBUG3 )
 	KheMonitorDebug(m, 2, 2, stderr);
       if( DEBUG5 )
-	KheFrameAssertNoClashes(days_frame);
+	KheFrameAssertNoClashes(days_frame, soln);
       if( KheMonitorCost(m) > 0 )  /* not redundant, cost may have changed */
       {
 	if( KheMonitorTag(m) == KHE_LIMIT_BUSY_TIMES_MONITOR_TAG )
@@ -758,7 +769,7 @@ bool KheSolnAssignRequestedResources(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
       if( DEBUG3 )
 	KheMonitorDebug(m, 2, 4, stderr);
       if( DEBUG5 )
-	KheFrameAssertNoClashes(days_frame);
+	KheFrameAssertNoClashes(days_frame, soln);
       if( KheMonitorCost(m) > 0 )
       {
 	if( KheMonitorTag(m) == KHE_LIMIT_BUSY_TIMES_MONITOR_TAG )
@@ -782,7 +793,7 @@ bool KheSolnAssignRequestedResources(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
   /* and return */
   KheSolnArenaEnd(soln, a);
   if( DEBUG5 )
-    KheFrameAssertNoClashes(days_frame);
+    KheFrameAssertNoClashes(days_frame, soln);
   if( DEBUG1 )
   {
     if( soln_changed )
