@@ -441,12 +441,20 @@ static bool KheRprNodeIsPSymmetrical(KHE_RPR_NODE node)
     for( j = 0;  j < KheEventResourceConstraintCount(er);  j++ )
     {
       c = KheEventResourceConstraint(er, j);
-      if( KheConstraintTag(c) == KHE_PREFER_RESOURCES_CONSTRAINT_TAG )
+      if( c == NULL ||
+	  KheConstraintTag(c) == KHE_PREFER_RESOURCES_CONSTRAINT_TAG )
       {
-	prc = (KHE_PREFER_RESOURCES_CONSTRAINT) c;
-	rg = KhePreferResourcesConstraintDomain(prc);
-	if( KheResourceGroupContains(rg,r1) != KheResourceGroupContains(rg,r2) )
-	  return false;
+	if( c != NULL )
+	{
+	  prc = (KHE_PREFER_RESOURCES_CONSTRAINT) c;
+	  rg = KhePreferResourcesConstraintDomain(prc);  /* NB prc != NULL */
+	  if( KheResourceGroupContains(rg,r1)!=KheResourceGroupContains(rg,r2) )
+	    return false;
+	}
+	else
+	{
+	  /* nothing to do here, rg is empty which is symmetrical */
+	}
       }
     }
   return true;
@@ -1362,7 +1370,7 @@ bool KheResourcePairReassign(KHE_SOLN soln, KHE_RESOURCE r1,
   KHE_RESOURCE r2, bool resource_invariant, bool fix_splits)
 {
   bool truncated, res;  HA_ARENA a;
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   res = KheResourcePairDoReassign(soln, r1, r2, resource_invariant,
     fix_splits, &truncated, a);
   KheSolnArenaEnd(soln, a);
@@ -1428,10 +1436,10 @@ void KheIncStats(KHE_OPTIONS options, bool success, bool truncated)
 /*                                                                           */
 /*****************************************************************************/
 
-static bool KheResourceTypeResourcePairRepairSplits(KHE_RESOURCE_TYPE rt,
-  KHE_TASKING tasking, KHE_OPTIONS options, HA_ARENA a)
+static bool KheResourceTypeResourcePairRepairSplits(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, KHE_OPTIONS options, HA_ARENA a)
 {
-  int i, j, k, pos;  KHE_TASK task;  KHE_EVENT_RESOURCE er;  KHE_SOLN soln;
+  int i, j, k, pos;  KHE_TASK task;  KHE_EVENT_RESOURCE er;
   ARRAY_KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR defects;  KHE_RESOURCE r1, r2;
   KHE_MONITOR m;  KHE_AVOID_SPLIT_ASSIGNMENTS_MONITOR asam;
   bool res, success, truncated, resource_invariant;
@@ -1443,11 +1451,10 @@ static bool KheResourceTypeResourcePairRepairSplits(KHE_RESOURCE_TYPE rt,
        resource_invariant ? "true" : "false");
 
   /* find the avoid split assignments defects */
-  soln = KheTaskingSoln(tasking);
   HaArrayInit(defects, a);
-  for( i = 0;  i < KheTaskingTaskCount(tasking);  i++ )
+  for( i = 0;  i < KheSolnTaskCount(soln);  i++ )
   {
-    task = KheTaskingTask(tasking, i);
+    task = KheSolnTask(soln, i);
     er = KheTaskEventResource(task);
     if( er != NULL && KheEventResourceResourceType(er) == rt )
       for( j = 0;  j < KheSolnEventResourceMonitorCount(soln, er);  j++ )
@@ -1556,11 +1563,10 @@ static bool KheResourceGroupResourcePairRepairAll(KHE_RESOURCE_GROUP rg,
 /*                                                                           */
 /*****************************************************************************/
 
-static bool KheResourceTypeResourcePairRepairAll(KHE_RESOURCE_TYPE rt,
-  KHE_TASKING tasking, KHE_OPTIONS options, bool partitions, HA_ARENA a)
+static bool KheResourceTypeResourcePairRepairAll(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, KHE_OPTIONS options, bool partitions, HA_ARENA a)
 {
-  bool res;  int i;  KHE_RESOURCE_GROUP rg;  KHE_SOLN soln;
-  soln = KheTaskingSoln(tasking);
+  bool res;  int i;  KHE_RESOURCE_GROUP rg;
   res = false;
   if( partitions && KheResourceTypeHasPartitions(rt) )
   {
@@ -1590,19 +1596,19 @@ static bool KheResourceTypeResourcePairRepairAll(KHE_RESOURCE_TYPE rt,
 /*                                                                           */
 /*****************************************************************************/
 
-static bool KheResourceTypeResourcePairRepair(KHE_RESOURCE_TYPE rt,
-  KHE_TASKING tasking, KHE_OPTIONS options, HA_ARENA a)
+static bool KheResourceTypeResourcePairRepair(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, KHE_OPTIONS options, HA_ARENA a)
 {
   char *opt;
   opt = KheOptionsGet(options, "rs_pair_select", "splits");
   if( strcmp(opt, "none") == 0 )
     return false;
   else if( strcmp(opt, "splits") == 0 )
-    return KheResourceTypeResourcePairRepairSplits(rt, tasking, options, a);
+    return KheResourceTypeResourcePairRepairSplits(soln, rt, options, a);
   else if( strcmp(opt, "partitions") == 0 )
-    return KheResourceTypeResourcePairRepairAll(rt, tasking, options, true, a);
+    return KheResourceTypeResourcePairRepairAll(soln, rt, options, true, a);
   else if( strcmp(opt, "all") == 0 )
-    return KheResourceTypeResourcePairRepairAll(rt, tasking, options, false, a);
+    return KheResourceTypeResourcePairRepairAll(soln, rt, options, false, a);
   else
   {
     HnAbort("KheResourceTypeResourcePairRepair: illegal resource_pair");
@@ -1658,10 +1664,10 @@ static bool KheResourceTypeResourcePairRepair(KHE_RESOURCE_TYPE rt,
 /*                                                                           */
 /*****************************************************************************/
 
-bool KheResourcePairRepair(KHE_TASKING tasking, KHE_OPTIONS options)
+bool KheResourcePairRepair(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
+  KHE_OPTIONS options)
 {
-  KHE_RESOURCE_TYPE rt;  int i;  KHE_SOLN soln;  KHE_INSTANCE ins;  bool res;
-  bool pair_off;  HA_ARENA a;
+  int i;  KHE_INSTANCE ins;  bool res, pair_off;  HA_ARENA a;
 
   /* quit immediately if resource pair repair is not wanted */
   pair_off = KheOptionsGetBool(options, "rs_pair_off", "false");
@@ -1669,10 +1675,7 @@ bool KheResourcePairRepair(KHE_TASKING tasking, KHE_OPTIONS options)
     return false;
 
   /* boilerplate */
-  soln = KheTaskingSoln(tasking);
-  a = KheSolnArenaBegin(soln, false);
-  /* a = HaArenaMa ke(); */
-  rt = KheTaskingResourceType(tasking);
+  a = KheSolnArenaBegin(soln);
   if( DEBUG3 )
     fprintf(stderr, "[ KheResourcePairRepair(%s, %s)\n",
       rt == NULL ? NULL : KheResourceTypeId(rt),
@@ -1682,7 +1685,7 @@ bool KheResourcePairRepair(KHE_TASKING tasking, KHE_OPTIONS options)
   res = false;
   if( rt != NULL )
   {
-    if( KheResourceTypeResourcePairRepair(rt, tasking, options, a) )
+    if( KheResourceTypeResourcePairRepair(soln, rt, options, a) )
       res = true;
   }
   else
@@ -1691,7 +1694,7 @@ bool KheResourcePairRepair(KHE_TASKING tasking, KHE_OPTIONS options)
     for( i = 0;  i < KheInstanceResourceTypeCount(ins);  i++ )
     {
       rt = KheInstanceResourceType(ins, i);
-      if( KheResourceTypeResourcePairRepair(rt, tasking, options, a) )
+      if( KheResourceTypeResourcePairRepair(soln, rt, options, a) )
 	res = true;
     }
   }
