@@ -118,7 +118,9 @@ struct khe_sr_task_set_rec {
 
 struct khe_split_solver_rec {
   HA_ARENA			arena;			/* arena             */
-  KHE_TASKING			tasking;		/* original tasking  */
+  KHE_SOLN			soln;			/* solution          */
+  KHE_RESOURCE_TYPE		resource_type;		/* resource type     */
+  /* KHE_TASKING		tasking; */		/* original tasking  */
   ARRAY_KHE_RESOURCE_NODE	resource_nodes;		/* resource nodes    */
   ARRAY_KHE_SR_TASK_SET		task_sets;		/* task sets         */
   KHE_PRIQUEUE			forced_task_nodes;	/* forced task nodes */
@@ -499,7 +501,7 @@ static void KheTaskNodeMerge(KHE_TASK_NODE tn1, KHE_TASK_NODE tn2)
 static void KheTaskSetAddTaskNode(KHE_SR_TASK_SET ts, KHE_TASK_NODE tn);
 
 static void KheTaskSetAddTaskNodes(KHE_SR_TASK_SET ts, KHE_TASK task,
-  KHE_TASKING tasking)
+  KHE_SOLN soln, KHE_RESOURCE_TYPE rt)
 {
   KHE_TASK child_task;
   while( KheTaskAssignedToCount(task) > 0 )
@@ -507,18 +509,20 @@ static void KheTaskSetAddTaskNodes(KHE_SR_TASK_SET ts, KHE_TASK task,
     child_task = KheTaskAssignedTo(task, 0);
     KheTaskAssignUnFix(child_task);
     KheTaskUnAssign(child_task);
-    KheTaskSetAddTaskNodes(ts, child_task, tasking);
+    KheTaskSetAddTaskNodes(ts, child_task, soln, rt);
   }
   if( KheTaskMeet(task) != NULL )
   {
     /* add a task node for task to ts */
     KheTaskSetAddTaskNode(ts, KheTaskNodeMake(ts, task));
+    /* ***
     if( KheTaskTasking(task) != tasking )
     {
       if( KheTaskTasking(task) != NULL )
 	KheTaskingDeleteTask(KheTaskTasking(task), task);
       KheTaskingAddTask(tasking, task);
     }
+    *** */
   }
   else
   {
@@ -567,7 +571,7 @@ static bool KheSRTaskSetMakeFromTask(KHE_SPLIT_SOLVER ss, KHE_TASK task,
 {
   KHE_SR_TASK_SET ts;
   ts = KheSRTaskSetMake(ss);
-  KheTaskSetAddTaskNodes(ts, task, ss->tasking);
+  KheTaskSetAddTaskNodes(ts, task, ss->soln, ss->resource_type);
   if( HaArrayCount(ts->task_nodes) > 0 )
   {
     *res = ts;
@@ -1537,12 +1541,12 @@ static void KheSplitSolverDeleteTaskSet(KHE_SPLIT_SOLVER ss, KHE_SR_TASK_SET ts)
 /*                                                                           */
 /*****************************************************************************/
 
-static KHE_SPLIT_SOLVER KheSplitSolverMake(KHE_TASKING tasking, HA_ARENA a)
+static KHE_SPLIT_SOLVER KheSplitSolverMake(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, HA_ARENA a)
 {
   KHE_SPLIT_SOLVER res;  int i, j, k, pos;  KHE_TASK task;
   KHE_INSTANCE ins;  KHE_RESOURCE r;  ARRAY_KHE_TASK unassigned_tasks;
   KHE_SR_TASK_SET ts;  KHE_TASK_NODE tn;  KHE_RESOURCE_NODE rn;
-  KHE_RESOURCE_TYPE rt;
   if( DEBUG7 )
     fprintf(stderr, "[ KheSplitSolverMake(tasking)\n");
 
@@ -1550,7 +1554,9 @@ static KHE_SPLIT_SOLVER KheSplitSolverMake(KHE_TASKING tasking, HA_ARENA a)
   /* a = HaArenaM ake(); */
   HaMake(res, a);
   res->arena = a;
-  res->tasking = tasking;
+  res->soln = soln;
+  res->resource_type = rt;
+  /* res->tasking = tasking; */
   HaArrayInit(res->resource_nodes, a);
   HaArrayInit(res->task_sets, a);
   res->forced_task_nodes = KhePriQueueMake(&KheTaskNodeKey,
@@ -1558,16 +1564,19 @@ static KHE_SPLIT_SOLVER KheSplitSolverMake(KHE_TASKING tasking, HA_ARENA a)
 
   /* gather the unassigned tasks of tasking */
   HaArrayInit(unassigned_tasks, a);
-  for( i = 0;  i < KheTaskingTaskCount(tasking);  i++ )
+  for( i = 0;  i < KheSolnTaskCount(soln);  i++ )
   {
-    task = KheTaskingTask(tasking, i);
-    if( KheTaskAsst(task) == NULL )
+    task = KheSolnTask(soln, i);
+    if( KheTaskResourceType(task) == rt )
     {
-      HaArrayAddLast(unassigned_tasks, task);
-      if( DEBUG7 )
+      if( KheTaskAsst(task) == NULL )
       {
-	fprintf(stderr, "  adding unassigned task ");
-	KheTaskDebug(task, 1, 0, stderr);
+	HaArrayAddLast(unassigned_tasks, task);
+	if( DEBUG7 )
+	{
+	  fprintf(stderr, "  adding unassigned task ");
+	  KheTaskDebug(task, 1, 0, stderr);
+	}
       }
     }
   }
@@ -1578,9 +1587,8 @@ static KHE_SPLIT_SOLVER KheSplitSolverMake(KHE_TASKING tasking, HA_ARENA a)
   HaArrayFree(unassigned_tasks);
 
   /* add one resource node for each relevant unpreassigned resource */
-  if( KheTaskingResourceType(tasking) != NULL )
+  if( rt != NULL )
   {
-    rt = KheTaskingResourceType(tasking);
     if( !KheResourceTypeDemandIsAllPreassigned(rt) )
       for( j = 0;  j < KheResourceTypeResourceCount(rt);  j++ )
       {
@@ -1590,7 +1598,7 @@ static KHE_SPLIT_SOLVER KheSplitSolverMake(KHE_TASKING tasking, HA_ARENA a)
   }
   else
   {
-    ins = KheSolnInstance(KheTaskingSoln(tasking));
+    ins = KheSolnInstance(soln);
     for( i = 0;  i < KheInstanceResourceTypeCount(ins);  i++ )
     {
       rt = KheInstanceResourceType(ins, i);
@@ -1795,21 +1803,19 @@ static void KheRemoveCompletedTaskSets(KHE_SPLIT_SOLVER ss)
 /*                                                                           */
 /*****************************************************************************/
 
-bool KheFindSplitResourceAssignments(KHE_TASKING tasking, KHE_OPTIONS options)
+bool KheFindSplitResourceAssignments(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, KHE_OPTIONS options)
 {
   KHE_SPLIT_SOLVER ss;  bool progressing;  int i, try;  KHE_SR_TASK_SET ts;
-  KHE_SOLN soln;  HA_ARENA a;
+  HA_ARENA a;
   if( DEBUG1 )
-    fprintf(stderr, "[ KheFindSplitResourceAssignments(%s) %d tasks\n",
-      KheTaskingResourceType(tasking) == NULL ? "~" :
-      KheResourceTypeId(KheTaskingResourceType(tasking)) == NULL ? "-" :
-      KheResourceTypeId(KheTaskingResourceType(tasking)),
-      KheTaskingTaskCount(tasking));
+    fprintf(stderr, "[ KheFindSplitResourceAssignments(%s)\n",
+      rt == NULL ? "~" : KheResourceTypeId(rt) == NULL ? "-" :
+      KheResourceTypeId(rt));
 
   /* make a split solver; carry out forced assignments and their extensions */
-  soln = KheTaskingSoln(tasking);
-  a = KheSolnArenaBegin(soln, false);
-  ss = KheSplitSolverMake(tasking, a);
+  a = KheSolnArenaBegin(soln);
+  ss = KheSplitSolverMake(soln, rt, a);
   KheClearForcedAssts(ss);
   do
   {
