@@ -64,15 +64,13 @@ typedef HA_ARRAY(KHE_TASK) ARRAY_KHE_TASK;
 /*                                                                           */
 /*****************************************************************************/
 
-static void KheTaskingDoPreferResourcesConstraintJob(KHE_TASKING tasking,
-  KHE_PREFER_RESOURCES_CONSTRAINT c, KHE_TASK_BOUND_GROUP tbg,
-  bool resource_invariant)
+static void KheDoPreferResourcesConstraintJob(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, KHE_PREFER_RESOURCES_CONSTRAINT c,
+  KHE_SOLN_ADJUSTER sa, bool resource_invariant)
 {
   KHE_TASK task;  KHE_RESOURCE_GROUP domain;  int i, j, init_count;
-  KHE_EVENT_RESOURCE er;  KHE_SOLN soln;  KHE_MARK mark;  bool success;
-  KHE_TASK_BOUND tb;
-  domain = KhePreferResourcesConstraintDomain(c);
-  soln = KheTaskingSoln(tasking);
+  KHE_EVENT_RESOURCE er;  KHE_MARK mark;  bool success;  KHE_TASK_BOUND tb;
+  domain = KhePreferResourcesConstraintDomain(c);  /* NB c != NULL */
   /* t = KheTransactionMake(soln); */
   for( i = 0;  i < KhePreferResourcesConstraintEventResourceCount(c);  i++ )
   {
@@ -80,18 +78,21 @@ static void KheTaskingDoPreferResourcesConstraintJob(KHE_TASKING tasking,
     for( j = 0;  j < KheEventResourceTaskCount(soln, er);  j++ )
     {
       task = KheTaskFirstUnFixed(KheEventResourceTask(soln, er, j));
-      if( task != NULL && KheTaskTasking(task) == tasking &&
+      if( task != NULL && KheTaskResourceType(task) == rt &&
 	  !KheResourceGroupDisjoint(KheTaskDomain(task), domain) )
       {
 	KheAtomicOperationBegin(soln, &mark, &init_count, resource_invariant);
 	/* success = KheTaskTightenDomain(task, domain, true); */
 	/* success = KheTaskBoundMake(tbg, task, domain, &tb); */
-	tb = KheTaskBoundMake(soln, domain);
+	tb = KheSolnAdjusterTaskBoundMake(sa, soln, domain);
+	/* tb = KheTaskBoundMake(soln, domain); */
 	success = KheTaskAddTaskBound(task, tb);
 	KheAtomicOperationEnd(soln, &mark, &init_count, resource_invariant,
 	  success);
+	/* *** already added to sa now
 	if( success && tbg != NULL )
 	  KheTaskBoundGroupAddTaskBound(tbg, tb);
+	*** */
       }
     }
   }
@@ -136,22 +137,21 @@ static int KheTaskIncreasingDomainSizeCmp(const void *t1, const void *t2)
 /*                                                                           */
 /*****************************************************************************/
 
-static void KheTaskingDoAvoidSplitAssignmentsConstraintJob(KHE_TASKING tasking,
-  KHE_AVOID_SPLIT_ASSIGNMENTS_CONSTRAINT c, KHE_TASK_BOUND_GROUP tbg,
-  bool resource_invariant, HA_ARENA a)
+static void KheDoAvoidSplitAssignmentsConstraintJob(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, KHE_AVOID_SPLIT_ASSIGNMENTS_CONSTRAINT c,
+  KHE_SOLN_ADJUSTER sa, bool resource_invariant, HA_ARENA a)
 {
   ARRAY_KHE_TASK tasks;  KHE_TASK task, task2;  int i, j, k, pos, init_count;
-  KHE_EVENT_RESOURCE er;  KHE_RESOURCE_GROUP domain;  KHE_SOLN soln;
+  KHE_EVENT_RESOURCE er;  KHE_RESOURCE_GROUP domain;
   KHE_MARK mark;  bool success;  KHE_TASK_BOUND tb;
   if( DEBUG8 )
     fprintf(stderr, "[ KheTaskingDoAvoidSplitAssignmentsConstraintJob()\n");
 
   HaArrayInit(tasks, a);
-  soln = KheTaskingSoln(tasking);
   /* t = KheTransactionMake(soln); */
   for( i = 0;  i < KheAvoidSplitAssignmentsConstraintEventGroupCount(c);  i++ )
   {
-    /* gather the distinct root tasks for event group i that lie in tasking */
+    /* gather the distinct root tasks for event group i that lie in */
     HaArrayClear(tasks);
     for( j=0; j<KheAvoidSplitAssignmentsConstraintEventResourceCount(c,i); j++ )
     {
@@ -159,7 +159,7 @@ static void KheTaskingDoAvoidSplitAssignmentsConstraintJob(KHE_TASKING tasking,
       for( k = 0;  k < KheEventResourceTaskCount(soln, er);  k++ )
       {
 	task = KheTaskFirstUnFixed(KheEventResourceTask(soln, er, k));
-	if( task != NULL && KheTaskTasking(task) == tasking &&
+	if( task != NULL && KheTaskResourceType(task) == rt &&
 	    !HaArrayContains(tasks, task, &pos) )
 	  HaArrayAddLast(tasks, task);
       }
@@ -189,7 +189,6 @@ static void KheTaskingDoAvoidSplitAssignmentsConstraintJob(KHE_TASKING tasking,
 	  if( KheTaskAssign(task, task2) )
 	  {
 	    KheTaskAssignFix(task);
-	    KheTaskingDeleteTask(tasking, task);
 	    HaArrayDeleteAndShift(tasks, j);
 	    j--;
 	    success = true;
@@ -211,9 +210,12 @@ static void KheTaskingDoAvoidSplitAssignmentsConstraintJob(KHE_TASKING tasking,
 	      HnAbort("KheTaskingDoAvoidSplitAssignmentsConstraintJob"
 		"internal error 1");
 	    *** */
-	    tb = KheTaskBoundMake(soln, KheTaskDomain(task2));
+	    tb = KheSolnAdjusterTaskBoundMake(sa, soln, KheTaskDomain(task2));
+	    /* tb = KheTaskBoundMake(soln, KheTaskDomain(task2)); */
+	    /* *** already added to sa now
 	    if( tbg != NULL )
 	      KheTaskBoundGroupAddTaskBound(tbg, tb);
+	    *** */
 	    if( !KheTaskAddTaskBound(task, tb) )
 	      HnAbort("KheTaskingDoAvoidSplitAssignmentsConstraintJob"
 		"internal error 1");
@@ -221,7 +223,6 @@ static void KheTaskingDoAvoidSplitAssignmentsConstraintJob(KHE_TASKING tasking,
 	      HnAbort("KheTaskingDoAvoidSplitAssignmentsConstraintJob"
 		"internal error 2");
 	    KheTaskAssignFix(task);
-	    KheTaskingDeleteTask(tasking, task);
 	    HaArrayDeleteAndShift(tasks, j);
 	    j--;
 	    success = true;
@@ -367,26 +368,28 @@ static bool KheDoAddTaskBound(KHE_TASK task, KHE_TASK_BOUND tb,
 /*                                                                           */
 /*****************************************************************************/
 
-static void KheTaskingDoLimitBusyTimesConstraintJob(KHE_TASKING tasking,
-  KHE_LIMIT_BUSY_TIMES_CONSTRAINT c, KHE_TASK_BOUND_GROUP tbg,
-  bool resource_invariant, HA_ARENA a)
+static void KheDoLimitBusyTimesConstraintJob(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, KHE_LIMIT_BUSY_TIMES_CONSTRAINT c,
+  KHE_SOLN_ADJUSTER sa, bool resource_invariant, HA_ARENA a)
 {
   KHE_TASK task;  KHE_RESOURCE_GROUP domain;  int i, j, k, m, n, p, offset;
-  KHE_EVENT_RESOURCE er;  KHE_SOLN soln;  KHE_TIME t;  bool success;
-  KHE_TASK_BOUND tb;  KHE_EVENT e;  KHE_RESOURCE_TYPE rt;  KHE_TIME_GROUP tg;
+  KHE_EVENT_RESOURCE er;  KHE_TIME t;  bool success;
+  KHE_TASK_BOUND tb;  KHE_EVENT e;  KHE_TIME_GROUP tg;
   if( DEBUG9 )
     fprintf(stderr, "[ KheTaskingDoLimitBusyTimesConstraintJob(-, %s, -, %s)\n",
       KheConstraintId((KHE_CONSTRAINT) c), resource_invariant ? "true":"false");
-  soln = KheTaskingSoln(tasking);
   domain = KheLimitBusyTimesConstraintComplementResources(c, soln);
   if( domain != NULL )
   {
     if( DEBUG9 )
       KheResourceGroupDebug(domain, 2, 2, stderr);
     rt = KheResourceGroupResourceType(domain);
-    tb = KheTaskBoundMake(soln, domain);
+    /* tb = KheTaskBoundMake(soln, domain); */
+    tb = KheSolnAdjusterTaskBoundMake(sa, soln, domain);
+    /* ***
     if( tbg != NULL )
       KheTaskBoundGroupAddTaskBound(tbg, tb);
+    *** */
     for( i = 0;  i < KheLimitBusyTimesConstraintAppliesToOffsetCount(c);  i++ )
     {
       offset = KheLimitBusyTimesConstraintAppliesToOffset(c, i);
@@ -441,27 +444,28 @@ static void KheTaskingDoLimitBusyTimesConstraintJob(KHE_TASKING tasking,
 /*                                                                           */
 /*****************************************************************************/
 
-static void KheTaskingDoClusterBusyTimesConstraintJob(KHE_TASKING tasking,
-  KHE_CLUSTER_BUSY_TIMES_CONSTRAINT c, KHE_TASK_BOUND_GROUP tbg,
-  bool resource_invariant, HA_ARENA a)
+static void KheDoClusterBusyTimesConstraintJob(KHE_SOLN soln,
+  KHE_RESOURCE_TYPE rt, KHE_CLUSTER_BUSY_TIMES_CONSTRAINT c,
+  KHE_SOLN_ADJUSTER sa, bool resource_invariant, HA_ARENA a)
 {
   KHE_TASK task;  KHE_RESOURCE_GROUP domain;  int i, j, k, m, n, p, offset;
-  KHE_EVENT_RESOURCE er;  KHE_SOLN soln;  KHE_TIME t;  bool success;
-  KHE_TASK_BOUND tb;  KHE_EVENT e;  KHE_RESOURCE_TYPE rt;  KHE_TIME_GROUP tg;
-  KHE_POLARITY po;
+  KHE_EVENT_RESOURCE er;  KHE_TIME t;  bool success;
+  KHE_TASK_BOUND tb;  KHE_EVENT e;  KHE_TIME_GROUP tg;  KHE_POLARITY po;
   if( DEBUG9 )
     fprintf(stderr, "[ KheTaskingDoClusterBusyTimesConstraintJob(%s, -, %s)\n",
       KheConstraintId((KHE_CONSTRAINT) c), resource_invariant ? "true":"false");
-  soln = KheTaskingSoln(tasking);
   domain = KheClusterBusyTimesConstraintComplementResources(c, soln);
   if( domain != NULL )
   {
     if( DEBUG9 )
       KheResourceGroupDebug(domain, 2, 2, stderr);
     rt = KheResourceGroupResourceType(domain);
+    tb = KheSolnAdjusterTaskBoundMake(sa, soln, domain);
+    /* ***
     tb = KheTaskBoundMake(soln, domain);
     if( tbg != NULL )
       KheTaskBoundGroupAddTaskBound(tbg, tb);
+    *** */
     for( i = 0;  i < KheClusterBusyTimesConstraintAppliesToOffsetCount(c); i++ )
     {
       offset = KheClusterBusyTimesConstraintAppliesToOffset(c, i);
@@ -525,104 +529,6 @@ static int KheConstraintDecreasingCombinedWeightCmp(const void *t1,
   return KheCostCmp(KheConstraintCombinedWeight(c2),
     KheConstraintCombinedWeight(c1));
 }
-
-
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheTaskingDoConstraintJobs(KHE_TASKING tasking,                     */
-/*    bool prefer_hard, bool prefer_soft, bool split_hard, bool split_soft,  */
-/*    bool limit_busy_hard, bool limit_busy_soft, KHE_TASK_BOUND_GROUP tbg,  */
-/*    bool resource_invariant, HA_ARENA a)                                   */
-/*                                                                           */
-/*  Apply to the tasks of tasking the constraint jobs requested.             */
-/*  Any time bounds created are to be added to tbg, if non-NULL.             */
-/*                                                                           */
-/*****************************************************************************/
-
-/* *** now inlined at its call point
-static void KheTaskingDoConstraintJobs(KHE_TASKING tasking,
-  bool prefer_hard, bool prefer_soft, bool split_hard, bool split_soft,
-  bool limit_busy_hard, bool limit_busy_soft, KHE_TASK_BOUND_GROUP tbg,
-  bool resource_invariant, HA_ARENA a)
-{
-  KHE_INSTANCE ins;  ARRAY_KHE_CONSTRAINT constraints;  KHE_CONSTRAINT c;
-  KHE_RESOURCE_TYPE rt;  KHE_PREFER_RESOURCES_CONSTRAINT prc;  int i;
-  KHE_RESOURCE_GROUP domain;  KHE_LIMIT_BUSY_TIMES_CONSTRAINT lbtc;
-
-  ** gather the constraints required **
-  ins = KheSolnInstance(KheTaskingSoln(tasking));
-  rt = KheTaskingResourceType(tasking);
-  HaArrayInit(constraints, a);
-  for( i = 0;  i < KheInstanceConstraintCount(ins);  i++ )
-  {
-    c = KheInstanceConstraint(ins, i);
-    if( KheConstraintCombinedWeight(c) > 0 ) switch( KheConstraintTag(c) )
-    {
-      case KHE_PREFER_RESOURCES_CONSTRAINT_TAG:
-
-	prc = (KHE_PREFER_RESOURCES_CONSTRAINT) c;
-	domain = KhePreferResourcesConstraintDomain(prc);
-	if( (rt == NULL || rt == KheResourceGroupResourceType(domain)) &&
-	    (KheConstraintRequired(c) ? prefer_hard : prefer_soft) )
-	  HaArrayAddLast(constraints, c);
-	break;
-
-      case KHE_AVOID_SPLIT_ASSIGNMENTS_CONSTRAINT_TAG:
-
-	if( KheConstraintRequired(c) ? split_hard : split_soft )
-	  HaArrayAddLast(constraints, c);
-	break;
-
-      case KHE_LIMIT_BUSY_TIMES_CONSTRAINT_TAG:
-
-	if( KheConstraintRequired(c) ? limit_busy_hard : limit_busy_soft )
-	{
-          lbtc = (KHE_LIMIT_BUSY_TIMES_CONSTRAINT) c;
-	  if( KheLimitBusyTimesConstraintMaximum(lbtc) == 0 )
-	    HaArrayAddLast(constraints, c);
-	}
-	break;
-
-      default:
-
-	** constraint of other types are not relevant **
-	break;
-    }
-  }
-
-  ** handle each constraint in turn, in decreasing combined weight order **
-  HaArraySort(constraints, &KheConstraintDecreasingCombinedWeightCmp);
-  HaArrayForEach(constraints, c, i)
-    switch( KheConstraintTag(c) )
-    {
-      case KHE_PREFER_RESOURCES_CONSTRAINT_TAG:
-
-        KheTaskingDoPreferResourcesConstraintJob(tasking,
-          (KHE_PREFER_RESOURCES_CONSTRAINT) c, tbg, resource_invariant);
-	break;
-
-      case KHE_AVOID_SPLIT_ASSIGNMENTS_CONSTRAINT_TAG:
-
-        KheTaskingDoAvoidSplitAssignmentsConstraintJob(tasking,
-          (KHE_AVOID_SPLIT_ASSIGNMENTS_CONSTRAINT) c, tbg,resource_invariant,a);
-	break;
-
-      case KHE_LIMIT_BUSY_TIMES_CONSTRAINT_TAG:
-
-        KheTaskingDoLimitBusyTimesConstraintJob(tasking,
-          (KHE_LIMIT_BUSY_TIMES_CONSTRAINT) c, tbg, resource_invariant, a);
-	break;
-
-      default:
-
-	HnAbort("KheTaskingDoConstraintJobs internal error");
-	break;
-    }
-
-  ** clean up and quit **
-  ** MArrayFree(constraints); **
-}
-*** */
 
 
 /*****************************************************************************/
@@ -679,56 +585,6 @@ static void CheckResourceTypes(ARRAY_KHE_RESOURCE_TYPE *resource_types,
 *** */
 
 
-/*****************************************************************************/
-/*                                                                           */
-/*  void KheTaskTreeMake(KHE_SOLN soln, KHE_TASK_JOB_TYPE tjt,               */
-/*    KHE_OPTIONS options)                                                   */
-/*                                                                           */
-/*  Make a task tree from the tasks of soln.                                 */
-/*                                                                           */
-/*****************************************************************************/
-
-bool KheTaskTreeMake(KHE_SOLN soln, KHE_OPTIONS options)
-{
-  ARRAY_KHE_RESOURCE_TYPE resource_types;  KHE_RESOURCE_TYPE rt;
-  KHE_INSTANCE ins;  int i;  KHE_TASKING tasking;  bool res;  HA_ARENA a;
-
-  /* get the resource types of soln's instance, in the required order */
-  ins = KheSolnInstance(soln);
-  if( DEBUG10 )
-    fprintf(stderr, "[ KheTaskTreeMake(%s %d, options)\n",
-      KheInstanceId(ins), KheSolnDiversifier(soln));
-  a = KheSolnArenaBegin(soln, false);
-  /* a = HaAre naMake(); */
-  HaArrayInit(resource_types, a);
-  for( i = 0;  i < KheInstanceResourceTypeCount(ins);  i++ )
-  {
-    rt = KheInstanceResourceType(ins, i);
-    HnAssert(rt != NULL, "KheTaskTreeMake internal error (i == %d)", i);
-    HaArrayAddLast(resource_types, rt);
-  }
-  HaArraySort(resource_types, &KheResourceTypeCmp);
-
-  /* build one tasking for each type, and apply KheTaskingMakeTaskTree to it */
-  HnAssert(KheSolnTaskingCount(soln) == 0, "KheTaskTreeMake: soln has tasking");
-  res = false;
-  HaArrayForEach(resource_types, rt, i)
-  {
-    tasking = KheTaskingMakeFromResourceType(soln, rt);
-    res = true;
-    KheTaskingMakeTaskTree(tasking, NULL, options);
-  }
-
-  /* clean up and exit */
-  KheSolnArenaEnd(soln, a);
-  /* HaArena Delete(a); */
-  /* MArrayFree(resource_types); */
-  if( DEBUG10 )
-    fprintf(stderr, "] KheTaskTreeMake returning %s\n", bool_show(res));
-  return res;
-}
-
-
 /*****************************************************************************/
 /*                                                                           */
 /*  KHE_TASKING KheTaskingMakeFromResourceType(KHE_SOLN soln,                */
@@ -738,6 +594,7 @@ bool KheTaskTreeMake(KHE_SOLN soln, KHE_OPTIONS options)
 /*                                                                           */
 /*****************************************************************************/
 
+/* ***
 KHE_TASKING KheTaskingMakeFromResourceType(KHE_SOLN soln, KHE_RESOURCE_TYPE rt)
 {
   int i;  KHE_TASK task;  KHE_TASKING res;
@@ -751,6 +608,7 @@ KHE_TASKING KheTaskingMakeFromResourceType(KHE_SOLN soln, KHE_RESOURCE_TYPE rt)
   }
   return res;
 }
+*** */
 
 
 /*****************************************************************************/
@@ -762,32 +620,41 @@ KHE_TASKING KheTaskingMakeFromResourceType(KHE_SOLN soln, KHE_RESOURCE_TYPE rt)
 /*                                                                           */
 /*****************************************************************************/
 
-bool KheTaskingMakeTaskTree(KHE_TASKING tasking,
-  KHE_TASK_BOUND_GROUP tbg, KHE_OPTIONS options)
+static bool KheTaskingMakeTaskTree(KHE_SOLN soln, KHE_RESOURCE_TYPE rt,
+  KHE_SOLN_ADJUSTER sa, KHE_OPTIONS options)
+/* ***
+bool KheTaskingMakeTaskTree(KHE_TASKING tasking, KHE_SOLN_ADJUSTER sa,
+  ** KHE_TASK_BOUND_GROUP tbg, ** KHE_OPTIONS options)
+*** */
 {
   int i;  KHE_TASK task;  KHE_RESOURCE r;  bool resource_invariant;
   bool prefer_hard, prefer_soft, split_hard, split_soft;  HA_ARENA a;
-  bool limit_busy_hard, limit_busy_soft;  KHE_SOLN soln;
+  bool limit_busy_hard, limit_busy_soft;
   KHE_INSTANCE ins;  ARRAY_KHE_CONSTRAINT constraints;  KHE_CONSTRAINT c;
-  KHE_RESOURCE_TYPE rt;  KHE_PREFER_RESOURCES_CONSTRAINT prc;
+  KHE_PREFER_RESOURCES_CONSTRAINT prc;
   KHE_RESOURCE_GROUP domain;  KHE_LIMIT_BUSY_TIMES_CONSTRAINT lbtc;
-   KHE_CLUSTER_BUSY_TIMES_CONSTRAINT cbtc;
+  KHE_CLUSTER_BUSY_TIMES_CONSTRAINT cbtc;  /* KHE_EVENT_RESOURCE er; */
   /* KHE_COST min_weight, weight; */
 
   if( DEBUG3 )
     fprintf(stderr, "[ KheTaskingMakeTaskTree(%s, tbg, options)\n",
-      KheTaskingResourceType(tasking) == NULL ? "~" :
-      KheResourceTypeId(KheTaskingResourceType(tasking)) == NULL ? "-" :
-      KheResourceTypeId(KheTaskingResourceType(tasking)));
+      rt == NULL ? "~" : KheResourceTypeId(rt) == NULL ? "-" :
+      KheResourceTypeId(rt));
 
   /* jobs to install existing domains and assignments */
   /* nothing to do here */
 
   /* job to assign preassigned tasks (some may fail, ignore that) */
+  /* ***
   for( i = 0;  i < KheTaskingTaskCount(tasking);  i++ )
-  {
     task = KheTaskingTask(tasking, i);
-    if( KheTaskIsPreassigned(task, &r) && KheTaskAsst(task) == NULL )
+  *** */
+  for( i = 0;  i < KheSolnTaskCount(soln);  i++ )
+  {
+    task = KheSolnTask(soln, i);
+    /* er = KheTaskEventResource(task); */
+    if( KheTaskResourceType(task) == rt &&
+	KheTaskIsPreassigned(task, &r) && KheTaskAsst(task) == NULL )
       KheTaskAssignResource(task, r);
   }
 
@@ -805,25 +672,23 @@ bool KheTaskingMakeTaskTree(KHE_TASKING tasking,
     "rs_task_tree_limit_busy_hard_off", false);
   limit_busy_soft = !KheOptionsGetBool(options,
     "rs_task_tree_split_limit_busy_soft_off", false);
-  soln = KheTaskingSoln(tasking);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaAren aMake(); */
 
   /* find the minimum weight of all constraints */
-  ins = KheSolnInstance(KheTaskingSoln(tasking));
+  ins = KheSolnInstance(soln);
   /* ***
   min_weight = KheCost(INT_MAX, INT_MAX);
   for( i = 0;  i < KheInstanceConstraintCount(ins);  i++ )
   {
     c = KheInstanceConstraint(ins, i);
-    weight = KheConstraintCombinedWeight(c);
+    weight = KheConstr aintCombinedWeight(c);
     if( weight > 0 && min_weight < weight )
       min_weight = weight;
   }
   *** */
 
   /* gather all constraints whose weight is greater than the minimum */
-  rt = KheTaskingResourceType(tasking);
   HaArrayInit(constraints, a);
   for( i = 0;  i < KheInstanceConstraintCount(ins);  i++ )
   {
@@ -834,7 +699,7 @@ bool KheTaskingMakeTaskTree(KHE_TASKING tasking,
 	case KHE_PREFER_RESOURCES_CONSTRAINT_TAG:
 
 	  prc = (KHE_PREFER_RESOURCES_CONSTRAINT) c;
-	  domain = KhePreferResourcesConstraintDomain(prc);
+	  domain = KhePreferResourcesConstraintDomain(prc);  /* NB c != NULL */
 	  if( (rt == NULL || rt == KheResourceGroupResourceType(domain)) &&
 	      (KheConstraintRequired(c) ? prefer_hard : prefer_soft) )
 	    HaArrayAddLast(constraints, c);
@@ -880,26 +745,29 @@ bool KheTaskingMakeTaskTree(KHE_TASKING tasking,
     {
       case KHE_PREFER_RESOURCES_CONSTRAINT_TAG:
 
-        KheTaskingDoPreferResourcesConstraintJob(tasking,
-          (KHE_PREFER_RESOURCES_CONSTRAINT) c, tbg, resource_invariant);
+        KheDoPreferResourcesConstraintJob(soln, rt,
+          (KHE_PREFER_RESOURCES_CONSTRAINT) c, sa, /*tbg,*/ resource_invariant);
 	break;
 
       case KHE_AVOID_SPLIT_ASSIGNMENTS_CONSTRAINT_TAG:
 
-        KheTaskingDoAvoidSplitAssignmentsConstraintJob(tasking,
-          (KHE_AVOID_SPLIT_ASSIGNMENTS_CONSTRAINT) c, tbg,resource_invariant,a);
+        KheDoAvoidSplitAssignmentsConstraintJob(soln, rt,
+          (KHE_AVOID_SPLIT_ASSIGNMENTS_CONSTRAINT) c, sa, /* tbg, */
+	  resource_invariant, a);
 	break;
 
       case KHE_LIMIT_BUSY_TIMES_CONSTRAINT_TAG:
 
-        KheTaskingDoLimitBusyTimesConstraintJob(tasking,
-          (KHE_LIMIT_BUSY_TIMES_CONSTRAINT) c, tbg, resource_invariant, a);
+        KheDoLimitBusyTimesConstraintJob(soln, rt,
+          (KHE_LIMIT_BUSY_TIMES_CONSTRAINT) c, sa, /* tbg, */
+	  resource_invariant, a);
 	break;
 
       case KHE_CLUSTER_BUSY_TIMES_CONSTRAINT_TAG:
 
-        KheTaskingDoClusterBusyTimesConstraintJob(tasking,
-          (KHE_CLUSTER_BUSY_TIMES_CONSTRAINT) c, tbg, resource_invariant, a);
+        KheDoClusterBusyTimesConstraintJob(soln, rt,
+          (KHE_CLUSTER_BUSY_TIMES_CONSTRAINT) c, sa, /* tbg, */
+	  resource_invariant, a);
 	break;
 
       default:
@@ -949,3 +817,52 @@ bool KheTaskingMakeTaskTree(KHE_TASKING tasking,
     fprintf(stderr, "] KheTaskingMakeTaskTree returning\n");
   return true;
 }
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheTaskTreeMake(KHE_SOLN soln, KHE_TASK_JOB_TYPE tjt,               */
+/*    KHE_OPTIONS options)                                                   */
+/*                                                                           */
+/*  Make a task tree from the tasks of soln.                                 */
+/*                                                                           */
+/*****************************************************************************/
+
+bool KheTaskTreeMake(KHE_SOLN soln, KHE_SOLN_ADJUSTER sa,
+  KHE_OPTIONS options)
+{
+  ARRAY_KHE_RESOURCE_TYPE resource_types;  KHE_RESOURCE_TYPE rt;
+  KHE_INSTANCE ins;  int i;  bool res;  HA_ARENA a;
+
+  /* get the resource types of soln's instance, in the required order */
+  ins = KheSolnInstance(soln);
+  if( DEBUG10 )
+    fprintf(stderr, "[ KheTaskTreeMake(%s %d, options)\n",
+      KheInstanceId(ins), KheSolnDiversifier(soln));
+  a = KheSolnArenaBegin(soln);
+  /* a = HaAre naMake(); */
+  HaArrayInit(resource_types, a);
+  for( i = 0;  i < KheInstanceResourceTypeCount(ins);  i++ )
+  {
+    rt = KheInstanceResourceType(ins, i);
+    HnAssert(rt != NULL, "KheTaskTreeMake internal error (i == %d)", i);
+    HaArrayAddLast(resource_types, rt);
+  }
+  HaArraySort(resource_types, &KheResourceTypeCmp);
+
+  /* build one task tree for each resource type */
+  res = false;
+  HaArrayForEach(resource_types, rt, i)
+  {
+    res = true;
+    KheTaskingMakeTaskTree(soln, rt, sa, options);
+  }
+
+  /* clean up and exit */
+  KheSolnArenaEnd(soln, a);
+  /* HaArena Delete(a); */
+  /* MArrayFree(resource_types); */
+  if( DEBUG10 )
+    fprintf(stderr, "] KheTaskTreeMake returning %s\n", bool_show(res));
+  return res;
+}
