@@ -888,7 +888,7 @@ void KheCoordinateLayers(KHE_NODE parent_node, bool with_domination)
 
   /* do the coordinating proper */
   soln = KheNodeSoln(parent_node);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaArena Make(); */
   ins = KheSolnInstance(KheNodeSoln(parent_node));
   for( i = 0;  i < KheInstanceResourceTypeCount(ins);  i++ )
