@@ -237,7 +237,7 @@ bool KheNodeMerge(KHE_NODE node1, KHE_NODE node2, KHE_NODE *res)
 
   /* move the children of node2 to node1, saving assignments first */
   soln = KheNodeSoln(node1);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaAr enaMake(); */
   HaArrayInit(assts, a);
   while( KheNodeChildCount(node2) > 0 )
@@ -528,7 +528,7 @@ bool KheNodeMove(KHE_NODE child_node, KHE_NODE parent_node)
     {
       meet = KheNodeMeet(child_node, i);
       if( KheMeetAsst(meet) != NULL )
-	KheMeetUnAssign(meet);
+	KheMeetUnAssign(meet);  /* part of node move, so should be OK */
     }
 
     /* move child_node to parent_node */
@@ -541,7 +541,7 @@ bool KheNodeMove(KHE_NODE child_node, KHE_NODE parent_node)
   {
     /* parent_node is ancestor; unassign meets, remembering targets */
     soln = KheNodeSoln(child_node);
-    a = KheSolnArenaBegin(soln, false);
+    a = KheSolnArenaBegin(soln);
     /* a = HaAren aMake(); */
     HaArrayInit(assts, a);
     for( i = 0;  i < KheNodeMeetCount(child_node);  i++ )
@@ -555,7 +555,7 @@ bool KheNodeMove(KHE_NODE child_node, KHE_NODE parent_node)
         KheMeetAsstSaveFull(&assts, meet, target_meet, offset);
       }
       if( KheMeetAsst(meet) != NULL )
-	KheMeetUnAssign(meet);
+	KheMeetUnAssign(meet);   /* part of node move; will be reassigned */
     }
 
     /* move child_node to parent_node */
@@ -590,7 +590,7 @@ bool KheNodeMove(KHE_NODE child_node, KHE_NODE parent_node)
     {
       meet = KheNodeMeet(child_node, i);
       if( KheMeetAsst(meet) != NULL )
-	KheMeetUnAssign(meet);
+	KheMeetUn Assign(meet);
     }
 
     ** move child_node to parent_node **
@@ -612,7 +612,7 @@ bool KheNodeMove(KHE_NODE child_node, KHE_NODE parent_node)
       HaArrayAddLast(target_meets, target_meet);
       HaArrayAddLast(target_offsets, target_offset);
       if( KheMeetAsst(meet) != NULL )
-	KheMeetUnAssign(meet);
+	KheMeetUn Assign(meet);
     }
 
     ** move child_node to parent_node **
@@ -708,7 +708,7 @@ KHE_NODE KheNodeVizierMake(KHE_NODE parent_node)
 
   /* make vizier_node and its meets; save assignments to vizier */
   soln = KheNodeSoln(parent_node);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaAren aMake(); */
   HaArrayInit(meet_assts, a);
   vizier_node = KheNodeMake(soln);
@@ -859,7 +859,7 @@ void KheNodeVizierDelete(KHE_NODE parent_node)
 
   /* save the assignments of the meets of the child nodes of vizier_node */
   soln = KheNodeSoln(vizier_node);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaAren aMake(); */
   HaArrayInit(meet_assts, a);
   for( i = 0;  i < KheNodeMeetCount(vizier_node);  i++ )
@@ -875,7 +875,7 @@ void KheNodeVizierDelete(KHE_NODE parent_node)
 	  KheMeetAsstSaveFull(&meet_assts, child_meet, parent_meet,
 	    KheMeetAsstOffset(vizier_meet) + KheMeetAsstOffset(child_meet));
       }
-      KheMeetUnAssign(vizier_meet);
+      KheMeetUnAssign(vizier_meet);  /* vizier meet, will be deleted */
     }
   }
 
@@ -969,7 +969,7 @@ static void KheMoveChildNode(KHE_NODE child_node, KHE_NODE vizier_node)
 	KheMeetDebug(vizier_meet, 3, 0, stderr);
     }
     if( vizier_meet != NULL )
-      KheMeetUnAssign(child_meet);
+      KheMeetUnAss ign(child_meet);
   }
 
   ** move the node **
