@@ -1190,7 +1190,7 @@ static void HandleOneMergeableSet(ARRAY_KHE_RUNAROUND_NODE *nodes,
 	x = HaArray(*nodes, i);
 	durn = KheNodeDuration(n->node) + KheNodeDuration(x->node);
 	if( n->mrd <= durn && durn == u && durn <= v &&
-	    KheTryRunarounds(n, nodes, i, i, rs,options,durn,nts, a) )
+	    KheTryRunarounds(n, nodes, i, i, rs, options, durn, nts, a) )
 	  done = true;
       }
 
@@ -1253,7 +1253,7 @@ void KheBuildRunarounds(KHE_NODE parent_node,
 
   /* sort out node templates */
   soln = KheNodeSoln(parent_node);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaAren aMake(); */
   nts = KheNodeTemplateSetMakeFromNode(parent_node, a);
   if( DEBUG4 )
