@@ -294,7 +294,7 @@ static void KheDurnInfoFree(KHE_DURN_INFO di)
 KHE_SPLIT_ANALYSER KheSplitAnalyserMake(KHE_SOLN soln)
 {
   KHE_SPLIT_ANALYSER res;  HA_ARENA a;
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaArena Make(); */
   HaMake(res, a);
   res->arena = a;
@@ -536,7 +536,7 @@ void KheSplitAnalyserAnalyse(KHE_SPLIT_ANALYSER sa, KHE_EVENT e)
   for( i = 0;  i < KheSolnEventMonitorCount(sa->soln, e);  i++ )
   {
     m = KheSolnEventMonitor(sa->soln, e, i);
-    if( KheConstraintCombinedWeight(KheMonitorConstraint(m)) > 0 )
+    if( KheMonitorCombinedWeight(m) > 0 )
       switch( KheMonitorTag(m) )
       {
 	case KHE_SPLIT_EVENTS_MONITOR_TAG:
