@@ -659,7 +659,7 @@ static void KheTimeEquivGroupPreassignedResources(KHE_TIME_EQUIV te,
 KHE_TIME_EQUIV KheTimeEquivMake(KHE_SOLN soln)
 {
   KHE_TIME_EQUIV res;  HA_ARENA a;
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaArenaMa ke(); */
   HaMake(res, a);
   res->arena = a;
