@@ -138,7 +138,7 @@ static bool KheMakeBestAsst(KHE_MEET meet, KHE_NODE parent_node)
 	    best_offset = offset;
 	    best_cost = cost;
 	  }
-	  KheMeetUnAssign(meet);
+	  KheMeetUnAssign(meet);  /* OK: experimental asst now being removed */
 	}
       }
     }
@@ -319,7 +319,7 @@ bool KheNodeUnAssignTimes(KHE_NODE parent_node, KHE_OPTIONS options)
     {
       meet = KheNodeMeet(child_node, j);
       if( KheMeetAsst(meet) != NULL )
-	if( !KheMeetUnAssign(meet) )
+	if( !KheMeetUnAssign(meet) )  /* in KheNodeUnAssignTimes, which is OK */
 	  return false;
     }
   }
@@ -345,7 +345,7 @@ bool KheLayerUnAssignTimes(KHE_LAYER layer, KHE_OPTIONS options)
     {
       meet = KheNodeMeet(child_node, j);
       if( KheMeetAsst(meet) != NULL )
-	if( !KheMeetUnAssign(meet) )
+	if( !KheMeetUnAssign(meet) )  /* in KheLayerUnAssignTimes, unused */
 	  return false;
     }
   }
@@ -444,8 +444,11 @@ bool KheNodePreassignedAssignTimes(KHE_NODE root_node,
 	  KheMeetDebug(meet, 1, -1, stderr);
 	  fprintf(stderr, ", %s)\n", KheTimeId(t) == NULL ? "-" : KheTimeId(t));
 	}
+	if( KheMeetAssignIsFixed(meet) )
+	  KheMeetAssignUnFix(meet);
 	if( !KheMeetAssignTime(meet, t) )
-	  res = false;
+	  HnAbort("KheNodePreassignedAssignTimes failed to assign meet");
+	res = false;  /* will never happen */
       }
     }
   }
@@ -524,8 +527,8 @@ bool KheSolnTryMeetUnAssignments(KHE_SOLN soln)
     target_meet = KheMeetAsst(meet);
     offset = KheMeetAsstOffset(meet);
     cost = KheSolnCost(soln);
-    if( target_meet != NULL && !KheMeetIsPreassigned(meet, NULL) &&
-	KheMeetUnAssign(meet) )
+    if( target_meet != NULL && !KheMeetIsAssignedPreassigned(meet, NULL) &&
+	KheMeetUnAssign(meet) )   /* meet not preassigned */
     {
       if( KheSolnCost(soln) < cost )
       {
