@@ -715,8 +715,8 @@ static void KheCalIdleMonitorSetPriority(KHE_CAL_IDLE_MONITOR cim)
     if( cim_min_durn < 0 ) cim_min_durn = 0;
     cim_max_dev = cim_avail - cim_min_durn - cim->monitor_maximum;
     if( cim_max_dev < 0 ) cim_max_dev = 0;
-    cim->priqueue_key = cim_max_dev * KheConstraintCombinedWeight(
-      (KHE_CONSTRAINT) KheLimitIdleTimesMonitorConstraint(cim->monitor));
+    cim->priqueue_key = cim_max_dev * KheMonitorCombinedWeight(
+      (KHE_MONITOR) cim->monitor);
   }
   else
     cim->priqueue_key = 0;
@@ -1006,7 +1006,7 @@ static bool KheCalVertexClusterMonitorIsCompatible(KHE_CAL_VERTEX cv,
   /* check weight */
   cbtc = KheClusterBusyTimesMonitorConstraint(cbtm);
   offset = KheClusterBusyTimesMonitorOffset(cbtm);
-  combined_weight = KheConstraintCombinedWeight((KHE_CONSTRAINT) cbtc);
+  combined_weight = KheMonitorCombinedWeight((KHE_MONITOR) cbtm);
   if( combined_weight == 0 || combined_weight < cv->graph->min_cluster_weight )
     return false;
 
@@ -1097,7 +1097,7 @@ static bool KheCalVertexIdleMonitorIsCompatible(KHE_CAL_VERTEX cv,
 
   /* check weight */
   litc = KheLimitIdleTimesMonitorConstraint(litm);
-  combined_weight = KheConstraintCombinedWeight((KHE_CONSTRAINT) litc);
+  combined_weight = KheMonitorCombinedWeight((KHE_MONITOR) litm);
   if( combined_weight == 0 || combined_weight < cv->graph->min_idle_weight )
     return false;
 
@@ -2026,7 +2026,7 @@ void KheSolnClusterAndLimitMeetDomains(KHE_SOLN soln,
   if( DEBUG1 )
     fprintf(stderr, "[ KheSolnClusterAndLimitMeetDomains(soln, ...)\n");
   HnAssert(slack>=1.0, "KheSolnClusterAndLimitMeetDomains: slack out of range");
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   cg = KheCalGraphMake(soln, min_cluster_weight, min_idle_weight,
     slack, mbg, options, a);
   if( DEBUG2 )
