@@ -338,11 +338,13 @@ static bool KheElmRepair(KHE_ELM elm, KHE_ELM_DEMAND d,
 /*  Add d and its edges to the matching graph, and try to repair if          */
 /*  necessary.  Use shift to vary the starting position of the repair.       */
 /*                                                                           */
+/*  Variable res is included just to keep the compiler happy.                */
+/*                                                                           */
 /*****************************************************************************/
 
 static void KheElmAddDemandToWMatch(KHE_ELM elm, KHE_ELM_DEMAND d, int shift)
 {
-  int pre_unmatched, post_unmatched;
+  int pre_unmatched, post_unmatched;  bool res;
 
   /* check the number of unmatched nodes before and after unfixing d */
   pre_unmatched = KheElmBestUnmatched(elm);
@@ -355,6 +357,7 @@ static void KheElmAddDemandToWMatch(KHE_ELM elm, KHE_ELM_DEMAND d, int shift)
     d->demand_group->wmatch_category, 1);
   KheWMatchEval(elm->wmatch, &post_unmatched, &cost);
   *** */
+  res = false;
   if( DEBUG1 )
   {
     fprintf(stderr, "[ KheAddDemand(elm, ");
@@ -369,7 +372,7 @@ static void KheElmAddDemandToWMatch(KHE_ELM elm, KHE_ELM_DEMAND d, int shift)
     if( KheNodeZoneCount(KheLayerParentNode(KheElmLayer(elm))) > 0 )
     {
       /* have zones, so try first to obey them, then not */
-      KheElmRepair(elm, d, true, KHE_ZONE_EXACT, shift) ||
+      res = KheElmRepair(elm, d, true, KHE_ZONE_EXACT, shift) ||
       KheElmRepair(elm, d, true, KHE_ZONE_INEXACT, shift) ||
       KheElmRepair(elm, d, false, KHE_ZONE_EXACT, shift) ||
       KheElmRepair(elm, d, false, KHE_ZONE_INEXACT, shift);
@@ -377,12 +380,12 @@ static void KheElmAddDemandToWMatch(KHE_ELM elm, KHE_ELM_DEMAND d, int shift)
     else
     {
       /* no zones, so don't take them into account when splitting */
-      KheElmRepair(elm, d, true, KHE_ZONE_IGNORE, shift) ||
+      res = KheElmRepair(elm, d, true, KHE_ZONE_IGNORE, shift) ||
       KheElmRepair(elm, d, false, KHE_ZONE_IGNORE, shift);
     }
   }
   if( DEBUG1 )
-    fprintf(stderr, "]\n");
+    fprintf(stderr, "] res = %s\n", res ? "true" : "false");
 }
 
 
