@@ -23,7 +23,7 @@
 /*  along with this program; if not, write to the Free Software              */
 /*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
 /*                                                                           */
-/*  FILE:         khe_kempe_move.c                                           */
+/*  FILE:         khe_st_kempe.c                                             */
 /*  DESCRIPTION:  Kempe and ejecting meet moves                              */
 /*                                                                           */
 /*****************************************************************************/
@@ -38,6 +38,7 @@
 #define DEBUG2 0
 #define DEBUG3 0
 #define DEBUG4 0
+#define DEBUG5 0
 
 typedef HA_ARRAY(KHE_MEET) ARRAY_KHE_MEET;
 
@@ -632,7 +633,7 @@ bool KheKempeMeetMove(KHE_MEET meet, KHE_MEET target_meet,
   int py, curr_step_start, offs, i, orig_start_offset;
   KHE_MEET orig_target;  KHE_TRACE t;  char *err;  KHE_MEET_SET ms;
   KHE_MONITOR m;  KHE_TASK task;  KHE_GROUP_MONITOR kempe_gm;
-  int step_count, phase_count;
+  int step_count, phase_count;  KHE_TIME time;
 
   if( DEBUG1 )
   {
@@ -668,7 +669,15 @@ bool KheKempeMeetMove(KHE_MEET meet, KHE_MEET target_meet,
   if( !KheMeetHasPreassignedTask(meet, &task) )
   {
     /* make the move */
-    if( !KheMeetMove(meet, target_meet, offset) )
+    if( DEBUG5 )
+      fprintf(stderr, "KheKempeMeetMove calling KheMeetMove (a)\n");
+    if( KheMeetIsAssignedPreassigned(meet, &time) )
+    {
+      if( DEBUG1 )
+	fprintf(stderr, "  ] KheKempeMeetMove returning false (preassigned)\n");
+      return false;
+    }
+    if( !KheMeetMove(meet, target_meet, offset) )  /* not preassigned */
     {
       if( DEBUG1 )
 	fprintf(stderr, "  ] KheKempeMeetMove returning false (trivial)\n");
@@ -712,7 +721,11 @@ bool KheKempeMeetMove(KHE_MEET meet, KHE_MEET target_meet,
   KheTraceBegin(t);
   if( DEBUG1 )
     KheKempeDebugOneMove(1, meet, target_meet, offset);
-  if( !KheMeetMove(meet, target_meet, offset) )
+  if( DEBUG5 )
+    fprintf(stderr, "KheKempeMeetMove calling KheMeetMove (b)\n");
+  if( KheMeetIsAssignedPreassigned(meet, &time) )
+    return KheKempeFail(ms, t, "move", meet);
+  if( !KheMeetMove(meet, target_meet, offset) )  /* not preassigned */
     return KheKempeFail(ms, t, "move", meet);
   *demand += KheMeetDemand(meet);
   step_count = phase_count = 1;
@@ -809,7 +822,11 @@ bool KheKempeMeetMove(KHE_MEET meet, KHE_MEET target_meet,
 	  from_frame[py].start_offset;
 	if( DEBUG1 )
           KheKempeDebugOneMove(py, meet, to_frame[py].target_meet, offs);
-	if( !KheMeetMove(meet, to_frame[py].target_meet, offs) )
+	if( DEBUG5 )
+	  fprintf(stderr, "KheKempeMeetMove calling KheMeetMove (c)\n");
+	if( KheMeetIsAssignedPreassigned(meet, &time) )
+	  return KheKempeFail(ms, t, "move", meet);
+	if( !KheMeetMove(meet, to_frame[py].target_meet, offs) )  /* not pre */
 	  return KheKempeFail(ms, t, "move", meet);
 	*demand += KheMeetDemand(meet);
 	step_count++;
@@ -1210,7 +1227,7 @@ bool KheEjectingMeetMove(KHE_MEET meet, KHE_MEET target_meet, int offset,
 {
   KHE_NODE node[2];  int init_irreg[2], final_irreg[2];  int i, j, pos;
   ARRAY_KHE_MEET meets;  KHE_TRACE t;  bool res;  KHE_SOLN soln;  HA_ARENA a;
-  KHE_MONITOR m, c;  KHE_TASK task;  KHE_GROUP_MONITOR kempe_gm;
+  KHE_MONITOR m, c;  KHE_TASK task;  KHE_GROUP_MONITOR kempe_gm;  KHE_TIME time;
 
   if( DEBUG4 )
   {
@@ -1235,7 +1252,15 @@ bool KheEjectingMeetMove(KHE_MEET meet, KHE_MEET target_meet, int offset,
   /* if there are no preassigned tasks, do an ordinary move */
   if( !KheMeetHasPreassignedTask(meet, &task) )
   {
-    res = KheMeetMove(meet, target_meet, offset);
+    if( DEBUG5 )
+      fprintf(stderr, "KheEjectingMeetMove calling KheMeetMove (d)\n");
+    if( KheMeetIsAssignedPreassigned(meet, &time) )
+    {
+      if( DEBUG4 )
+	fprintf(stderr, "  ] KheEjectingMeetMove returning false (preass)\n");
+      return false;
+    }
+    res = KheMeetMove(meet, target_meet, offset);  /* not preassigned */
     *demand += KheMeetDemand(meet);
     if( DEBUG4 )
       fprintf(stderr, "  ] KheEjectingMeetMove returning %s (basic move)\n",
@@ -1273,11 +1298,15 @@ bool KheEjectingMeetMove(KHE_MEET meet, KHE_MEET target_meet, int offset,
 
   /* move meet */
   soln = KheMeetSoln(meet);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaA renaMake(); */
   HaArrayInit(meets, a);
   KheTraceBegin(t);
-  if( !KheMeetMove(meet, target_meet, offset) )
+  if( DEBUG5 )
+    fprintf(stderr, "KheEjectingMeetMove calling KheMeetMove (e)\n");
+  if( KheMeetIsAssignedPreassigned(meet, &time) )
+    return KheEjectingFail(soln, a, t, "move", meet);
+  if( !KheMeetMove(meet, target_meet, offset) )  /* not preassigned */
     return KheEjectingFail(soln, a, t, "move", meet);
   *demand += KheMeetDemand(meet);
   if( DEBUG4 )
@@ -1352,8 +1381,12 @@ bool KheEjectingMeetMove(KHE_MEET meet, KHE_MEET target_meet, int offset,
 
   /* eject all these meets */
   HaArrayForEach(meets, meet, i)
-    if( !KheMeetUnAssign(meet) )
+  {
+    if( KheMeetIsAssignedPreassigned(meet, &time) )
       return KheEjectingFail(soln, a, t, "unassign", NULL);
+    if( !KheMeetUnAssign(meet) )  /* OK; meet is not preassigned */
+      return KheEjectingFail(soln, a, t, "unassign", NULL);
+  }
   KheSolnArenaEnd(soln, a);
   /* HaAre naDelete(a); */
   /* MArrayFree(meets); */
@@ -1416,7 +1449,7 @@ bool KheEjectingMeetMoveTime(KHE_MEET meet, KHE_TIME t,
 bool KheBasicMeetMove(KHE_MEET meet, KHE_MEET target_meet,
   int offset, bool preserve_regularity, int *demand)
 {
-  KHE_NODE node[2];  int init_irreg[2], final_irreg[2];
+  KHE_NODE node[2];  int init_irreg[2], final_irreg[2];  KHE_TIME time;
 
   if( DEBUG4 )
   {
@@ -1454,7 +1487,15 @@ bool KheBasicMeetMove(KHE_MEET meet, KHE_MEET target_meet,
   }
 
   /* move meet */
-  if( !KheMeetMove(meet, target_meet, offset) )
+  if( DEBUG5 )
+    fprintf(stderr, "KheBasicMeetMove calling KheMeetMove (f)\n");
+  if( KheMeetIsAssignedPreassigned(meet, &time) )
+  {
+    if( DEBUG4 )
+      fprintf(stderr, "  ] KheBasicMeetMove returning false (preasst)");
+    return false;
+  }
+  if( !KheMeetMove(meet, target_meet, offset) )  /* not preassigned */
   {
     if( DEBUG4 )
       fprintf(stderr, "  ] KheBasicMeetMove returning false (move)");
