@@ -463,7 +463,7 @@ static void KheDemandNodeUnAssign(KHE_DEMAND_NODE dn)
 {
   KHE_MEET meet;  int i;
   HaArrayForEach(dn->meets, meet, i)
-    KheMeetUnAssign(meet);
+    KheMeetUnAssign(meet);  /* layer matching, not sure about this */
 }
 
 
@@ -588,7 +588,7 @@ static bool KheDemandNodeEdgeFn(KHE_DEMAND_NODE dn,
 
     /* unassign everything that was assigned and return true if all */
     for( j = 0;  j < i;  j++ )
-      KheMeetUnAssign(HaArray(dn->meets, j));
+      KheMeetUnAssign(HaArray(dn->meets, j));  /* OK, unassigns prev asst */
     /* *cost = (cost_after - cost_before) + COST_ADJUST; */
     *cost = cost_after;
     return i == HaArrayCount(dn->meets);
@@ -999,7 +999,7 @@ bool KheLayerNodeMatchingNodeRepairTimes(KHE_NODE parent_node,
   KHE_LAYER_NODE_SOLVER lns;  bool add_layers;  KHE_SOLN soln;
   KHE_COST initial_cost, final_cost;  HA_ARENA a;
   soln = KheNodeSoln(parent_node);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   initial_cost = KheSolnCost(soln);
   if( DEBUG1 )
     fprintf(stderr, "[ KheLayerNodeMatchingNodeRepairTimes(Node %d): %.5f\n",
@@ -1042,7 +1042,7 @@ bool KheLayerNodeMatchingLayerRepairTimes(KHE_LAYER layer, KHE_OPTIONS options)
   KHE_LAYER_NODE_SOLVER lns;  KHE_SOLN soln;
   KHE_COST initial_cost, final_cost;  HA_ARENA a;
   soln = KheLayerSoln(layer);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   initial_cost = KheSolnCost(soln);
   if( DEBUG1 )
     fprintf(stderr, "[ KheLayerNodeMatchingLayerRepairTimes(Layer %d): %.5f\n",
