@@ -40,6 +40,7 @@
 #define DEBUG7 0
 #define DEBUG8 0
 #define DEBUG9 0
+#define DEBUG10 0
 
 typedef HA_ARRAY(KHE_MEET) ARRAY_KHE_MEET;
 typedef HA_ARRAY(KHE_NODE) ARRAY_KHE_NODE;
@@ -410,7 +411,7 @@ static bool KheNodeParallelAssignTimes(KHE_NODE child_node, KHE_LAYER layer)
 
   /* find the nodes of layer that overlap child_node */
   soln = KheNodeSoln(child_node);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaArena Make(); */
   HaArrayInit(nodes, a);
   for( i = 0;  i < KheLayerChildNodeCount(layer);  i++ )
@@ -501,7 +502,7 @@ bool KheLayerParallelAssignTimes(KHE_LAYER layer, KHE_OPTIONS options)
 
   /* get the layer_infos and sort them by decreasing no of preassigned meets */
   soln = KheLayerSoln(layer);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaAr enaMake(); */
   HaArrayInit(layer_infos, a);
   parent_node = KheLayerParentNode(layer);
@@ -706,12 +707,32 @@ static void KheSolnDebugDefectsWithTag(KHE_SOLN soln, KHE_MONITOR_TAG tag)
 
 /*****************************************************************************/
 /*                                                                           */
-/*  void KheLayerRepair(KHE_LAYER layer, KHE_OPTIONS options, KHE_BACKOFF bk)*/
+/*  void KheLayerDebugUnassigned(KHE_LAYER layer, int verbosity,             */
+/*    int indent, FILE *fp)                                                  */
 /*                                                                           */
-/*  Carry out the repair of layer requested by option time_layer_repair.     */
+/*  Debug print of the unassigned children of layer.                         */
 /*                                                                           */
 /*****************************************************************************/
 
+static void KheLayerDebugUnassigned(KHE_LAYER layer, int verbosity,
+  int indent, FILE *fp)
+{
+  int i, j;  KHE_NODE child_node;  KHE_MEET meet;
+  fprintf(stderr, "%*s[ unassigned meets of layer:\n", indent, "");
+  for( i = 0;  i < KheLayerChildNodeCount(layer);  i++ )
+  {
+    child_node = KheLayerChildNode(layer, i);
+    for( j = 0;  j < KheNodeMeetCount(child_node);  j++ )
+    {
+      meet = KheNodeMeet(child_node, j);
+      if( KheMeetAsst(meet) == NULL )
+	KheMeetDebug(meet, verbosity, indent + 2, fp);
+    }
+  }
+  fprintf(stderr, "%*s]\n", indent, "");
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  void KheLayerRepair(KHE_LAYER layer, KHE_LAYER_REPAIR_TYPE repair_type,  */
@@ -724,6 +745,11 @@ static void KheSolnDebugDefectsWithTag(KHE_SOLN soln, KHE_MONITOR_TAG tag)
 static void KheLayerRepair(KHE_LAYER layer, KHE_LAYER_REPAIR_TYPE repair_type,
   KHE_OPTIONS options, KHE_BACKOFF bk)
 {
+  if( DEBUG10 )
+  {
+    fprintf(stderr, "[ KheLayerRepair(layer, ...), layer:\n");
+    KheLayerDebugUnassigned(layer, 2, 2, stderr);
+  }
   switch( repair_type )
   {
     case KHE_LAYER_REPAIR_NONE:
@@ -735,19 +761,25 @@ static void KheLayerRepair(KHE_LAYER layer, KHE_LAYER_REPAIR_TYPE repair_type,
 
       /* repair the layer with exponential backoff */
       if( KheBackoffAcceptOpportunity(bk) )
-	KheBackoffResult(bk, KheEjectionChainLayerRepairTimes(layer, options));
+	KheBackoffResult(bk,
+	  KheEjectionChainLayerRepairTimes(layer, options, NULL));
       break;
 
     case KHE_LAYER_REPAIR_ALL:
 
       /* repair the layer without exponential backoff */
-      KheEjectionChainLayerRepairTimes(layer, options);
+      KheEjectionChainLayerRepairTimes(layer, options, NULL);
       break;
 
     default:
 
       HnAbort("KheLayerRepair: invalid repair_type (%d)", repair_type);
-
+  }
+  if( DEBUG10 )
+  {
+    fprintf(stderr, "  at end of KheLayerRepair:\n");
+    KheLayerDebugUnassigned(layer, 2, 2, stderr);
+    fprintf(stderr, "] KheLayerRepair returning\n");
   }
 }
 
@@ -840,13 +872,15 @@ static void KheOneLayerAsstDebug(KHE_NODE parent_node, int i,
   bool reversed, FILE *fp)
 {
   KHE_LAYER layer;  int j, k;  KHE_NODE child_node;  KHE_MEET meet;
-  KHE_SOLN soln;
+  KHE_SOLN soln;  bool all_assigned;
   soln = KheNodeSoln(parent_node);
   layer = KheNodeChildLayer(parent_node, i);
-  fprintf(stderr, "  after assigning %slayer %d of %d (%d assigned of %d):\n",
+  all_assigned = (KheLayerAssignedDuration(layer) >= KheLayerDuration(layer));
+  fprintf(stderr, "  after assigning %slayer %d of %d (%d assigned of %d%s):\n",
     reversed ? "reversed " : "", i, KheNodeChildLayerCount(parent_node),
-    KheLayerAssignedDuration(layer), KheLayerDuration(layer));
-  KheLayerDebug(layer, 1, 4, stderr);
+    KheLayerAssignedDuration(layer), KheLayerDuration(layer),
+    all_assigned ? "" : " - not all assigned");
+  KheLayerDebug(layer, all_assigned ? 1 : 3, 4, stderr);
   for( j = 0;  j < KheLayerChildNodeCount(layer);  j++ )
   {
     child_node = KheLayerChildNode(layer, j);
@@ -856,13 +890,13 @@ static void KheOneLayerAsstDebug(KHE_NODE parent_node, int i,
       if( KheMeetAsst(meet) == NULL )
       {
 	fprintf(stderr, "    unassigned meet ");
-	KheMeetDebug(meet, 1, 0, stderr);
+	KheMeetDebug(meet, 3, 0, stderr);
       }
     }
   }
   KheSolnCostByTypeDebug(soln, 2, 4, stderr);
-  fflush(stderr);
   /* ***
+  fflush(stderr);
   KheSolnDebugDefectsWithTag(soln, KHE_ORDINARY_DEMAND_MONITOR_TAG);
   KheSolnDebugDefectsWithTag(soln, KHE_SPREAD_EVENTS_MONITOR_TAG);
   KheSolnDebugDefectsWithTag(soln, KHE_EVENNESS_MONITOR_TAG);
@@ -1063,7 +1097,7 @@ bool KheNodeLayeredAssignTimes(KHE_NODE parent_node, KHE_OPTIONS options)
     fprintf(stderr, "[ KheNodeLayeredAssignTimes(Node %d, %s)\n",
       KheNodeSolnIndex(parent_node), no_node_regularity ? "true" : "false");
   soln = KheNodeSoln(parent_node);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaAre naMake(); */
   ins = KheSolnInstance(soln);
   best_sec = no_node_regularity ? NULL : BestSpreadEventsConstraint(ins);
