@@ -293,7 +293,7 @@ static void KheTargetRangeGroup(ARRAY_KHE_TARGET_RANGE *target_ranges)
 KHE_MEET_SET_SOLVER KheMeetSetSolveBegin(KHE_SOLN soln, int max_meets)
 {
   KHE_MEET_SET_SOLVER res;  HA_ARENA a;
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaAren aMake(); */
   HaMake(res, a);
   res->arena = a;
@@ -383,7 +383,7 @@ static bool KheMeetSetMeetAssign(KHE_MEET_SET_SOLVER mss,
   /* fail if tixels increase */
   if( KheSolnMatchingDefectCount(mss->soln) > mss->init_tixels )
   {
-    KheMeetUnAssign(meet);
+    KheMeetUnAssign(meet);   /* OK, unassigns previous asst */
     return false;
   }
 
@@ -433,7 +433,7 @@ static void KheMeetSetDoSolve(KHE_MEET_SET_SOLVER mss, int pos, int *end_index)
         if( KheMeetSetMeetAssign(mss, mo->meet, tr->target_meet, offs) )
 	{
           KheMeetSetDoSolve(mss, pos + 1, &junk);
-	  KheMeetUnAssign(mo->meet);
+	  KheMeetUnAssign(mo->meet);   /* OK, unassigns previous asst */
 	  if( mss->node_count > MAX_NODE_COUNT )
 	    goto TRUNCATE;
 	}
@@ -525,7 +525,7 @@ bool KheMeetSetSolveEnd(KHE_MEET_SET_SOLVER mss)
   mark = KheMarkBegin(mss->soln);
   HaArrayForEach(mss->meet_objs, mo, i)
     if( KheMeetAsst(mo->meet) != NULL )
-      KheMeetUnAssign(mo->meet);
+      KheMeetUnAssign(mo->meet);  /* not sure; used by fuzzy meet moves */
   KheMeetSetDoSolve(mss, 0, &end_index);
   success = mss->best_cost < mss->init_cost;
   KheMarkEnd(mark, !success);
@@ -564,13 +564,13 @@ bool KheMeetSetSolveEnd(KHE_MEET_SET_SOLVER mss)
 
 
 /* *** old version, uses transactions
-bool KheMeetSetSolveEnd(KHE_MEET_SET_SOLVER mss)
+bool KheMeet SetSolveEnd(KHE_MEET_SET_SOLVER mss)
 {
   KHE_MEET meet, target_meet;  int i, end_index;  KHE_TARGET_RANGE tr;
   KHE_MEET_OBJ mo;  bool res;
 
   if( DEBUG1 )
-    fprintf(stderr, "[ KheMeetSetSolveEnd(mss):\n");
+    fprintf(stderr, "[ KheMeetS etSolveEnd(mss):\n");
 
   ** find and group target ranges **
   HaArrayForEach(mss->meet_objs, mo, i)
@@ -597,7 +597,7 @@ bool KheMeetSetSolveEnd(KHE_MEET_SET_SOLVER mss)
   KheTransactionBegin(mss->init_trans);
   HaArrayForEach(mss->meet_objs, mo, i)
     if( KheMeetAsst(mo->meet) != NULL )
-      KheMeetUnAssign(mo->meet);
+      KheMeetUn Assign(mo->meet);
   KheTransactionEnd(mss->init_trans);
 
   ** carry out the tree search and finalize the assignment, new or old **
@@ -799,7 +799,7 @@ bool KheFuzzyMeetMove(KHE_MEET meet, KHE_MEET target_meet, int offset,
 
   /* get the meets and resources within the given limits */
   soln = KheMeetSoln(meet);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   /* a = HaArena Make(); */
   HnAssert(KheMeetAsst(meet) != NULL, "KheFuzzyMeetMove: meet is not assigned");
   HaArrayInit(meets, a);
