@@ -892,7 +892,7 @@ static bool KheTryAsst(KHE_MEET meet, KHE_MEET target_meet, int target_offset,
 	}
 	return true;
       }
-      KheMeetUnAssign(meet);
+      KheMeetUnAssign(meet);  /* OK; undoes previous asst */
     }
   }
   return false;
@@ -1246,7 +1246,7 @@ bool KheRunaroundNodeAssignTimes(KHE_NODE parent_node, KHE_OPTIONS options)
 
   /* build a runaround solver for this problem */
   soln = KheNodeSoln(parent_node);
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   rs = KheRunaroundSolverMake(parent_node, a);
   if( DEBUG1 )
     KheRunaroundSolverDebug(rs, 2, 2, stderr);
