@@ -729,7 +729,7 @@ static bool KheMeetObjectAssign(KHE_MEET_OBJ mo, int time_index)
   ** fail if the number of unassigned demand tixels has increased **
   if( KheSolnMatchingDefectCount(mo->solver->soln) > unassigned_tixels )
   {
-    ** KheMeetUnAssignTime(mo->meet); **
+    ** KheMeetUnAss ignTime(mo->meet); **
     return false;
   }
 
@@ -738,7 +738,7 @@ static bool KheMeetObjectAssign(KHE_MEET_OBJ mo, int time_index)
   if( mo->solver->unassignable > 0 )
   {
     KheMeetObjUnApplyExclusions(mo, time_index);
-    ** KheMeetUnAssignTime(mo->meet); **
+    ** KheMeetUnAss ignTime(mo->meet); **
     return false;
   }
 
@@ -762,7 +762,7 @@ static bool KheMeetObjectAssign(KHE_MEET_OBJ mo, int time_index)
 static void KheMeetObjectUnAssign(KHE_MEET_OBJ mo, int time_index)
 {
   KheMeetObjUnApplyExclusions(mo, time_index);
-  ** KheMeetUnAssignTime(mo->meet); **
+  ** KheMeetUnAss ignTime(mo->meet); **
 }
 *** */
 
@@ -974,7 +974,7 @@ bool KheTreeSearchLayerRepairTimes(KHE_SOLN soln, KHE_RESOURCE r)
       KheResourceId(r));
 
   /* make a solver */
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
   tls = KheTreeLayerSolverMake(soln, r, a);
 
   /* reduce domains, using meet domains and resource unavailable times */
@@ -1007,7 +1007,7 @@ bool KheTreeSearchLayerRepairTimes(KHE_SOLN soln, KHE_RESOURCE r)
   KheMarkAddBestPath(tls->root_mark, 1);  /* curr soln is one option */
   HaArrayForEach(tls->meet_objs, mo1, i)
     if( KheMeetAsst(mo1->meet) != NULL )
-      KheMeetUnAssign(mo1->meet);
+      KheMeetUnAssign(mo1->meet);  /* part of KheTreeSearchLayerRepairTimes */
   KheTreeLayerSolve(tls, 0, &end_index);
   KheMarkUndo(tls->root_mark);
   KhePathRedo(KheMarkPath(tls->root_mark, 0));
@@ -1035,12 +1035,12 @@ bool KheTreeSearchLayerRepairTimes(KHE_SOLN soln, KHE_RESOURCE r)
 
 
 /* *** old version, uses transactions
-bool KheTreeSearchLayerRepairTimes(KHE_SOLN soln, KHE_RESOURCE r)
+bool KheTreeSearchLa yerRepairTimes(KHE_SOLN soln, KHE_RESOURCE r)
 {
   KHE_TREE_LAYER_SOLVER tls;  int i, j, end_index;  KHE_MEET_OBJ mo1, mo2;
   KHE_EXCLUSION_SET es;  bool res;  KHE_TIME time;
   if( DEBUG1 )
-    fprintf(stderr, "[ KheTreeSearchLayerRepairTimes(soln, %s)\n",
+    fprintf(stderr, "[ KheTreeSearc hLayerRepairTimes(soln, %s)\n",
       KheResourceId(r));
 
   ** make a solver **
@@ -1075,7 +1075,7 @@ bool KheTreeSearchLayerRepairTimes(KHE_SOLN soln, KHE_RESOURCE r)
   KheTransactionBegin(tls->init_trans);
   HaArrayForEach(tls->meet_objs, mo1, i)
     if( KheMeetAsst(mo1->meet) != NULL )
-      KheMeetUnAssign(mo1->meet);
+      KheMeetUnA ssign(mo1->meet);
   KheTransactionEnd(tls->init_trans);
 
   ** carry out the tree search and finalize the assignment, new or old **
@@ -1087,10 +1087,10 @@ bool KheTreeSearchLayerRepairTimes(KHE_SOLN soln, KHE_RESOURCE r)
     {
       time = KheInstanceTime(mo1->instance, mo1->best_asst);
       if( !KheMeetAssignTime(mo1->meet, time) )
-	HnAbort("KheTreeSearchLayerRepairTimes internal error 1");
+	HnAbort("KheTreeSearchLa yerRepairTimes internal error 1");
     }
     HnAssert(KheSolnCost(soln) == tls->bes t_cost,
-      "KheTreeSearchLayerRepairTimes internal error 2");
+      "KheTreeSearchLay erRepairTimes internal error 2");
     res = true;
   }
   else
@@ -1102,18 +1102,18 @@ bool KheTreeSearchLayerRepairTimes(KHE_SOLN soln, KHE_RESOURCE r)
 
   ** delete the solver and return **
   HnAssert(KheSolnCost(soln) <= tls->init_cost,
-    "KheTreeSearchLayerRepairTimes internal error 2");
+    "KheTreeSearchLayer RepairTimes internal error 2");
   if( DEBUG1 )
   {
     fprintf(stderr, "  assign %d, bottom %d, end %d\n",
 	tls->assign_count, tls->bottom_count, end_index);
     if( res )
       fprintf(stderr,
-	"] KheTreeSearchLayerRepairTimes returning true (%.5f -> %.5f)\n",
+	"] KheTreeSearchLay erRepairTimes returning true (%.5f -> %.5f)\n",
 	KheCostShow(tls->init_cost), KheCostShow(tls->bes t_cost));
     else
       fprintf(stderr,
-	"] KheTreeSearchLayerRepairTimes ret false\n");
+	"] KheTreeSearchLay erRepairTimes ret false\n");
   }
   KheTreeLayerSolverDelete(tls);
   return res;
