@@ -28,6 +28,7 @@
 /*                                                                           */
 /*****************************************************************************/
 #include "khe_solvers.h"
+#define min(a, b) ((a) < (b) ? (a) : (b))
 
 #define DEBUG1 0
 #define DEBUG2 0
@@ -212,6 +213,20 @@ static KHE_MONITOR_FRAME KheMonitorFrameMake(KHE_FRAME frame, HA_ARENA a)
 /*                                                                           */
 /*****************************************************************************/
 
+static void KheMonitorFrameAddResourceMonitor(KHE_MONITOR_FRAME mf,
+  KHE_MONITOR m, KHE_TIME first_time, KHE_TIME last_time)
+{
+  int first_index, last_index, index;  KHE_MONITOR_FRAME_PART part;
+  first_index = KheFrameTimeIndex(mf->frame, first_time);
+  last_index = KheFrameTimeIndex(mf->frame, last_time);
+  for( index = first_index;  index <= last_index;  index++ )
+  {
+    part = HaArray(mf->parts, index);
+    HaArrayAddLast(part->resource_monitors, m);
+  }
+}
+
+/* old version
 static void KheMonitorFrameAddResourceMonitor(KHE_MONITOR_FRAME mf,
   KHE_MONITOR m, KHE_TIME time)
 {
@@ -223,6 +238,7 @@ static void KheMonitorFrameAddResourceMonitor(KHE_MONITOR_FRAME mf,
     HaArrayAddLast(part->resource_monitors, m);
   }
 }
+*** */
 
 
 /*****************************************************************************/
@@ -268,8 +284,7 @@ static void KheMonitorFrameAddAndDetachTaskMonitor(KHE_MONITOR_FRAME mf,
 static void KheMonitorFrameAddAndCutOffAllResourceMonitors(KHE_MONITOR_FRAME mf,
   KHE_RESOURCE_GROUP rg, KHE_SOLN soln)
 {
-  KHE_TIME t;  int i, j;  KHE_CLUSTER_BUSY_TIMES_MONITOR cbtm;
-  KHE_RESOURCE r;  KHE_LIMIT_ACTIVE_INTERVALS_MONITOR laim;  KHE_MONITOR m;
+  KHE_TIME first_time, last_time;  int i, j;  KHE_RESOURCE r;  KHE_MONITOR m;
   if( DEBUG6 )
     fprintf(stderr, "[ KheMonitorFrameAddAndCutOffAllResourceMonitors(...):\n");
   for( i = 0;  i < KheResourceGroupResourceCount(rg);  i++ )
@@ -278,6 +293,12 @@ static void KheMonitorFrameAddAndCutOffAllResourceMonitors(KHE_MONITOR_FRAME mf,
     for( j = 0;  j < KheSolnResourceMonitorCount(soln, r);  j++ )
     {
       m = KheSolnResourceMonitor(soln, r, j);
+      if( KheMonitorCutoffTimeRange(m, &first_time, &last_time) )
+      {
+	KheMonitorSetCutoffTime(m, NULL);
+        KheMonitorFrameAddResourceMonitor(mf, m, first_time, last_time);
+      }
+      /* ***
       if( KheMonitorTag(m) == KHE_LIMIT_ACTIVE_INTERVALS_MONITOR_TAG )
       {
 	laim = (KHE_LIMIT_ACTIVE_INTERVALS_MONITOR) m;
@@ -297,6 +318,7 @@ static void KheMonitorFrameAddAndCutOffAllResourceMonitors(KHE_MONITOR_FRAME mf,
 	if( DEBUG6 )
 	  KheMonitorDebug(m, 2, 2, stderr);
       }
+      *** */
     }
   }
   if( DEBUG6 )
@@ -719,6 +741,268 @@ static void DebugStart(KHE_SOLN soln, KHE_OPTIONS options, KHE_FRAME frame,
 }
 
 
+/*****************************************************************************/
+/*                                                                           */
+/*  KHE_TIME_SWEEP_OPTIONS - options for controlling time sweep              */
+/*                                                                           */
+/*****************************************************************************/
+
+typedef struct khe_time_sweep_options_rec {
+  KHE_FRAME	frame;
+  int		frame_len;
+  float		daily_time_limit;
+  /* KHE_TIMER	daily_timer; */
+  KHE_TIMER	st_cutoffs;
+  float		time_cutoffs;
+  KHE_TIMER	st_matches;
+  float		time_matches;
+  bool		edge_adjust1_off;
+  bool		edge_adjust2_off;
+  bool		edge_adjust3_off;
+  bool		edge_adjust4_off;
+  bool		ejection_off;
+  int		lookahead;
+  bool		preserve_existing;
+  bool		cutoff_off;
+  bool		redo_off;
+  bool		rematch_off;
+  int		rematch_max_groups;
+} *KHE_TIME_SWEEP_OPTIONS;
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*  void KheTimeSweepDoAssignResources(KHE_SOLN soln, KHE_RESOURCE_GROUP rg, */
+/*    KHE_TIME_SWEEP_OPTIONS tso, KHE_OPTIONS options, HA_ARENA a)           */
+/*                                                                           */
+/*  Do most of the work of KheTimeSweepAssignResources.                      */
+/*                                                                           */
+/*****************************************************************************/
+
+static void KheTimeSweepDoAssignResources(KHE_SOLN soln, KHE_RESOURCE_GROUP rg,
+  KHE_TIME_SWEEP_OPTIONS tso, KHE_OPTIONS options, HA_ARENA a)
+{
+  KHE_RESOURCE_MATCHING_SOLVER rms;  KHE_TIME_GROUP tg;
+  int frame_pos, init_len;  KHE_TIMER daily_timer;
+  int count, p, i, redo_count;  KHE_COST init_cost;
+  bool rematch_redo;  ARRAY_KHE_RESOURCE_MATCHING_DEMAND_SET demand_sets;
+  KHE_RESOURCE_MATCHING_DEMAND_SET rmds;  KHE_MONITOR_FRAME mf;
+  rms = KheResourceMatchingSolverMake(soln, rg, a);
+
+  if( DEBUG8 )
+    KheFrameAssertNoClashes(tso->frame);
+
+  /* install cutoffs, if required */
+  if( !tso->cutoff_off )
+  {
+    mf = KheMonitorFrameMake(tso->frame, a);
+    KheMonitorFrameAddAndCutOffAllResourceMonitors(mf, rg, soln);
+    for( frame_pos = 0;  frame_pos < tso->lookahead;  frame_pos++ )
+      KheMonitorFrameUpdateResourceMonitorCutoffs(mf, frame_pos);
+    KheMonitorFrameAddAndDetachTaskMonitors(mf, soln);
+    if( DEBUG5 )
+      KheMonitorFrameDebug(mf, 2, stderr);
+  }
+  else
+    mf = NULL;  /* keep compiler happy */
+  rematch_redo = false;
+
+  /* create timer for daily time limit, if required */
+  if( tso->daily_time_limit >= 0.0 )
+  {
+    daily_timer = KheOptionsAddTimer(options, "daily",
+      tso->daily_time_limit);
+    if( DEBUG2 )
+      fprintf(stderr,
+	"  KheTimeSweepAssignResources daily time limit %.1f secs\n",
+        tso->daily_time_limit);
+  }
+  else
+    daily_timer = NULL;
+
+  /* get demand sets up to just before position lookahead */
+  HaArrayInit(demand_sets, a);
+  init_len = min(tso->lookahead, tso->frame_len);
+  for( frame_pos = 0;  frame_pos < init_len;  frame_pos++ )
+  {
+    rmds = KheResourceMatchingDemandSetMake(rms, tso->preserve_existing);
+    tg = KheFrameTimeGroup(tso->frame, frame_pos);
+    KheResourceMatchingDemandSetAddTimeGroup(rmds, tg);
+    HaArrayAddLast(demand_sets, rmds);
+  }
+
+  /* do one solve for each time group of frame */
+  for( frame_pos = 0;  frame_pos < tso->frame_len;  frame_pos++ )
+  {
+    DebugStart(soln, options, tso->frame, frame_pos, frame_pos, "construct");
+    if( DEBUG8 )
+      KheFrameAssertNoClashes(tso->frame);
+
+    /* start daily time limit, if requested */
+    if( tso->daily_time_limit >= 0.0 )
+      KheTimerResetStartTime(daily_timer);
+
+    /* get demand set for position frame_pos + lookahead  */
+    if( frame_pos + tso->lookahead < tso->frame_len )
+    {
+      rmds = KheResourceMatchingDemandSetMake(rms, tso->preserve_existing);
+      tg = KheFrameTimeGroup(tso->frame, frame_pos + tso->lookahead);
+      KheResourceMatchingDemandSetAddTimeGroup(rmds, tg);
+      HaArrayAddLast(demand_sets, rmds);
+    }
+
+    /* install cutoffs, unless cutoff_off */
+    if( !tso->cutoff_off )
+    {
+      if( DEBUG3 )
+	KheTimerResetStartTime(tso->st_cutoffs);
+      if( frame_pos + tso->lookahead < tso->frame_len )
+	KheMonitorFrameUpdateResourceMonitorCutoffs(mf,
+	  frame_pos + tso->lookahead);
+      KheMonitorFrameAttachTaskMonitors(mf, frame_pos);
+      if( DEBUG3 )
+	tso->time_cutoffs += KheTimerElapsedTime(tso->st_cutoffs);
+    }
+
+    /* do the solve */
+    if( DEBUG3 )
+      KheTimerResetStartTime(tso->st_matches);
+    DebugStart(soln, options, tso->frame, frame_pos, frame_pos, "solve");
+    KheResourceMatchingSolverSolveWithLookahead(rms, &demand_sets, frame_pos,
+      min(frame_pos + tso->lookahead, tso->frame_len - 1),
+      tso->edge_adjust1_off, tso->edge_adjust2_off, tso->edge_adjust3_off,
+      tso->edge_adjust4_off, tso->ejection_off, /* nocost_off, */ options);
+    DebugStart(soln, options, tso->frame, frame_pos, frame_pos, "after-solve");
+    KheDebugTimetable(soln, "after solving %d", frame_pos);
+    if( DEBUG8 )
+      KheFrameAssertNoClashes(tso->frame);
+
+    /* clear demand set for frame_pos, if not being saved */
+    if( !KheSaveDemandSets(tso->frame) )
+    {
+      rmds = HaArray(demand_sets, frame_pos);
+      KheResourceMatchingDemandSetDelete(rmds);
+      HaArrayPut(demand_sets, frame_pos, NULL);
+    }
+    if( DEBUG3 )
+      tso->time_matches += KheTimerElapsedTime(tso->st_matches);
+
+    /* rematch, unless rematch_off */
+    if( !tso->rematch_off && !KheOptionsTimeLimitReached(options) )
+    {
+      rmds = KheResourceMatchingDemandSetMake(rms, false);
+      count = min(tso->rematch_max_groups, frame_pos + 1);
+      do
+      {
+	if( DEBUG4 )
+	  fprintf(stderr, "  and rematching 2 .. %d parts\n", count);
+	init_cost = KheSolnCost(soln);
+	for( p = 2;  p <= count && !KheOptionsTimeLimitReached(options);  p++ )
+	{
+	  for( i = 0;  i < p;  i++ )
+	  {
+	    tg = KheFrameTimeGroup(tso->frame, frame_pos - i);
+	    KheResourceMatchingDemandSetAddTimeGroup(rmds, tg);
+	  }
+	  DebugStart(soln, options, tso->frame, frame_pos - p + 1, frame_pos,
+	    "rematch");
+	  if( DEBUG3 )
+	    KheTimerResetStartTime(tso->st_matches);
+	  if( DEBUG8 )
+	    KheFrameAssertNoClashes(tso->frame);
+	  KheResourceMatchingSolverSolve(rms, rmds, tso->edge_adjust1_off,
+	    tso->edge_adjust2_off, tso->edge_adjust3_off, tso->edge_adjust4_off,
+	    tso->ejection_off, /* nocost_off, */ options);
+          KheDebugTimetable(soln, "after rematch %d .. %d",
+	    frame_pos - p + 1, frame_pos);
+	  if( DEBUG8 )
+	    KheFrameAssertNoClashes(tso->frame);
+          KheResourceMatchingDemandSetClear(rmds);
+	  if( DEBUG3 )
+	    tso->time_matches += KheTimerElapsedTime(tso->st_matches);
+	}
+      } while( rematch_redo && KheSolnCost(soln) < init_cost );
+      KheResourceMatchingDemandSetDelete(rmds);
+    }
+    if( DEBUG11 )
+    {
+      fprintf(stderr, "  after time sweep of time group %d:\n", frame_pos);
+      KheFrameAssignmentsDebug(tso->frame, options, 2, stderr);
+    }
+  }
+  if( tso->daily_time_limit >= 0.0 )
+    KheOptionsDeleteTimer(options, daily_timer);
+
+  /* remove cutoffs, unless cutoff_off */
+  if( !tso->cutoff_off )
+  {
+    if( DEBUG3 )
+      KheTimerResetStartTime(tso->st_cutoffs);
+    KheMonitorFrameRemoveResourceMonitorCutoffs(mf);
+    if( DEBUG3 )
+      tso->time_cutoffs += KheTimerElapsedTime(tso->st_cutoffs);
+  }
+
+  /* if time permits, redo while improving (no cutoffs, no rematch) */
+  /* although redo no more than twice, because of diminishing returns */
+  if( !tso->redo_off && !KheOptionsTimeLimitReached(options) )
+  {
+    redo_count = 0;
+    do
+    {
+      init_cost = KheSolnCost(soln);
+      if( tso->daily_time_limit >= 0.0 )
+	daily_timer = KheOptionsAddTimer(options, "daily",
+	  tso->daily_time_limit/2);
+      for( frame_pos = 0;  frame_pos < tso->frame_len;  frame_pos++ )
+      {
+	/* daily time limit, if requested */
+	if( tso->daily_time_limit >= 0.0 )
+	  KheTimerResetStartTime(daily_timer);
+
+	/* get time group and demand set for frame_pos */
+	tg = KheFrameTimeGroup(tso->frame, frame_pos);
+	if( KheSaveDemandSets(tso->frame) )
+	  rmds = HaArray(demand_sets, frame_pos);
+	else
+	{
+	  rmds = KheResourceMatchingDemandSetMake(rms, tso->preserve_existing);
+	  KheResourceMatchingDemandSetAddTimeGroup(rmds, tg);
+	}
+
+	/* solve tg */
+	DebugStart(soln, options, tso->frame, frame_pos, frame_pos, "redo");
+	if( DEBUG3 )
+	  KheTimerResetStartTime(tso->st_matches);
+	if( DEBUG8 )
+	  KheFrameAssertNoClashes(tso->frame);
+	KheResourceMatchingSolverSolve(rms, rmds, tso->edge_adjust1_off,
+	  tso->edge_adjust2_off, tso->edge_adjust3_off, tso->edge_adjust4_off,
+	  tso->ejection_off, /* nocost_off, */ options);
+	if( DEBUG8 )
+	  KheFrameAssertNoClashes(tso->frame);
+	redo_count++;
+	if( !KheSaveDemandSets(tso->frame) )
+	  KheResourceMatchingDemandSetClear(rmds);
+	if( DEBUG3 )
+	  tso->time_matches += KheTimerElapsedTime(tso->st_matches);
+      }
+      if( tso->daily_time_limit >= 0.0 )
+	KheOptionsDeleteTimer(options, daily_timer);
+      if( DEBUG11 )
+      {
+	fprintf(stderr, "  after time sweep of time group %d:\n", frame_pos);
+	KheFrameAssignmentsDebug(tso->frame, options, 2, stderr);
+      }
+    } while( redo_count < 2 && KheSolnCost(soln) < init_cost &&
+	!KheOptionsTimeLimitReached(options) );
+  }
+
+  /* debug, reclaim all memory, and return */
+  KheResourceMatchingSolverDelete(rms);
+}
+
+
 /*****************************************************************************/
 /*                                                                           */
 /*  bool KheTimeSweepAssignResources(KHE_SOLN soln, KHE_RESOURCE_GROUP rg,   */
@@ -727,21 +1011,13 @@ static void DebugStart(KHE_SOLN soln, KHE_OPTIONS options, KHE_FRAME frame,
 /*  Carry out time sweep assignment of the resources of rg in soln.          */
 /*                                                                           */
 /*****************************************************************************/
-#define min(a, b) ((a) < (b) ? (a) : (b))
 
 bool KheTimeSweepAssignResources(KHE_SOLN soln, KHE_RESOURCE_GROUP rg,
   KHE_OPTIONS options)
 {
-  KHE_RESOURCE_MATCHING_SOLVER rms;  KHE_FRAME frame;  KHE_TIME_GROUP tg;
-  int lookahead, frame_pos, frame_len, init_len, rematch_max_groups;
-  int count, p, i, redo_count;  KHE_COST init_cost, global_init_cost;
-  bool edge_adjust1_off, edge_adjust2_off, edge_adjust3_off, edge_adjust4_off;
-  bool ejection_off, two_phase;
-  bool preserve_existing, /* nocost_off, */ cutoff_off, redo_off, rematch_off;
-  bool rematch_redo;  ARRAY_KHE_RESOURCE_MATCHING_DEMAND_SET demand_sets;
-  KHE_RESOURCE_MATCHING_DEMAND_SET rmds;  KHE_MONITOR_FRAME mf;
-  KHE_TIMER st_all, st_cutoffs, st_matches, daily_timer;  HA_ARENA a;
-  float time_all, time_cutoffs, time_matches, daily_time_limit;
+  struct khe_time_sweep_options_rec tsor;  KHE_RESOURCE_GROUP rg1, rg2;
+  KHE_COST global_init_cost;  bool two_phase;  KHE_TIMER st_all;  HA_ARENA a;
+  float time_all;
 
   if( DEBUG1 )
     fprintf(stderr, "[ KheTimeSweepAssignResources(soln, rg, options)\n");
@@ -753,268 +1029,70 @@ bool KheTimeSweepAssignResources(KHE_SOLN soln, KHE_RESOURCE_GROUP rg,
   KheDebugTimetable(soln, "at start of KheTimeSweepAssignResources");
 
   /* get an arena */
-  a = KheSolnArenaBegin(soln, false);
+  a = KheSolnArenaBegin(soln);
 
   /* initialize for debugging running time, if wanted */
   if( DEBUG3 )
   {
     st_all = KheTimerMake("ts_all", KHE_NO_TIME, a);
-    st_cutoffs = KheTimerMake("ts_cutoffs", KHE_NO_TIME, a);
-    st_matches = KheTimerMake("ts_matches", KHE_NO_TIME, a);
-    time_all = time_cutoffs = time_matches = 0.0;
+    tsor.st_cutoffs = KheTimerMake("ts_cutoffs", KHE_NO_TIME, a);
+    tsor.st_matches = KheTimerMake("ts_matches", KHE_NO_TIME, a);
   }
+  else
+    st_all = tsor.st_cutoffs = tsor.st_matches = NULL;
+  time_all = tsor.time_cutoffs = tsor.time_matches = 0.0;
 
-  /* get the options */
-  daily_time_limit = KheTimeFromString(KheOptionsGet(options,
-    "rs_time_sweep_daily_time_limit", "-"));
-  edge_adjust1_off = KheOptionsGetBool(options,
+  /* get the time sweep options */
+  tsor.frame = KheOptionsFrame(options, "gs_common_frame", soln);
+  tsor.frame_len = KheFrameTimeGroupCount(tsor.frame);
+  tsor.daily_time_limit = KheTimeFromString(KheOptionsGet(options,
+    "rs_time_sweep_daily_time_limit", "3"));
+  /* tsor.daily_timer = NULL; */
+  tsor.edge_adjust1_off = KheOptionsGetBool(options,
     "rs_time_sweep_edge_adjust1_off", false);
-  edge_adjust2_off = KheOptionsGetBool(options,
+  tsor.edge_adjust2_off = KheOptionsGetBool(options,
     "rs_time_sweep_edge_adjust2_off", false);
-  edge_adjust3_off = KheOptionsGetBool(options,
+  tsor.edge_adjust3_off = KheOptionsGetBool(options,
     "rs_time_sweep_edge_adjust3_off", false);
-  edge_adjust4_off = KheOptionsGetBool(options,
+  tsor.edge_adjust4_off = KheOptionsGetBool(options,
     "rs_time_sweep_edge_adjust4_off", false);
-  ejection_off = KheOptionsGetBool(options,
+  tsor.ejection_off = KheOptionsGetBool(options,
     "rs_time_sweep_ejection_off", false);
-  /* ***
-  nocost_off = KheOptionsGetBool(options,
-    "rs_time_sweep _nocost_off", false);
-  *** */
-  lookahead = KheOptionsGetInt(options, "rs_time_sweep_lookahead", 0);
-  preserve_existing = ! KheOptionsGetBool(options,
+  tsor.lookahead = KheOptionsGetInt(options,
+    "rs_time_sweep_lookahead", 0);
+  tsor.preserve_existing = ! KheOptionsGetBool(options,
     "rs_time_sweep_preserve_existing_off", false);
-  cutoff_off = KheOptionsGetBool(options, "rs_time_sweep_cutoff_off", false);
-  redo_off = KheOptionsGetBool(options, "rs_time_sweep_redo_off", false);
-  rematch_off = KheOptionsGetBool(options, "rs_time_sweep_rematch_off", false);
-  rematch_max_groups = KheOptionsGetInt(options,
-    "rs_time_sweep_rematch_max_groups", KheDefaultRematchMaxGroups(frame));
-  rematch_redo = false;
-  two_phase = KheOptionsGetBool(options, "rs_time_sweep_two_phase", false);
+  tsor.cutoff_off = KheOptionsGetBool(options,
+    "rs_time_sweep_cutoff_off", false);
+  tsor.redo_off = KheOptionsGetBool(options,
+    "rs_time_sweep_redo_off", false);
+  tsor.rematch_off = KheOptionsGetBool(options,
+    "rs_time_sweep_rematch_off", false);
+  tsor.rematch_max_groups = KheOptionsGetInt(options,
+    "rs_time_sweep_rematch_max_groups", KheDefaultRematchMaxGroups(tsor.frame));
+  two_phase = KheOptionsGetBool(options,
+    "rs_time_sweep_two_phase", false);
 
-  /* get the solver and frame */
-  rms = KheResourceMatchingSolverMake(soln, rg, a);
-  frame = KheOptionsFrame(options, "gs_common_frame", soln);
-  frame_len = KheFrameTimeGroupCount(frame);
+  /* do the actual solving */
   global_init_cost = KheSolnCost(soln);
-  if( DEBUG8 )
-    KheFrameAssertNoClashes(frame);
-
-  /* install cutoffs, if required */
-  if( !cutoff_off )
+  if( two_phase && KheClassifyResourcesByWorkload(soln, rg, &rg1, &rg2) )
   {
-    mf = KheMonitorFrameMake(frame, a);
-    KheMonitorFrameAddAndCutOffAllResourceMonitors(mf, rg, soln);
-    for( frame_pos = 0;  frame_pos < lookahead;  frame_pos++ )
-      KheMonitorFrameUpdateResourceMonitorCutoffs(mf, frame_pos);
-    KheMonitorFrameAddAndDetachTaskMonitors(mf, soln);
-    if( DEBUG5 )
-      KheMonitorFrameDebug(mf, 2, stderr);
+    KheTimeSweepDoAssignResources(soln, rg1, &tsor, options, a);
+    KheTimeSweepDoAssignResources(soln, rg2, &tsor, options, a);
   }
   else
-    mf = NULL;  /* keep compiler happy */
-
-  /* create timer for daily time limit, if required */
-  if( daily_time_limit >= 0.0 )
-  {
-    daily_timer = KheOptionsAddTimer(options, "daily", daily_time_limit);
-    if( DEBUG2 )
-      fprintf(stderr,
-	"  KheTimeSweepAssignResources daily time limit %.1f secs\n",
-        daily_time_limit);
-  }
-  else
-    daily_timer = NULL;
-
-  /* get demand sets up to just before position lookahead */
-  HaArrayInit(demand_sets, a);
-  init_len = min(lookahead, frame_len);
-  for( frame_pos = 0;  frame_pos < init_len;  frame_pos++ )
-  {
-    rmds = KheResourceMatchingDemandSetMake(rms, preserve_existing);
-    tg = KheFrameTimeGroup(frame, frame_pos);
-    KheResourceMatchingDemandSetAddTimeGroup(rmds, tg);
-    HaArrayAddLast(demand_sets, rmds);
-  }
-
-  /* do one solve for each time group of frame */
-  for( frame_pos = 0;  frame_pos < frame_len;  frame_pos++ )
-  {
-    DebugStart(soln, options, frame, frame_pos, frame_pos, "construct");
-    if( DEBUG8 )
-      KheFrameAssertNoClashes(frame);
-
-    /* start daily time limit, if requested */
-    if( daily_time_limit >= 0.0 )
-      KheTimerResetStartTime(daily_timer);
-
-    /* get demand set for position frame_pos + lookahead  */
-    if( frame_pos + lookahead < frame_len )
-    {
-      rmds = KheResourceMatchingDemandSetMake(rms, preserve_existing);
-      tg = KheFrameTimeGroup(frame, frame_pos + lookahead);
-      KheResourceMatchingDemandSetAddTimeGroup(rmds, tg);
-      HaArrayAddLast(demand_sets, rmds);
-    }
-
-    /* install cutoffs, unless cutoff_off */
-    if( !cutoff_off )
-    {
-      if( DEBUG3 )
-	KheTimerResetStartTime(st_cutoffs);
-      if( frame_pos + lookahead < frame_len )
-	KheMonitorFrameUpdateResourceMonitorCutoffs(mf, frame_pos + lookahead);
-      KheMonitorFrameAttachTaskMonitors(mf, frame_pos);
-      if( DEBUG3 )
-	time_cutoffs += KheTimerElapsedTime(st_cutoffs);
-    }
-
-    /* do the solve */
-    if( DEBUG3 )
-      KheTimerResetStartTime(st_matches);
-    DebugStart(soln, options, frame, frame_pos, frame_pos, "solve");
-    KheResourceMatchingSolverSolveWithLookahead(rms, &demand_sets, frame_pos,
-      min(frame_pos + lookahead, frame_len - 1), edge_adjust1_off,
-      edge_adjust2_off, edge_adjust3_off, edge_adjust4_off, ejection_off,
-      /* nocost_off, */ options);
-    DebugStart(soln, options, frame, frame_pos, frame_pos, "after-solve");
-    KheDebugTimetable(soln, "after solving %d", frame_pos);
-    if( DEBUG8 )
-      KheFrameAssertNoClashes(frame);
-
-    /* clear demand set for frame_pos, if not being saved */
-    if( !KheSaveDemandSets(frame) )
-    {
-      rmds = HaArray(demand_sets, frame_pos);
-      KheResourceMatchingDemandSetDelete(rmds);
-      HaArrayPut(demand_sets, frame_pos, NULL);
-    }
-    if( DEBUG3 )
-      time_matches += KheTimerElapsedTime(st_matches);
-
-    /* rematch, unless rematch_off */
-    if( !rematch_off && !KheOptionsTimeLimitReached(options) )
-    {
-      rmds = KheResourceMatchingDemandSetMake(rms, false);
-      count = min(rematch_max_groups, frame_pos + 1);
-      do
-      {
-	if( DEBUG4 )
-	  fprintf(stderr, "  and rematching 2 .. %d parts\n", count);
-	init_cost = KheSolnCost(soln);
-	for( p = 2;  p <= count && !KheOptionsTimeLimitReached(options);  p++ )
-	{
-	  for( i = 0;  i < p;  i++ )
-	  {
-	    tg = KheFrameTimeGroup(frame, frame_pos - i);
-	    KheResourceMatchingDemandSetAddTimeGroup(rmds, tg);
-	  }
-	  DebugStart(soln, options, frame, frame_pos - p + 1, frame_pos,
-	    "rematch");
-	  if( DEBUG3 )
-	    KheTimerResetStartTime(st_matches);
-	  if( DEBUG8 )
-	    KheFrameAssertNoClashes(frame);
-	  KheResourceMatchingSolverSolve(rms, rmds, edge_adjust1_off,
-	    edge_adjust2_off, edge_adjust3_off, edge_adjust4_off,
-	    ejection_off, /* nocost_off, */ options);
-          KheDebugTimetable(soln, "after rematch %d .. %d",
-	    frame_pos - p + 1, frame_pos);
-	  if( DEBUG8 )
-	    KheFrameAssertNoClashes(frame);
-          KheResourceMatchingDemandSetClear(rmds);
-	  if( DEBUG3 )
-	    time_matches += KheTimerElapsedTime(st_matches);
-	}
-      } while( rematch_redo && KheSolnCost(soln) < init_cost );
-      KheResourceMatchingDemandSetDelete(rmds);
-    }
-    if( DEBUG11 )
-    {
-      fprintf(stderr, "  after time sweep of time group %d:\n", frame_pos);
-      KheFrameAssignmentsDebug(frame, options, 2, stderr);
-    }
-  }
-  if( daily_time_limit > 0.0 )
-    KheOptionsDeleteTimer(options, daily_timer);
-
-  /* remove cutoffs, unless cutoff_off */
-  if( !cutoff_off )
-  {
-    if( DEBUG3 )
-      KheTimerResetStartTime(st_cutoffs);
-    KheMonitorFrameRemoveResourceMonitorCutoffs(mf);
-    if( DEBUG3 )
-      time_cutoffs += KheTimerElapsedTime(st_cutoffs);
-  }
-
-  /* if time permits, redo while improving (no cutoffs, no rematch) */
-  /* although redo no more than twice, because of diminishing returns */
-  if( !redo_off && !KheOptionsTimeLimitReached(options) )
-  {
-    redo_count = 0;
-    do
-    {
-      init_cost = KheSolnCost(soln);
-      if( daily_time_limit >= 0.0 )
-	daily_timer = KheOptionsAddTimer(options, "daily", daily_time_limit/2);
-      for( frame_pos = 0;  frame_pos < frame_len;  frame_pos++ )
-      {
-	/* daily time limit, if requested */
-	if( daily_time_limit >= 0.0 )
-	  KheTimerResetStartTime(daily_timer);
-
-	/* get time group and demand set for frame_pos */
-	tg = KheFrameTimeGroup(frame, frame_pos);
-	if( KheSaveDemandSets(frame) )
-	  rmds = HaArray(demand_sets, frame_pos);
-	else
-	{
-	  rmds = KheResourceMatchingDemandSetMake(rms, preserve_existing);
-	  KheResourceMatchingDemandSetAddTimeGroup(rmds, tg);
-	}
-
-	/* solve tg */
-	DebugStart(soln, options, frame, frame_pos, frame_pos, "redo");
-	if( DEBUG3 )
-	  KheTimerResetStartTime(st_matches);
-	if( DEBUG8 )
-	  KheFrameAssertNoClashes(frame);
-	KheResourceMatchingSolverSolve(rms, rmds, edge_adjust1_off,
-	  edge_adjust2_off, edge_adjust3_off, edge_adjust4_off,
-	  ejection_off, /* nocost_off, */ options);
-	if( DEBUG8 )
-	  KheFrameAssertNoClashes(frame);
-	redo_count++;
-	if( !KheSaveDemandSets(frame) )
-	  KheResourceMatchingDemandSetClear(rmds);
-	if( DEBUG3 )
-	  time_matches += KheTimerElapsedTime(st_matches);
-      }
-      if( daily_time_limit > 0.0 )
-	KheOptionsDeleteTimer(options, daily_timer);
-      if( DEBUG11 )
-      {
-	fprintf(stderr, "  after time sweep of time group %d:\n", frame_pos);
-	KheFrameAssignmentsDebug(frame, options, 2, stderr);
-      }
-    } while( redo_count < 2 && KheSolnCost(soln) < init_cost &&
-	!KheOptionsTimeLimitReached(options) );
-  }
-
-  /* debug, reclaim all memory, and return */
+    KheTimeSweepDoAssignResources(soln, rg, &tsor, options, a);
   if( DEBUG3 )
   {
     time_all += KheTimerElapsedTime(st_all);
     fprintf(stderr, "  timing:  all = %.1f, cutoffs = %.1f, matches = %.1f\n",
-      time_all, time_cutoffs, time_matches);
+      time_all, tsor.time_cutoffs, tsor.time_matches);
   }
-  if( DEBUG1 )
-    fprintf(stderr, "] KheTimeSweepAssignResources ret. %s (%.5f -> %.5f)\n",
-      KheSolnCost(soln) < global_init_cost ? "true" : "false",
-      KheCostShow(global_init_cost), KheCostShow(KheSolnCost(soln)));
-  KheResourceMatchingSolverDelete(rms);
+
+  /* give up the arena */
   KheSolnArenaEnd(soln, a);
+
+  /* debug print and exit */
   KheDebugTimetable(soln, "at end of KheTimeSweepAssignResources");
   if( DEBUG10 )
   {
@@ -1024,7 +1102,11 @@ bool KheTimeSweepAssignResources(KHE_SOLN soln, KHE_RESOURCE_GROUP rg,
   if( DEBUG11 )
   {
     fprintf(stderr, "  at end of time sweep:\n");
-    KheFrameAssignmentsDebug(frame, options, 2, stderr);
+    KheFrameAssignmentsDebug(tsor.frame, options, 2, stderr);
   }
+  if( DEBUG1 )
+    fprintf(stderr, "] KheTimeSweepAssignResources ret. %s (%.5f -> %.5f)\n",
+      KheSolnCost(soln) < global_init_cost ? "true" : "false",
+      KheCostShow(global_init_cost), KheCostShow(KheSolnCost(soln)));
   return KheSolnCost(soln) < global_init_cost;
 }
